@@ -46,7 +46,7 @@ done
 # ### APR's libtool. deferring to a second round of change...
 #
 
-libtoolize="`./build/PrintPath glibtoolize libtoolize`"
+libtoolize="`./build/PrintPath glibtoolize libtoolize libtoolize15`"
 
 if [ "x$libtoolize" = "x" ]; then
     echo "libtoolize not found in path"
@@ -56,10 +56,10 @@ fi
 $libtoolize --copy --automake
 
 ltpath="`dirname $libtoolize`"
-ltfile="`cd $ltpath/../share/aclocal ; pwd`"/libtool.m4
+ltfile=${LIBTOOL_M4-`cd $ltpath/../share/aclocal ; pwd`/libtool.m4}
 
 if [ ! -f $ltfile ]; then
-    echo "$ltfile not found"
+    echo "$ltfile not found (try setting the LIBTOOL_M4 environment variable)"
     exit 1
 fi
 
@@ -84,23 +84,23 @@ fi
 
 if test -n "$SKIP_DEPS"; then
   echo "Creating build-outputs.mk (no dependencies)..."
-  ./gen-make.py -s build.conf || gen_failed=1
+  /usr/bin/env python ./gen-make.py -s build.conf || gen_failed=1
 
   ### if apr and apr-util are not subdirs, then this fails. only do it
   ### for the release (from dist.sh; for now)
   if test -n "$RELEASE_MODE"; then
     echo "Creating MSVC files (no dependencies)..."
-    ./gen-make.py -t dsp -s build.conf || gen_failed=1
+    /usr/bin/env python ./gen-make.py -t dsp -s build.conf || gen_failed=1
   fi
 else
   echo "Creating build-outputs.mk..."
-  ./gen-make.py build.conf || gen_failed=1
+  /usr/bin/env python ./gen-make.py build.conf || gen_failed=1
 
   ### if apr and apr-util are not subdirs, then this fails. only do it
   ### for the release (from dist.sh; for now)
   if test -n "$RELEASE_MODE"; then
     echo "Creating MSVC files..."
-    ./gen-make.py -t dsp -s build.conf || gen_failed=1
+    /usr/bin/env python ./gen-make.py -t dsp -s build.conf || gen_failed=1
   fi
 fi
 
@@ -163,7 +163,6 @@ echo "./configure --enable-maintainer-mode"
 echo "./configure --disable-shared"
 echo "./configure --enable-maintainer-mode --disable-shared"
 echo ""
-echo "Note:  this build will create the Subversion shared libraries and a"
-echo "       command-line client.  If you wish to build a Subversion server,"
-echo "       you will need Apache 2.0.  See the INSTALL file for details."
+echo "Note:  If you wish to run a Subversion HTTP server, you will need"
+echo "Apache 2.0.  See the INSTALL file for details."
 echo ""
