@@ -24,51 +24,89 @@
 #include "JNIUtil.h"
 #include "Pool.h"
 
+/**
+ * Constructor
+ *
+ * @see Path::Path (const std::string &)
+ * @param path Path string
+ */
 Path::Path (const char * path)
 {
-init (path);
+    init (path);
 }
 
-Path::Path (const std::string & path) 
+/**
+ * Constructor that takes a string as parameter.
+ * The string is converted to subversion internal
+ * representation. The string is copied.
+ *
+ * @param path Path string
+ */
+Path::Path (const std::string & path)
 {
-init (path.c_str ());
+    init (path.c_str ());
 }
 
-Path::Path (const Path & path) 
+/**
+ * Copy constructor
+ *
+ * @param path Path to be copied
+ */
+Path::Path (const Path & path)
 {
-init (path.c_str ());
+    init (path.c_str ());
 }
 
+/**
+ * initialize the class
+ *
+ * @param path Path string
+ */
 void
 Path::init (const char * path)
 {
-	if(*path == 0)
-	{
-		m_path = "";
-	}
-	else
-	{
-		const char * int_path = svn_path_internal_style (path, JNIUtil::getRequestPool()->pool() );
+    if(*path == 0)
+    {
+        m_path = "";
+    }
+    else
+    {
+        m_error_occured = JNIUtil::preprocessPath(path, 
+            JNIUtil::getRequestPool()->pool() );
 
-		m_path = int_path;
-	}
+        m_path = path;
+    }
 }
 
+/**
+ * @return Path string
+ */
 const std::string &
 Path::path () const
 {
-return m_path;
+    return m_path;
 }
 
-const char * 
+/**
+ * @return Path string as c string
+ */
+const char *
 Path::c_str() const
 {
-return m_path.c_str ();
+    return m_path.c_str ();
 }
 
-Path& 
+/**
+ * Assignment operator
+ */
+Path&
 Path::operator=(const Path & path)
 {
-init (path.c_str ());
-return *this;
+    init (path.c_str ());
+    return *this;
+}
+
+svn_error_t *Path::error_occured() const
+{
+    return m_error_occured;
 }
