@@ -151,8 +151,16 @@ jobjectArray SVNClient::list(const char *url, Revision &revision, bool recurse)
         return NULL;
     }
 
+    Path urlPath(url);
+    svn_error_t *Err = urlPath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return NULL;
+    }
+    
     apr_hash_t *dirents;
-    svn_error_t *Err = svn_client_ls (&dirents, url, 
+    Err = svn_client_ls (&dirents, urlPath.c_str(), 
                          const_cast<svn_opt_revision_t*>(revision.revision ()),
                          recurse, ctx, requestPool.pool());
     if (Err == NULL)
@@ -260,13 +268,19 @@ jobjectArray SVNClient::status(const char *path, bool descend, bool onServer,
     {
         return NULL;
     }
+    Path checkedPath(path);
+    svn_error_t *Err = checkedPath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return NULL;
+    }
 
     rev.kind = svn_opt_revision_unspecified;
     statusBaton.pool = requestPool.pool();
-    const char *intPath = svn_path_internal_style (path, requestPool.pool());
 
-    svn_error_t *Err = svn_client_status (
-                             &youngest, intPath, &rev, statusReceiver, 
+    Err = svn_client_status (
+                             &youngest, checkedPath.c_str(), &rev, statusReceiver, 
                              &statusBaton, descend ? TRUE : FALSE,
                              getAll ? TRUE : FALSE,
                              onServer ? TRUE : FALSE,
@@ -341,9 +355,15 @@ jobject SVNClient::singleStatus(const char *path, bool onServer)
 
     rev.kind = svn_opt_revision_unspecified;
     statusBaton.pool = requestPool.pool();
-    const char *intPath = svn_path_internal_style (path, requestPool.pool());
+    Path intPath(path);
+    svn_error_t *Err = intPath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return NULL;
+    }
 
-    svn_error_t *Err = svn_client_status (&youngest, intPath, &rev, 
+    Err = svn_client_status (&youngest, intPath.c_str(), &rev, 
                              statusReceiver, &statusBaton,
                              FALSE,
                              TRUE,  // get_All
@@ -411,14 +431,20 @@ jobjectArray SVNClient::logMessages(const char *path, Revision &revisionStart,
         return NULL;
     }
 
-    m_lastPath = svn_path_internal_style (path, requestPool.pool());
-    Targets target (m_lastPath.c_str () );
     svn_client_ctx_t *ctx = getContext(NULL);
     if(ctx == NULL)
     {
         return NULL;
     }
-    svn_error_t *Err = svn_client_log (target.array (requestPool),
+    Targets target (path);
+    const apr_array_header_t *targets = target.array(requestPool);
+    svn_error_t *Err = target.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return NULL;
+    }
+    Err = svn_client_log (targets,
                         revisionStart.revision (),
                         revisionEnd.revision (),
                         discoverPaths,
@@ -488,7 +514,20 @@ jlong SVNClient::checkout(const char *moduleName, const char *destPath,
         return -1;
     }
 
-    m_lastPath = svn_path_internal_style (destPath, apr_pool);
+    Path url(moduleName);
+    Path path(destPath);
+    svn_error_t *Err = url.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return -1;
+    }
+    Err = path.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return -1;
+    }
     svn_revnum_t retval;
 
     svn_client_ctx_t *ctx = getContext(NULL);
@@ -497,8 +536,8 @@ jlong SVNClient::checkout(const char *moduleName, const char *destPath,
         return -1;
     }
 
-    svn_error_t *Err = svn_client_checkout (&retval, moduleName,
-                                 m_lastPath.c_str (),
+	Err = svn_client_checkout (&retval, url.c_str(),
+                                 path.c_str (),
                                  revision.revision (),
                                  recurse, ctx,
                                  apr_pool);
@@ -523,15 +562,21 @@ void SVNClient::remove(Targets &targets, const char *message, bool force)
     svn_client_commit_info_t *commit_info = NULL;
     Pool requestPool;
     apr_pool_t * apr_pool = requestPool.pool ();
-//    m_lastPath = path;
     svn_client_ctx_t *ctx = getContext(message);
     if(ctx == NULL)
     {
         return;
     }
+    const apr_array_header_t *targets2 = targets.array(requestPool);
+    svn_error_t *Err = targets.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
 
-    svn_error_t *Err = svn_client_delete (&commit_info, 
-                                          targets.array(requestPool), force,
+    Err = svn_client_delete (&commit_info, 
+                                          targets2, force,
                                           ctx, apr_pool);
     if(Err != NULL)
          JNIUtil::handleSVNError(Err);
@@ -549,14 +594,20 @@ void SVNClient::revert(const char *path, bool recurse)
         return;
     }
 
-    m_lastPath = svn_path_internal_style (path, apr_pool);
     svn_client_ctx_t *ctx = getContext(NULL);
-    Targets target (m_lastPath.c_str () );
+    Targets target (path);
+    const apr_array_header_t *targets = target.array(requestPool);
+    svn_error_t *Err = target.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
     if(ctx == NULL)
     {
         return;
     }
-    svn_error_t *Err = svn_client_revert (target.array(requestPool), recurse, 
+    Err = svn_client_revert (targets, recurse, 
                                           ctx, apr_pool);
 
     if(Err != NULL)
@@ -575,13 +626,19 @@ void SVNClient::add(const char *path, bool recurse)
         return;
     }
 
-    m_lastPath = svn_path_internal_style (path, apr_pool);
+    Path intPath(path);
+    svn_error_t *Err = intPath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
     svn_client_ctx_t *ctx = getContext(NULL);
     if(ctx == NULL)
     {
         return;
     }
-    svn_error_t *Err = svn_client_add (m_lastPath.c_str (), recurse, ctx, 
+    Err = svn_client_add (intPath.c_str (), recurse, ctx, 
                                        apr_pool);
 
     if(Err != NULL)
@@ -599,14 +656,20 @@ jlong SVNClient::update(const char *path, Revision &revision, bool recurse)
         return -1;
     }
 
-    m_lastPath = svn_path_internal_style (path, apr_pool);
-       svn_client_ctx_t *ctx = getContext(NULL);
+    Path intPath(path);
+    svn_error_t *Err = intPath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return -1;
+    }
+    svn_client_ctx_t *ctx = getContext(NULL);
     svn_revnum_t retval;
     if(ctx == NULL)
     {
         return -1;
     }
-    svn_error_t *Err = svn_client_update (&retval, m_lastPath.c_str (),
+    Err = svn_client_update (&retval, intPath.c_str (),
                                           revision.revision (),
                                           recurse,
                                           ctx,
@@ -626,13 +689,20 @@ jlong SVNClient::commit(Targets &targets, const char *message, bool recurse)
     Pool requestPool;
     apr_pool_t * apr_pool = requestPool.pool ();
     svn_client_commit_info_t *commit_info = NULL;
+    const apr_array_header_t *targets2 = targets.array(requestPool);
+    svn_error_t *Err = targets.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return NULL;
+    }
     svn_client_ctx_t *ctx = getContext(message);
     if(ctx == NULL)
     {
         return -1;
     }
-    svn_error_t *Err = svn_client_commit (&commit_info,
-                                          targets.array (requestPool),
+    Err = svn_client_commit (&commit_info,
+                                          targets2,
                                           !recurse, ctx, apr_pool);
     if(Err != NULL)
          JNIUtil::handleSVNError(Err);
@@ -660,8 +730,20 @@ void SVNClient::copy(const char *srcPath, const char *destPath,
         return;
     }
 
-    Path sourcePath = srcPath;
-    m_lastPath = svn_path_internal_style (destPath, apr_pool);
+    Path sourcePath(srcPath);
+    svn_error_t *Err = sourcePath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
+    Path destinationPath(destPath);
+    Err = destinationPath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
 
     svn_client_commit_info_t *commit_info = NULL;
        svn_client_ctx_t *ctx = getContext(message);
@@ -670,10 +752,10 @@ void SVNClient::copy(const char *srcPath, const char *destPath,
         return;
     }
 
-    svn_error_t *Err = svn_client_copy (&commit_info,
+    Err = svn_client_copy (&commit_info,
                              sourcePath.c_str (),
                              revision.revision(),
-                             m_lastPath.c_str (),
+                             destinationPath.c_str (),
                              ctx,
                              apr_pool);
     if(Err != NULL)
@@ -698,8 +780,22 @@ void SVNClient::move(const char *srcPath, const char *destPath,
         return;
     }
     svn_client_commit_info_t *commit_info = NULL;
-    Path sourcePath = srcPath;
-    m_lastPath = svn_path_internal_style (destPath, apr_pool);
+    Path sourcePath(srcPath);
+    svn_error_t *Err = sourcePath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
+
+    Path destinationPath(destPath);
+    Err = destinationPath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
+
     svn_client_ctx_t *ctx = getContext(message);
     if(ctx == NULL)
     {
@@ -707,10 +803,10 @@ void SVNClient::move(const char *srcPath, const char *destPath,
     }
 
 
-    svn_error_t *Err = svn_client_move (&commit_info,
+    Err = svn_client_move (&commit_info,
                                         sourcePath.c_str (),
                                         revision.revision (),
-                                        m_lastPath.c_str (),
+                                        destinationPath.c_str (),
                                         force,
                                         ctx,
                                         apr_pool);
@@ -728,9 +824,16 @@ void SVNClient::mkdir(Targets &targets, const char *message)
     {
         return;
     }
+    const apr_array_header_t *targets2 = targets.array(requestPool);
+    svn_error_t *Err = targets.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
 
-    svn_error_t *Err = svn_client_mkdir (&commit_info,
-                                         targets.array(requestPool),
+    Err = svn_client_mkdir (&commit_info,
+                                         targets2,
                                          ctx,
                                          apr_pool);
 
@@ -748,13 +851,20 @@ void SVNClient::cleanup(const char *path)
         JNIUtil::throwNullPointerException("path");
         return;
     }
-    m_lastPath = svn_path_internal_style (path, apr_pool);
+    Path intPath(path);
+    svn_error_t *Err = intPath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
+
     svn_client_ctx_t *ctx = getContext(NULL);
     if(ctx == NULL)
     {
         return;
     }
-    svn_error_t *Err = svn_client_cleanup (m_lastPath.c_str (), ctx, apr_pool);
+    Err = svn_client_cleanup (intPath.c_str (), ctx, apr_pool);
 
     if(Err != NULL)
          JNIUtil::handleSVNError(Err);
@@ -770,13 +880,19 @@ void SVNClient::resolved(const char *path, bool recurse)
         JNIUtil::throwNullPointerException("path");
         return;
     }
-    m_lastPath = svn_path_internal_style (path, apr_pool);
+    Path intPath(path);
+    svn_error_t *Err = intPath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
     svn_client_ctx_t *ctx = getContext(NULL);
     if(ctx == NULL)
     {
         return;
     }
-    svn_error_t *Err = svn_client_resolved (m_lastPath.c_str (),
+    Err = svn_client_resolved (intPath.c_str (),
                                             recurse,
                                             ctx,
                                             apr_pool);
@@ -801,16 +917,28 @@ jlong SVNClient::doExport(const char *srcPath, const char *destPath,
         JNIUtil::throwNullPointerException("destPath");
         return -1;
     }
-    Path sourcePath = srcPath;
-    m_lastPath = svn_path_internal_style (destPath, apr_pool);
+    Path sourcePath(srcPath);
+    svn_error_t *Err = sourcePath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return -1;
+    }
+    Path destinationPath(destPath);
+    Err = destinationPath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return -1;
+    }
     svn_revnum_t retval;
     svn_client_ctx_t *ctx = getContext(NULL);
     if(ctx == NULL)
     {
         return -1;
     }
-    svn_error_t *Err = svn_client_export (&retval, sourcePath.c_str (),
-                                          m_lastPath.c_str (),
+    Err = svn_client_export (&retval, sourcePath.c_str (),
+                                          destinationPath.c_str (),
                                           const_cast<svn_opt_revision_t*>(
                                                  revision.revision ()),
                                           force,
@@ -841,15 +969,29 @@ jlong SVNClient::doSwitch(const char *path, const char *url,
         JNIUtil::throwNullPointerException("url");
         return -1;
     }
-    m_lastPath = svn_path_internal_style (path, apr_pool);
+    Path intUrl(url);
+    svn_error_t *Err = intUrl.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return -1;
+    }
+    Path intPath(path);
+    Err = intPath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return -1;
+    }
+
     svn_revnum_t retval;
-       svn_client_ctx_t *ctx = getContext(NULL);
+    svn_client_ctx_t *ctx = getContext(NULL);
     if(ctx == NULL)
     {
         return -1;
     }
-    svn_error_t *Err = svn_client_switch (&retval, m_lastPath.c_str (),
-                                          url,
+    Err = svn_client_switch (&retval, intPath.c_str (),
+                                          intUrl.c_str(),
                                           revision.revision (),
                                           recurse,
                                           ctx,
@@ -878,7 +1020,21 @@ void SVNClient::doImport(const char *path, const char *url,
         JNIUtil::throwNullPointerException("url");
         return;
     }
-    m_lastPath = svn_path_internal_style (path, apr_pool);
+    Path intPath(path);
+    svn_error_t *Err = intPath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
+    Path intUrl(url);
+    Err = intUrl.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
+
     svn_client_commit_info_t *commit_info = NULL;
     svn_client_ctx_t *ctx = getContext(message);
     if(ctx == NULL)
@@ -886,9 +1042,9 @@ void SVNClient::doImport(const char *path, const char *url,
         return;
     }
 
-    svn_error_t *Err = svn_client_import (&commit_info,
-                                          m_lastPath.c_str (),
-                                          url,
+    Err = svn_client_import (&commit_info,
+                                          intPath.c_str (),
+                                          intUrl.c_str(),
                                           !recurse,
                                           ctx,
                                           apr_pool);
@@ -919,20 +1075,41 @@ void SVNClient::merge(const char *path1, Revision &revision1,
         return;
     }
     apr_pool_t * apr_pool = requestPool.pool ();
-    m_lastPath = svn_path_internal_style (localPath, apr_pool);
-    Path srcPath1 = path1;
+    Path intLocalPath(localPath);
+    svn_error_t *Err = intLocalPath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
+
+    Path srcPath1(path1);
+    Err = srcPath1.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
+
     Path srcPath2 = path2;
+    Err = srcPath2.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
+
     svn_client_ctx_t *ctx = getContext(NULL);
     if(ctx == NULL)
     {
         return;
     }
 
-    svn_error_t *Err = svn_client_merge (srcPath1.c_str (),
+    Err = svn_client_merge (srcPath1.c_str (),
                                          revision1.revision (),
                                          srcPath2.c_str (),
                                          revision2.revision (),
-                                         localPath,
+                                         intLocalPath.c_str(),
                                          recurse,
                                          FALSE,   // ignore_ancestry
                                          force,
@@ -962,7 +1139,13 @@ jobject SVNClient::propertyGet(jobject jthis, const char *path, const char *name
         return NULL;
     }
     apr_pool_t * apr_pool = requestPool.pool ();
-    m_lastPath = svn_path_internal_style (path, apr_pool);
+    Path intPath(path);
+    svn_error_t *Err = intPath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return NULL;
+    }
 
     Revision rev(Revision::START);
     svn_client_ctx_t *ctx = getContext(NULL);
@@ -972,9 +1155,9 @@ jobject SVNClient::propertyGet(jobject jthis, const char *path, const char *name
     }
 
     apr_hash_t *props;
-    svn_error_t *Err = svn_client_propget(&props,
+    Err = svn_client_propget(&props,
                                           name,
-                                          m_lastPath.c_str(),
+                                          intPath.c_str(),
                                           rev.revision(),
                                           FALSE,
                                           ctx,
@@ -1012,7 +1195,13 @@ jobjectArray SVNClient::properties(jobject jthis, const char *path)
         return NULL;
     }
     apr_pool_t * apr_pool = requestPool.pool ();
-    m_lastPath = svn_path_internal_style (path, apr_pool);
+    Path intPath(path);
+    svn_error_t *Err = intPath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return NULL;
+    }
 
     Revision rev(Revision::START);
     svn_client_ctx_t *ctx = getContext(NULL);
@@ -1021,8 +1210,8 @@ jobjectArray SVNClient::properties(jobject jthis, const char *path)
         return NULL;
     }
 
-    svn_error_t *Err = svn_client_proplist (&props,
-                                            m_lastPath.c_str (),
+    Err = svn_client_proplist (&props,
+                                            intPath.c_str (),
                                             rev.revision(),
                                             FALSE,
                                             ctx,
@@ -1228,6 +1417,21 @@ void SVNClient::diff(const char *target1, Revision &revision1,
     if(ctx == NULL)
         return;
 
+    Path intTarget1(target1);
+    svn_error_t *Err = intTarget1.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
+    Path intTarget2(target2);
+    Err = intTarget2.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
+
     apr_file_t *outfile = NULL;
     apr_status_t rv;
     rv = apr_file_open(&outfile, 
@@ -1245,13 +1449,11 @@ void SVNClient::diff(const char *target1, Revision &revision1,
     // we don't use any options
     options = svn_cstring_split ("", " \t\n\r", TRUE, requestPool.pool());
 
-    svn_error_t *Err = svn_client_diff (
+    Err = svn_client_diff (
                             options,    // options
-                            svn_path_internal_style (target1, 
-                                                     requestPool.pool()),
+                            intTarget1.c_str(),
                             revision1.revision(),
-                            svn_path_internal_style (target2, 
-                                                     requestPool.pool()) ,
+                            intTarget2.c_str(),
                             revision2.revision(),
                             recurse ? TRUE : FALSE,
                             TRUE,  // ignore_ancestry
@@ -1859,12 +2061,19 @@ jobject SVNClient::createJavaProperty(jobject jthis, const char *path,
 void SVNClient::propertySet(const char *path, const char *name, 
                             svn_string_t *value, bool recurse)
 {
-    svn_error_t * error = svn_client_propset (name, value, 
-                                svn_path_internal_style (path, 
-                                            JNIUtil::getRequestPool()->pool()),
+    Path intPath(path);
+    svn_error_t *Err = intPath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
+
+    Err = svn_client_propset (name, value, 
+                                intPath.c_str(),
                                 recurse, JNIUtil::getRequestPool()->pool());
-    if(error != NULL)
-        JNIUtil::handleSVNError(error);
+    if(Err!= NULL)
+        JNIUtil::handleSVNError(Err);
 }
 
 jbyteArray SVNClient::fileContent(const char *path, Revision &revision)
@@ -1875,6 +2084,14 @@ jbyteArray SVNClient::fileContent(const char *path, Revision &revision)
         JNIUtil::throwNullPointerException("path");
         return NULL;
     }
+    Path intPath(path);
+    svn_error_t *Err = intPath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return NULL;
+    }
+
     path = svn_path_internal_style (path, requestPool.pool());
 
     svn_stream_t *read_stream = NULL;
@@ -1885,10 +2102,8 @@ jbyteArray SVNClient::fileContent(const char *path, Revision &revision)
     // the server
     {
 
-        const char *ori_path = svn_path_internal_style(path, 
-                                                       requestPool.pool());
         const char *base_path;
-        svn_error_t *err = svn_wc_get_pristine_copy_path (ori_path,
+        svn_error_t *err = svn_wc_get_pristine_copy_path (intPath.c_str(),
                                &base_path,
                                requestPool.pool());
         if(err != NULL)
@@ -1920,18 +2135,16 @@ jbyteArray SVNClient::fileContent(const char *path, Revision &revision)
     // (not good)
     {
 
-        const char *ori_path = svn_path_internal_style(path, 
-                                                       requestPool.pool());
         apr_file_t *file = NULL;
         apr_finfo_t finfo;
-        apr_status_t apr_err = apr_stat(&finfo, ori_path,
+        apr_status_t apr_err = apr_stat(&finfo, intPath.c_str(),
                                    APR_FINFO_MIN, requestPool.pool());
         if(apr_err)
         {
             JNIUtil::handleAPRError(apr_err, _("open file"));
             return NULL;
         }
-        apr_err = apr_file_open(&file, ori_path, APR_READ, 0, 
+        apr_err = apr_file_open(&file, intPath.c_str(), APR_READ, 0, 
                                 requestPool.pool());
         if(apr_err)
         {
@@ -1951,7 +2164,7 @@ jbyteArray SVNClient::fileContent(const char *path, Revision &revision)
         svn_stringbuf_t *buf = svn_stringbuf_create("", requestPool.pool());
         read_stream = svn_stream_from_stringbuf(buf, requestPool.pool());
         svn_error_t *err = svn_client_cat (read_stream,
-                path, revision.revision(), ctx, requestPool.pool());
+                intPath.c_str(), revision.revision(), ctx, requestPool.pool());
         if(err != NULL)
         {
             JNIUtil::handleSVNError(err);
@@ -2073,7 +2286,13 @@ jobject SVNClient::revProperty(jobject jthis, const char *path,
         return NULL;
     }
     apr_pool_t * apr_pool = requestPool.pool ();
-    m_lastPath = svn_path_internal_style (path, apr_pool);
+    Path intPath(path);
+    svn_error_t *Err = intPath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return NULL;
+    }
 
     svn_client_ctx_t *ctx = getContext(NULL);
     if(ctx == NULL)
@@ -2083,11 +2302,11 @@ jobject SVNClient::revProperty(jobject jthis, const char *path,
     const char *URL;
     svn_string_t *propval;
     svn_revnum_t set_rev;
-    svn_error_t * error = svn_client_url_from_path (&URL, path, apr_pool);
+    Err = svn_client_url_from_path (&URL, intPath.c_str(), apr_pool);
 
-    if(error != SVN_NO_ERROR)
+    if(Err != SVN_NO_ERROR)
     {
-        JNIUtil::handleSVNError(error);
+        JNIUtil::handleSVNError(Err);
         return NULL;
     }
 
@@ -2099,12 +2318,12 @@ jobject SVNClient::revProperty(jobject jthis, const char *path,
         return NULL;
     }
 
-    error = svn_client_revprop_get (name, &propval,
+    Err = svn_client_revprop_get (name, &propval,
                                     URL, rev.revision(),
                                     &set_rev, ctx, apr_pool);
-    if(error != SVN_NO_ERROR)
+    if(Err != SVN_NO_ERROR)
     {
-        JNIUtil::handleSVNError(error);
+        JNIUtil::handleSVNError(Err);
         return NULL;
     }
     if(propval == NULL)
@@ -2132,7 +2351,30 @@ void SVNClient::relocate(const char *from, const char *to, const char *path,
         JNIUtil::throwNullPointerException("to");
         return;
     }
-    m_lastPath = svn_path_internal_style (path, apr_pool);
+    Path intPath(path);
+    svn_error_t *Err = intPath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
+
+    Path intFrom(from);
+    Err = intFrom.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
+
+    Path intTo(to);
+    Err = intTo.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
+
 
     svn_client_ctx_t *ctx = getContext(NULL);
     if(ctx == NULL)
@@ -2141,12 +2383,12 @@ void SVNClient::relocate(const char *from, const char *to, const char *path,
     }
 
 
-    svn_error_t * error = svn_client_relocate (path != NULL? path:"", from, to, 
+    Err = svn_client_relocate (intPath.c_str(), intFrom.c_str(), intTo.c_str(), 
                                                recurse, ctx, apr_pool);
 
-    if(error != SVN_NO_ERROR)
+    if(Err != SVN_NO_ERROR)
     {
-        JNIUtil::handleSVNError(error);
+        JNIUtil::handleSVNError(Err);
         return;
     }
 }
@@ -2177,7 +2419,13 @@ jbyteArray SVNClient::blame(const char *path, Revision &revisionStart,
         return NULL;
     }
     apr_pool_t * apr_pool = requestPool.pool ();
-    m_lastPath = svn_path_internal_style (path, apr_pool);
+    Path intPath(path);
+    svn_error_t *Err = intPath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return NULL;
+    }
 
     svn_client_ctx_t *ctx = getContext(NULL);
     if(ctx == NULL)
@@ -2186,16 +2434,16 @@ jbyteArray SVNClient::blame(const char *path, Revision &revisionStart,
     }
     svn_stringbuf_t *buf = svn_stringbuf_create("", apr_pool);
     svn_stream_t *read_stream = svn_stream_from_stringbuf(buf, apr_pool);
-    svn_error_t * error = svn_client_blame (path,
+    Err = svn_client_blame (intPath.c_str(),
                                             revisionStart.revision(),
                                             revisionEnd.revision(),
                                             blame_receiver,
                                             read_stream,
                                             ctx,
                                             apr_pool);
-    if(error != SVN_NO_ERROR)
+    if(Err != SVN_NO_ERROR)
     {
-        JNIUtil::handleSVNError(error);
+        JNIUtil::handleSVNError(Err);
         return NULL;
     }
     size_t size = buf->len;
@@ -2211,13 +2459,13 @@ jbyteArray SVNClient::blame(const char *path, Revision &revisionStart,
     {
         return NULL;
     }
-    svn_error_t *err = svn_stream_read (read_stream, (char *)retdata,
+    Err = svn_stream_read (read_stream, (char *)retdata,
                               &size);
 
-    if(err != NULL)
+    if(Err != NULL)
     {
         env->ReleaseByteArrayElements(ret, retdata, 0);
-        JNIUtil::handleSVNError(err);
+        JNIUtil::handleSVNError(Err);
         return NULL;
     }
     env->ReleaseByteArrayElements(ret, retdata, 0);
@@ -2250,23 +2498,29 @@ void SVNClient::blame(const char *path, Revision &revisionStart,
         return;
     }
     apr_pool_t * apr_pool = requestPool.pool ();
-    m_lastPath = svn_path_internal_style (path, apr_pool);
+    Path intPath(path);
+    svn_error_t *Err = intPath.error_occured();
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
+        return;
+    }
 
     svn_client_ctx_t *ctx = getContext(NULL);
     if(ctx == NULL)
     {
         return;
     }
-    svn_error_t * error = svn_client_blame (path,
+    Err = svn_client_blame (intPath.c_str(),
                                             revisionStart.revision(),
                                             revisionEnd.revision(),
                                             blame_receiver2,
                                             callback,
                                             ctx,
                                             apr_pool);
-    if(error != SVN_NO_ERROR)
+    if(Err != SVN_NO_ERROR)
     {
-        JNIUtil::handleSVNError(error);
+        JNIUtil::handleSVNError(Err);
         return;
     }
 }
@@ -2316,19 +2570,25 @@ jobject SVNClient::info(const char *path)
         return NULL;
     }
     apr_pool_t * apr_pool = requestPool.pool ();
-    const char *intpath = svn_path_internal_style (path, apr_pool);
-    
-    svn_error_t *err = svn_wc_adm_probe_open2(&adm_access, NULL, intpath, 
-        FALSE, 0, apr_pool);
-    if(err != NULL)
+    Path intPath(path);
+    svn_error_t *Err = intPath.error_occured();
+    if(Err != NULL)
     {
-        JNIUtil::handleSVNError(err);
+        JNIUtil::handleSVNError(Err);
         return NULL;
     }
-    err = svn_wc_entry(&entry, intpath, adm_access, FALSE, apr_pool);
-    if(err != NULL)
+    
+    Err = svn_wc_adm_probe_open2(&adm_access, NULL, intPath.c_str(), 
+        FALSE, 0, apr_pool);
+    if(Err != NULL)
     {
-        JNIUtil::handleSVNError(err);
+        JNIUtil::handleSVNError(Err);
+        return NULL;
+    }
+    Err = svn_wc_entry(&entry, intPath.c_str(), adm_access, FALSE, apr_pool);
+    if(Err != NULL)
+    {
+        JNIUtil::handleSVNError(Err);
         return NULL;
     }
     return createJavaInfo(entry);
@@ -2526,3 +2786,86 @@ jint SVNClient::mapScheduleKind(int schedule)
         return org_tigris_subversion_javahl_ScheduleKind_normal;
     }
 }
+struct version_status_baton
+{
+  svn_revnum_t min_rev;   /* lowest revision found. */
+  svn_revnum_t max_rev;   /* highest revision found. */
+  svn_boolean_t switched; /* is anything switched? */
+  svn_boolean_t modified; /* is anything modified? */
+  svn_boolean_t committed; /* examine last committed revisions */
+  svn_boolean_t done;     /* note completion of our task. */
+  const char *wc_path;    /* path whose URL we're looking for. */
+  const char *wc_url;     /* URL for the path whose URL we're looking for. */
+  apr_pool_t *pool;       /* pool in which to store alloc-needy things. */
+};
+
+/* This implements `svn_cancel_func_t'. */
+static svn_error_t *
+cancel (void *baton)
+{
+  struct version_status_baton *sb = (version_status_baton *)baton;
+  if (sb->done)
+    return svn_error_create (SVN_ERR_CANCELLED, NULL, "Finished");
+  else
+    return SVN_NO_ERROR;
+}
+
+/* An svn_wc_status_func_t callback function for anaylyzing status
+   structures. */
+static void
+analyze_status (void *baton,
+                const char *path,
+                svn_wc_status_t *status)
+{
+  struct version_status_baton *sb = (version_status_baton *)baton;
+  
+  if (sb->done)
+    return;
+
+  if (! status->entry)
+    return;
+
+  /* Added files have a revision of no interest */
+  if (status->text_status != svn_wc_status_added)
+    {
+      svn_revnum_t item_rev = (sb->committed
+                               ? status->entry->cmt_rev
+                               : status->entry->revision);
+
+      if (sb->min_rev == SVN_INVALID_REVNUM || item_rev < sb->min_rev)
+        sb->min_rev = item_rev;
+
+      if (sb->max_rev == SVN_INVALID_REVNUM || item_rev > sb->max_rev)
+        sb->max_rev = item_rev;
+    }
+
+  sb->switched |= status->switched;
+  sb->modified |= (status->text_status != svn_wc_status_normal);
+  sb->modified |= (status->prop_status != svn_wc_status_normal
+                   && status->prop_status != svn_wc_status_none);
+  
+  if (sb->wc_path 
+      && (! sb->wc_url) 
+      && (strcmp (path, sb->wc_path) == 0)
+      && (status->entry))
+    sb->wc_url = apr_pstrdup (sb->pool, status->entry->url);
+}
+
+
+/* This implements `svn_wc_notify_func_t'. */
+static void
+notify (void *baton,
+        const char *path,
+        svn_wc_notify_action_t action,
+        svn_node_kind_t kind,
+        const char *mime_type,
+        svn_wc_notify_state_t content_state,
+        svn_wc_notify_state_t prop_state,
+        svn_revnum_t revision)
+{
+  struct version_status_baton *sb = (version_status_baton *)baton;
+  if ((action == svn_wc_notify_status_external)
+      || (action == svn_wc_notify_status_completed))
+    sb->done = TRUE;
+}
+
