@@ -68,7 +68,9 @@ svn_cl__add (apr_getopt_t *os,
     {
       const char *target = ((const char **) (targets->elts))[i];
 
-      err = svn_client_add (target, (! opt_state->nonrecursive), ctx, subpool);
+      SVN_ERR (svn_cl__check_cancel (ctx->cancel_baton));
+      err = svn_client_add2 (target, (! opt_state->nonrecursive), 
+                             opt_state->force, ctx, subpool);
       if (err)
         {
           if (err->apr_err == SVN_ERR_ENTRY_EXISTS)
@@ -79,7 +81,6 @@ svn_cl__add (apr_getopt_t *os,
           else
             return err;
         }
-      SVN_ERR (svn_cl__check_cancel (ctx->cancel_baton));
       svn_pool_clear (subpool);
     }
 
