@@ -35,6 +35,7 @@
 
 #include "svn_wc.h"
 #include "svn_client.h"
+#include "svn_cmdline.h"
 #include "svn_string.h"
 #include "svn_path.h"
 #include "svn_delta.h"
@@ -50,34 +51,38 @@
 
 
 
-void
-svn_cl__print_commit_info (svn_client_commit_info_t *commit_info)
+svn_error_t *
+svn_cl__print_commit_info (svn_client_commit_info_t *commit_info,
+                           apr_pool_t *pool)
 {
   if ((commit_info) 
       && (SVN_IS_VALID_REVNUM (commit_info->revision)))
-    printf ("\nCommitted revision %" SVN_REVNUM_T_FMT ".\n",
-            commit_info->revision);
+    SVN_ERR (svn_cmdline_printf (pool, _("\nCommitted revision %ld.\n"),
+                                 commit_info->revision));
 
-  return;
+  return SVN_NO_ERROR;
 }
 
 
 svn_error_t *
-svn_cl__edit_externally (const char **edited_contents /* UTF-8! */,
+svn_cl__edit_externally (svn_string_t **edited_contents /* UTF-8! */,
                          const char **tmpfile_left /* UTF-8! */,
                          const char *editor_cmd,
                          const char *base_dir /* UTF-8! */,
-                         const char *contents /* UTF-8! */,
+                         const svn_string_t *contents /* UTF-8! */,
                          const char *prefix,
                          apr_hash_t *config,
+                         svn_boolean_t as_text,
+                         const char *encoding,
                          apr_pool_t *pool)
 {
   const char *editor = NULL;
   const char *cmd;
   apr_file_t *tmp_file;
   const char *tmpfile_name;
-  const char *contents_native, *tmpfile_native;
+  const char *tmpfile_native;
   const char *tmpfile_apr, *base_dir_apr;
+  svn_string_t *translated_contents;
   apr_status_t apr_err, apr_err2;
   apr_size_t written;
   apr_finfo_t finfo_before, finfo_after;
@@ -120,17 +125,34 @@ svn_cl__edit_externally (const char **edited_contents /* UTF-8! */,
   if (! editor)
     return svn_error_create 
       (SVN_ERR_CL_NO_EXTERNAL_EDITOR, NULL,
-       "None of the environment variables SVN_EDITOR, VISUAL or EDITOR is "
-       "set, and no 'editor-cmd' run-time configuration option was found");
+       _("None of the environment variables SVN_EDITOR, VISUAL or EDITOR is "
+         "set, and no 'editor-cmd' run-time configuration option was found"));
 
-  /* Convert file contents from UTF-8 */
-  SVN_ERR (svn_utf_cstring_from_utf8 (&contents_native, contents, pool));
+  /* Convert file contents from UTF-8/LF if desired. */
+  if (as_text)
+    {
+      const char *translated;
+      SVN_ERR (svn_subst_translate_cstring (contents->data, &translated,
+                                            APR_EOL_STR, FALSE,
+                                            NULL, FALSE, pool));
+      translated_contents = svn_string_create ("", pool);
+      if (encoding)
+        SVN_ERR (svn_utf_cstring_from_utf8_ex (&translated_contents->data,
+                                               translated, encoding, NULL,
+                                               pool));
+      else
+        SVN_ERR (svn_utf_cstring_from_utf8 (&translated_contents->data,
+                                            translated, pool));
+      translated_contents->len = strlen (translated_contents->data);
+    }
+  else
+    translated_contents = svn_string_dup (contents, pool);
 
   /* Move to BASE_DIR to avoid getting characters that need quoting
      into tmpfile_name */
   apr_err = apr_filepath_get (&old_cwd, APR_FILEPATH_NATIVE, pool);
   if (apr_err)
-    return svn_error_wrap_apr (apr_err, "Can't get working directory");
+    return svn_error_wrap_apr (apr_err, _("Can't get working directory"));
 
   /* APR doesn't like "" directories */
   if (base_dir[0] == '\0')
@@ -141,7 +163,7 @@ svn_cl__edit_externally (const char **edited_contents /* UTF-8! */,
   if (apr_err)
     {
       return svn_error_wrap_apr
-        (apr_err, "Can't change working directory to '%s'", base_dir);
+        (apr_err, _("Can't change working directory to '%s'"), base_dir);
     }
 
   /*** From here on, any problems that occur require us to cd back!! ***/
@@ -157,8 +179,8 @@ svn_cl__edit_externally (const char **edited_contents /* UTF-8! */,
        the file we just created!! ***/
 
   /* Dump initial CONTENTS to TMP_FILE. */
-  apr_err = apr_file_write_full (tmp_file, contents_native, 
-                                 strlen (contents_native), &written);
+  apr_err = apr_file_write_full (tmp_file, translated_contents->data,
+                                 translated_contents->len, &written);
 
   apr_err2 = apr_file_close (tmp_file);
   if (! apr_err)
@@ -167,7 +189,8 @@ svn_cl__edit_externally (const char **edited_contents /* UTF-8! */,
   /* Make sure the whole CONTENTS were written, else return an error. */
   if (apr_err)
     {
-      err = svn_error_wrap_apr (apr_err, "Can't write to '%s'", tmpfile_name);
+      err = svn_error_wrap_apr (apr_err, _("Can't write to '%s'"), 
+                                tmpfile_name);
       goto cleanup;
     }
 
@@ -181,7 +204,7 @@ svn_cl__edit_externally (const char **edited_contents /* UTF-8! */,
                       APR_FINFO_MTIME | APR_FINFO_SIZE, pool);
   if (apr_err)
     {
-      err = svn_error_wrap_apr (apr_err, "Can't stat '%s'", tmpfile_name);
+      err = svn_error_wrap_apr (apr_err, _("Can't stat '%s'"), tmpfile_name);
       goto cleanup;
     }
 
@@ -196,7 +219,7 @@ svn_cl__edit_externally (const char **edited_contents /* UTF-8! */,
       /* Extracting any meaning from sys_err is platform specific, so just
          use the raw value. */
       err =  svn_error_createf (SVN_ERR_EXTERNAL_PROGRAM, NULL,
-                                "system('%s') returned %d", cmd, sys_err);
+                                _("system('%s') returned %d"), cmd, sys_err);
       goto cleanup;
     }
   
@@ -205,7 +228,7 @@ svn_cl__edit_externally (const char **edited_contents /* UTF-8! */,
                       APR_FINFO_MTIME | APR_FINFO_SIZE, pool);
   if (apr_err)
     {
-      err = svn_error_wrap_apr (apr_err, "Can't stat '%s'", tmpfile_name);
+      err = svn_error_wrap_apr (apr_err, _("Can't stat '%s'"), tmpfile_name);
       goto cleanup;
     }
   
@@ -218,7 +241,16 @@ svn_cl__edit_externally (const char **edited_contents /* UTF-8! */,
       if (err)
         goto cleanup;
 
-      *edited_contents = edited_contents_s->data;
+      *edited_contents = svn_string_create_from_buf (edited_contents_s, pool);
+
+      /* Translate back to UTF8/LF if desired. */
+      if (as_text)
+        {
+          err = svn_subst_translate_string (edited_contents, *edited_contents,
+                                            encoding, pool);
+          if (err)
+            goto cleanup;
+        }
     }
   else
     {
@@ -252,7 +284,7 @@ svn_cl__edit_externally (const char **edited_contents /* UTF-8! */,
   if (apr_err)
     {
       svn_handle_error (svn_error_wrap_apr
-                        (apr_err, "Can't restore working directory"),
+                        (apr_err, _("Can't restore working directory")),
                         stderr, TRUE /* fatal */);
     }
 
@@ -290,7 +322,7 @@ svn_cl__make_log_msg_baton (void **baton,
              a deliberate encoding, and even if it is, we still
              can't handle it. */
           return svn_error_create (SVN_ERR_CL_BAD_LOG_MESSAGE, NULL,
-                                   "Log message contains a zero byte");
+                                   _("Log message contains a zero byte"));
         }
       lmb->message = opt_state->filedata->data;
     }      
@@ -348,7 +380,8 @@ svn_cl__cleanup_log_msg (void *log_msg_baton,
      svn_error_create (commit_err->apr_err,
                        svn_error_createf (commit_err->apr_err, NULL,
                                           "   '%s'", lmb->tmpfile_left),
-                       "Your commit message was left in a temporary file:"));
+                       _("Your commit message was left in "
+                         "a temporary file:")));
   return commit_err;
 }
 
@@ -399,7 +432,7 @@ truncate_buffer_at_prefix (apr_size_t *new_len,
 }
 
 
-#define EDITOR_EOF_PREFIX  "--This line, and those below, will be ignored--"
+#define EDITOR_EOF_PREFIX  _("--This line, and those below, will be ignored--")
 
 /* This function is of type svn_client_get_commit_log_t. */
 svn_error_t *
@@ -409,11 +442,15 @@ svn_cl__get_log_message (const char **log_msg,
                          void *baton,
                          apr_pool_t *pool)
 {
-  const char *default_msg =
-    APR_EOL_STR EDITOR_EOF_PREFIX APR_EOL_STR APR_EOL_STR;
+  svn_stringbuf_t *default_msg = NULL;
   struct log_msg_baton *lmb = baton;
   svn_stringbuf_t *message = NULL;
-  
+
+  /* Set default message.  */
+  default_msg = svn_stringbuf_create(APR_EOL_STR, pool);
+  svn_stringbuf_appendcstr(default_msg, EDITOR_EOF_PREFIX);
+  svn_stringbuf_appendcstr(default_msg, APR_EOL_STR APR_EOL_STR);
+
   *tmp_file = NULL;
   if (lmb->message)
     {
@@ -446,9 +483,9 @@ svn_cl__get_log_message (const char **log_msg,
          get one.  Note that svn_cl__edit_externally will still return
          a UTF-8'ized log message. */
       int i;
-      svn_stringbuf_t *tmp_message = svn_stringbuf_create (default_msg, pool);
+      svn_stringbuf_t *tmp_message = svn_stringbuf_dup (default_msg, pool);
       svn_error_t *err = SVN_NO_ERROR;
-      const char *msg2 = NULL;  /* ### shim for svn_cl__edit_externally */
+      svn_string_t *msg_string = svn_string_create ("", pool);
 
       for (i = 0; i < commit_items->nelts; i++)
         {
@@ -489,23 +526,14 @@ svn_cl__get_log_message (const char **log_msg,
           svn_stringbuf_appendcstr (tmp_message, APR_EOL_STR);
         }
 
-      /* Use the external edit to get a log message. */
-      err = svn_cl__edit_externally (&msg2, &lmb->tmpfile_left,
-                                     lmb->editor_cmd, lmb->base_dir,
-                                     tmp_message->data, "svn-commit",
-                                     lmb->config, pool);
+      msg_string->data = tmp_message->data;
+      msg_string->len = tmp_message->len;
 
-      /* Clean up the log message into UTF8/LF before giving it to
-         libsvn_client. */
-      if (msg2)
-        {
-          svn_string_t *new_logval = svn_string_create ("", pool);
-          new_logval->data = msg2;
-          new_logval->len = strlen (msg2);
-          SVN_ERR (svn_subst_translate_string (&new_logval, new_logval,
-                                               NULL, pool));
-          msg2 = new_logval->data;
-        }        
+      /* Use the external edit to get a log message. */
+      err = svn_cl__edit_externally (&msg_string, &lmb->tmpfile_left,
+                                     lmb->editor_cmd, lmb->base_dir,
+                                     msg_string, "svn-commit",
+                                     lmb->config, TRUE, NULL, pool);
 
       /* Dup the tmpfile path into its baton's pool. */
       *tmp_file = lmb->tmpfile_left = apr_pstrdup (lmb->pool, 
@@ -516,14 +544,14 @@ svn_cl__get_log_message (const char **log_msg,
         {
           if (err->apr_err == SVN_ERR_CL_NO_EXTERNAL_EDITOR)
             err = svn_error_quick_wrap 
-              (err, "Could not use external editor to fetch log message; "
-               "consider setting the $SVN_EDITOR environment variable "
-               "or using the --message (-m) or --file (-F) options");
+              (err, _("Could not use external editor to fetch log message; "
+                      "consider setting the $SVN_EDITOR environment variable "
+                      "or using the --message (-m) or --file (-F) options"));
           return err;
         }
 
-      if (msg2)
-        message = svn_stringbuf_create (msg2, pool);
+      if (msg_string)
+        message = svn_stringbuf_create_from_string (msg_string, pool);
 
       /* Strip the prefix from the buffer. */
       if (message)
@@ -551,8 +579,8 @@ svn_cl__get_log_message (const char **log_msg,
         {
           const char *reply;
           svn_cl__prompt_user (&reply,
-                               "\nLog message unchanged or not specified\n"
-                               "a)bort, c)ontinue, e)dit\n", pool);
+                               _("\nLog message unchanged or not specified\n"
+                                 "a)bort, c)ontinue, e)dit\n"), pool);
           if (reply)
             {
               char letter = apr_tolower (reply[0]);
