@@ -1,7 +1,7 @@
 /**
  * @copyright
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -33,6 +33,7 @@
 #include "svn_string.h"
 #include "svn_error.h"
 #include "svn_io.h"
+#include "svn_version.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -40,6 +41,13 @@ extern "C" {
 
 
 
+/**
+ * Get libsvn_delta version information.
+ * @since New in 1.1.
+ */
+const svn_version_t *svn_delta_version (void);
+
+
 /**  Text deltas.
  *
  * A text delta represents the difference between two strings of
@@ -110,7 +118,8 @@ enum svn_delta_action {
 };
 
 /** A single text delta instruction.  */
-typedef struct svn_txdelta_op_t {
+typedef struct svn_txdelta_op_t
+{
   enum svn_delta_action action_code;
   apr_size_t offset;
   apr_size_t length;
@@ -135,7 +144,8 @@ typedef struct svn_txdelta_op_t {
  * property allows us to apply deltas to non-seekable source streams
  * without making a full copy of the source stream.
  */
-typedef struct svn_txdelta_window_t {
+typedef struct svn_txdelta_window_t
+{
 
   /** The offset of the source view for this window.  */
   svn_filesize_t sview_offset;
@@ -216,6 +226,22 @@ void svn_txdelta (svn_txdelta_stream_t **stream,
                   apr_pool_t *pool);
 
 
+/** 
+ * @since New in 1.1.
+ *
+ * Return a writable stream which, when fed target data, will send
+ * delta windows to @a handler/@a handler_baton which transform the
+ * data in @a source to the target data.  As usual, the window handler
+ * will receive a NULL window to signify the end of the window stream.
+ * The stream handler functions will read data from @a source as
+ * necessary.
+ */
+svn_stream_t *svn_txdelta_target_push (svn_txdelta_window_handler_t handler,
+                                       void *handler_baton,
+                                       svn_stream_t *source,
+                                       apr_pool_t *pool);
+
+
 /** Send the contents of @a string to window-handler @a handler/@a baton. 
  * This is effectively a 'copy' operation, resulting in delta windows that 
  * make the target equivalent to the value of @a string.
@@ -233,7 +259,7 @@ svn_error_t *svn_txdelta_send_string (const svn_string_t *string,
  *
  * If @a digest is non-null, populate it with the md5 checksum for the
  * fulltext that was deltified (@a digest must be at least
- * @c MD5_DIGESTSIZE bytes long).
+ * @c APR_MD5_DIGESTSIZE bytes long).
  *
  * All temporary allocation is performed in @a pool.
  */
@@ -261,7 +287,7 @@ svn_error_t *svn_txdelta_send_txstream (svn_txdelta_stream_t *txstream,
  * @a *handler_baton is set to the value to pass as the @a baton argument to
  * @a *handler.
  *
- * If @a result_digest is non-null, it points to MD5_DIGESTSIZE bytes
+ * If @a result_digest is non-null, it points to APR_MD5_DIGESTSIZE bytes
  * of storage, and the final call to @a handler populates it with the
  * MD5 digest of the resulting fulltext.
  *
@@ -309,6 +335,35 @@ svn_stream_t *svn_txdelta_parse_svndiff (svn_txdelta_window_handler_t handler,
                                          svn_boolean_t error_on_early_close,
                                          apr_pool_t *pool);
 
+/**
+ * @since New in 1.1.
+ *
+ * Read and parse one delta window in svndiff format from the
+ * readable stream @a stream and place it in @a *window, allocating
+ * the result in @a pool.  The caller must take responsibility for
+ * stripping off the four-byte 'SVN<ver>' header at the beginning of
+ * the svndiff document before reading the first window, and must
+ * provide the version number (the value of the fourth byte) to each
+ * invocation of this routine with the @a svndiff_version argument. */
+svn_error_t *svn_txdelta_read_svndiff_window (svn_txdelta_window_t **window,
+                                              svn_stream_t *stream,
+                                              int svndiff_version,
+                                              apr_pool_t *pool);
+
+/**
+ * @since New in 1.1.
+ *
+ * Skip one delta window in svndiff format in the file @a file.  and
+ * place it in @a *window, allocating the result in @a pool.  The
+ * caller must take responsibility for stripping off the four-byte
+ * 'SVN<ver>' header at the beginning of the svndiff document before
+ * reading or skipping the first window, and must provide the version
+ * number (the value of the fourth byte) to each invocation of this
+ * routine with the @a svndiff_version argument. */
+svn_error_t *svn_txdelta_skip_svndiff_window (apr_file_t *file,
+                                              int svndiff_version,
+                                              apr_pool_t *pool);
+
 /** @} */
 
 
@@ -380,9 +435,10 @@ svn_stream_t *svn_txdelta_parse_svndiff (svn_txdelta_window_handler_t handler,
  * baton global to the entire delta edit.  If there is a target
  * revision that needs to be set for this operation, the producer
  * should called the @c set_target_revision function at this point.
- * Next, the producer should pass this @a edit_baton to the @c open_root
- * function, to get a baton representing root of the tree being
- * edited.
+ *
+ * Next, if there are any tree deltas to express, the producer should
+ * pass the @a edit_baton to the @c open_root function, to get a baton
+ * representing root of the tree being edited.
  *
  * Most of the callbacks work in the obvious way:
  *
@@ -518,8 +574,19 @@ svn_stream_t *svn_txdelta_parse_svndiff (svn_txdelta_window_handler_t handler,
  * passed to @c open_directory and @c add_directory definitely does not
  * have the proper lifetime). In general, it is recommended to simply
  * avoid keeping a parent directory baton in a file baton.
+ *
+ *
+ * <h3>Errors</h3>
+ *
+ * At least one implementation of the editor interface is
+ * asynchronous; an error from one operation may be detected some
+ * number of operations later.  As a result, an editor driver must not
+ * assume that an error from an editing function resulted from the
+ * particular operation being detected.  Moreover, once an editing
+ * function returns an error, the edit is dead; the only further
+ * operation which may be called on the editor is abort_edit.
  */
-typedef struct
+typedef struct svn_delta_editor_t
 {
   /** Set the target revision for this edit to @a target_revision.  This
    * call, if used, should precede all other editor calls.
@@ -617,6 +684,15 @@ typedef struct
                                    apr_pool_t *pool);
 
 
+  /** In the directory represented by @a parent_baton, indicate that
+   * @a path is present as a subdirectory in the edit source, but
+   * cannot be conveyed to the edit consumer (perhaps because of
+   * authorization restrictions).
+   */
+  svn_error_t *(*absent_directory) (const char *path,
+                                    void *parent_baton,
+                                    apr_pool_t *pool);
+
   /** We are going to add a new file named @a path.  The callback can
    * store a baton for this new file in @a **file_baton; whatever value
    * it stores there should be passed through to @c apply_textdelta.
@@ -710,6 +786,15 @@ typedef struct
                               const char *text_checksum,
                               apr_pool_t *pool);
 
+  /** In the directory represented by @a parent_baton, indicate that
+   * @a path is present as a file in the edit source, but cannot be
+   * conveyed to the edit consumer (perhaps because of authorization
+   * restrictions).
+   */
+  svn_error_t *(*absent_file) (const char *path,
+                               void *parent_baton,
+                               apr_pool_t *pool);
+
   /** All delta processing is done.  Call this, with the @a edit_baton for
    * the entire edit.
    */
@@ -751,7 +836,7 @@ svn_error_t *svn_delta_noop_window_handler (svn_txdelta_window_t *window,
 /** Return a cancellation editor that wraps @a wrapped_editor.
  *
  * The @a editor will call @a cancel_func with @a cancel_baton when each of 
- * it's functions are called, continuing on to call the corresponding wrapped 
+ * its functions is called, continuing on to call the corresponding wrapped 
  * function if it returns @c SVN_NO_ERROR.
  *
  * If @a cancel_func is @c NULL, @a *editor is set to @a wrapped_editor and 
