@@ -1,7 +1,7 @@
 /**
  * @copyright
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -37,6 +37,13 @@ extern "C" {
 #endif /* __cplusplus */
 
 
+/**
+ * Get libsvn_fs version information.
+ * @since New in 1.1.
+ */
+const svn_version_t *svn_fs_version (void);
+
+
 /* Opening and creating filesystems.  */
 
 
@@ -46,37 +53,29 @@ typedef struct svn_fs_t svn_fs_t;
 
 /** Filesystem configuration options. */
 #define SVN_FS_CONFIG_BDB_TXN_NOSYNC            "bdb-txn-nosync"
+#define SVN_FS_CONFIG_BDB_LOG_AUTOREMOVE        "bdb-log-autoremove"
 
-
-/** Create a new filesystem object in @a pool.
- *
- * It doesn't refer to any actual repository yet; you need to invoke
- * @c svn_fs_open_* or @c svn_fs_create_* on it for that to happen. If
- * @a fs_config is not @c NULL, the options it contains modify the
- * behaviour of the filesystem. The interpretation of @a fs_config is
- * specific to the filesystem back-end.
- *
- * @note The lifetime of @a fs_config must not be shorter than @a
- * pool's. It's a good idea to allocate @a fs_config from @a pool or
- * one of its ancestors.
- *
- * @note You probably don't want to use this directly, especially not
- * if it's followed immediately by a call to @c svn_fs_open_berkeley().
- * Take a look at @c svn_repos_open() instead.
- */
-svn_fs_t *svn_fs_new (apr_hash_t *fs_config, apr_pool_t *pool);
+/* @since New in 1.1. */
+#define SVN_FS_CONFIG_FS_TYPE                   "fs-type"
+/* @since New in 1.1. */
+#define SVN_FS_TYPE_BDB                         "bdb"
+/* @since New in 1.1. */
+#define SVN_FS_TYPE_FSFS                        "fsfs"
 
 
 /** The type of a warning callback function.  @a baton is the value specified
  * in the call to @c svn_fs_set_warning_func; the filesystem passes it through
- * to the callback.  @a msg contains the warning message.
+ * to the callback.  @a err contains the warning message.
+ *
+ * The callback function should not clear the error that is passed to it;
+ * its caller should do that.
  */
 typedef void (*svn_fs_warning_callback_t) (void *baton, svn_error_t *err);
 
 
 /** Provide a callback function, @a warning, that @a fs should use to 
  * report (non-fatal) errors.  To print an error, the filesystem will call
- * @a warning, passing it @a baton and the error.
+ * @a warning, passing it @a warning_baton and the error.
  *
  * By default, this is set to a function that will crash the process.
  * Dumping to @c stderr or <tt>/dev/tty</tt> is not acceptable default 
@@ -89,58 +88,94 @@ void svn_fs_set_warning_func (svn_fs_t *fs,
 
 
 
-/** Subversion filesystems based on Berkeley DB.
- * 
- * There are many possible ways to implement the Subversion filesystem
- * interface.  You could implement it directly using ordinary POSIX
- * filesystem operations; you could build it using an SQL server as a
- * back end; you could build it on RCS; and so on.
+
+/**
+ * @since New in 1.1.
  *
- * The functions on this page create filesystem objects that use
- * Berkeley DB (http://www.sleepycat.com) to store their data.
- * Berkeley DB supports transactions and recoverability, making it
- * well-suited for Subversion.
+ * Create a new, empty Subversion filesystem, stored in the directory
+ * @a path, and return a pointer to it in @a *fs_p.  @a path must not
+ * currently exist, but its parent must exist.  If @a fs_config is not
+ * @c NULL, the options it contains modify the behavior of the
+ * filesystem.  The interpretation of @a fs_config is specific to the
+ * filesystem back-end.  The new filesystem may be closed by
+ * destroying @a pool.
  *
- * A Berkeley DB ``environment'' is a Unix directory containing
- * database files, log files, backing files for shared memory buffers,
- * and so on --- everything necessary for a complex database
- * application.  Each Subversion filesystem lives in a single Berkeley
- * DB environment.
+ * @note The lifetime of @a fs_config must not be shorter than @a
+ * pool's. It's a good idea to allocate @a fs_config from @a pool or
+ * one of its ancestors.
  *
- * @defgroup svn_fs_bdb berkeley bd filesystems
- * @{
+ * If @a fs_config contains a value for @c SVN_FS_CONFIG_FS_TYPE, that
+ * value determines the filesystem type for the new filesystem.
+ * Currently defined values are:
+ *
+ *   SVN_FS_TYPE_BDB   Berkeley-DB implementation
+ *   SVN_FS_TYPE_FSFS  Native-filesystem implementation
+ *
+ * Otherwise, the BDB filesystem type is assumed.  Once the filesystem
+ * is created, its type will be recorded so that other functions will
+ * know how to operate on it.
  */
+svn_error_t *svn_fs_create (svn_fs_t **fs_p, const char *path,
+                            apr_hash_t *fs_config, apr_pool_t *pool);
 
-/** Create a new, empty Subversion filesystem, stored in a Berkeley DB
- * environment under @a path, a utf8-encoded path.  Make @a fs refer to 
- * this new filesystem.  @a fs provides the memory pool, warning function,
- * etc.  If @a path exists, it must be an empty directory.
- */
-svn_error_t *svn_fs_create_berkeley (svn_fs_t *fs, const char *path);
-
-
-/** Make @a fs refer to the Berkeley DB-based Subversion filesystem at
- * @a path.  @a path is utf8-encoded, and must refer to a file or directory
- * created by @c svn_fs_create_berkeley.
+/** 
+ * @since New in 1.1.
+ *
+ * Open a Subversion filesystem located in the directory @a path, and
+ * return a pointer to it in @a *fs_p.  If @a fs_config is not @c
+ * NULL, the options it contains modify the behavior of the
+ * filesystem.  The interpretation of @a fs_config is specific to the
+ * filesystem back-end.  The opened filesystem may be closed by
+ * destroying @a pool.
+ *
+ * @note The lifetime of @a fs_config must not be shorter than @a
+ * pool's. It's a good idea to allocate @a fs_config from @a pool or
+ * one of its ancestors.
  *
  * Only one thread may operate on any given filesystem object at once.
  * Two threads may access the same filesystem simultaneously only if
- * they open separate filesystem objects.  
+ * they open separate filesystem objects.
  *
- * NOTE: you probably don't want to use this directly, especially not
- * if it's immediately preceded by a call to @c svn_fs_new().  Take a
- * look at @c svn_repos_open() instead.
+ * NOTE: you probably don't want to use this directly.  Take a look at
+ * @c svn_repos_open() instead.
  */
-svn_error_t *svn_fs_open_berkeley (svn_fs_t *fs, const char *path);
+svn_error_t *svn_fs_open (svn_fs_t **fs_p, const char *path,
+                          apr_hash_t *config, apr_pool_t *pool);
 
-
-/** Return the utf8-encoded path to @a fs's repository, allocated in
- * @a pool.  Note: this is just what was passed to
- * @c svn_fs_create_berkeley() or @a svn_fs_open_berkeley() -- might be
- * absolute, might not.
+/** 
+ * @since New in 1.1.
+ *
+ * Return the path to @a fs's repository, allocated in @a pool.
+ * Note: this is just what was passed to @c svn_fs_create() or
+ * @a svn_fs_open() -- might be absolute, might not.
  */
-const char *svn_fs_berkeley_path (svn_fs_t *fs, apr_pool_t *pool);
+const char *svn_fs_path (svn_fs_t *fs, apr_pool_t *pool);
 
+/**
+ * @since New in 1.1.
+ *
+ * Delete the filesystem at @a path. */
+svn_error_t *svn_fs_delete_fs (const char *path, apr_pool_t *pool);
+
+/** 
+ * @since New in 1.1.
+ *
+ * Copy a possibly live Subversion filesystem from @a src_path to
+ * @a dest_path.  If @a clean is @c TRUE, perform cleanup on the
+ * source filesystem as part of the copy operation; currently, this
+ * means deleting copied, unused logfiles for a Berkeley DB source
+ * filesystem.
+ */
+svn_error_t *svn_fs_hotcopy (const char *src_path, const char *dest_path,
+                             svn_boolean_t clean, apr_pool_t *pool);
+
+/** Subversion filesystems based on Berkeley DB.
+ *
+ * The following functions are specific to Berkeley DB filesystems.
+ *
+ * @defgroup svn_fs_bdb berkeley db filesystems
+ * @{
+ */
 
 /** Register an error handling function for Berkeley DB error messages.
  * If a Berkeley DB error occurs, the filesystem will call @a handler
@@ -160,13 +195,6 @@ svn_error_t *svn_fs_set_berkeley_errcall (svn_fs_t *fs,
                                           void (*handler) (const char *errpfx,
                                                            char *msg));
 
-/** Delete the Berkeley DB-based filesystem @a path.  This deletes the
- * database files, log files, shared memory segments, etc.  @a path should
- * refer to a file or directory created by @c svn_fs_create_berkeley.
- */
-svn_error_t *svn_fs_delete_berkeley (const char *PATH, apr_pool_t *pool);
-
-
 /** Perform any necessary non-catastrophic recovery on a Berkeley
  * DB-based Subversion filesystem, stored in the environment @a path.
  * Do any necessary allocation within @a pool.
@@ -192,6 +220,55 @@ svn_error_t *svn_fs_delete_berkeley (const char *PATH, apr_pool_t *pool);
 svn_error_t *svn_fs_berkeley_recover (const char *path,
                                       apr_pool_t *pool);
 
+
+/** Set @a *logfiles to array of <tt>const char *</tt> log file names
+ * of Berkeley DB-based Subversion filesystem.
+ *
+ * If @a only_unused is used is @c TRUE, @a *logfiles will contain
+ * only the names of Berkeley DB log files still in use by the
+ * filesystem.  Otherwise, all log files (used and unused) are returned.
+ *
+ * This function wraps the Berkeley DB 'log_archive' function
+ * called by the db_archive binary.  Repository administrators may
+ * want to run this function periodically and delete the unused log
+ * files, as a way of reclaiming disk space.
+ */
+svn_error_t *svn_fs_berkeley_logfiles (apr_array_header_t **logfiles,
+                                       const char *path,
+                                       svn_boolean_t only_unused,
+                                       apr_pool_t *pool);
+
+
+/**
+ * The following functions are similar to their generic counterparts,
+ * but only work on Berkeley DB filesystems.
+ *
+ * @defgroup svn_fs_bdb_deprecated berkeley db filesystem compatibility
+ * @{
+ */
+
+/** @deprecated Provided for backward compatibility with the 1.0.0 API. */
+svn_fs_t *svn_fs_new (apr_hash_t *fs_config, apr_pool_t *pool);
+
+/** @deprecated Provided for backward compatibility with the 1.0.0 API. */
+svn_error_t *svn_fs_create_berkeley (svn_fs_t *fs, const char *path);
+
+/** @deprecated Provided for backward compatibility with the 1.0.0 API. */
+svn_error_t *svn_fs_open_berkeley (svn_fs_t *fs, const char *path);
+
+/** @deprecated Provided for backward compatibility with the 1.0.0 API. */
+const char *svn_fs_berkeley_path (svn_fs_t *fs, apr_pool_t *pool);
+
+/** @deprecated Provided for backward compatibility with the 1.0.0 API. */
+svn_error_t *svn_fs_delete_berkeley (const char *path, apr_pool_t *pool);
+
+/** @deprecated Provided for backward compatibility with the 1.0.0 API. */
+svn_error_t *svn_fs_hotcopy_berkeley (const char *src_path, 
+                                      const char *dest_path, 
+                                      svn_boolean_t clean_logs,
+                                      apr_pool_t *pool);
+/** @} */
+
 /** @} */
 
 
@@ -237,19 +314,16 @@ int svn_fs_compare_ids (const svn_fs_id_t *a, const svn_fs_id_t *b);
 
 /** Return non-zero IFF the nodes associated with @a id1 and @a id2 are
  * related, else return zero.  
- *
- * NOTE: While this might seem redundant in the presence of
- * @c svn_fs_compare_ids (looking for a return value != -1), it is
- * slightly faster to run if the equality case is not interesting to
- * you.
  */
-int svn_fs_check_related (const svn_fs_id_t *id1,
-                          const svn_fs_id_t *id2);
+svn_boolean_t svn_fs_check_related (const svn_fs_id_t *id1,
+                                    const svn_fs_id_t *id2);
 
 
-/** Parse the @a len bytes at @a data as a node revision @a id.  Return zero 
- * if the bytes are not a properly-formed @a id.  Allocate the parsed @a id 
- * in @a pool.
+/** @deprecated Provided for backward compatibility with the 1.0.0 API.
+ *
+ * NOTE: This function is not guaranteed to work with all filesystem
+ * types.  There is currently no un-deprecated equivalent; contact the
+ * Subversion developers if you have a need for it.
  */
 svn_fs_id_t *svn_fs_parse_id (const char *data, 
                               apr_size_t len,
@@ -350,9 +424,7 @@ typedef struct svn_fs_txn_t svn_fs_txn_t;
  * When committed, this transaction will create a new revision.
  *
  * Allocate the new transaction in @a pool; when @a pool is freed, the new
- * transaction will be closed (neither committed nor aborted).  You
- * can also close the transaction explicitly, using
- * @c svn_fs_close_txn.
+ * transaction will be closed (neither committed nor aborted).
  *
  *<pre>   >> Note: if you're building a txn for committing, you probably <<
  *   >> don't want to call this directly.  Instead, call            <<
@@ -382,11 +454,7 @@ svn_error_t *svn_fs_begin_txn (svn_fs_txn_t **txn_p,
  * the conflict in @a txn, with the same lifetime as @a txn;
  * otherwise, set @a *conflict_p to null.
  *
- * If the commit succeeds, it frees @a txn, and any temporary resources
- * it holds.  Any root objects (see below) referring to the root
- * directory of @a txn become invalid; performing any operation on them
- * other than closing them will produce an @c SVN_ERR_FS_DEAD_TRANSACTION
- * error.
+ * If the commit succeeds, @a txn is invalid.
  *
  * If the commit fails, @a txn is still valid; you can make more
  * operations to resolve the conflict, or call @c svn_fs_abort_txn to
@@ -400,18 +468,36 @@ svn_error_t *svn_fs_begin_txn (svn_fs_txn_t **txn_p,
  */
 svn_error_t *svn_fs_commit_txn (const char **conflict_p,
                                 svn_revnum_t *new_rev,
-                                svn_fs_txn_t *txn);
+                                svn_fs_txn_t *txn,
+                                apr_pool_t *pool);
 
 
-/** Abort the transaction @a txn.  Any changes made in @a txn are discarded,
- * and the filesystem is left unchanged.
+/** Abort the transaction @a txn.  Any changes made in @a txn are
+ * discarded, and the filesystem is left unchanged.  Use @a pool for
+ * any necessary allocations.
  *
- * If the abort succeeds, it frees @a txn, and any temporary resources
- * it holds.  Any root objects referring to @a txn's root directory
- * become invalid; performing any operation on them other than closing
- * them will produce an @c SVN_ERR_FS_DEAD_TRANSACTION error.
+ * Use @a pool for any necessary allocations.
+ *
+ * NOTE: This function first sets the state of the transaction to
+ * "dead", and then attempts to the purge the txn and any related data
+ * from the filesystem.  If some part of the cleanup process fails,
+ * the transaction and some portion of its data may remain in the
+ * database after this function returns.  Use @c svn_fs_purge_txn() to
+ * retry the transaction cleanup.
  */
-svn_error_t *svn_fs_abort_txn (svn_fs_txn_t *txn);
+svn_error_t *svn_fs_abort_txn (svn_fs_txn_t *txn,
+                               apr_pool_t *pool);
+
+
+/** Cleanup the dead transaction in @a fs whose ID is @a txn_id.  Use
+ * @a pool for all allocations.  If the transaction is not yet dead,
+ * the error @c SVN_ERR_FS_TRANSACTION_NOT_DEAD is returned.  (The
+ * caller probably forgot to abort the transaction, or the cleanup
+ * step of that abort failed for some reason.)
+ */
+svn_error_t *svn_fs_purge_txn (svn_fs_t *fs,
+                               const char *txn_id,
+                               apr_pool_t *pool);
 
 
 /** Set @a *name_p to the name of the transaction @a txn, as a
@@ -422,14 +508,6 @@ svn_error_t *svn_fs_txn_name (const char **name_p,
                               apr_pool_t *pool);
 
 
-/** Return the filesystem to which @a txn belongs.  */
-svn_fs_t *svn_fs_txn_fs (svn_fs_txn_t *txn);
-
-
-/** Return @a txn's pool.  */
-apr_pool_t *svn_fs_txn_pool (svn_fs_txn_t *txn);
-
-
 /** Return @a txn's base revision.  If @a txn's base root id is an mutable
  * node, return 0.
  */
@@ -444,9 +522,7 @@ svn_revnum_t svn_fs_txn_base_revision (svn_fs_txn_t *txn);
  * the error returned.
  *
  * Allocate the new transaction in @a pool; when @a pool is freed, the new
- * transaction will be closed (neither committed nor aborted).  You
- * can also close the transaction explicitly, using
- * @c svn_fs_close_txn.
+ * transaction will be closed (neither committed nor aborted).
  */
 svn_error_t *svn_fs_open_txn (svn_fs_txn_t **txn,
                               svn_fs_t *fs,
@@ -454,13 +530,6 @@ svn_error_t *svn_fs_open_txn (svn_fs_txn_t **txn,
                               apr_pool_t *pool);
 
 
-/** Close the transaction @a txn.  This is neither an abort nor a commit;
- * the state of the transaction so far is stored in the filesystem, to
- * be opened again later.
- */
-svn_error_t *svn_fs_close_txn (svn_fs_txn_t *txn);
-
-
 /** Set @a *names_p to an array of <tt>const char *</tt> @a ids which are the 
  * names of all the currently active transactions in the filesystem @a fs.
  * Allocate the array in @a pool.
@@ -546,26 +615,24 @@ void svn_fs_close_root (svn_fs_root_t *root);
 svn_fs_t *svn_fs_root_fs (svn_fs_root_t *root);
 
 
-/** Return true iff @a root is a transaction root.  */
-int svn_fs_is_txn_root (svn_fs_root_t *root);
+/** Return @c TRUE iff @a root is a transaction root.  */
+svn_boolean_t svn_fs_is_txn_root (svn_fs_root_t *root);
 
-/** Return true iff @a root is a revision root.  */
-int svn_fs_is_revision_root (svn_fs_root_t *root);
+/** Return @c TRUE iff @a root is a revision root.  */
+svn_boolean_t svn_fs_is_revision_root (svn_fs_root_t *root);
 
 
-/** If @a root is the root of a transaction, return a pointer to the name
- * of the transaction; otherwise, return zero.  The name is owned by
- * @a root, and will be freed when @a root is closed.
+/** If @a root is the root of a transaction, return the name of the
+ * transaction, allocated in @a pool; otherwise, return null.
  */
 const char *svn_fs_txn_root_name (svn_fs_root_t *root,
                                   apr_pool_t *pool);
 
 
 /** If @a root is the root of a revision, return the revision number.
- * Otherwise, it is the root of a transaction, so return the revision
- * on which the transaction is based.
+ * Otherwise, return @c SVN_INVALID_REVNUM.
  */
-svn_revnum_t svn_fs_root_revision (svn_fs_root_t *root);
+svn_revnum_t svn_fs_revision_root_revision (svn_fs_root_t *root);
 
 /** @} */
 
@@ -628,10 +695,10 @@ typedef struct svn_fs_path_change_t
   svn_fs_path_change_kind_t change_kind;
 
   /** were there text mods? */
-  int text_mod;
+  svn_boolean_t text_mod;
 
   /** were there property mods? */
-  int prop_mod;
+  svn_boolean_t prop_mod;
 
 } svn_fs_path_change_t;
 
@@ -662,14 +729,35 @@ svn_error_t *svn_fs_check_path (svn_node_kind_t *kind_p,
                                 apr_pool_t *pool);
 
 
-/** Allocate and return an array @a *revs of @c svn_revnum_t revisions in
- * which @a path under @a root was modified.  Use @a pool for all 
- * allocations.  The array of @a *revs are sorted in descending order.
- * All duplicates will also be removed.
- * 
- * If @a cross_copy_history is not set, this function will halt the
- * search for revisions in which @a path was changed when it detects
- * that the path was copied.
+/** An opaque node history object. */
+typedef struct svn_fs_history_t svn_fs_history_t;
+
+
+/** Set @a *history_p to an opaque node history object which
+ * represents @a path under @a root.  @a root must be a revision root.
+ * Use @a pool for all allocations.
+ */
+svn_error_t *svn_fs_node_history (svn_fs_history_t **history_p,
+                                  svn_fs_root_t *root,
+                                  const char *path,
+                                  apr_pool_t *pool);
+
+
+/** Set @a *prev_history_t to an opaque node history object which
+ * represents the previous (or "next oldest") interesting history
+ * location for the filesystem node represented by @a history, or @c
+ * NULL if no such previous history exists.  If @a cross_copies is @c
+ * FALSE, also return @c NULL if stepping backwards in history to @a
+ * prev_history_t would cross a filesystem copy operation.  
+ *
+ * NOTE: If this is the first call to svn_fs_history_prev() for the @a
+ * history object, it could return a history object whose location is
+ * the same as the original.  This will happen if the original
+ * location was an interested one (where the node was modified, or
+ * took place in a copy event).  This behavior allows looping callers
+ * to avoid the calling svn_fs_history_location() on the object
+ * returned by svn_fs_node_history(), and instead go ahead and begin
+ * calling svn_fs_history_prev().
  *
  * NOTE: This function uses node-id ancestry alone to determine
  * modifiedness, and therefore does NOT claim that in any of the
@@ -681,26 +769,35 @@ svn_error_t *svn_fs_check_path (svn_node_kind_t *kind_p,
  * @a root is a revision root based on revision X, and @a path was
  * modified in some revision(s) younger than X, those revisions
  * younger than X will not be included for @a path.  */
-svn_error_t *svn_fs_revisions_changed (apr_array_header_t **revs,
-                                       svn_fs_root_t *root,
-                                       const char *path,
-                                       int cross_copy_history,
-                                       apr_pool_t *pool);
+svn_error_t *svn_fs_history_prev (svn_fs_history_t **prev_history_p,
+                                  svn_fs_history_t *history,
+                                  svn_boolean_t cross_copies,
+                                  apr_pool_t *pool);
 
 
-/** Set @a *is_dir to non-zero iff @a path in @a root is a directory.
+/** Set @a *path and @a *revision to the path and revision,
+ * respectively, of the @a history object.  Use @a pool for all
+ * allocations. 
+ */
+svn_error_t *svn_fs_history_location (const char **path,
+                                      svn_revnum_t *revision,
+                                      svn_fs_history_t *history,
+                                      apr_pool_t *pool);
+                                      
+
+/** Set @a *is_dir to @c TRUE iff @a path in @a root is a directory.
  * Do any necessary temporary allocation in @a pool.
  */
-svn_error_t *svn_fs_is_dir (int *is_dir,
+svn_error_t *svn_fs_is_dir (svn_boolean_t *is_dir,
                             svn_fs_root_t *root,
                             const char *path,
                             apr_pool_t *pool);
 
 
-/** Set @a *is_file to non-zero iff @a path in @a root is a file.
+/** Set @a *is_file to @c TRUE iff @a path in @a root is a file.
  * Do any necessary temporary allocation in @a pool.
  */
-svn_error_t *svn_fs_is_file (int *is_file,
+svn_error_t *svn_fs_is_file (svn_boolean_t *is_file,
                              svn_fs_root_t *root,
                              const char *path,
                              apr_pool_t *pool);
@@ -730,6 +827,18 @@ svn_error_t *svn_fs_node_created_rev (svn_revnum_t *revision,
                                       const char *path,
                                       apr_pool_t *pool);
 
+/** Set @a *created_path to the path at which @a path under @a root was
+ * created.  Use @a pool for all allocations.  Callers may use this
+ * function in conjunction with svn_fs_node_created_rev() perform a
+ * reverse lookup of the mapping of (path, revision) -> node-id that
+ * svn_fs_node_id() performs.
+ */
+svn_error_t *svn_fs_node_created_path (const char **created_path,
+                                       svn_fs_root_t *root,
+                                       const char *path,
+                                       apr_pool_t *pool);
+
+
 /** Set @a *value_p to the value of the property named @a propname of 
  * @a path in @a root.  If the node has no property by that name, set 
  * @a *value_p to zero.  Allocate the result in @a pool.
@@ -775,7 +884,7 @@ svn_error_t *svn_fs_change_node_prop (svn_fs_root_t *root,
  * same.  Both paths must exist under their respective roots, and both
  * roots must be in the same filesystem.
  */
-svn_error_t *svn_fs_props_changed (int *changed_p,
+svn_error_t *svn_fs_props_changed (svn_boolean_t *changed_p,
                                    svn_fs_root_t *root1,
                                    const char *path1,
                                    svn_fs_root_t *root2,
@@ -870,90 +979,27 @@ svn_error_t *svn_fs_merge (const char **conflict_p,
                            apr_pool_t *pool);
 
 
-
-/** Compare the nodes @a root1:path1 and @a root2:path2, and determine if
- * they are "different".  Return the answer in @a is_different.
- *
- * We define two nodes to be "different" if:
- *
- *     - they are different node types, or
- *
- *     - if both files, they have different node-revision-ids, or 
- * 
- *     - if both dirs, they have different entry lists.
- *
- * (Note that there is a small chance of getting a false positive: two
- * different node-rev-ids don't *necessarily* have different contents.
- * But right now it's not worth doing byte-for-byte comparisons.  This
- * problem will go away when we have deltified storage.)
- */
-svn_error_t *svn_fs_is_different (int *is_different,
-                                  svn_fs_root_t *root1,
-                                  const char *path1,
-                                  svn_fs_root_t *root2,
-                                  const char *path2,
-                                  apr_pool_t *pool);
-
-
-
-/* Deltification of Storage.  */
-
-
-/** Examine the data associated with @a path under @a root, and offer the
- * filesystem a chance store that data in a deltified fashion.  @a root
- * is a revision root.
- *
- * If @a path represents a directory, deltify @a path's properties and list
- * of entries, and if @a recursive is non-zero, perform this operation
- * recursively on @a path's children.
- *
- * If @a path represents a file, deltify @a path's properties and text
- * contents (and ignore the @a recursive argument).
- *
- * Use @a pool for all necessary allocations.
- */
-svn_error_t *svn_fs_deltify (svn_fs_root_t *root,
-                             const char *path,
-                             int recursive,
-                             apr_pool_t *pool);
-
-
-/** Ensure that the data associated with @a path under @a root is stored as
- * fulltext (that is, in an undeltified fashion).  If this is already
- * the case, do nothing.  @a root is a revision root.
- *
- * If @a path represents a directory, un-deltify @a path's properties and 
- * list of entries, and if @a recursive is non-zero, perform this operation
- * recursively on @a path's children.
- *
- * If @a path represents a file, un-deltify @a path's properties and text
- * contents (and ignore the @a recursive argument).
- *
- * Use @a pool for all necessary allocations.
- */
-svn_error_t *svn_fs_undeltify (svn_fs_root_t *root,
-                               const char *path,
-                               int recursive,
-                               apr_pool_t *pool);
-
-
 
 /* Directories.  */
 
 
 /** The type of a Subversion directory entry.  */
-typedef struct svn_fs_dirent_t {
+typedef struct svn_fs_dirent_t
+{
 
   /** The name of this directory entry.  */
-  char *name;
+  const char *name;
 
   /** The node revision ID it names.  */
-  svn_fs_id_t *id;
+  const svn_fs_id_t *id;
+
+  /** The node kind. */
+  svn_node_kind_t kind;
 
 } svn_fs_dirent_t;
 
 
-/** Set @a *table_p to a newly allocated APR hash table containing the
+/** Set @a *entries_p to a newly allocated APR hash table containing the
  * entries of the directory at @a path in @a root.  The keys of the table
  * are entry names, as byte strings, excluding the final null
  * character; the table's values are pointers to @c svn_fs_dirent_t
@@ -976,21 +1022,6 @@ svn_error_t *svn_fs_make_dir (svn_fs_root_t *root,
                               apr_pool_t *pool);
                               
 
-/** Delete the node named @a path in @a root.  @a root must be the root of
- * a transaction, not a revision.  Do any necessary temporary allocation
- * in @a pool.
- *
- * If the node being deleted is a directory, it must be empty, else
- * the error @c SVN_ERR_DIR_NOT_EMPTY is returned.
- *
- * Attempting to remove the root dir also results in an error,
- * @c SVN_ERR_FS_ROOT_DIR, even if the dir is empty.
- */
-svn_error_t *svn_fs_delete (svn_fs_root_t *root,
-                            const char *path,
-                            apr_pool_t *pool);
-
-
 /** Delete the node named @a path in @a root.  If the node being deleted is
  * a directory, its contents will be deleted recursively.  @a root must be
  * the root of a transaction, not of a revision.  Use @a pool for
@@ -1006,20 +1037,12 @@ svn_error_t *svn_fs_delete (svn_fs_root_t *root,
  * If return @c SVN_ERR_FS_NO_SUCH_ENTRY, then the basename of @a path is
  * missing from its parent, that is, the final target of the deletion
  * is missing.
- */
-svn_error_t *svn_fs_delete_tree (svn_fs_root_t *root,
-                                 const char *path,
-                                 apr_pool_t *pool);
-
-
-/** Move the node named @a from to @a to, both in @a root.  @a root must be 
- * the root of a transaction, not a revision.
  *
- * Do any necessary temporary allocation in @a pool.
+ * Attempting to remove the root dir also results in an error,
+ * @c SVN_ERR_FS_ROOT_DIR, even if the dir is empty.
  */
-svn_error_t *svn_fs_rename (svn_fs_root_t *root,
-                            const char *from,
-                            const char *to,
+svn_error_t *svn_fs_delete (svn_fs_root_t *root,
+                            const char *path,
                             apr_pool_t *pool);
 
 
@@ -1036,10 +1059,11 @@ svn_error_t *svn_fs_rename (svn_fs_root_t *root,
  * difference (modulo node revision ID's) between copying @a from and
  * simply adding a reference to it.  So the operation takes place in
  * constant time.  However, there's no reason not to extend this to
- * mutable nodes --- it's just more code.)
+ * mutable nodes --- it's just more code.)  Further, @a to_root and @a
+ * from_root must represent the same filesystem.
  *
  * Note: to do a copy without preserving copy history, use
- * @c svn_fs_link().
+ * @c svn_fs_revision_link().
  *
  * Do any necessary temporary allocation in @a pool.
  */
@@ -1054,9 +1078,9 @@ svn_error_t *svn_fs_copy (svn_fs_root_t *from_root,
  * the PATH.  You cannot use @c svn_fs_copied_from() later to find out
  * where this copy came from.
  *
- * Use @c svn_fs_link() in situations where you don't care about the copy
- * history, and where @a to_path and @a from_path are the same, because it
- * is cheaper than @c svn_fs_copy().
+ * Use @c svn_fs_revision_link() in situations where you don't care
+ * about the copy history, and where @a to_path and @a from_path are
+ * the same, because it is cheaper than @c svn_fs_copy().
  */
 svn_error_t *svn_fs_revision_link (svn_fs_root_t *from_root,
                                    svn_fs_root_t *to_root,
@@ -1075,7 +1099,7 @@ svn_error_t *svn_fs_file_length (svn_filesize_t *length_p,
 
 
 /** Put the MD5 checksum of file @a path into @a digest, which points
- * to @c MD5_DIGESTSIZE bytes of storage.  Use @a pool only for temporary
+ * to @c APR_MD5_DIGESTSIZE bytes of storage.  Use @a pool only for temporary
  * allocations.
  *
  * If the filesystem does not have a prerecorded checksum for @a path,
@@ -1111,10 +1135,11 @@ svn_error_t *svn_fs_file_md5_checksum (unsigned char digest[],
                                        apr_pool_t *pool);
 
 
-/** Set @a *contents to a readable generic stream will yield the contents
- * of the file @a path in @a root.  Allocate the stream in @a pool.  You can
- * only use @a *contents for as long as the underlying filesystem is
- * open.  If @a path is not a file, return @c SVN_ERR_FS_NOT_FILE.
+/** Set @a *contents to a readable generic stream that will yield the
+ * contents of the file @a path in @a root.  Allocate the stream in 
+ * @a pool.  You can only use @a *contents for as long as the underlying
+ * filesystem is open.  If @a path is not a file, return 
+ * @c SVN_ERR_FS_NOT_FILE.
  *
  * If @a root is the root of a transaction, it is possible that the
  * contents of the file @a path will change between calls to
@@ -1220,7 +1245,7 @@ svn_error_t *svn_fs_apply_text (svn_stream_t **contents_p,
  * same.  Both paths must exist under their respective roots, and both
  * roots must be in the same filesystem. 
  */
-svn_error_t *svn_fs_contents_changed (int *changed_p,
+svn_error_t *svn_fs_contents_changed (svn_boolean_t *changed_p,
                                       svn_fs_root_t *root1,
                                       const char *path1,
                                       svn_fs_root_t *root2,
@@ -1242,6 +1267,18 @@ svn_error_t *svn_fs_youngest_rev (svn_revnum_t *youngest_p,
                                   apr_pool_t *pool);
 
 
+/** Deltify predecessors of paths modified in @a revision in
+ * filesystem @a fs.  Use @a pool for all allocations. 
+ * 
+ * NOTE:  This can be a time-consuming process, depending the breadth
+ * of the changes made in @a revision, and the depth of the history of
+ * those changed paths. 
+ */
+svn_error_t *svn_fs_deltify_revision (svn_fs_t *fs,
+                                      svn_revnum_t revision,
+                                      apr_pool_t *pool);
+
+
 /** Set @a *value_p to the value of the property named @a propname on
  * revision @a rev in the filesystem @a fs.  If @a rev has no property by 
  * that name, set @a *value_p to zero.  Allocate the result in @a pool.
@@ -1298,30 +1335,28 @@ svn_error_t *svn_fs_change_rev_prop (svn_fs_t *fs,
  * Allocate @a *stream_p, and do any necessary temporary allocation, in
  * @a pool.
  */
-svn_error_t *
-svn_fs_get_file_delta_stream (svn_txdelta_stream_t **stream_p,
-                              svn_fs_root_t *source_root,
-                              const char *source_path,
-                              svn_fs_root_t *target_root,
-                              const char *target_path,
-                              apr_pool_t *pool);
+svn_error_t *svn_fs_get_file_delta_stream (svn_txdelta_stream_t **stream_p,
+                                           svn_fs_root_t *source_root,
+                                           const char *source_path,
+                                           svn_fs_root_t *target_root,
+                                           const char *target_path,
+                                           apr_pool_t *pool);
 
 
 
 /* UUID manipulation. */
 
-/** Populate @a *uuid with the UUID associated with @a fs. */
-svn_error_t *
-svn_fs_get_uuid(svn_fs_t *fs,
-                const char **uuid,
-                apr_pool_t *pool);
+/** Populate @a *uuid with the UUID associated with @a fs.  Allocate
+    @a *uuid in @a pool.  */
+svn_error_t *svn_fs_get_uuid (svn_fs_t *fs,
+                              const char **uuid,
+                              apr_pool_t *pool);
 
 
-/** Associate @a *uuid with @a fs. */
-svn_error_t *
-svn_fs_set_uuid(svn_fs_t *fs,
-                const char *uuid,
-                apr_pool_t *pool);
+/** Associate @a *uuid with @a fs.  Use @a pool for any scratchwork. */
+svn_error_t *svn_fs_set_uuid (svn_fs_t *fs,
+                              const char *uuid,
+                              apr_pool_t *pool);
 
 
 /* Non-historical properties.  */
