@@ -1,7 +1,7 @@
 /**
  * @copyright
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -39,15 +39,6 @@ extern "C" {
 #endif /* __cplusplus */
 
 
-
-/* ### NOTE: Although all svn_io functions claim to conversion from/to
-   UTF-8, in fact they only do this if Subversion was configured with
-   the `--enable-utf8' flag.  We expect (?) this flag to go away
-   soon, because UTF-8 can't really be optional if it's to be
-   depended on for interoperability.  The flag is only there so that
-   development of the UTF-8 code doesn't affect others until said
-   code is ready for prime time.  */
-
 
 /** Determine the @a kind of @a path.
  *
@@ -68,6 +59,17 @@ svn_error_t *svn_io_check_path (const char *path,
                                 svn_node_kind_t *kind,
                                 apr_pool_t *pool);
 
+/**
+ * @since New in 1.1.
+ *
+ * Like svn_io_check_path(), but also set *is_special to @c TRUE if
+ * the path is not a normal file.
+ */
+svn_error_t *svn_io_check_special_path (const char *path,
+                                        svn_node_kind_t *kind,
+                                        svn_boolean_t *is_special,
+                                        apr_pool_t *pool);
+
 /** Like svn_io_check_path(), but resolve symlinks.  This returns the
     same varieties of @a kind as svn_io_check_path(). */ 
 svn_error_t *svn_io_check_resolved_path (const char *path,
@@ -86,7 +88,7 @@ svn_error_t *svn_io_check_resolved_path (const char *path,
  *
  * The first attempt will just append @a suffix.  If the result is not
  * a unique name, then subsequent attempts will append a dot,
- * followed by an iteration number ("1", then "2", and so on),
+ * followed by an iteration number ("2", then "3", and so on),
  * followed by the suffix.  For example, if @a path is
  *
  *    tests/t1/A/D/G/pi
@@ -128,6 +130,36 @@ svn_error_t *svn_io_open_unique_file (apr_file_t **f,
                                       svn_boolean_t delete_on_close,
                                       apr_pool_t *pool);
 
+/**
+ * @since New in 1.1.
+ *
+ * Like svn_io_open_unique_file, except that instead of creating a
+ * file, a symlink is generated that references the path @a dest.
+ */
+svn_error_t *svn_io_create_unique_link (const char **unique_name_p,
+                                        const char *path,
+                                        const char *dest,
+                                        const char *suffix,
+                                        apr_pool_t *pool);
+
+
+/**
+ * @since New in 1.1.
+ *
+ * Set @a dest to the path that the symlink at @a path references.
+ * Allocate the string from @a pool.
+ */
+svn_error_t *svn_io_read_link (svn_string_t **dest,
+                               const char *path,
+                               apr_pool_t *pool);
+
+
+/** Set @a dir to a directory path (allocated in @a pool) deemed
+ * usable for the creation of temporary files and subdirectories.
+ */
+svn_error_t *svn_io_temp_dir (const char **dir,
+                              apr_pool_t *pool);
+
 
 /** Copy @a src to @a dst atomically.  Overwrite @a dst if it exists, else
  * create it.  Both @a src and @a dst are utf8-encoded filenames.  If
@@ -138,6 +170,18 @@ svn_error_t *svn_io_copy_file (const char *src,
                                svn_boolean_t copy_perms,
                                apr_pool_t *pool);
 
+/** 
+ * @since New in 1.1.
+ *
+ * Copy symbolic link @a src to @a dst atomically.  Overwrite @a dst
+ * if it exists, else create it.  Both @a src and @a dst are
+ * utf8-encoded filenames.  After copying, the @a dst link will point
+ * to the same thing @a src does.
+ */
+svn_error_t *svn_io_copy_link (const char *src,
+                               const char *dst,
+                               apr_pool_t *pool);
+
 
 /** Recursively copy directory @a src into @a dst_parent, as a new entry named
  * @a dst_basename.  If @a dst_basename already exists in @a dst_parent, 
@@ -199,13 +243,29 @@ svn_error_t *svn_io_set_file_read_only (const char *path,
 /** Make a file as writable as the operating system allows.
  * @a path is the utf8-encoded path to the file.  If @a ignore_enoent is
  * @c TRUE, don't fail if the target file doesn't exist.
+ * @warning On Unix this function will do the equivlanet of chmod a+w path.
+ * If this is not what you want you should not use this function, but rather
+ * use apr_file_perms_set().
  */
 svn_error_t *svn_io_set_file_read_write (const char *path,
                                          svn_boolean_t ignore_enoent,
                                          apr_pool_t *pool);
 
-/** Toggle a file's "executability", as much as the operating system
- * allows.  @a path is the utf8-encoded path to the file.  If @a executable
+/** Toggle a file's "executability".
+ *
+ * When making the file executable on operating systems with unix style
+ * permissions, never add an execute permission where there is not
+ * already a read permission: that is, only make the file executable
+ * for the user, group or world if the corresponding read permission
+ * is already set for user, group or world.
+ *
+ * When making the file non-executable on operating systems with unix style
+ * permissions, remove all execute permissions.
+ *
+ * On other operating systems, toggle the file's "executability" as much as
+ * the operating system allows.
+ *
+ * @a path is the utf8-encoded path to the file.  If @a executable
  * is @c TRUE, then make the file executable.  If @c FALSE, make in
  * non-executable.  If @a ignore_enoent is @c TRUE, don't fail if the target
  * file doesn't exist.
@@ -234,8 +294,9 @@ svn_error_t *svn_io_is_file_executable(svn_boolean_t *executable,
  *
  * When the file is out of lines, @c APR_EOF will be returned.
  */
-apr_status_t
-svn_io_read_length_line (apr_file_t *file, char *buf, apr_size_t *limit);
+svn_error_t *
+svn_io_read_length_line (apr_file_t *file, char *buf, apr_size_t *limit,
+                         apr_pool_t *pool);
 
 
 /** Set @a *apr_time to the time of last modification of the contents of the
@@ -248,6 +309,17 @@ svn_error_t *svn_io_file_affected_time (apr_time_t *apr_time,
                                         const char *path,
                                         apr_pool_t *pool);
 
+/** Set the timestamp of file @a path to @a apr_time.  @a path is
+ *  utf8-encoded.
+ *
+ * Note: this is the APR mtime which corresponds to the traditional mtime
+ * on Unix, and the last write time on Windows.
+ */
+svn_error_t *svn_io_set_file_affected_time (apr_time_t apr_time,
+                                            const char *path,
+                                            apr_pool_t *pool);
+
+
 
 /** Set @a *different_p to non-zero if @a file1 and @a file2 have different
  * sizes, else set to zero.  Both @a file1 and @a file2 are utf8-encoded.
@@ -265,7 +337,7 @@ svn_error_t *svn_io_filesizes_different_p (svn_boolean_t *different_p,
 
 
 /** Put the md5 checksum of @a file into @a digest.
- * @a digest points to @c MD5_DIGESTSIZE bytes of storage.
+ * @a digest points to @c APR_MD5_DIGESTSIZE bytes of storage.
  * Use @a pool only for temporary allocations.
  */
 svn_error_t *svn_io_file_checksum (unsigned char digest[],
@@ -273,6 +345,67 @@ svn_error_t *svn_io_file_checksum (unsigned char digest[],
                                    apr_pool_t *pool);
 
 
+/* Set @a *same to non-zero if @a file1 and @a file2 have the same
+ * contents, else set it to zero.  Use @a pool for temporary allocations.
+ */
+svn_error_t *svn_io_files_contents_same_p (svn_boolean_t *same,
+                                           const char *file1,
+                                           const char *file2,
+                                           apr_pool_t *pool);
+
+/** Create file at @a file with contents @a contents.
+ * will be created.  Path @a file is utf8-encoded.
+ * Use @a pool for memory allocations.
+ */
+svn_error_t *svn_io_file_create (const char *file,
+                                 const char *contents,
+                                 apr_pool_t *pool);
+
+/**
+ * @deprecated Provided for backward compatibility with the 1.0.0 API.
+ *
+ * Lock file at @a lock_file. If @exclusive is TRUE,
+ * obtain exclusive lock, otherwise obtain shared lock.
+ * Lock will be automaticaly released when @a pool is cleared or destroyed.
+ * Use @a pool for memory allocations.
+ */
+svn_error_t *svn_io_file_lock (const char *lock_file,
+                               svn_boolean_t exclusive,
+                               apr_pool_t *pool);
+
+/**
+ * @since New in 1.1.
+ *
+ * Lock file at @a lock_file. If @exclusive is TRUE,
+ * obtain exclusive lock, otherwise obtain shared lock.
+ *
+ * If @a nonblocking is TRUE, do not wait for the lock if it
+ * is not available: throw an error instead.
+ *
+ * Lock will be automaticaly released when @a pool is cleared or destroyed.
+ * Use @a pool for memory allocations.
+ */
+svn_error_t *svn_io_file_lock2 (const char *lock_file,
+                                svn_boolean_t exclusive,
+                                svn_boolean_t nonblocking,
+                                apr_pool_t *pool);
+/**
+ * @since New in 1.1.
+ *
+ * Flush any unwritten data from @a file to disk.  Use @a pool for
+ *  memory allocations.
+ */
+svn_error_t *svn_io_file_flush_to_disk (apr_file_t *file,
+                                        apr_pool_t *pool);
+
+/** Copy file @a file from location @a src_path to location @a dest_path.
+ * Use @a pool for memory allocations.
+ */
+svn_error_t *svn_io_dir_file_copy (const char *src_path, 
+                                   const char *dest_path, 
+                                   const char *file,
+                                   apr_pool_t *pool);
+
 
 /** Generic byte-streams
  *
@@ -393,18 +526,30 @@ svn_error_t *svn_stream_printf (svn_stream_t *stream,
                                 ...)
        __attribute__ ((format(printf, 3, 4)));
 
-/** Allocate @a *stringbuf in @a pool, and read one line from @a stream 
- * into it. The '\\n' is read from the stream, but is not added to the end 
- * of the stringbuf.  Instead, the stringbuf ends with a usual '\\0'.
+/** Allocate @a *stringbuf in @a pool, and read into it one line (terminated
+ * by @a eol) from @a stream. The line-terminator is read from the stream,
+ * but is not added to the end of the stringbuf.  Instead, the stringbuf
+ * ends with a usual '\\0'.
  *
- * If @a stream runs out of bytes before encountering a '\\n', then set
- * @a *stringbuf to @c NULL and return no error.
+ * If @a stream runs out of bytes before encountering a line-terminator,
+ * then set @a *eof to @c TRUE, otherwise set @a *eof to FALSE.
  */
 svn_error_t *
 svn_stream_readline (svn_stream_t *stream,
                      svn_stringbuf_t **stringbuf,
+                     const char *eol,
+                     svn_boolean_t *eof,
                      apr_pool_t *pool);
 
+/**
+ * @since New in 1.1.
+ *
+ * Read the contents of the readable stream @a from and write them to the
+ * writable stream @a to.
+ */
+svn_error_t *svn_stream_copy (svn_stream_t *from, svn_stream_t *to,
+                              apr_pool_t *pool);
+
 /** @} */
 
 /** Sets @a *result to a string containing the contents of @a filename, a
@@ -462,7 +607,9 @@ typedef svn_error_t * (*svn_io_walk_func_t) (void *baton,
  * rooted at @a dirname, a utf8-encoded path. For each file or directory,
  * @a walk_func is invoked, passing in the @a walk_baton, the utf8-encoded
  * full path to the entry, an @c apr_finfo_t structure, and a temporary
- * pool for allocations.
+ * pool for allocations.  For any directory, @a walk_func will be invoked
+ * on the directory itself before being invoked on any subdirectories or
+ * files within the directory.
  *
  * The set of information passed to @a walk_func is specified by @a wanted,
  * and the items specified by @c APR_FINFO_TYPE and @c APR_FINFO_NAME.
@@ -523,7 +670,7 @@ svn_error_t *svn_io_run_cmd (const char *path,
  */
 svn_error_t *svn_io_run_diff (const char *dir,
                               const char *const *user_args,
-                              const int num_user_args,
+                              int num_user_args,
                               const char *label1,
                               const char *label2,
                               const char *from,
@@ -594,6 +741,54 @@ svn_io_file_open (apr_file_t **new_file, const char *fname,
                   apr_pool_t *pool);
 
 
+/** Wrapper for @c apr_file_close(), which see. */
+svn_error_t *
+svn_io_file_close (apr_file_t *file, apr_pool_t *pool);
+
+
+/** Wrapper for @c apr_file_getc(), which see. */
+svn_error_t *
+svn_io_file_getc (char *ch, apr_file_t *file, apr_pool_t *pool);
+
+
+/** Wrapper for @c apr_file_info_get(), which see. */
+svn_error_t *
+svn_io_file_info_get (apr_finfo_t *finfo, apr_int32_t wanted, 
+                      apr_file_t *file, apr_pool_t *pool);
+
+
+/** Wrapper for @c apr_file_read(), which see. */
+svn_error_t *
+svn_io_file_read (apr_file_t *file, void *buf, 
+                  apr_size_t *nbytes, apr_pool_t *pool);
+
+
+/** Wrapper for @c apr_file_read_full(), which see. */
+svn_error_t *
+svn_io_file_read_full (apr_file_t *file, void *buf, 
+                       apr_size_t nbytes, apr_size_t *bytes_read,
+                       apr_pool_t *pool);
+
+
+/** Wrapper for @c apr_file_seek(), which see. */
+svn_error_t *
+svn_io_file_seek (apr_file_t *file, apr_seek_where_t where, 
+                  apr_off_t *offset, apr_pool_t *pool);
+
+
+/** Wrapper for @c apr_file_write(), which see. */
+svn_error_t *
+svn_io_file_write (apr_file_t *file, const void *buf, 
+                   apr_size_t *nbytes, apr_pool_t *pool);
+
+
+/** Wrapper for @c apr_file_write_full(), which see. */
+svn_error_t *
+svn_io_file_write_full (apr_file_t *file, const void *buf, 
+                        apr_size_t nbytes, apr_size_t *bytes_written,
+                        apr_pool_t *pool);
+
+
 /** Wrapper for @c apr_stat(), which see.  @a fname is utf8-encoded. */
 svn_error_t *
 svn_io_stat (apr_finfo_t *finfo, const char *fname,
@@ -618,6 +813,19 @@ svn_error_t *
 svn_io_dir_make_hidden (const char *path, apr_fileperms_t perm,
                         apr_pool_t *pool);
 
+/**
+ * @since New in 1.1.
+ *
+ * Same as svn_io_dir_make, but attempts to set the sgid on the
+ * directory on systems that support it.  Does not return an error if
+ * the attempt to set the sgid bit fails.  On Unix filesystems,
+ * setting the sgid bit on a directory ensures that files and
+ * subdirectories created within inherit group ownership from the
+ * parent instead of from the primary gid. */
+svn_error_t *
+svn_io_dir_make_sgid (const char *path, apr_fileperms_t perm,
+                      apr_pool_t *pool);
+
 /** Wrapper for @c apr_dir_open(), which see.  @a dirname is utf8-encoded. */
 svn_error_t *
 svn_io_dir_open (apr_dir_t **new_dir, const char *dirname, apr_pool_t *pool);
@@ -643,14 +851,6 @@ svn_io_dir_read (apr_finfo_t *finfo,
                  apr_pool_t *pool);
 
 
-/** Wrapper for @c apr_file_printf(), which see.  @a format is a utf8-encoded
- * string after it is formatted, so this function can convert it to
- * native encoding before printing.
- */
-svn_error_t *
-svn_io_file_printf (apr_file_t *fptr, const char *format, ...);
-
-
 
 /** Version/format files. 
  *
@@ -668,7 +868,8 @@ svn_io_read_version_file (int *version, const char *path, apr_pool_t *pool);
 
 /** Create (or overwrite) the file at @a path with new contents,
  * formatted as a non-negative integer @a version followed by a single
- * newline.  Use @a pool for all allocations.
+ * newline.  On successful completion the file will be read-only.  Use
+ * @a pool for all allocations.
  */
 svn_error_t *
 svn_io_write_version_file (const char *path, int version, apr_pool_t *pool);
