@@ -1,7 +1,7 @@
 /**
  * @copyright
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -18,7 +18,8 @@
  * @file svn_path.h
  * @brief A path manipulation library
  *
- * All incoming and outgoing paths are in UTF-8.
+ * All incoming and outgoing paths are non-null and in UTF-8, unless
+ * otherwise documented.
  * 
  * No result path ever ends with a separator, no matter whether the
  * path is a file or directory, because we always canonicalize() it.
@@ -69,11 +70,12 @@ const char *svn_path_local_style (const char *path, apr_pool_t *pool);
  * use this function for constructing URLs, or for relative URLs or
  * repository paths.
  *
- * This function is NOT appropriate for native (local) file paths. Only
- * for "internal" paths, since it uses '/' for the separator. Further,
- * an absolute path (for @a component) is based on a leading '/' character.
- * Thus, an "absolute URI" for the @a component won't be detected. An
- * absolute URI can only be used for the base.
+ * This function is NOT appropriate for native (local) file
+ * paths. Only for "internal" canonicalized paths, since it uses '/'
+ * for the separator. Further, an absolute path (for @a component) is
+ * based on a leading '/' character.  Thus, an "absolute URI" for the
+ * @a component won't be detected. An absolute URI can only be used
+ * for the base.
  */
 char *svn_path_join (const char *base,
                      const char *component,
@@ -92,10 +94,11 @@ char *svn_path_join (const char *base,
 char *svn_path_join_many (apr_pool_t *pool, const char *base, ...);
 
 
-/** Get the basename of the specified @a path.  The basename is defined as
- * the last component of the path (ignoring any trailing slashes).  If
- * the @a path is root ("/"), then that is returned.  Otherwise, the
- * returned value will have no slashes in it.
+/** Get the basename of the specified canonicalized @a path.  The
+ * basename is defined as the last component of the path (ignoring any
+ * trailing slashes).  If the @a path is root ("/"), then that is
+ * returned.  Otherwise, the returned value will have no slashes in
+ * it.
  *
  * Example: svn_path_basename("/foo/bar") -> "bar"
  *
@@ -105,18 +108,23 @@ char *svn_path_join_many (apr_pool_t *pool, const char *base, ...);
  */
 char *svn_path_basename (const char *path, apr_pool_t *pool);
 
-/** Get the dirname of the specified @a path, defined as the path with its
- * basename removed.
+/** Get the dirname of the specified canonicalized @a path, defined as
+ * the path with its basename removed.
  *
  * Get the dirname of the specified @a path, defined as the path with its
  * basename removed.  If @a path is root ("/"), it is returned unchanged.
  *
- * The returned basename will be allocated in @a pool.
+ * The returned dirname will be allocated in @a pool.
  */
 char *svn_path_dirname (const char *path, apr_pool_t *pool);
 
-/** Add a @a component (a null-terminated C-string) to @a path.  @a component 
- * is allowed to contain directory separators.
+/** Return the number of components in the canonicalized @a path. */
+apr_size_t
+svn_path_component_count (const char *path);
+
+/** Add a @a component (a null-terminated C-string) to the
+ * canonicalized @a path.  @a component is allowed to contain
+ * directory separators.
  *
  * If @a path is non-empty, append the appropriate directory separator
  * character, and then @a component.  If @a path is empty, simply set it to
@@ -127,11 +135,15 @@ char *svn_path_dirname (const char *path, apr_pool_t *pool);
 void svn_path_add_component (svn_stringbuf_t *path, 
                              const char *component);
 
-/** Remove one component off the end of @a path. */
+/** Remove one component off the end of the canonicalized @a path. */
 void svn_path_remove_component (svn_stringbuf_t *path);
 
+/** Remove @a n components off the end of the canonizalized @a path.
+ * Equivalent to calling @c svn_remove_component @a n times. */
+void svn_path_remove_components (svn_stringbuf_t *path, apr_size_t n);
 
-/** Divide @a path into @a *dirpath and @a *base_name, allocated in @a pool.
+/** Divide the canonicalized @a path into @a *dirpath and @a
+ * *base_name, allocated in @a pool.
  *
  * If @a dirpath or @a base_name is null, then don't set that one.
  *
@@ -144,7 +156,7 @@ void svn_path_remove_component (svn_stringbuf_t *path);
  *   examples:
  *             - <pre>"/foo/bar/baz"  ==>  "/foo/bar" and "baz"</pre>
  *             - <pre>"/bar"          ==>  "/"  and "bar"</pre>
- *             - <pre>"/"             ==>  "/"  and ""</pre>
+ *             - <pre>"/"             ==>  "/"  and "/"</pre>
  *             - <pre>"bar"           ==>  ""   and "bar"</pre>
  *             - <pre>""              ==>  ""   and ""</pre>
  */
@@ -161,12 +173,16 @@ void svn_path_split (const char *path,
 int svn_path_is_empty (const char *path);
 
 
-/** Return a new path like @a path, but with any trailing separators that don't
- * affect @a path's meaning removed. Will convert a "." path to "".  Allocate
- * the new path in @a pool if anything changed, else just return @a path.
+/** Return a new path (or URL) like @a path, but transformed such that
+ * some types of path specification redundancies are removed.
  *
- * (At some future point, this may make other semantically inoperative
- * transformations.)
+ * This involves collapsing redundant "/./" and "/../" elements,
+ * removing multiple adjacent separator characters, removing trailing
+ * separator characters, and possibly other semantically inoperative
+ * transformations.
+ *
+ * The returned path may be statically allocated, equal to @a path, or
+ * allocated from @a pool.
  */
 const char *svn_path_canonicalize (const char *path, apr_pool_t *pool);
 
@@ -177,8 +193,9 @@ const char *svn_path_canonicalize (const char *path, apr_pool_t *pool);
 int svn_path_compare_paths (const char *path1, const char *path2);
 
 
-/** Return the longest common path shared by both @a path1 and @a path2.  If
- * there's no common ancestor, return the empty path.
+/** Return the longest common path shared by two canonicalized paths,
+ * @a path1 and @a path2.  If there's no common ancestor, return the
+ * empty path.
  *
  * @a path1 and @a path2 may be URLs.  In order for two URLs to have 
  * a common ancestor, they must (a) have the same protocol (since two URLs 
@@ -190,8 +207,8 @@ char *svn_path_get_longest_ancestor (const char *path1,
                                      const char *path2,
                                      apr_pool_t *pool);
 
-/** Convert @a relative path to an absolute path and return the results in
- * @a *pabsolute, allocated in @a pool.
+/** Convert @a relative canonicalized path to an absolute path and
+ * return the results in @a *pabsolute, allocated in @a pool.
  *
  * @a relative may be a URL, in which case no attempt is made to convert it, 
  * and a copy of the URL is returned. 
@@ -201,10 +218,11 @@ svn_path_get_absolute (const char **pabsolute,
                        const char *relative,
                        apr_pool_t *pool);
 
-/** Return the path part of @a path in @a *pdirectory, and the file part in 
- * @a *pfile.  If @a path is a directory, set @a *pdirectory to @a path, and 
- * @a *pfile to the empty string.  If @a path does not exist it is treated 
- * as if it is a file, since directories do not normally vanish.
+/** Return the path part of the canonicalized @a path in @a
+ * *pdirectory, and the file part in @a *pfile.  If @a path is a
+ * directory, set @a *pdirectory to @a path, and @a *pfile to the
+ * empty string.  If @a path does not exist it is treated as if it is
+ * a file, since directories do not normally vanish.
  */
 svn_error_t *
 svn_path_split_if_file(const char *path,
@@ -212,13 +230,13 @@ svn_path_split_if_file(const char *path,
                        const char **pfile,
                        apr_pool_t *pool);
 
-/** Find the common prefix of the paths in @a targets (an array of @a
- * const char *'s), and remove redundant paths if @a
+/** Find the common prefix of the canonicalized paths in @a targets
+ * (an array of @a const char *'s), and remove redundant paths if @a
  * remove_redundancies is true.
  *
  *   - Set @a *pcommon to the absolute path of the path or URL common to
  *     all of the targets.  If the targets have no common prefix, or
- *     are a mix of URLs and local paths, set @a *pbasename to the
+ *     are a mix of URLs and local paths, set @a *pcommon to the
  *     empty string.
  *
  *   - If @a pcondensed_targets is non-null, set @a *pcondensed_targets
@@ -237,10 +255,10 @@ svn_path_split_if_file(const char *path,
  *     to an array containing zero elements.  Else if
  *     @a pcondensed_targets is null, leave it alone.
  *
- * If there are no items in @a targets, set @a *pbasename and (if
+ * If there are no items in @a targets, set @a *pcommon and (if
  * applicable) @a *pcondensed_targets to @c NULL.
  *
- * NOTE: There is no guarantee that @a *pbasename is within a working
+ * NOTE: There is no guarantee that @a *pcommon is within a working
  * copy.  */
 svn_error_t *
 svn_path_condense_targets (const char **pcommon,
@@ -250,11 +268,11 @@ svn_path_condense_targets (const char **pcommon,
                            apr_pool_t *pool);
 
 
-/** Copy a list of @a targets, one at a time, into @a pcondensed_targets,
- * omitting any targets that are found earlier in the list, or whose
- * ancestor is found earlier in the list.  Ordering of targets in the
- * original list is preserved in the condensed list of targets.  Use
- * @a pool for any allocations.  
+/** Copy a list of canonicalized @a targets, one at a time, into @a
+ * pcondensed_targets, omitting any targets that are found earlier in
+ * the list, or whose ancestor is found earlier in the list.  Ordering
+ * of targets in the original list is preserved in the condensed list
+ * of targets.  Use @a pool for any allocations.
  *
  * How does this differ in functionality from @c svn_path_condense_targets?
  *
@@ -293,9 +311,10 @@ svn_path_remove_redundancies (apr_array_header_t **pcondensed_targets,
                               apr_pool_t *pool);
 
 
-/** Decompose @a path into an array of <tt>const char *</tt> components, 
- * allocated in @a pool.  If @a path is absolute, the first component will 
- * be a lone dir separator (the root directory).
+/** Decompose the canonicalized @a path into an array of <tt>const
+ * char *</tt> components, allocated in @a pool.  If @a path is
+ * absolute, the first component will be a lone dir separator (the
+ * root directory).
  */
 apr_array_header_t *svn_path_decompose (const char *path,
                                         apr_pool_t *pool);
@@ -309,6 +328,16 @@ apr_array_header_t *svn_path_decompose (const char *path,
 svn_boolean_t svn_path_is_single_path_component (const char *name);
 
 
+/**
+ * @since New in 1.1.
+ *
+ * Test to see if a backpath, i.e. '..', is present in @a path.
+ * If not, return @c FALSE.
+ * If so, return @c TRUE.
+ */
+svn_boolean_t svn_path_is_backpath_present (const char *path);
+
+
 /** Test if @a path2 is a child of @a path1.
  * If not, return @c NULL.
  * If so, return a copy of the remainder path, allocated in @a pool.
@@ -357,6 +386,24 @@ const char *svn_path_url_add_component (const char *url,
                                         const char *component,
                                         apr_pool_t *pool);
 
+/** @since New in 1.1.
+ * Convert @a iri (Internationalized URI) to an URI.
+ * The return value may be the same as @a iri if it was already
+ * a URI.  Else, allocate the return value in @a pool. */
+const char *svn_path_uri_from_iri (const char *iri,
+                                   apr_pool_t *pool);
+
+/** @since New in 1.1.
+ * URI-encode certain characters in @a uri that are not valid in an URI, but
+ * doesn't have any special meaning in @a uri at their positions.  If no
+ * characters need escaping, just return @a uri.
+ *
+ * NOTE: Currently, this function escapes <, >, ", space, {, }, |, \, ^, and `.
+ * This may be extended in the future to do context-dependent escaping.
+ */
+const char *svn_path_uri_autoescape (const char *uri,
+                                     apr_pool_t *pool);
+
 /** @} */
 
 /** Charset conversion stuff
