@@ -25,12 +25,15 @@
 #include "svn_client.h"
 #include "svn_string.h"
 #include "svn_error.h"
+#include "svn_pools.h"
 #include "svn_subst.h"
 #include "svn_io.h"
 #include "svn_time.h"
 #include "svn_path.h"
 #include "client.h"
 
+#include "svn_private_config.h"
+
 
 /*** Code. ***/
 
@@ -42,7 +45,7 @@ svn_client_cat (svn_stream_t *out,
                 apr_pool_t *pool)
 {
   svn_ra_plugin_t *ra_lib;
-  void *ra_baton, *session;
+  void *session;
   svn_revnum_t rev;
   svn_node_kind_t url_kind;
   svn_string_t *eol_style;
@@ -50,33 +53,16 @@ svn_client_cat (svn_stream_t *out,
   apr_hash_t *props;
   const char *url;
 
-  SVN_ERR (svn_client_url_from_path (&url, path_or_url, pool));
-  if (! url)
-    return svn_error_createf (SVN_ERR_ENTRY_MISSING_URL, NULL,
-                              "'%s' has no URL", path_or_url);
+  /* Get an RA plugin for this filesystem object. */
+  SVN_ERR (svn_client__ra_lib_from_path (&ra_lib, &session, &rev,
+                                         &url, path_or_url, revision,
+                                         ctx, pool));
 
-
-  /* Get the RA library that handles URL. */
-  SVN_ERR (svn_ra_init_ra_libs (&ra_baton, pool));
-  SVN_ERR (svn_ra_get_ra_library (&ra_lib, ra_baton, url, pool));
-
-  /* Open a repository session to the URL. */
-  SVN_ERR (svn_client__open_ra_session (&session, ra_lib, url, NULL, NULL,
-                                        NULL, FALSE, FALSE,
-                                        ctx, pool));
-
-  /* Resolve REVISION into a real revnum. */
-  SVN_ERR (svn_client__get_revision_number (&rev, ra_lib, session,
-                                            revision, path_or_url, pool));
-  if (! SVN_IS_VALID_REVNUM (rev))
-    SVN_ERR (ra_lib->get_latest_revnum (session, &rev, pool));
-
-  /* Decide if the URL is a file or directory. */
+  /* Make sure the object isn't a directory. */
   SVN_ERR (ra_lib->check_path (session, "", rev, &url_kind, pool));
-
   if (url_kind == svn_node_dir)
     return svn_error_createf(SVN_ERR_CLIENT_IS_DIRECTORY, NULL,
-                             "URL '%s' refers to a directory", url);
+                             _("URL '%s' refers to a directory"), url);
 
   /* Grab some properties we need to know in order to figure out if anything 
      special needs to be done with this file. */
@@ -116,7 +102,7 @@ svn_client_cat (svn_stream_t *out,
       /* rewind our stream. */
       apr_err = apr_file_seek (tmp_file, APR_SET, &off);
       if (apr_err)
-        return svn_error_wrap_apr (apr_err, "Can't seek in '%s'",
+        return svn_error_wrap_apr (apr_err, _("Can't seek in '%s'"),
                                    tmp_filename);
 
       if (eol_style)
