@@ -24,6 +24,7 @@
 
 #include <string.h>
 #include <assert.h>
+#include "svn_private_config.h"
 #include "svn_wc.h"
 #include "svn_client.h"
 #include "svn_string.h"
@@ -35,6 +36,8 @@
 
 #include "client.h"
 
+#include "svn_private_config.h"
+
 
 /*** Code. ***/
 
@@ -77,7 +80,7 @@ wc_to_wc_copy (const char *src_path,
   SVN_ERR (svn_io_check_path (src_path, &src_kind, pool));
   if (src_kind == svn_node_none)
     return svn_error_createf (SVN_ERR_NODE_UNKNOWN_KIND, NULL,
-                              "Path '%s' does not exist", src_path);
+                              _("Path '%s' does not exist"), src_path);
 
   /* If DST_PATH does not exist, then its basename will become a new
      file or dir added to its parent (possibly an implicit '.').  If
@@ -96,7 +99,7 @@ wc_to_wc_copy (const char *src_path,
     }
   else
     return svn_error_createf (SVN_ERR_ENTRY_EXISTS, NULL,
-                              "File '%s' already exists", dst_path);
+                              _("File '%s' already exists"), dst_path);
 
   if (is_move)
     {
@@ -104,9 +107,9 @@ wc_to_wc_copy (const char *src_path,
 
       svn_path_split (src_path, &src_parent, NULL, pool);
 
-      SVN_ERR (svn_wc_adm_open (&src_access, NULL, src_parent, TRUE,
-                                src_kind == svn_node_dir,
-                                pool));
+      SVN_ERR (svn_wc_adm_open2 (&src_access, NULL, src_parent, TRUE,
+                                 src_kind == svn_node_dir ? -1 : 0,
+                                 pool));
 
       /* Need to avoid attempting to open the same dir twice when source
          and destination overlap. */
@@ -129,20 +132,20 @@ wc_to_wc_copy (const char *src_path,
             }
           else
             {
-              SVN_ERR (svn_wc_adm_open (&adm_access, NULL, dst_parent,
-                                        TRUE, FALSE, pool));
+              SVN_ERR (svn_wc_adm_open2 (&adm_access, NULL, dst_parent,
+                                         TRUE, 0, pool));
             }
         }
 
       if (!force)
         /* Ensure there are no "awkward" files. */
         SVN_ERR_W (svn_client__can_delete (src_path, ctx, pool),
-                   "Move will not be attempted unless forced");
+                   _("Move will not be attempted unless forced"));
     }
   else 
     {
-      SVN_ERR (svn_wc_adm_open (&adm_access, NULL, dst_parent, TRUE, FALSE,
-                                pool));
+      SVN_ERR (svn_wc_adm_open2 (&adm_access, NULL, dst_parent, TRUE,
+                                 0, pool));
     }
                               
   /* Perform the copy and (optionally) delete. */
@@ -197,7 +200,7 @@ path_driver_cb_func (void **dir_baton,
                      apr_pool_t *pool)
 {
   struct path_driver_cb_baton *cb_baton = callback_baton;
-  svn_boolean_t do_delete = TRUE, do_add = FALSE;
+  svn_boolean_t do_delete = FALSE, do_add = FALSE;
 
   /* Initialize return value. */
   *dir_baton = NULL;
@@ -318,7 +321,7 @@ repos_to_repos_copy (svn_client_commit_info_t **commit_info,
   /* We can't move something into itself, period. */
   if (svn_path_is_empty (src_rel) && is_move)
     return svn_error_createf (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-                              "Cannot move URL '%s' into itself", src_url);
+                              _("Cannot move URL '%s' into itself"), src_url);
 
   /* Get the RA vtable that matches URL. */
   SVN_ERR (svn_ra_init_ra_libs (&ra_baton, pool));
@@ -347,8 +350,8 @@ repos_to_repos_copy (svn_client_commit_info_t **commit_info,
         {
           return svn_error_createf
             (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-             "Source and dest appear not to be in the same repository "
-	     "(src: '%s'; dst: '%s')",
+             _("Source and dest appear not to be in the same repository "
+               "(src: '%s'; dst: '%s')"),
              src_url, dst_url);
         }
       else
@@ -378,7 +381,7 @@ repos_to_repos_copy (svn_client_commit_info_t **commit_info,
   if (src_kind == svn_node_none)
     return svn_error_createf 
       (SVN_ERR_FS_NOT_FOUND, NULL,
-       "Path '%s' does not exist in revision '%" SVN_REVNUM_T_FMT "'",
+       _("Path '%s' does not exist in revision %ld"),
        src_url, src_revnum);
 
   /* Figure out the basename that will result from this operation. */
@@ -391,7 +394,7 @@ repos_to_repos_copy (svn_client_commit_info_t **commit_info,
     {
       /* We disallow the overwriting of files. */
       return svn_error_createf (SVN_ERR_FS_ALREADY_EXISTS, NULL,
-                                "Path '%s' already exists.", dst_rel);
+                                _("Path '%s' already exists"), dst_rel);
     }
   else if (dst_kind == svn_node_dir)
     {
@@ -407,12 +410,12 @@ repos_to_repos_copy (svn_client_commit_info_t **commit_info,
                                    &attempt_kind, pool));
       if (attempt_kind != svn_node_none)
         return svn_error_createf (SVN_ERR_FS_ALREADY_EXISTS, NULL,
-                                  "Path '%s' already exists", dst_rel);
+                                  _("Path '%s' already exists"), dst_rel);
     }
   else
     {
       return svn_error_createf (SVN_ERR_NODE_UNKNOWN_KIND, NULL,
-                                "Unrecognized node kind of '%s'", dst_url);
+                                _("Unrecognized node kind of '%s'"), dst_url);
     }
 
   /* Create a new commit item and add it to the array. */
@@ -421,13 +424,20 @@ repos_to_repos_copy (svn_client_commit_info_t **commit_info,
       svn_client_commit_item_t *item;
       const char *tmp_file;
       apr_array_header_t *commit_items 
-        = apr_array_make (pool, 1, sizeof (item));
+        = apr_array_make (pool, 2, sizeof (item));
       
       item = apr_pcalloc (pool, sizeof (*item));
       item->url = svn_path_join (top_url, dst_rel, pool);
       item->state_flags = SVN_CLIENT_COMMIT_ITEM_ADD;
       (*((svn_client_commit_item_t **) apr_array_push (commit_items))) = item;
-      
+      if (is_move && (! resurrection))
+        {
+          item = apr_pcalloc (pool, sizeof (*item));
+          item->url = svn_path_join (top_url, src_rel, pool);
+          item->state_flags = SVN_CLIENT_COMMIT_ITEM_DELETE;
+          (*((svn_client_commit_item_t **) apr_array_push (commit_items))) = 
+            item;
+        }
       SVN_ERR ((*ctx->log_msg_func) (&message, &tmp_file, commit_items, 
                                      ctx->log_msg_baton, pool));
       if (! message)
@@ -528,7 +538,7 @@ reconcile_errors (svn_error_t *commit_err,
   if (commit_err)
     {
       commit_err = svn_error_quick_wrap 
-        (commit_err, "Commit failed (details follow):");
+        (commit_err, _("Commit failed (details follow):"));
       err = commit_err;
     }
 
@@ -536,14 +546,14 @@ reconcile_errors (svn_error_t *commit_err,
      that follow. */
   else
     err = svn_error_create (SVN_ERR_BASE, NULL,
-                            "Commit succeeded, but other errors follow:");
+                            _("Commit succeeded, but other errors follow:"));
 
   /* If there was an unlock error... */
   if (unlock_err)
     {
       /* Wrap the error with some headers. */
       unlock_err = svn_error_quick_wrap 
-        (unlock_err, "Error unlocking locked dirs (details follow):");
+        (unlock_err, _("Error unlocking locked dirs (details follow):"));
 
       /* Append this error to the chain. */
       svn_error_compose (err, unlock_err);
@@ -554,7 +564,7 @@ reconcile_errors (svn_error_t *commit_err,
     {
       /* Wrap the error with some headers. */
       cleanup_err = svn_error_quick_wrap 
-        (cleanup_err, "Error in post-commit clean-up (details follow):");
+        (cleanup_err, _("Error in post-commit clean-up (details follow):"));
 
       /* Append this error to the chain. */
       svn_error_compose (err, cleanup_err);
@@ -594,8 +604,8 @@ wc_to_repos_copy (svn_client_commit_info_t **commit_info,
      paths everywhere. */
   SVN_ERR (svn_path_get_absolute (&base_path, src_path, pool));
 
-  SVN_ERR (svn_wc_adm_probe_open (&adm_access, NULL, base_path, FALSE, TRUE,
-                                  pool));
+  SVN_ERR (svn_wc_adm_probe_open2 (&adm_access, NULL, base_path,
+                                   FALSE, -1, pool));
 
   /* Split the DST_URL into an anchor and target. */
   svn_path_split (dst_url, &anchor, &target, pool);
@@ -633,7 +643,7 @@ wc_to_repos_copy (svn_client_commit_info_t **commit_info,
       /* DST_URL is an existing file, which can't be overwritten or
          used as a container, so error out. */
       return svn_error_createf (SVN_ERR_FS_ALREADY_EXISTS, NULL,
-                                "File '%s' already exists", dst_url);
+                                _("File '%s' already exists"), dst_url);
     }
 
   /* Create a new commit item and add it to the array. */
@@ -779,12 +789,12 @@ repos_to_wc_copy (const char *src_url,
       if (SVN_IS_VALID_REVNUM (src_revnum))
         return svn_error_createf
           (SVN_ERR_FS_NOT_FOUND, NULL,
-           "Path '%s' not found in revision '%" SVN_REVNUM_T_FMT "'",
+           _("Path '%s' not found in revision %ld"),
            src_url, src_revnum);
       else
         return svn_error_createf
           (SVN_ERR_FS_NOT_FOUND, NULL,
-           "Path '%s' not found in head revision", src_url);
+           _("Path '%s' not found in head revision"), src_url);
     }
 
   /* There are two interfering sets of cases to watch out for here:
@@ -821,7 +831,7 @@ repos_to_wc_copy (const char *src_url,
   else if (dst_kind != svn_node_none)  /* must be a file */
     {
       return svn_error_createf (SVN_ERR_ENTRY_EXISTS, NULL,
-                                "File '%s' already exists", dst_path);
+                                _("File '%s' already exists"), dst_path);
     }
 
   /* Now that dst_path has possibly been reset, check that there's
@@ -829,10 +839,10 @@ repos_to_wc_copy (const char *src_url,
   SVN_ERR (svn_io_check_path (dst_path, &dst_kind, pool));
   if (dst_kind != svn_node_none)
     return svn_error_createf (SVN_ERR_WC_OBSTRUCTED_UPDATE, NULL,
-                              "'%s' is in the way", dst_path);
+                              _("'%s' is in the way"), dst_path);
 
-  SVN_ERR (svn_wc_adm_probe_open (&adm_access, NULL, dst_path, TRUE, FALSE,
-                                  pool));
+  SVN_ERR (svn_wc_adm_probe_open2 (&adm_access, NULL, dst_path, TRUE,
+                                   0, pool));
 
   /* We've already checked for physical obstruction by a working file.
      But there could also be logical obstruction by an entry whose
@@ -844,7 +854,7 @@ repos_to_wc_copy (const char *src_url,
     if (ent && (ent->kind != svn_node_dir))
       return svn_error_createf
         (SVN_ERR_WC_OBSTRUCTED_UPDATE, NULL,
-         "Entry for '%s' exists (though the working file is missing)",
+         _("Entry for '%s' exists (though the working file is missing)"),
          dst_path);
   }
 
@@ -902,8 +912,8 @@ repos_to_wc_copy (const char *src_url,
              should be the copyfrom_revision when we commit later. */
           const svn_wc_entry_t *d_entry;
           svn_wc_adm_access_t *dst_access;
-          SVN_ERR (svn_wc_adm_open (&dst_access, adm_access, dst_path,
-                                    TRUE, TRUE, pool));
+          SVN_ERR (svn_wc_adm_open2 (&dst_access, adm_access, dst_path,
+                                     TRUE, -1, pool));
           SVN_ERR (svn_wc_entry (&d_entry, dst_path, dst_access, FALSE, pool));
           src_revnum = d_entry->revision;
         }
@@ -931,8 +941,10 @@ repos_to_wc_copy (const char *src_url,
              Unfortunately, svn_wc_add() is such a mess that it chokes
              at the moment when we pass a NULL copyfromurl. */
           
-          return svn_error_create (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-                                   "Source URL is from foreign repository");
+          return svn_error_createf
+            (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
+             _("Source URL '%s' is from foreign repository; "
+               "leaving it as a disjoint WC"), src_url);
         }
     } /* end directory case */
 
@@ -950,7 +962,7 @@ repos_to_wc_copy (const char *src_url,
       fstream = svn_stream_from_aprfile (fp, pool);
       SVN_ERR (ra_lib->get_file
                (sess, "", src_revnum, fstream, &real_rev, &new_props, pool));
-      svn_stream_close (fstream);
+      SVN_ERR (svn_stream_close (fstream));
       SVN_ERR (svn_io_file_close (fp, pool));
 
       /* If SRC_REVNUM is invalid (HEAD), then REAL_REV is now the
@@ -965,6 +977,20 @@ repos_to_wc_copy (const char *src_url,
                 same_repositories ? src_url : NULL,
                 same_repositories ? src_revnum : SVN_INVALID_REVNUM,
                 pool));
+
+      /* Ideally, svn_wc_add_repos_file() would take a notify function
+         and baton, and we wouldn't have to make this call here.
+         However, the situation is... complicated.  See issue #1552
+         for the full story. */
+      if (ctx->notify_func)
+        (*ctx->notify_func) (ctx->notify_baton,
+                             dst_path,
+                             svn_wc_notify_add,
+                             src_kind,
+                             NULL,
+                             svn_wc_notify_state_unknown,
+                             svn_wc_notify_state_unknown,
+                             SVN_INVALID_REVNUM);
     }
   
   SVN_ERR (svn_wc_adm_close (adm_access));
@@ -993,7 +1019,7 @@ setup_copy (svn_client_commit_info_t **commit_info,
       && svn_path_is_child (src_path, dst_path, pool))
     return svn_error_createf
       (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-       "Cannot copy path '%s' into its own child '%s'",
+       _("Cannot copy path '%s' into its own child '%s'"),
        src_path, dst_path);
 
   if (is_move)
@@ -1003,7 +1029,7 @@ setup_copy (svn_client_commit_info_t **commit_info,
           if (strcmp (src_path, dst_path) == 0)
             return svn_error_createf
               (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-               "Cannot move path '%s' into itself",
+               _("Cannot move path '%s' into itself"),
                src_path);
         }
       else
@@ -1011,7 +1037,7 @@ setup_copy (svn_client_commit_info_t **commit_info,
           /* Disallow moves between the working copy and the repository. */
           return svn_error_create 
             (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-             "No support for repos <--> working copy moves");
+             _("No support for repos <--> working copy moves"));
         }
 
       /* It doesn't make sense to specify revisions in a move. */
@@ -1026,7 +1052,7 @@ setup_copy (svn_client_commit_info_t **commit_info,
         {
           return svn_error_create
             (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-             "Cannot specify revisions with move operations");
+             _("Cannot specify revisions with move operations"));
         }
     }
   else
@@ -1040,8 +1066,9 @@ setup_copy (svn_client_commit_info_t **commit_info,
                  entries file. */
               svn_wc_adm_access_t *adm_access;  /* ### FIXME local */
               const svn_wc_entry_t *entry;
-              SVN_ERR (svn_wc_adm_probe_open (&adm_access, NULL, src_path,
-                                              FALSE, FALSE, pool));
+              SVN_ERR (svn_wc_adm_probe_open2 (&adm_access, NULL,
+                                               src_path, FALSE, 0,
+                                               pool));
               SVN_ERR (svn_wc_entry (&entry, src_path, adm_access, FALSE,
                                      pool));
               SVN_ERR (svn_wc_adm_close (adm_access));
@@ -1049,7 +1076,13 @@ setup_copy (svn_client_commit_info_t **commit_info,
               if (! entry)
                 return svn_error_createf
                   (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
-                   "'%s' is not under version control", src_path);
+                   _("'%s' is not under version control"), src_path);
+
+              if (! entry->url)
+                return svn_error_createf
+                  (SVN_ERR_ENTRY_MISSING_URL, NULL,
+                   _("'%s' does not seem to have a URL associated with it"),
+                   src_path);
 
               src_path = entry->url;
               src_is_url = TRUE;
