@@ -2,7 +2,7 @@
  * externals.c:  handle the svn:externals property
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -36,162 +36,26 @@
 
 
 
-/* One external item.  This usually represents one line from an
-   svn:externals description but with the path and URL canonicalized. */
-struct external_item
-{
-  /* The name of the subdirectory into which this external should be
-     checked out.  This is relative to the parent directory that holds
-     this external item.  (Note that these structs are often stored in
-     hash tables with the target dirs as keys, so this field will
-     often be redundant.) */
-  const char *target_dir;
-
-  /* Where to check out from. */
-  const char *url;
-
-  /* What revision to check out.  The only valid kinds for this are
-     svn_opt_revision_number, svn_opt_revision_date, and
-     svn_opt_revision_head. */
-  svn_opt_revision_t revision;
-};
-
-
-/* Set *EXTERNALS_P to a hash table whose keys are target subdir
- * names, and values are `struct external_item *' objects,
- * based on DESC.
- *
- * The format of EXTERNALS is the same as for values of the directory
- * property SVN_PROP_EXTERNALS, which see.
- *
- * Allocate the table, keys, and values in POOL.
- *
- * If the format of DESC is invalid, don't touch *EXTERNALS_P and
- * return SVN_ERR_CLIENT_INVALID_EXTERNALS_DESCRIPTION.
- *
- * Use PARENT_DIRECTORY only in constructing error strings.
- */
-static svn_error_t *
-parse_externals_description (apr_hash_t **externals_p,
-                             const char *parent_directory,
-                             const char *desc,
-                             apr_pool_t *pool)
-{
-  apr_hash_t *externals = apr_hash_make (pool);
-  apr_array_header_t *lines = svn_cstring_split (desc, "\n\r", TRUE, pool);
-  int i;
-  
-  for (i = 0; i < lines->nelts; i++)
-    {
-      const char *line = APR_ARRAY_IDX (lines, i, const char *);
-      apr_array_header_t *line_parts;
-      struct external_item *item;
-
-      if ((! line) || (line[0] == '#'))
-        continue;
-
-      /* else proceed */
-
-      line_parts = svn_cstring_split (line, " \t", TRUE, pool);
-
-      item = apr_palloc (pool, sizeof (*item));
-
-      if (line_parts->nelts < 2)
-        goto parse_error;
-
-      else if (line_parts->nelts == 2)
-        {
-          /* No "-r REV" given. */
-          item->target_dir = APR_ARRAY_IDX (line_parts, 0, const char *);
-          item->url = APR_ARRAY_IDX (line_parts, 1, const char *);
-          item->revision.kind = svn_opt_revision_head;
-        }
-      else if ((line_parts->nelts == 3) || (line_parts->nelts == 4))
-        {
-          /* We're dealing with one of these two forms:
-           * 
-           *    TARGET_DIR  -rN  URL
-           *    TARGET_DIR  -r N  URL
-           * 
-           * Handle either way.
-           */
-
-          const char *r_part_1 = NULL, *r_part_2 = NULL;
-
-          item->target_dir = APR_ARRAY_IDX (line_parts, 0, const char *);
-          item->revision.kind = svn_opt_revision_number;
-
-          if (line_parts->nelts == 3)
-            {
-              r_part_1 = APR_ARRAY_IDX (line_parts, 1, const char *);
-              item->url = APR_ARRAY_IDX (line_parts, 2, const char *);
-            }
-          else  /* nelts == 4 */
-            {
-              r_part_1 = APR_ARRAY_IDX (line_parts, 1, const char *);
-              r_part_2 = APR_ARRAY_IDX (line_parts, 2, const char *);
-              item->url = APR_ARRAY_IDX (line_parts, 3, const char *);
-            }
-
-          if ((! r_part_1) || (r_part_1[0] != '-') || (r_part_1[1] != 'r'))
-            goto parse_error;
-
-          if (! r_part_2)  /* "-rN" */
-            {
-              if (strlen (r_part_1) < 3)
-                goto parse_error;
-              else
-                item->revision.value.number = SVN_STR_TO_REV (r_part_1 + 2);
-            }
-          else             /* "-r N" */
-            {
-              if (strlen (r_part_2) < 1)
-                goto parse_error;
-              else
-                item->revision.value.number = SVN_STR_TO_REV (r_part_2);
-            }
-        }
-      else    /* too many items on line */
-        goto parse_error;
-
-      if (0)
-        {
-        parse_error:
-          return svn_error_createf
-            (SVN_ERR_CLIENT_INVALID_EXTERNALS_DESCRIPTION, NULL,
-             "error parsing " SVN_PROP_EXTERNALS " property on '%s':\n"
-             "Invalid line: '%s'", parent_directory, line);
-        }
-
-      item->target_dir = svn_path_canonicalize (item->target_dir, pool);
-      item->url = svn_path_canonicalize (item->url, pool);
-
-      apr_hash_set (externals, item->target_dir, APR_HASH_KEY_STRING, item);
-    }
-
-  *externals_p = externals;
-
-  return SVN_NO_ERROR;
-}
-
-
 /* Closure for handle_external_item_change. */
 struct handle_external_item_change_baton
 {
-  /* As returned by parse_externals_description(). */
+  /* As returned by svn_wc_parse_externals_description(). */
   apr_hash_t *new_desc;
   apr_hash_t *old_desc;
 
   /* The directory that has this externals property. */
   const char *parent_dir;
 
-  /* Passed through to svn_client_checkout(). */
+  /* Passed through to svn_client_* functions. */
   svn_client_ctx_t *ctx;
 
   /* If set, then run update on items that didn't change. */
   svn_boolean_t update_unchanged;
   svn_boolean_t *timestamp_sleep;
+  svn_boolean_t is_export;
 
+  /* A scratchwork pool -- do not put anything in here that needs to
+     outlive the hash diffing callback! */
   apr_pool_t *pool;
 };
 
@@ -206,8 +70,8 @@ struct handle_external_item_change_baton
    we could get away with an "update -r" on the external, instead of
    a re-checkout. */
 static svn_boolean_t
-compare_external_items (struct external_item *new_item,
-                        struct external_item *old_item)
+compare_external_items (svn_wc_external_item_t *new_item,
+                        svn_wc_external_item_t *old_item)
 {
   if ((strcmp (new_item->target_dir, old_item->target_dir) != 0)
       || (strcmp (new_item->url, old_item->url) != 0)
@@ -239,10 +103,10 @@ relegate_external (const char *path,
   svn_error_t *err;
   svn_wc_adm_access_t *adm_access;
 
-  SVN_ERR (svn_wc_adm_open (&adm_access, NULL, path, TRUE, FALSE, pool));
+  SVN_ERR (svn_wc_adm_open2 (&adm_access, NULL, path, TRUE, -1, pool));
   err = svn_wc_remove_from_revision_control (adm_access,
                                              SVN_WC_ENTRY_THIS_DIR,
-                                             TRUE,
+                                             TRUE, FALSE,
                                              cancel_func,
                                              cancel_baton,
                                              pool);
@@ -296,22 +160,23 @@ relegate_external (const char *path,
 }
 
 
-/* This implements the `svn_hash_diff_func_t' interface.
-   BATON is of type `struct handle_external_item_change_baton *'.  */
+/* This implements the 'svn_hash_diff_func_t' interface.
+   BATON is of type 'struct handle_external_item_change_baton *'.  */
 static svn_error_t *
 handle_external_item_change (const void *key, apr_ssize_t klen,
                              enum svn_hash_diff_key_status status,
                              void *baton)
 {
   struct handle_external_item_change_baton *ib = baton;
-  struct external_item *old_item, *new_item;
+  svn_wc_external_item_t *old_item, *new_item;
+  const char *parent;
   const char *path = svn_path_join (ib->parent_dir,
                                     (const char *) key, ib->pool);
 
   /* Don't bother to check status, since we'll get that for free by
      attempting to retrieve the hash values anyway.  */
 
-  if (ib->old_desc)
+  if ((ib->old_desc) && (! ib->is_export))
     old_item = apr_hash_get (ib->old_desc, key, klen);
   else
     old_item = NULL;
@@ -350,11 +215,8 @@ handle_external_item_change (const void *key, apr_ssize_t klen,
     {
       /* The target dir might have multiple components.  Guarantee
          the path leading down to the last component. */
-      {
-        const char *checkout_parent;
-        svn_path_split (path, &checkout_parent, NULL, ib->pool);
-        SVN_ERR (svn_io_make_dir_recursively (checkout_parent, ib->pool));
-      }
+      svn_path_split (path, &parent, NULL, ib->pool);
+      SVN_ERR (svn_io_make_dir_recursively (parent, ib->pool));
 
       /* If we were handling renames the fancy way, then  before
          checking out a new subdir here, we would somehow learn if
@@ -374,14 +236,22 @@ handle_external_item_change (const void *key, apr_ssize_t klen,
                                  svn_wc_notify_state_unknown,
                                  SVN_INVALID_REVNUM);
 
-      SVN_ERR (svn_client__checkout_internal
-               (new_item->url,
-                path,
-                &(new_item->revision),
-                TRUE, /* recurse */
-                ib->timestamp_sleep,
-                ib->ctx,
-                ib->pool));
+      if (ib->is_export)
+        /* ### It should be okay to "force" this export.  Externals
+           only get created in subdirectories of versioned
+           directories, so an external directory couldn't already
+           exist before the parent export process unless a versioned
+           directory above it did, which means the user would have
+           already had to force these creations to occur. */
+        SVN_ERR (svn_client_export (NULL, new_item->url, path,
+                                    &(new_item->revision),
+                                    TRUE, ib->ctx, ib->pool));
+      else
+        SVN_ERR (svn_client__checkout_internal (NULL, new_item->url, path,
+                                                &(new_item->revision),
+                                                TRUE, /* recurse */
+                                                ib->timestamp_sleep,
+                                                ib->ctx, ib->pool));
     }
   else if (! new_item)
     {
@@ -392,15 +262,15 @@ handle_external_item_change (const void *key, apr_ssize_t klen,
       svn_error_t *err;
       svn_wc_adm_access_t *adm_access;
 
-      SVN_ERR (svn_wc_adm_open (&adm_access, NULL, path, TRUE, TRUE,
-                                ib->pool));
+      SVN_ERR (svn_wc_adm_open2 (&adm_access, NULL, path, TRUE, -1,
+                                 ib->pool));
 
       /* We don't use relegate_external() here, because we know that
          nothing else in this externals description (at least) is
          going to need this directory, and therefore it's better to
          leave stuff where the user expects it. */
       err = svn_wc_remove_from_revision_control
-        (adm_access, SVN_WC_ENTRY_THIS_DIR, TRUE,
+        (adm_access, SVN_WC_ENTRY_THIS_DIR, TRUE, FALSE,
          ib->ctx->cancel_func, ib->ctx->cancel_baton, ib->pool);
 
       /* ### Ugly. Unlock only if not going to return an error. Revisit */
@@ -436,21 +306,19 @@ handle_external_item_change (const void *key, apr_ssize_t klen,
                                  svn_wc_notify_state_unknown,
                                  SVN_INVALID_REVNUM);
 
-      SVN_ERR (svn_client__checkout_internal
-               (new_item->url,
-                path,
-                &(new_item->revision),
-                TRUE, /* recurse */
-                ib->timestamp_sleep,
-                ib->ctx,
-                ib->pool));
+      SVN_ERR (svn_client__checkout_internal (NULL, new_item->url, path,
+                                              &(new_item->revision),
+                                              TRUE, /* recurse */
+                                              ib->timestamp_sleep,
+                                              ib->ctx, ib->pool));
     }
   else if (ib->update_unchanged)
     {
       /* Exact same item is present in both hashes, and caller wants
          to update such unchanged items. */
-
-      svn_error_t *err;
+      svn_wc_adm_access_t *adm_access;
+      const svn_wc_entry_t *ext_entry;
+      svn_node_kind_t kind;
 
       /* First notify that we're about to handle an external. */
       if (ib->ctx->notify_func)
@@ -463,43 +331,66 @@ handle_external_item_change (const void *key, apr_ssize_t klen,
                                  svn_wc_notify_state_unknown,
                                  SVN_INVALID_REVNUM);
 
-      /* Try an update, but if no such dir, then check out instead. */
-      err = svn_client__update_internal (path,
-                                         &(new_item->revision),
-                                         TRUE, /* recurse */
-                                         ib->timestamp_sleep,
-                                         ib->ctx,
-                                         ib->pool);
-
-      if (err && (err->apr_err == SVN_ERR_ENTRY_NOT_FOUND))
+      /* Next, verify that the external working copy matches
+         (URL-wise) what is supposed to be on disk. */
+      SVN_ERR (svn_io_check_path (path, &kind, ib->pool));
+      if (kind == svn_node_dir)
         {
-          /* No problem.  Probably user added this external item, but
-             hasn't updated since then, so they don't actually have a
-             working copy of it yet.  Just check it out. */
-          
-          svn_error_clear (err);
+          SVN_ERR (svn_wc_adm_open2 (&adm_access, NULL, path, TRUE, -1,
+                                     ib->pool));
+          SVN_ERR (svn_wc_entry (&ext_entry, path, adm_access, 
+                                 FALSE, ib->pool));
+          SVN_ERR (svn_wc_adm_close (adm_access));
 
+          /* If we have what appears to be a version controlled
+             subdir, and its top-level URL matches that of our
+             externals definition, perform an update. */
+          if (ext_entry && (strcmp (ext_entry->url, new_item->url) == 0))
+            {
+              SVN_ERR (svn_client__update_internal (NULL, path,
+                                                    &(new_item->revision),
+                                                    TRUE, /* recurse */
+                                                    ib->timestamp_sleep,
+                                                    ib->ctx, ib->pool));
+            }
+          /* If, however, the URLs don't match, we need to relegate
+             the one subdir, and then checkout a new one. */
+          else
+            {
+              /* Buh-bye, old and busted ... */
+              SVN_ERR (relegate_external (path,
+                                          ib->ctx->cancel_func,
+                                          ib->ctx->cancel_baton,
+                                          ib->pool));
+              /* ... Hello, new hotness. */
+              SVN_ERR (svn_client__checkout_internal (NULL, new_item->url,
+                                                      path,
+                                                      &(new_item->revision),
+                                                      TRUE, /* recurse */
+                                                      ib->timestamp_sleep,
+                                                      ib->ctx, ib->pool));
+            }
+        }
+      else /* Not a directory at all -- just try the checkout. */
+        {
           /* The target dir might have multiple components.  Guarantee
              the path leading down to the last component. */
-          {
-            const char *parent;
-            svn_path_split (path, &parent, NULL, ib->pool);
-            SVN_ERR (svn_io_make_dir_recursively (parent, ib->pool));
-          }
-          
-          SVN_ERR (svn_client__checkout_internal
-                   (new_item->url,
-                    path,
-                    &(new_item->revision),
-                    TRUE, /* recurse */
-                    ib->timestamp_sleep,
-                    ib->ctx,
-                    ib->pool));
+          svn_path_split (path, &parent, NULL, ib->pool);
+          SVN_ERR (svn_io_make_dir_recursively (parent, ib->pool));
+
+          /* Checking out... */
+          SVN_ERR (svn_client__checkout_internal (NULL, new_item->url, path,
+                                                  &(new_item->revision),
+                                                  TRUE, /* recurse */
+                                                  ib->timestamp_sleep,
+                                                  ib->ctx, ib->pool));
         }
-      else if (err)
-        return err;
     }
 
+  /* Clear IB->pool -- we only use it for scratchwork (and this will
+     close any RA sessions still open in this pool). */
+  svn_pool_clear (ib->pool);
+
   return SVN_NO_ERROR;
 }
 
@@ -515,13 +406,14 @@ struct handle_externals_desc_change_baton
   svn_client_ctx_t *ctx;
   svn_boolean_t update_unchanged;
   svn_boolean_t *timestamp_sleep;
+  svn_boolean_t is_export;
 
   apr_pool_t *pool;
 };
 
 
-/* This implements the `svn_hash_diff_func_t' interface.
-   BATON is of type `struct handle_externals_desc_change_baton *'.  
+/* This implements the 'svn_hash_diff_func_t' interface.
+   BATON is of type 'struct handle_externals_desc_change_baton *'.  
 */
 static svn_error_t *
 handle_externals_desc_change (const void *key, apr_ssize_t klen,
@@ -531,30 +423,82 @@ handle_externals_desc_change (const void *key, apr_ssize_t klen,
   struct handle_externals_desc_change_baton *cb = baton;
   struct handle_external_item_change_baton ib;
   const char *old_desc_text, *new_desc_text;
-  apr_hash_t *old_desc, *new_desc;
+  apr_array_header_t *old_desc, *new_desc;
+  apr_hash_t *old_desc_hash, *new_desc_hash;
+  int i;
+  svn_wc_external_item_t *item;
 
   if ((old_desc_text = apr_hash_get (cb->externals_old, key, klen)))
-    SVN_ERR (parse_externals_description (&old_desc, (const char *) key,
-                                          old_desc_text, cb->pool));
+    SVN_ERR (svn_wc_parse_externals_description2 (&old_desc, (const char *) key,
+                                                  old_desc_text, cb->pool));
   else
     old_desc = NULL;
 
   if ((new_desc_text = apr_hash_get (cb->externals_new, key, klen)))
-    SVN_ERR (parse_externals_description (&new_desc, (const char *) key,
-                                          new_desc_text, cb->pool));
+    SVN_ERR (svn_wc_parse_externals_description2 (&new_desc, (const char *) key,
+                                                  new_desc_text, cb->pool));
   else
     new_desc = NULL;
 
-  ib.old_desc          = old_desc;
-  ib.new_desc          = new_desc;
+  old_desc_hash = apr_hash_make (cb->pool);
+  new_desc_hash = apr_hash_make (cb->pool);
+
+  /* Create hashes of our two externals arrays so that we can
+     efficiently generate a diff for them. */
+  for (i = 0; old_desc && (i < old_desc->nelts); i++)
+    {
+      item = APR_ARRAY_IDX (old_desc, i, svn_wc_external_item_t *);
+
+      apr_hash_set (old_desc_hash, item->target_dir,
+                    APR_HASH_KEY_STRING, item);
+    }
+  
+  for (i = 0; new_desc && (i < new_desc->nelts); i++)
+    {
+      item = APR_ARRAY_IDX (new_desc, i, svn_wc_external_item_t *);
+
+      apr_hash_set (new_desc_hash, item->target_dir,
+                    APR_HASH_KEY_STRING, item);
+    }
+
+  ib.old_desc          = old_desc_hash;
+  ib.new_desc          = new_desc_hash;
   ib.parent_dir        = (const char *) key;
   ib.ctx               = cb->ctx;
   ib.update_unchanged  = cb->update_unchanged;
+  ib.is_export         = cb->is_export;
   ib.timestamp_sleep   = cb->timestamp_sleep;
-  ib.pool              = cb->pool;
+  ib.pool              = svn_pool_create (cb->pool);
 
-  SVN_ERR (svn_hash_diff (old_desc, new_desc,
-                          handle_external_item_change, &ib, cb->pool));
+  /* We must use a custom version of svn_hash_diff so that the diff
+     entries are processed in the order they were originally specified
+     in the svn:external properties. */
+
+  for (i = 0; old_desc && (i < old_desc->nelts); i++)
+    {
+      item = APR_ARRAY_IDX (old_desc, i, svn_wc_external_item_t *);
+
+      if (apr_hash_get (new_desc_hash, item->target_dir, APR_HASH_KEY_STRING))
+        SVN_ERR (handle_external_item_change (item->target_dir,
+                                              APR_HASH_KEY_STRING,
+                                              svn_hash_diff_key_both, &ib));
+      else
+        SVN_ERR (handle_external_item_change (item->target_dir,
+                                              APR_HASH_KEY_STRING,
+                                              svn_hash_diff_key_a, &ib));
+    }
+  for (i = 0; new_desc && (i < new_desc->nelts); i++)
+    {
+      item = APR_ARRAY_IDX (new_desc, i, svn_wc_external_item_t *);
+      if (! apr_hash_get (old_desc_hash, item->target_dir, APR_HASH_KEY_STRING))
+        SVN_ERR (handle_external_item_change (item->target_dir,
+                                              APR_HASH_KEY_STRING,
+                                              svn_hash_diff_key_b, &ib));
+    }
+  
+  /* Now destroy the subpool we pass to the hash differ.  This will
+     close any remaining RA sessions used by the hash diff callback. */
+  svn_pool_destroy (ib.pool);
 
   return SVN_NO_ERROR;
 }
@@ -577,10 +521,127 @@ svn_client__handle_externals (svn_wc_traversal_info_t *traversal_info,
   cb.ctx               = ctx;
   cb.update_unchanged  = update_unchanged;
   cb.timestamp_sleep   = timestamp_sleep;
+  cb.is_export         = FALSE;
   cb.pool              = pool;
 
-  SVN_ERR (svn_hash_diff (externals_old, externals_new,
+  SVN_ERR (svn_hash_diff (cb.externals_old, cb.externals_new,
                           handle_externals_desc_change, &cb, pool));
 
   return SVN_NO_ERROR;
 }
+
+
+svn_error_t *
+svn_client__fetch_externals (apr_hash_t *externals,
+                             svn_boolean_t is_export,
+                             svn_boolean_t *timestamp_sleep,
+                             svn_client_ctx_t *ctx,
+                             apr_pool_t *pool)
+{
+  struct handle_externals_desc_change_baton cb;
+
+  cb.externals_new     = externals;
+  cb.externals_old     = apr_hash_make (pool);
+  cb.ctx               = ctx;
+  cb.update_unchanged  = TRUE;
+  cb.timestamp_sleep   = timestamp_sleep;
+  cb.is_export         = is_export;
+  cb.pool              = pool;
+
+  SVN_ERR (svn_hash_diff (cb.externals_old, cb.externals_new,
+                          handle_externals_desc_change, &cb, pool));
+
+  return SVN_NO_ERROR;
+}
+
+
+svn_error_t *
+svn_client__do_external_status (svn_wc_traversal_info_t *traversal_info,
+                                svn_wc_status_func_t status_func,
+                                void *status_baton,
+                                svn_boolean_t get_all,
+                                svn_boolean_t update,
+                                svn_boolean_t no_ignore,
+                                svn_client_ctx_t *ctx,
+                                apr_pool_t *pool)
+{
+  apr_hash_t *externals_old, *externals_new;
+  apr_hash_index_t *hi;
+  apr_pool_t *subpool = svn_pool_create (pool);
+
+  /* Get the values of the svn:externals properties. */
+  svn_wc_edited_externals (&externals_old, &externals_new, traversal_info);
+
+  /* Loop over the hash of new values (we don't care about the old
+     ones).  This is a mapping of versioned directories to property
+     values. */
+  for (hi = apr_hash_first (pool, externals_new); 
+       hi; 
+       hi = apr_hash_next (hi))
+    {
+      apr_array_header_t *exts;
+      const void *key;
+      void *val;
+      const char *path;
+      const char *propval;
+      apr_pool_t *iterpool;
+      int i;
+
+      /* Clear the subpool. */
+      svn_pool_clear (subpool);
+
+      apr_hash_this (hi, &key, NULL, &val);
+      path = key;
+      propval = val;
+
+      /* Parse the svn:externals property value.  This results in a
+         hash mapping subdirectories to externals structures. */
+      SVN_ERR (svn_wc_parse_externals_description2 (&exts, path, 
+                                                    propval, subpool));
+
+      /* Make a sub-pool of SUBPOOL. */
+      iterpool = svn_pool_create (subpool);
+
+      /* Loop over the subdir array. */
+      for (i = 0; exts && (i < exts->nelts); i++)
+        {
+          const char *fullpath;
+          svn_wc_external_item_t *external;
+          svn_node_kind_t kind;
+
+          svn_pool_clear (iterpool);
+
+          external = APR_ARRAY_IDX (exts, i, svn_wc_external_item_t *);
+          fullpath = svn_path_join (path, external->target_dir, iterpool);
+
+          /* If the external target directory doesn't exist on disk,
+             just skip it. */
+          SVN_ERR (svn_io_check_path (fullpath, &kind, iterpool));
+          if (kind != svn_node_dir)
+            continue;
+
+          /* Tell the client we're staring an external status set. */
+          if (ctx->notify_func)
+            (ctx->notify_func) (ctx->notify_baton,
+                                fullpath,
+                                svn_wc_notify_status_external,
+                                svn_node_unknown,
+                                NULL,
+                                svn_wc_notify_state_unknown,
+                                svn_wc_notify_state_unknown,
+                                SVN_INVALID_REVNUM);
+
+          /* And then do the status. */
+          SVN_ERR (svn_client_status (NULL, fullpath, 
+                                      &(external->revision),
+                                      status_func, status_baton, 
+                                      TRUE, get_all, update, no_ignore, 
+                                      ctx, iterpool));
+        }
+    } 
+  
+  /* Destroy SUBPOOL and (implicitly) ITERPOOL. */
+  apr_pool_destroy (subpool);
+
+  return SVN_NO_ERROR;
+}
