@@ -24,6 +24,8 @@
 #include "svn_client.h"
 #include "svn_path.h"
 
+#include "svn_private_config.h"
+
 static svn_error_t *
 get_dir_contents (apr_hash_t *dirents,
                   const char *dir,
@@ -31,6 +33,7 @@ get_dir_contents (apr_hash_t *dirents,
                   svn_ra_plugin_t *ra_lib,
                   void *session,
                   svn_boolean_t recurse,
+                  svn_client_ctx_t *ctx,
                   apr_pool_t *pool)
 {
   apr_hash_t *tmpdirents;
@@ -43,7 +46,10 @@ get_dir_contents (apr_hash_t *dirents,
                               NULL, NULL, pool));
   else
     return svn_error_create (SVN_ERR_RA_NOT_IMPLEMENTED, NULL,
-                             "No get_dir() available for URL schema");
+                             _("No get_dir() available for URL schema"));
+
+  if (ctx->cancel_func)
+    SVN_ERR (ctx->cancel_func (ctx->cancel_baton));
 
   for (hi = apr_hash_first (pool, tmpdirents);
        hi;
@@ -63,7 +69,7 @@ get_dir_contents (apr_hash_t *dirents,
 
       if (recurse && the_ent->kind == svn_node_dir)
         SVN_ERR (get_dir_contents (dirents, path, rev, ra_lib, session,
-                                   recurse, pool));
+                                   recurse, ctx, pool));
     }
 
   return SVN_NO_ERROR;
@@ -78,33 +84,15 @@ svn_client_ls (apr_hash_t **dirents,
                apr_pool_t *pool)
 {
   svn_ra_plugin_t *ra_lib;  
-  void *ra_baton, *session;
+  void *session;
   svn_revnum_t rev;
   svn_node_kind_t url_kind;
   const char *url;
 
-  SVN_ERR (svn_client_url_from_path (&url, path_or_url, pool));
-  if (! url)
-    return svn_error_createf (SVN_ERR_ENTRY_MISSING_URL, NULL,
-                              "'%s' has no URL", path_or_url);
-
-  /* Get the RA library that handles URL. */
-  SVN_ERR (svn_ra_init_ra_libs (&ra_baton, pool));
-  SVN_ERR (svn_ra_get_ra_library (&ra_lib, ra_baton, url, pool));
-
-  /* Open a repository session to the URL. */
-  SVN_ERR (svn_client__open_ra_session (&session, ra_lib, url,
-                                        NULL,
-                                        NULL, NULL, FALSE, TRUE, 
-                                        ctx, pool));
-
-  /* Resolve REVISION into a real revnum. */
-  SVN_ERR (svn_client__get_revision_number
-           (&rev, ra_lib, session, revision, 
-            (path_or_url == url) ? NULL : path_or_url, pool));
-
-  if (! SVN_IS_VALID_REVNUM (rev))
-    SVN_ERR (ra_lib->get_latest_revnum (session, &rev, pool));
+  /* Get an RA plugin for this filesystem object. */
+  SVN_ERR (svn_client__ra_lib_from_path (&ra_lib, &session, &rev,
+                                         &url, path_or_url, revision,
+                                         ctx, pool));
 
   /* Decide if the URL is a file or directory. */
   SVN_ERR (ra_lib->check_path (session, "", rev, &url_kind, pool));
@@ -114,7 +102,7 @@ svn_client_ls (apr_hash_t **dirents,
       *dirents = apr_hash_make (pool);
 
       SVN_ERR (get_dir_contents (*dirents, "", rev, ra_lib, session, recurse,
-                                 pool));
+                                 ctx, pool));
     }
   else if (url_kind == svn_node_file)
     {
@@ -124,6 +112,10 @@ svn_client_ls (apr_hash_t **dirents,
 
       /* Re-open the session to the file's parent instead. */
       svn_path_split (url, &parent_url, &base_name, pool);
+      /* 'base_name' is now the last component of an URL, but we want
+         to use it as a plain file name. Therefore, we must URI-decode
+         it. */
+      base_name = svn_path_uri_decode(base_name, pool);
       SVN_ERR (svn_client__open_ra_session (&session, ra_lib, parent_url,
                                             NULL,
                                             NULL, NULL, FALSE, TRUE, 
@@ -135,20 +127,22 @@ svn_client_ls (apr_hash_t **dirents,
                                   NULL, NULL, pool));
       else
         return svn_error_create (SVN_ERR_RA_NOT_IMPLEMENTED, NULL,
-                                 "No get_dir() available for URL schema");
+                                 _("No get_dir() available for URL schema"));
 
       /* Copy the relevant entry into the caller's hash. */
       *dirents = apr_hash_make (pool);
       the_ent = apr_hash_get (parent_ents, base_name, APR_HASH_KEY_STRING);
       if (the_ent == NULL)
-        return svn_error_create (SVN_ERR_FS_NOT_FOUND, NULL,
-                                 "URL non-existent in that revision");
-        
+        return svn_error_createf (SVN_ERR_FS_NOT_FOUND, NULL,
+                                  _("URL '%s' non-existent in that revision"),
+                                  url);
+
       apr_hash_set (*dirents, base_name, APR_HASH_KEY_STRING, the_ent);
     }
   else
-    return svn_error_create (SVN_ERR_FS_NOT_FOUND, NULL,
-                             "URL non-existent in that revision");
+    return svn_error_createf (SVN_ERR_FS_NOT_FOUND, NULL,
+                              _("URL '%s' non-existent in that revision"),
+                              url);
 
   return SVN_NO_ERROR;
 }
