@@ -30,7 +30,9 @@
 #include "svn_client.h"
 #include "client.h"
 #include "svn_path.h"
+#include "svn_pools.h"
 
+#include "svn_private_config.h"
 
 
 /*** Code. ***/
@@ -143,8 +145,8 @@ svn_client_propset (const char *propname,
   if (is_revision_prop_name (propname))
     {
       return svn_error_createf (SVN_ERR_CLIENT_PROPERTY_NAME, NULL,
-                                "Revision property '%s' not allowed "
-                                "in this context", propname);
+                                _("Revision property '%s' not allowed "
+                                  "in this context"), propname);
     }
 
   if (svn_path_is_url (target))
@@ -153,19 +155,20 @@ svn_client_propset (const char *propname,
          if it's ever to support setting properties remotely. */
       return svn_error_createf
         (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-         "Setting property on non-local target '%s' not yet supported",
+         _("Setting property on non-local target '%s' not yet supported"),
          target);
     }
 
   if (propval && ! is_valid_prop_name (propname))
     return svn_error_createf (SVN_ERR_CLIENT_PROPERTY_NAME, NULL,
-                              "Bad property name: '%s'", propname);
+                              _("Bad property name: '%s'"), propname);
 
-  SVN_ERR (svn_wc_adm_probe_open (&adm_access, NULL, target, TRUE, TRUE, pool));
+  SVN_ERR (svn_wc_adm_probe_open2 (&adm_access, NULL, target, TRUE,
+                                   recurse ? -1 : 0, pool));
   SVN_ERR (svn_wc_entry (&node, target, adm_access, FALSE, pool));
   if (!node)
     return svn_error_createf (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
-                              "'%s' is not under version control", 
+                              _("'%s' is not under version control"), 
                               target);
 
   if (recurse && node->kind == svn_node_dir)
@@ -209,11 +212,11 @@ svn_client_revprop_set (const char *propname,
       && strchr (propval->data, '\n') != NULL 
       && (! force))
     return svn_error_create (SVN_ERR_CLIENT_REVISION_AUTHOR_CONTAINS_NEWLINE,
-                             NULL, "Value will not be set unless forced");
+                             NULL, _("Value will not be set unless forced"));
 
   if (propval && ! is_valid_prop_name (propname))
     return svn_error_createf (SVN_ERR_CLIENT_PROPERTY_NAME, NULL,
-                              "Bad property name: '%s'", propname);
+                              _("Bad property name: '%s'"), propname);
 
   /* Open an RA session for the URL. Note that we don't have a local
      directory, nor a place to put temp files. */
@@ -376,11 +379,12 @@ maybe_convert_to_url (const char **new_target,
       else
         pdir = target;
       
-      SVN_ERR (svn_wc_adm_open (&adm_access, NULL, pdir, FALSE, FALSE, pool));
+      SVN_ERR (svn_wc_adm_open2 (&adm_access, NULL, pdir, FALSE,
+                                 0, pool));
       SVN_ERR (svn_wc_entry (&entry, target, adm_access, FALSE, pool));
       if (! entry)
         return svn_error_createf (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
-                                  "'%s' is not under version control", 
+                                  _("'%s' is not under version control"), 
                                   target);
       *new_target = entry->url;
     }
@@ -434,7 +438,7 @@ remote_propget (apr_hash_t *props,
     {
       return svn_error_createf
         (SVN_ERR_NODE_UNKNOWN_KIND, NULL,
-         "Unknown node kind for '%s'",
+         _("Unknown node kind for '%s'"),
          svn_path_join (target_prefix, target_relative, pool));
     }
   
@@ -495,6 +499,7 @@ svn_client_propget (apr_hash_t **props,
   svn_wc_adm_access_t *adm_access;
   const svn_wc_entry_t *node;
   const char *utarget;  /* target, or the url for target */
+  const char *url;
   svn_revnum_t revnum;
 
   *props = apr_hash_make (pool);
@@ -505,55 +510,18 @@ svn_client_propget (apr_hash_t **props,
      requested property information is not available locally. */
   if (svn_path_is_url (utarget))
     {
-      void *ra_baton, *session;
+      void *session;
       svn_ra_plugin_t *ra_lib;
       svn_node_kind_t kind;
-      svn_opt_revision_t new_revision;  /* only used in one case */
 
-      SVN_ERR (svn_ra_init_ra_libs (&ra_baton, pool));
-      SVN_ERR (svn_ra_get_ra_library (&ra_lib, ra_baton, utarget, pool));
-      SVN_ERR (svn_client__open_ra_session (&session, ra_lib, utarget,
-                                            NULL, NULL, NULL,
-                                            FALSE, FALSE, ctx, pool));
-
-      /* Default to HEAD. */
-      if (revision->kind == svn_opt_revision_unspecified)
-        {
-          new_revision.kind = svn_opt_revision_head;
-          revision = &new_revision;
-        }
-
-      /* Handle the various different kinds of revisions. */
-      if ((revision->kind == svn_opt_revision_head)
-          || (revision->kind == svn_opt_revision_date)
-          || (revision->kind == svn_opt_revision_number))
-        {
-          SVN_ERR (svn_client__get_revision_number
-                   (&revnum, ra_lib, session, revision, NULL, pool));
-        }
-      else if (revision->kind == svn_opt_revision_previous)
-        {
-          if (svn_path_is_url (target))
-            {
-              return svn_error_createf
-                (SVN_ERR_ILLEGAL_TARGET, NULL,
-                 "'%s' is a URL, but revision kind requires a working copy",
-                 target);
-            }
-
-          /* target is a working copy path */
-          SVN_ERR (svn_client__get_revision_number
-                   (&revnum, NULL, NULL, revision, target, pool));
-        }
-      else
-        {
-          return svn_error_create
-            (SVN_ERR_CLIENT_BAD_REVISION, NULL, "Unknown revision kind");
-        }
+      /* Get an RA plugin for this filesystem object. */
+      SVN_ERR (svn_client__ra_lib_from_path (&ra_lib, &session, &revnum,
+                                             &url, target, revision,
+                                             ctx, pool));
 
       SVN_ERR (ra_lib->check_path (session, "", revnum, &kind, pool));
 
-      SVN_ERR (remote_propget (*props, propname, utarget, "",
+      SVN_ERR (remote_propget (*props, propname, url, "",
                                kind, revnum, ra_lib, session,
                                recurse, pool));
     }
@@ -561,12 +529,13 @@ svn_client_propget (apr_hash_t **props,
     {
       svn_boolean_t pristine;
 
-      SVN_ERR (svn_wc_adm_probe_open (&adm_access, NULL, target, FALSE, TRUE,
-                                      pool));
+      SVN_ERR (svn_wc_adm_probe_open2 (&adm_access, NULL, target,
+                                       FALSE, recurse ? -1 : 0, pool));
       SVN_ERR (svn_wc_entry (&node, target, adm_access, FALSE, pool));
       if (! node)
-        return svn_error_createf (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
-                                  "'%s' is not under version control", target);
+        return svn_error_createf 
+          (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
+           _("'%s' is not under version control"), target);
       
       SVN_ERR (svn_client__get_revision_number
                (&revnum, NULL, NULL, revision, target, pool));
@@ -677,7 +646,10 @@ push_props_on_list (apr_array_header_t *list,
  * REVNUM, obtained using RA_LIB and SESSION.  The item->node_name
  * will be "TARGET_PREFIX/TARGET_RELATIVE", and the value will be a
  * hash mapping 'const char *' property names onto 'svn_string_t *'
- * property values.  Allocate the new item and its contents in POOL.
+ * property values.  
+ *
+ * Allocate the new item and its contents in POOL.
+ * Do all looping, recursion, and temporary work in SCRATCHPOOL.
  *
  * KIND is the kind of the node at "TARGET_PREFIX/TARGET_RELATIVE".
  *
@@ -692,55 +664,68 @@ remote_proplist (apr_array_header_t *proplist,
                  svn_ra_plugin_t *ra_lib,
                  void *session,
                  svn_boolean_t recurse,
-                 apr_pool_t *pool)
+                 apr_pool_t *pool,
+                 apr_pool_t *scratchpool)
 {
   apr_hash_t *dirents;
-  apr_hash_t *prop_hash;
+  apr_hash_t *prop_hash, *final_hash;
   apr_hash_index_t *hi;
-  
+ 
   if (kind == svn_node_dir)
     {
       SVN_ERR (ra_lib->get_dir (session, target_relative, revnum,
                                 (recurse ? &dirents : NULL),
-                                NULL, &prop_hash, pool));
+                                NULL, &prop_hash, scratchpool));
     }
   else if (kind == svn_node_file)
     {
       SVN_ERR (ra_lib->get_file (session, target_relative, revnum,
-                                 NULL, NULL, &prop_hash, pool));
+                                 NULL, NULL, &prop_hash, scratchpool));
     }
   else
     {
       return svn_error_createf
         (SVN_ERR_NODE_UNKNOWN_KIND, NULL,
-         "Unknown node kind for '%s'",
+         _("Unknown node kind for '%s'"),
          svn_path_join (target_prefix, target_relative, pool));
     }
   
-  /* Filter out non-regular properties, since the RA layer
-     returns all kinds. */
-  for (hi = apr_hash_first (pool, prop_hash);
+  /* Filter out non-regular properties, since the RA layer returns all
+     kinds.  Copy regular properties keys/vals from the prop_hash
+     allocated in SCRATCHPOOL to the "final" hash allocated in POOL. */
+  final_hash = apr_hash_make (pool);
+  for (hi = apr_hash_first (scratchpool, prop_hash);
        hi;
        hi = apr_hash_next (hi))
     {
       const void *key;
       apr_ssize_t klen;
-      svn_prop_kind_t prop_kind;
-      
-      apr_hash_this (hi, &key, &klen, NULL);
+      void *val;
+      svn_prop_kind_t prop_kind;      
+      const char *name;
+      svn_string_t *value;
+
+      apr_hash_this (hi, &key, &klen, &val);
       prop_kind = svn_property_kind (NULL, (const char *) key);
       
-      if (prop_kind != svn_prop_regular_kind)
-        apr_hash_set (prop_hash, key, klen, NULL);
+      if (prop_kind == svn_prop_regular_kind)
+        {
+          name = apr_pstrdup (pool, (const char *) key);
+          value = svn_string_dup ((svn_string_t *) val, pool);
+          apr_hash_set (final_hash, name, klen, value);
+        }
     }
   
-  push_props_on_list (proplist, prop_hash,
-                      svn_path_join (target_prefix, target_relative, pool),
+  push_props_on_list (proplist, final_hash,
+                      svn_path_join (target_prefix, target_relative,
+                                     scratchpool),
                       pool);
   
   if (recurse && (kind == svn_node_dir) && (apr_hash_count (dirents) > 0))
     {
-      for (hi = apr_hash_first (pool, dirents);
+      apr_pool_t *subpool = svn_pool_create (scratchpool);
+      
+      for (hi = apr_hash_first (scratchpool, dirents);
            hi;
            hi = apr_hash_next (hi))
         {
@@ -750,12 +735,14 @@ remote_proplist (apr_array_header_t *proplist,
           svn_dirent_t *this_ent;
           const char *new_target_relative;
 
+          svn_pool_clear (subpool);
+
           apr_hash_this (hi, &key, NULL, &val);
           this_name = key;
           this_ent = val;
 
           new_target_relative = svn_path_join (target_relative,
-                                               this_name, pool);
+                                               this_name, subpool);
 
           SVN_ERR (remote_proplist (proplist,
                                     target_prefix,
@@ -765,8 +752,11 @@ remote_proplist (apr_array_header_t *proplist,
                                     ra_lib,
                                     session,
                                     recurse,
-                                    pool));
+                                    pool,
+                                    subpool));
         }
+
+      svn_pool_destroy (subpool);
     }
 
   return SVN_NO_ERROR;
@@ -852,6 +842,7 @@ svn_client_proplist (apr_array_header_t **props,
   svn_wc_adm_access_t *adm_access;
   const svn_wc_entry_t *node;
   const char *utarget;  /* target, or the url for target */
+  const char *url;
   svn_revnum_t revnum;
 
   *props = apr_array_make (pool, 5, sizeof (svn_client_proplist_item_t *));
@@ -862,68 +853,32 @@ svn_client_proplist (apr_array_header_t **props,
      requested property information is not available locally. */
   if (svn_path_is_url (utarget))
     {
-      void *ra_baton, *session;
+      void *session;
       svn_ra_plugin_t *ra_lib;
       svn_node_kind_t kind;
-      svn_opt_revision_t new_revision;  /* only used in one case */
-
-      SVN_ERR (svn_ra_init_ra_libs (&ra_baton, pool));
-      SVN_ERR (svn_ra_get_ra_library (&ra_lib, ra_baton, utarget, pool));
-      SVN_ERR (svn_client__open_ra_session (&session, ra_lib, utarget,
-                                            NULL, NULL, NULL,
-                                            FALSE, FALSE, ctx, pool));
-
-      /* Default to HEAD. */
-      if (revision->kind == svn_opt_revision_unspecified)
-        {
-          new_revision.kind = svn_opt_revision_head;
-          revision = &new_revision;
-        }
-
-      /* Handle the various different kinds of revisions. */
-      if ((revision->kind == svn_opt_revision_head)
-          || (revision->kind == svn_opt_revision_date)
-          || (revision->kind == svn_opt_revision_number))
-        {
-          SVN_ERR (svn_client__get_revision_number
-                   (&revnum, ra_lib, session, revision, NULL, pool));
-        }
-      else if (revision->kind == svn_opt_revision_previous)
-        {
-          if (svn_path_is_url (target))
-            {
-              return svn_error_createf
-                (SVN_ERR_ILLEGAL_TARGET, NULL,
-                 "'%s' is a URL, but revision kind requires a working copy",
-                 target);
-            }
-
-          /* target is a working copy path */
-          SVN_ERR (svn_client__get_revision_number
-                   (&revnum, NULL, NULL, revision, target, pool));
-        }
-      else
-        {
-          return svn_error_create
-            (SVN_ERR_CLIENT_BAD_REVISION, NULL, "Unknown revision kind");
-        }
 
+      /* Get an RA plugin for this filesystem object. */
+      SVN_ERR (svn_client__ra_lib_from_path (&ra_lib, &session, &revnum,
+                                             &url, target, revision,
+                                             ctx, pool));
+      
       SVN_ERR (ra_lib->check_path (session, "", revnum, &kind, pool));
 
-      SVN_ERR (remote_proplist (*props, utarget, "",
+      SVN_ERR (remote_proplist (*props, url, "",
                                 kind, revnum, ra_lib, session,
-                                recurse, pool));
+                                recurse, pool, svn_pool_create (pool)));
     }
   else  /* working copy path */
     {
       svn_boolean_t pristine;
 
-      SVN_ERR (svn_wc_adm_probe_open (&adm_access, NULL, target, FALSE, TRUE,
-                                      pool));
+      SVN_ERR (svn_wc_adm_probe_open2 (&adm_access, NULL, target,
+                                       FALSE, recurse ? -1 : 0, pool));
       SVN_ERR (svn_wc_entry (&node, target, adm_access, FALSE, pool));
       if (! node)
-        return svn_error_createf (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
-                                  "'%s' is not under version control", target);
+        return svn_error_createf 
+          (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
+           _("'%s' is not under version control"), target);
       
       SVN_ERR (svn_client__get_revision_number
                (&revnum, NULL, NULL, revision, target, pool));
