@@ -33,6 +33,7 @@
 #include "svn_config.h"
 #include "client.h"
 
+#include "svn_private_config.h"
 
 
 /*** Code. ***/
@@ -61,13 +62,14 @@ svn_client_switch (svn_revnum_t *result_rev,
 {
   const svn_ra_reporter_t *reporter;
   void *report_baton;
-  const svn_wc_entry_t *entry, *session_entry;
+  const svn_wc_entry_t *entry;
   const char *URL, *anchor, *target;
   void *ra_baton, *session;
   svn_ra_plugin_t *ra_lib;
   svn_revnum_t revnum;
   svn_error_t *err = SVN_NO_ERROR;
-  svn_wc_adm_access_t *adm_access;
+  svn_wc_adm_access_t *adm_access, *dir_access;
+  svn_node_kind_t kind;
   const char *diff3_cmd;
   svn_boolean_t timestamp_sleep = FALSE;  
   svn_boolean_t use_commit_times;
@@ -84,62 +86,37 @@ svn_client_switch (svn_revnum_t *result_rev,
                   SVN_CONFIG_OPTION_DIFF3_CMD, NULL);
 
   /* See if the user wants last-commit timestamps instead of current ones. */
-  svn_config_get_bool (cfg, &use_commit_times, SVN_CONFIG_SECTION_MISCELLANY,
-                       SVN_CONFIG_OPTION_USE_COMMIT_TIMES, FALSE);
+  SVN_ERR (svn_config_get_bool (cfg, &use_commit_times,
+                                SVN_CONFIG_SECTION_MISCELLANY,
+                                SVN_CONFIG_OPTION_USE_COMMIT_TIMES, FALSE));
 
   /* Sanity check.  Without these, the switch is meaningless. */
   assert (path);
   assert (switch_url && (switch_url[0] != '\0'));
 
-  /* ### Note: we don't pass the `recurse' flag as the tree_lock
-     argument to probe_open below, only because the RA layer is
-     planning to blindly invalidate all wcprops below path anyway, and
-     it needs a full tree lock to do so.  If someday the RA layer gets
-     smarter about this, then we can start passing `recurse' below
-     again.  See issue #1000 and related commits for details. */
-  SVN_ERR (svn_wc_adm_probe_open (&adm_access, NULL, path, TRUE, TRUE, pool));
-  SVN_ERR (svn_wc_entry (&entry, path, adm_access, FALSE, pool));
-  
+  /* Use PATH to get the update's anchor and targets and get a write lock */
+  SVN_ERR (svn_wc_get_actual_target (path, &anchor, &target, pool));
+
+  /* Get a write-lock on the anchor and target.  We need a lock on
+     the whole target tree so we can invalidate wcprops on it. */
+  SVN_ERR (svn_wc_adm_open2 (&adm_access, NULL, anchor, TRUE,
+                             *target ? 0 : -1, pool));
+  SVN_ERR (svn_io_check_path (path, &kind, pool));
+  if (*target && (kind == svn_node_dir))
+    SVN_ERR (svn_wc_adm_open2 (&dir_access, adm_access, path,
+                               TRUE, -1, pool));
+  else
+    dir_access = adm_access;
+
+  SVN_ERR (svn_wc_entry (&entry, anchor, adm_access, FALSE, pool));
   if (! entry)
-    return svn_error_createf
-      (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
-       "'%s' is not under version control", path);
-
+    return svn_error_createf (SVN_ERR_UNVERSIONED_RESOURCE, NULL, 
+                              _("'%s' is not under version control"), anchor);
   if (! entry->url)
-    return svn_error_createf
-      (SVN_ERR_ENTRY_MISSING_URL, NULL,
-       "Entry '%s' has no URL", path);
+    return svn_error_createf (SVN_ERR_ENTRY_MISSING_URL, NULL,
+                              _("Directory '%s' has no URL"), anchor);
 
-  if (entry->kind == svn_node_file)
-    {
-      SVN_ERR (svn_wc_get_actual_target (path, &anchor, &target, pool));
-      
-      /* get the parent entry */
-      SVN_ERR (svn_wc_entry (&session_entry, anchor, adm_access, FALSE, pool));
-      if (! session_entry)
-        return svn_error_createf
-          (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
-           "'%s' is not under version control", anchor);
-
-      if (! session_entry->url)
-        return svn_error_createf
-          (SVN_ERR_ENTRY_MISSING_URL, NULL,
-           "Directory '%s' has no URL", anchor);
-    }
-  else if (entry->kind == svn_node_dir)
-    {
-      /* Unlike 'svn up', we do *not* split the local path into an
-         anchor/target pair.  We do this because we know that the
-         target isn't going to be deleted, because we're doing a
-         switch.  This means the update editor gets anchored on PATH
-         itself, and thus PATH's name will never change, which is
-         exactly what we want. */
-      anchor = path;
-      target = "";
-      session_entry = entry;
-    }
-
-  URL = apr_pstrdup (pool, session_entry->url);
+  URL = apr_pstrdup (pool, entry->url);
 
   /* Get revnum set to something meaningful, so we can fetch the
      switch editor. */
@@ -182,7 +159,7 @@ svn_client_switch (svn_revnum_t *result_rev,
      We pass NULL for traversal_info because this is a switch, not an
      update, and therefore we don't want to handle any externals
      except the ones directly affected by the switch. */ 
-  err = svn_wc_crawl_revisions (path, adm_access, reporter, report_baton,
+  err = svn_wc_crawl_revisions (path, dir_access, reporter, report_baton,
                                 TRUE, recurse, use_commit_times,
                                 ctx->notify_func, ctx->notify_baton,
                                 NULL, /* no traversal info */
