@@ -2,7 +2,7 @@
  * commit.c :  routines for committing changes to the server
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -32,6 +32,7 @@
 #endif
 #include <assert.h>
 
+#include <ne_socket.h>
 #include <ne_request.h>
 #include <ne_props.h>
 #include <ne_basic.h>
@@ -46,6 +47,8 @@
 #include "svn_dav.h"
 #include "svn_base64.h"
 
+#include "svn_private_config.h"
+
 #include "ra_dav.h"
 
 
@@ -72,6 +75,7 @@ typedef struct
   const char *vsn_url;
   const char *wr_url;
   const char *local_path;
+  apr_pool_t *pool; /* pool in which this resource is allocated. */
 
 } resource_t;
 
@@ -80,9 +84,6 @@ typedef struct
   svn_ra_session_t *ras;
   const char *activity_url;
 
-  /* ### resources may not be needed */
-  apr_hash_t *resources;        /* URL (const char *) -> RESOURCE_T */
-
   apr_hash_t *valid_targets;
 
   svn_ra_get_wc_prop_func_t get_func;
@@ -97,15 +98,9 @@ typedef struct
   /* Log message for the commit. */
   const char *log_msg;
 
-  /* The new revision created by this commit. */
-  svn_revnum_t *new_rev;
-
-  /* The date (according to the repository) of this commit. */
-  const char **committed_date;
-
-  /* The author (also according to the repository) of this commit. */
-  const char **committed_author;
-
+  /* The commit callback and baton */
+  svn_commit_callback_t callback;
+  void *callback_baton;
 } commit_ctx_t;
 
 typedef struct
@@ -136,10 +131,31 @@ static const ne_propname fetch_props[] =
 
 static const ne_propname log_message_prop = { SVN_DAV_PROP_NS_SVN, "log" };
 
-static svn_error_t * simple_request(svn_ra_session_t *ras, const char *method,
-                                    const char *url, int *code,
+/* perform a deep copy of BASE into POOL, and return the result. */
+static resource_t * dup_resource(resource_t *base, apr_pool_t *pool)
+{
+  resource_t *rsrc = apr_pcalloc(pool, sizeof(*rsrc));
+  rsrc->pool = pool;
+  rsrc->revision = base->revision;
+  rsrc->url = base->url ?
+    apr_pstrdup(pool, base->url) : NULL;
+  rsrc->vsn_url = base->vsn_url ?
+    apr_pstrdup(pool, base->vsn_url) : NULL;
+  rsrc->wr_url = base->wr_url ?
+    apr_pstrdup(pool, base->wr_url) : NULL;
+  rsrc->local_path = base->local_path ? 
+    apr_pstrdup(pool, base->local_path) : NULL;
+  return rsrc;
+}
+
+static svn_error_t * simple_request(svn_ra_session_t *ras, 
+                                    const char *method,
+                                    const char *url, 
+                                    int *code,
                                     apr_hash_t *extra_headers,
-                                    int okay_1, int okay_2)
+                                    int okay_1, 
+                                    int okay_2, 
+                                    apr_pool_t *pool)
 {
   ne_request *req;
 
@@ -148,7 +164,7 @@ static svn_error_t * simple_request(svn_ra_session_t *ras, const char *method,
   if (req == NULL)
     {
       return svn_error_createf(SVN_ERR_RA_DAV_CREATING_REQUEST, NULL,
-                               "Could not create a request (%s '%s')",
+                               _("Could not create a request (%s '%s')"),
                                method, url);
     }
 
@@ -156,12 +172,12 @@ static svn_error_t * simple_request(svn_ra_session_t *ras, const char *method,
   if (extra_headers != NULL)
     {
       apr_hash_index_t *hi;
-      for (hi = apr_hash_first (ras->pool, extra_headers);
-           hi; hi = apr_hash_next (hi))
+      for (hi = apr_hash_first(pool, extra_headers);
+           hi; hi = apr_hash_next(hi))
         {
           const void *key;
           void *val;
-          apr_hash_this (hi, &key, NULL, &val);
+          apr_hash_this(hi, &key, NULL, &val);
           ne_add_request_header(req, (const char *) key, (const char *) val); 
         }
     }
@@ -169,11 +185,23 @@ static svn_error_t * simple_request(svn_ra_session_t *ras, const char *method,
   /* run the request and get the resulting status code (and svn_error_t) */
   SVN_ERR( svn_ra_dav__request_dispatch(code, req, ras->sess,
                                         method, url, okay_1, okay_2,
-                                        ras->pool) );
+                                        pool) );
 
   return SVN_NO_ERROR;
 }
 
+
+static svn_error_t * delete_activity(void *edit_baton,
+                                     apr_pool_t *pool)
+{
+  commit_ctx_t *cc = edit_baton;
+  return simple_request(cc->ras, "DELETE", cc->activity_url, NULL,
+                        NULL, 204 /* No Content */, 404 /* Not Found */, pool);
+}
+
+
+/* Get the version resource URL for RSRC, storing it in
+   RSRC->vsn_url.  Use POOL for all temporary allocations. */
 static svn_error_t * get_version_url(commit_ctx_t *cc,
                                      resource_t *rsrc,
                                      svn_boolean_t force,
@@ -194,8 +222,8 @@ static svn_error_t * get_version_url(commit_ctx_t *cc,
                                pool) );
       if (vsn_url_value != NULL)
         {
-          rsrc->vsn_url = vsn_url_value->data;
-          return NULL;
+          rsrc->vsn_url = apr_pstrdup (rsrc->pool, vsn_url_value->data);
+          return SVN_NO_ERROR;
         }
 
       /* whoops. it wasn't there. go grab it from the server. */
@@ -236,12 +264,12 @@ static svn_error_t * get_version_url(commit_ctx_t *cc,
       return svn_error_create(APR_EGENERAL, NULL,
                               "Could not fetch the Version Resource URL "
                               "(needed during an import or when it is "
-                              "missing from the local, cached props).");
+                              "missing from the local, cached props)");
     }
 
   /* ensure we get the proper lifetime for this URL since it is going into
      a resource object. */
-  rsrc->vsn_url = apr_pstrdup(cc->ras->pool, url_str->data);
+  rsrc->vsn_url = apr_pstrdup(rsrc->pool, url_str->data);
 
   if (cc->push_func != NULL)
     {
@@ -258,10 +286,10 @@ static svn_error_t * get_version_url(commit_ctx_t *cc,
 
 /* When FORCE is true, then we force a query to the server, ignoring any
    cached property. */
-static svn_error_t * get_activity_collection(
-  commit_ctx_t *cc,
-  const svn_string_t **activity_coll,
-  svn_boolean_t force)
+static svn_error_t * get_activity_collection(commit_ctx_t *cc,
+                                             const svn_string_t **collection,
+                                             svn_boolean_t force,
+                                             apr_pool_t *pool)
 {
   if (!force && cc->get_func != NULL)
     {
@@ -272,10 +300,10 @@ static svn_error_t * get_activity_collection(
       SVN_ERR( (*cc->get_func)(cc->cb_baton,
                                "",
                                SVN_RA_DAV__LP_ACTIVITY_COLL,
-                               activity_coll,
-                               cc->ras->pool) );
+                               collection,
+                               pool) );
 
-      if (*activity_coll != NULL)
+      if (*collection != NULL)
         {
           /* the property was there. return it. */
           return SVN_NO_ERROR;
@@ -285,10 +313,10 @@ static svn_error_t * get_activity_collection(
     }
 
   /* use our utility function to fetch the activity URL */
-  SVN_ERR( svn_ra_dav__get_activity_collection(activity_coll,
+  SVN_ERR( svn_ra_dav__get_activity_collection(collection,
                                                cc->ras,
                                                cc->ras->root.path,
-                                               cc->ras->pool) );
+                                               pool) );
 
   if (cc->push_func != NULL)
     {
@@ -296,14 +324,15 @@ static svn_error_t * get_activity_collection(
       SVN_ERR( (*cc->push_func)(cc->cb_baton,
                                "",
                                SVN_RA_DAV__LP_ACTIVITY_COLL,
-                               *activity_coll,
-                               cc->ras->pool) );
+                               *collection,
+                               pool) );
     }
 
   return SVN_NO_ERROR;
 }
 
-static svn_error_t * create_activity(commit_ctx_t *cc)
+static svn_error_t * create_activity(commit_ctx_t *cc,
+                                     apr_pool_t *pool)
 {
   const svn_string_t * activity_collection;
   apr_uuid_t uuid;
@@ -317,23 +346,22 @@ static svn_error_t * create_activity(commit_ctx_t *cc)
 
   /* get the URL where we'll create activities, construct the URL
      for the activity, and create the activity. */
-  SVN_ERR( get_activity_collection(cc, &activity_collection, FALSE) );
+  SVN_ERR( get_activity_collection(cc, &activity_collection, FALSE, pool) );
   url = svn_path_url_add_component(activity_collection->data, 
-                                   uuid_buf, cc->ras->pool);
+                                   uuid_buf, pool);
   SVN_ERR( simple_request(cc->ras, "MKACTIVITY", url, &code, NULL,
-                          201 /* Created */,
-                          404 /* Not Found */) );
+                          201 /* Created */, 404 /* Not Found */, pool) );
 
   /* if we get a 404, then it generally means that the cached activity
      collection no longer exists. Retry the sequence, but force a query
      to the server for the activity collection. */
   if (code == 404)
     {
-      SVN_ERR( get_activity_collection(cc, &activity_collection, TRUE) );
+      SVN_ERR( get_activity_collection(cc, &activity_collection, TRUE, pool) );
       url = svn_path_url_add_component(activity_collection->data, 
-                                       uuid_buf, cc->ras->pool);
+                                       uuid_buf, pool);
       SVN_ERR( simple_request(cc->ras, "MKACTIVITY", url, &code,
-                              NULL, 201, 0) );
+                              NULL, 201, 0, pool) );
     }
 
   cc->activity_url = url;
@@ -341,7 +369,7 @@ static svn_error_t * create_activity(commit_ctx_t *cc)
   return SVN_NO_ERROR;
 }
 
-/* add a child resource. TEMP_POOL should be as "temporary" as possible,
+/* Add a child resource.  POOL should be as "temporary" as possible,
    but probably not as far as requiring a new temp pool. */
 static svn_error_t * add_child(resource_t **child,
                                commit_ctx_t *cc,
@@ -349,10 +377,8 @@ static svn_error_t * add_child(resource_t **child,
                                const char *name,
                                int created,
                                svn_revnum_t revision,
-                               apr_pool_t *temp_pool)
+                               apr_pool_t *pool)
 {
-  /* use ras->pool for the proper lifetime */
-  apr_pool_t *pool = cc->ras->pool;
   resource_t *rsrc;
 
   /* ### todo:  This from Yoshiki Hayashi <yoshiki@xemacs.org>:
@@ -363,6 +389,7 @@ static svn_error_t * add_child(resource_t **child,
   */
 
   rsrc = apr_pcalloc(pool, sizeof(*rsrc));
+  rsrc->pool = pool;
   rsrc->revision = revision;
   rsrc->url = svn_path_url_add_component(parent->url, name, pool);
   rsrc->local_path = svn_path_join(parent->local_path, name, pool);
@@ -379,9 +406,7 @@ static svn_error_t * add_child(resource_t **child,
      This means it has a VR URL already, and the WR URL won't exist
      until it's "checked out". */
   else
-    SVN_ERR( get_version_url(cc, rsrc, FALSE, temp_pool) );
-
-  apr_hash_set(cc->resources, rsrc->url, APR_HASH_KEY_STRING, rsrc);
+    SVN_ERR( get_version_url(cc, rsrc, FALSE, pool) );
 
   *child = rsrc;
   return SVN_NO_ERROR;
@@ -391,7 +416,8 @@ static svn_error_t * do_checkout(commit_ctx_t *cc,
                                  const char *vsn_url,
                                  svn_boolean_t allow_404,
                                  int *code,
-                                 char **locn)
+                                 char **locn,
+                                 apr_pool_t *pool)
 {
   ne_request *req;
   const char *body;
@@ -411,7 +437,7 @@ static svn_error_t * do_checkout(commit_ctx_t *cc,
     }
 
   /* ### store this into cc to avoid pool growth */
-  body = apr_psprintf(cc->ras->pool,
+  body = apr_psprintf(pool,
                       "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
                       "<D:checkout xmlns:D=\"DAV:\">"
                       "<D:activity-set>"
@@ -431,62 +457,69 @@ static svn_error_t * do_checkout(commit_ctx_t *cc,
                                       "CHECKOUT", vsn_url,
                                       201 /* Created */,
                                       allow_404 ? 404 /* Not Found */ : 0,
-                                      cc->ras->pool);
+                                      pool);
 }
 
-/* check out the specified resource (if it hasn't been checked out yet) */
+
 static svn_error_t * checkout_resource(commit_ctx_t *cc,
-                                       resource_t *res,
-                                       svn_boolean_t allow_404)
+                                       resource_t *rsrc,
+                                       svn_boolean_t allow_404, 
+                                       apr_pool_t *pool)
 {
   int code;
   char *locn = NULL;
   ne_uri parse;
   svn_error_t *err;
 
-  if (res->wr_url != NULL)
+  if (rsrc->wr_url != NULL)
     {
       /* already checked out! */
       return NULL;
     }
 
   /* check out the Version Resource */
-  err = do_checkout(cc, res->vsn_url, allow_404, &code, &locn);
+  err = do_checkout(cc, rsrc->vsn_url, allow_404, &code, &locn, pool);
 
   /* possibly run the request again, with a re-fetched Version Resource */
   if (err == NULL && allow_404 && code == 404)
     {
+      /* free the LOCN if it got assigned. */
+      if (locn)
+        free(locn);
+
       /* re-fetch, forcing a query to the server */
-      SVN_ERR( get_version_url(cc, res, TRUE, cc->ras->pool) );
+      SVN_ERR( get_version_url(cc, rsrc, TRUE, pool) );
 
       /* do it again, but don't allow a 404 this time */
-      err = do_checkout(cc, res->vsn_url, FALSE, &code, &locn);
+      err = do_checkout(cc, rsrc->vsn_url, FALSE, &code, &locn, pool);
     }
 
   /* special-case when conflicts occur */
   if (err)
     {
+      /* free the LOCN if it got assigned. */
+      if (locn)
+        free(locn);
+      
       if (err->apr_err == SVN_ERR_FS_CONFLICT)
         return svn_error_createf
           (err->apr_err, err,
-           "Your file or directory '%s' is probably out-of-date.",
-           res->local_path);
+           "Your file or directory '%s' is probably out-of-date",
+           rsrc->local_path);
       return err;
     }
 
   /* we got the header, right? */
   if (locn == NULL)
-    {
-      return svn_error_create(SVN_ERR_RA_DAV_REQUEST_FAILED, NULL,
-                              "The CHECKOUT response did not contain a "
-                              "Location: header.");
-    }
+    return svn_error_create(SVN_ERR_RA_DAV_REQUEST_FAILED, NULL,
+                            "The CHECKOUT response did not contain a "
+                            "'Location:' header");
 
   /* The location is an absolute URI. We want just the path portion. */
   /* ### what to do with the rest? what if it points somewhere other
      ### than the current session? */
   ne_uri_parse(locn, &parse);
-  res->wr_url = apr_pstrdup(cc->ras->pool, parse.path);
+  rsrc->wr_url = apr_pstrdup(rsrc->pool, parse.path);
   ne_uri_free(&parse);
   free(locn);
 
@@ -590,9 +623,9 @@ static svn_error_t * commit_open_root(void *edit_baton,
   resource_baton_t *root;
   resource_t *rsrc;
 
-  /* create the root resource. no wr_url (yet). use ras->pool for the
-     proper lifetime of the resource. */
-  rsrc = apr_pcalloc(cc->ras->pool, sizeof(*rsrc));
+  /* create the root resource. no wr_url (yet). */
+  rsrc = apr_pcalloc(dir_pool, sizeof(*rsrc));
+  rsrc->pool = dir_pool;
 
   /* ### should this be 'base_revision' here? we might not always be
      ### working against the head! (think "properties"). */
@@ -603,8 +636,6 @@ static svn_error_t * commit_open_root(void *edit_baton,
 
   SVN_ERR( get_version_url(cc, rsrc, FALSE, dir_pool) );
 
-  apr_hash_set(cc->resources, rsrc->url, APR_HASH_KEY_STRING, rsrc);
-
   root = apr_pcalloc(dir_pool, sizeof(*root));
   root->pool = dir_pool;
   root->cc = cc;
@@ -629,31 +660,25 @@ static svn_error_t * commit_delete_entry(const char *path,
 
   if (SVN_IS_VALID_REVNUM(revision))
     {
-      const char *revstr = apr_psprintf(pool, "%" SVN_REVNUM_T_FMT, revision);
+      const char *revstr = apr_psprintf(pool, "%ld", revision);
       extra_headers = apr_hash_make(pool);
       apr_hash_set(extra_headers, SVN_DAV_VERSION_NAME_HEADER,
                    APR_HASH_KEY_STRING, revstr);
     }
 
   /* get the URL to the working collection */
-  SVN_ERR( checkout_resource(parent->cc, parent->rsrc, TRUE) );
+  SVN_ERR( checkout_resource(parent->cc, parent->rsrc, TRUE, pool) );
 
   /* create the URL for the child resource */
   child = svn_path_url_add_component(parent->rsrc->wr_url, name, pool);
 
-  /* Note: the child cannot have a resource stored in the resources table
-     because of the editor traversal rules. That is: this is the first time
-     we have seen anything about the child, and we're deleting it. As a
-     corollary, we know the child hasn't been checked out. */
-
-  /* delete the child resource */
   /* ### 404 is ignored, because mod_dav_svn is effectively merging
      against the HEAD revision on-the-fly.  In such a universe, a
      failed deletion (because it's already missing) is OK;  deletion
      is an idempotent merge operation. */
   SVN_ERR( simple_request(parent->cc->ras, "DELETE", child, &code,
                           extra_headers,
-                          204 /* Created */, 404 /* Not Found */) );
+                          204 /* Created */, 404 /* Not Found */, pool) );
 
   /* Add this path to the valid targets hash. */
   add_valid_target (parent->cc, path, svn_nonrecursive);
@@ -672,25 +697,28 @@ static svn_error_t * commit_add_dir(const char *path,
   resource_baton_t *child;
   int code;
   const char *name = svn_path_basename(path, dir_pool);
+  apr_pool_t *workpool = svn_pool_create(dir_pool);
+  resource_t *rsrc = NULL;
 
   /* check out the parent resource so that we can create the new collection
      as one of its children. */
-  SVN_ERR( checkout_resource(parent->cc, parent->rsrc, TRUE) );
+  SVN_ERR( checkout_resource(parent->cc, parent->rsrc, TRUE, dir_pool) );
 
   /* create a child object that contains all the resource urls */
   child = apr_pcalloc(dir_pool, sizeof(*child));
   child->pool = dir_pool;
   child->cc = parent->cc;
   child->created = TRUE;
-  SVN_ERR( add_child(&child->rsrc, parent->cc, parent->rsrc,
-                     name, 1, SVN_INVALID_REVNUM, dir_pool) );
+  SVN_ERR( add_child(&rsrc, parent->cc, parent->rsrc,
+                     name, 1, SVN_INVALID_REVNUM, workpool) );
+  child->rsrc = dup_resource(rsrc, dir_pool);
 
   if (! copyfrom_path)
     {
       /* This a new directory with no history, so just create a new,
          empty collection */
       SVN_ERR( simple_request(parent->cc->ras, "MKCOL", child->rsrc->wr_url,
-                              &code, NULL, 201 /* Created */, 0) );
+                              &code, NULL, 201 /* Created */, 0, workpool) );
     }
   else
     {
@@ -708,7 +736,7 @@ static svn_error_t * commit_add_dir(const char *path,
                                              parent->cc->ras->sess,
                                              copyfrom_path,
                                              copyfrom_revision,
-                                             dir_pool));
+                                             workpool));
 
 
       /* Combine the BC-URL and relative path; this is the main
@@ -717,7 +745,7 @@ static svn_error_t * commit_add_dir(const char *path,
          part of the child object. */
       copy_src = svn_path_url_add_component(bc_url.data,
                                             bc_relative.data, 
-                                            dir_pool);
+                                            workpool);
 
       /* Have neon do the COPY. */
       status = ne_copy(parent->cc->ras->sess,
@@ -730,7 +758,7 @@ static svn_error_t * commit_add_dir(const char *path,
         {
           const char *msg = apr_psprintf(dir_pool, "COPY of %s", path);
           return svn_ra_dav__convert_error(parent->cc->ras->sess,
-                                           msg, status);
+                                           msg, status, workpool);
         }
     }
 
@@ -738,6 +766,7 @@ static svn_error_t * commit_add_dir(const char *path,
   add_valid_target (parent->cc, path, 
                     copyfrom_path ? svn_recursive : svn_nonrecursive);
 
+  svn_pool_destroy(workpool);
   *child_baton = child;
   return SVN_NO_ERROR;
 }
@@ -751,12 +780,16 @@ static svn_error_t * commit_open_dir(const char *path,
   resource_baton_t *parent = parent_baton;
   resource_baton_t *child = apr_pcalloc(dir_pool, sizeof(*child));
   const char *name = svn_path_basename(path, dir_pool);
+  apr_pool_t *workpool = svn_pool_create(dir_pool);
+  resource_t *rsrc = NULL;
 
   child->pool = dir_pool;
   child->cc = parent->cc;
   child->created = FALSE;
-  SVN_ERR( add_child(&child->rsrc, parent->cc, parent->rsrc,
-                     name, 0, base_revision, dir_pool) );
+
+  SVN_ERR( add_child(&rsrc, parent->cc, parent->rsrc,
+                     name, 0, base_revision, workpool) );
+  child->rsrc = dup_resource(rsrc, dir_pool);
 
   /*
   ** Note: open_dir simply means that a change has occurred somewhere
@@ -767,7 +800,7 @@ static svn_error_t * commit_open_dir(const char *path,
   **       will not be used: a true replacement is modeled with a "delete"
   **       followed by an "add".
   */
-
+  svn_pool_destroy(workpool);
   *child_baton = child;
   return SVN_NO_ERROR;
 }
@@ -781,10 +814,10 @@ static svn_error_t * commit_change_dir_prop(void *dir_baton,
 
   /* record the change. it will be applied at close_dir time. */
   /* ### we should put this into the dir_baton's pool */
-  record_prop_change(dir->cc->ras->pool, dir, name, value);
+  record_prop_change(dir->pool, dir, name, value);
 
   /* do the CHECKOUT sooner rather than later */
-  SVN_ERR( checkout_resource(dir->cc, dir->rsrc, TRUE) );
+  SVN_ERR( checkout_resource(dir->cc, dir->rsrc, TRUE, pool) );
 
   /* Add this path to the valid targets hash. */
   add_valid_target (dir->cc, dir->rsrc->local_path, svn_nonrecursive);
@@ -814,6 +847,8 @@ static svn_error_t * commit_add_file(const char *path,
   resource_baton_t *parent = parent_baton;
   resource_baton_t *file;
   const char *name = svn_path_basename(path, file_pool);
+  apr_pool_t *workpool = svn_pool_create(file_pool);
+  resource_t *rsrc = NULL;
 
   /*
   ** To add a new file into the repository, we CHECKOUT the parent
@@ -825,15 +860,16 @@ static svn_error_t * commit_add_file(const char *path,
   */
 
   /* Do the parent CHECKOUT first */
-  SVN_ERR( checkout_resource(parent->cc, parent->rsrc, TRUE) );
+  SVN_ERR( checkout_resource(parent->cc, parent->rsrc, TRUE, workpool) );
 
   /* Construct a file_baton that contains all the resource urls. */
   file = apr_pcalloc(file_pool, sizeof(*file));
   file->pool = file_pool;
   file->cc = parent->cc;
   file->created = TRUE;
-  SVN_ERR( add_child(&file->rsrc, parent->cc, parent->rsrc,
-                     name, 1, SVN_INVALID_REVNUM, file_pool) );
+  SVN_ERR( add_child(&rsrc, parent->cc, parent->rsrc,
+                     name, 1, SVN_INVALID_REVNUM, workpool) );
+  file->rsrc = dup_resource(rsrc, file_pool);
 
   /* If the parent directory existed before this commit then there may be a
      file with this URL already. We need to ensure such a file does not
@@ -842,7 +878,7 @@ static svn_error_t * commit_add_file(const char *path,
      half of a "replace".  
 
      ### For now, we'll assume that if this path has already been
-     added to the valid targets hash, that addition occured during the
+     added to the valid targets hash, that addition occurred during the
      "delete" phase (if that's not the case, this editor is being
      driven incorrectly, as we should never visit the same path twice
      except in a delete+add situation). */
@@ -853,17 +889,16 @@ static svn_error_t * commit_add_file(const char *path,
       svn_error_t *err = svn_ra_dav__get_starting_props(&res,
                                                         file->cc->ras->sess,
                                                         file->rsrc->url, NULL,
-                                                        file_pool);
+                                                        workpool);
       if (!err)
         {
           /* If the PROPFIND succeeds the file already exists */
           return svn_error_createf(SVN_ERR_RA_DAV_ALREADY_EXISTS, NULL,
-                                   "file '%s' already exists", file->rsrc->url);
+                                   _("File '%s' already exists"), 
+                                   file->rsrc->url);
         }
-      else if (err->apr_err == SVN_ERR_RA_DAV_REQUEST_FAILED)
+      else if (err->apr_err == SVN_ERR_RA_DAV_PATH_NOT_FOUND)
         {
-          /* ### TODO: This is what we get if the file doesn't exist
-             but an explicit not-found error might be better */
           svn_error_clear(err);
         }
       else
@@ -899,7 +934,7 @@ static svn_error_t * commit_add_file(const char *path,
                                              parent->cc->ras->sess,
                                              copyfrom_path,
                                              copyfrom_revision,
-                                             file_pool));
+                                             workpool));
 
 
       /* Combine the BC-URL and relative path; this is the main
@@ -908,7 +943,7 @@ static svn_error_t * commit_add_file(const char *path,
          part of the file_baton. */
       copy_src = svn_path_url_add_component(bc_url.data, 
                                             bc_relative.data, 
-                                            file_pool);
+                                            workpool);
 
       /* Have neon do the COPY. */
       status = ne_copy(parent->cc->ras->sess,
@@ -921,13 +956,15 @@ static svn_error_t * commit_add_file(const char *path,
         {
           const char *msg = apr_psprintf(file_pool, "COPY of %s", path);
           return svn_ra_dav__convert_error(parent->cc->ras->sess,
-                                           msg, status);
+                                           msg, status, workpool);
         }
     }
 
   /* Add this path to the valid targets hash. */
   add_valid_target (parent->cc, path, svn_nonrecursive);
 
+  svn_pool_destroy(workpool);
+
   /* return the file_baton */
   *file_baton = file;
   return SVN_NO_ERROR;
@@ -942,16 +979,19 @@ static svn_error_t * commit_open_file(const char *path,
   resource_baton_t *parent = parent_baton;
   resource_baton_t *file;
   const char *name = svn_path_basename(path, file_pool);
+  apr_pool_t *workpool = svn_pool_create(file_pool);
+  resource_t *rsrc = NULL;
 
   file = apr_pcalloc(file_pool, sizeof(*file));
   file->pool = file_pool;
   file->cc = parent->cc;
   file->created = FALSE;
-  SVN_ERR( add_child(&file->rsrc, parent->cc, parent->rsrc,
-                     name, 0, base_revision, file_pool) );
+  SVN_ERR( add_child(&rsrc, parent->cc, parent->rsrc,
+                     name, 0, base_revision, workpool) );
+  file->rsrc = dup_resource(rsrc, file_pool);
 
   /* do the CHECKOUT now. we'll PUT the new file contents later on. */
-  SVN_ERR( checkout_resource(parent->cc, file->rsrc, TRUE) );
+  SVN_ERR( checkout_resource(parent->cc, file->rsrc, TRUE, workpool) );
 
   /* ### wait for apply_txdelta before doing a PUT. it might arrive a
      ### "long time" from now. certainly after many other operations, so
@@ -959,12 +999,14 @@ static svn_error_t * commit_open_file(const char *path,
      ### so... anything else to do here? what about the COPY case?
   */
 
+  svn_pool_destroy(workpool);
   *file_baton = file;
   return SVN_NO_ERROR;
 }
 
 static svn_error_t * commit_stream_write(void *baton,
-                                         const char *data, apr_size_t *len)
+                                         const char *data, 
+                                         apr_size_t *len)
 {
   put_baton_t *pb = baton;
   apr_status_t status;
@@ -972,8 +1014,8 @@ static svn_error_t * commit_stream_write(void *baton,
   /* drop the data into our temp file */
   status = apr_file_write_full(pb->tmpfile, data, *len, NULL);
   if (status)
-    return svn_error_create(status, NULL,
-                            "Could not write svndiff to temp file.");
+    return svn_error_wrap_apr(status, 
+                              _("Could not write svndiff to temp file"));
 
   return SVN_NO_ERROR;
 }
@@ -1004,7 +1046,8 @@ commit_apply_txdelta(void *file_baton,
   /* Use the client callback to create a tmpfile. */
   SVN_ERR(file->cc->ras->callbacks->open_tmp_file 
           (&baton->tmpfile, 
-           file->cc->ras->callback_baton));
+           file->cc->ras->callback_baton,
+           file->pool));
 
   /* ### register a cleanup on file_pool which closes the file; this
      ### will ensure that the file always gets tossed, even if we exit
@@ -1030,10 +1073,10 @@ static svn_error_t * commit_change_file_prop(void *file_baton,
 
   /* record the change. it will be applied at close_file time. */
   /* ### we should put this into the file_baton's pool */
-  record_prop_change(file->cc->ras->pool, file, name, value);
+  record_prop_change(file->pool, file, name, value);
 
   /* do the CHECKOUT sooner rather than later */
-  SVN_ERR( checkout_resource(file->cc, file->rsrc, TRUE) );
+  SVN_ERR( checkout_resource(file->cc, file->rsrc, TRUE, pool) );
 
   /* Add this path to the valid targets hash. */
   add_valid_target (file->cc, file->rsrc->local_path, svn_nonrecursive);
@@ -1062,7 +1105,7 @@ static svn_error_t * commit_close_file(void *file_baton,
       if (req == NULL)
         {
           return svn_error_createf(SVN_ERR_RA_DAV_CREATING_REQUEST, NULL,
-                                   "Could not create a PUT request (%s)",
+                                   _("Could not create a PUT request (%s)"),
                                    url);
         }
       
@@ -1081,6 +1124,7 @@ static svn_error_t * commit_close_file(void *file_baton,
       if (err)
         {
           apr_file_close(pb->tmpfile);
+          ne_request_destroy(req);
           return err;
         }
       
@@ -1109,34 +1153,50 @@ static svn_error_t * commit_close_edit(void *edit_baton,
                                        apr_pool_t *pool)
 {
   commit_ctx_t *cc = edit_baton;
+  svn_revnum_t new_rev;
+  const char *committed_date;
+  const char *committed_author;
 
-  /* ### different pool? */
-  SVN_ERR( svn_ra_dav__merge_activity(cc->new_rev,
-                                      cc->committed_date,
-                                      cc->committed_author,
+  SVN_ERR( svn_ra_dav__merge_activity(&new_rev,
+                                      &committed_date,
+                                      &committed_author,
                                       cc->ras,
                                       cc->ras->root.path,
                                       cc->activity_url,
                                       cc->valid_targets,
                                       cc->disable_merge_response,
-                                      cc->ras->pool) );
-
+                                      pool) );
+  SVN_ERR( delete_activity(edit_baton, pool) );
   SVN_ERR( svn_ra_dav__maybe_store_auth_info(cc->ras) );
 
+  if (new_rev != SVN_INVALID_REVNUM)
+    SVN_ERR( cc->callback (new_rev, committed_date, committed_author,
+                           cc->callback_baton));
+
   return SVN_NO_ERROR;
 }
 
+
+static svn_error_t * commit_abort_edit(void *edit_baton,
+                                       apr_pool_t *pool)
+{
+  return delete_activity(edit_baton, pool);
+}
+
+
 static svn_error_t * apply_log_message(commit_ctx_t *cc,
-                                       const char *log_msg)
+                                       const char *log_msg,
+                                       apr_pool_t *pool)
 {
-  apr_pool_t *pool = cc->ras->pool;
   const svn_string_t *vcc;
   const svn_string_t *baseline_url;
   resource_t baseline_rsrc = { SVN_INVALID_REVNUM };
   ne_proppatch_operation po[2] = { { 0 } };
   int rv;
   svn_stringbuf_t *xml_data;
-
+  svn_error_t *err = NULL;
+  int retry_count = 5;
+  
   /* ### this whole sequence can/should be replaced with an expand-property
      ### REPORT when that is available on the server. */
 
@@ -1147,16 +1207,43 @@ static svn_error_t * apply_log_message(commit_ctx_t *cc,
   /* ### we should use DAV:apply-to-version on the CHECKOUT so we can skip
      ### retrieval of the baseline */
 
-  /* Get the Baseline from the DAV:checked-in value */
-  SVN_ERR( svn_ra_dav__get_one_prop(&baseline_url, cc->ras->sess, vcc->data, 
-                                    NULL, &svn_ra_dav__checked_in_prop, pool));
+  do {
 
-  baseline_rsrc.vsn_url = baseline_url->data;
-  SVN_ERR( checkout_resource(cc, &baseline_rsrc, FALSE) );
+    svn_error_clear(err);
+
+    /* Get the latest baseline from VCC's DAV:checked-in property.
+       This should give us the HEAD revision of the moment. */
+    SVN_ERR( svn_ra_dav__get_one_prop(&baseline_url, cc->ras->sess,
+                                      vcc->data, NULL,
+                                      &svn_ra_dav__checked_in_prop, pool));
+    baseline_rsrc.pool = pool;
+    baseline_rsrc.vsn_url = baseline_url->data;
+    
+    /* To set the log message, we must checkout the latest baseline
+       and get back a mutable "working" baseline.  */
+    err = checkout_resource(cc, &baseline_rsrc, FALSE, pool);
+
+    /* There's a small chance of a race condition here, if apache is
+       experiencing heavy commit concurrency or if the network has
+       long latency.  It's possible that the value of HEAD changed
+       between the time we fetched the latest baseline and the time we
+       checkout that baseline.  If that happens, apache will throw us
+       a BAD_BASELINE error (deltaV says you can only checkout the
+       latest baseline).  We just ignore that specific error and
+       retry a few times, asking for the latest baseline again. */
+    if (err && err->apr_err != SVN_ERR_APMOD_BAD_BASELINE)
+      return err;
+
+  } while (err && (--retry_count > 0));
+
+  /* Yikes, if we couldn't hold onto HEAD after a few retries, throw a
+     real error.*/
+  if (err)
+    return err;
 
   /* XML-Escape the log message. */
   xml_data = NULL;           /* Required by svn_xml_escape_*. */
-  svn_xml_escape_cdata_cstring(&xml_data, log_msg, cc->ras->pool);
+  svn_xml_escape_cdata_cstring(&xml_data, log_msg, pool);
 
   po[0].name = &log_message_prop;
   po[0].type = ne_propset;
@@ -1165,24 +1252,21 @@ static svn_error_t * apply_log_message(commit_ctx_t *cc,
   rv = ne_proppatch(cc->ras->sess, baseline_rsrc.wr_url, po);
   if (rv != NE_OK)
     {
-      const char *msg = apr_psprintf(cc->ras->pool,
-                                     "applying log message to %s",
+      const char *msg = apr_psprintf(pool, _("applying log message to %s"),
                                      baseline_rsrc.wr_url);
-      return svn_ra_dav__convert_error(cc->ras->sess, msg, rv);
+      return svn_ra_dav__convert_error(cc->ras->sess, msg, rv, pool);
     }
 
   return SVN_NO_ERROR;
 }
 
-svn_error_t * svn_ra_dav__get_commit_editor(
-  void *session_baton,
-  const svn_delta_editor_t **editor,
-  void **edit_baton,
-  svn_revnum_t *new_rev,
-  const char **committed_date,
-  const char **committed_author,
-  const char *log_msg,
-  apr_pool_t *pool)
+svn_error_t * svn_ra_dav__get_commit_editor(void *session_baton,
+                                            const svn_delta_editor_t **editor,
+                                            void **edit_baton,
+                                            const char *log_msg,
+                                            svn_commit_callback_t callback,
+                                            void *callback_baton,
+                                            apr_pool_t *pool)
 {
   svn_ra_session_t *ras = session_baton;
   svn_delta_editor_t *commit_editor;
@@ -1191,15 +1275,13 @@ svn_error_t * svn_ra_dav__get_commit_editor(
   /* Build the main commit editor's baton. */
   cc = apr_pcalloc(pool, sizeof(*cc));
   cc->ras = ras;
-  cc->resources = apr_hash_make(pool);
   cc->valid_targets = apr_hash_make(pool);
   cc->get_func = ras->callbacks->get_wc_prop;
   cc->push_func = ras->callbacks->push_wc_prop;
   cc->cb_baton = ras->callback_baton;
   cc->log_msg = log_msg;
-  cc->new_rev = new_rev;
-  cc->committed_date = committed_date;
-  cc->committed_author = committed_author;
+  cc->callback = callback;
+  cc->callback_baton = callback_baton;
 
   /* If the caller didn't give us any way of storing wcprops, then
      there's no point in getting back a MERGE response full of VR's. */
@@ -1214,13 +1296,13 @@ svn_error_t * svn_ra_dav__get_commit_editor(
   ** We will check out all further resources within the context of this
   ** activity.
   */
-  SVN_ERR( create_activity(cc) );
+  SVN_ERR( create_activity(cc, pool) );
 
   /*
   ** Find the latest baseline resource, check it out, and then apply the
   ** log message onto the thing.
   */
-  SVN_ERR( apply_log_message(cc, log_msg) );
+  SVN_ERR( apply_log_message(cc, log_msg, pool) );
 
   /*
   ** Set up the editor.
@@ -1242,6 +1324,7 @@ svn_error_t * svn_ra_dav__get_commit_editor(
   commit_editor->change_file_prop = commit_change_file_prop;
   commit_editor->close_file = commit_close_file;
   commit_editor->close_edit = commit_close_edit;
+  commit_editor->abort_edit = commit_abort_edit;
 
   *editor = commit_editor;
   *edit_baton = cc;
