@@ -1,7 +1,7 @@
 /* dump.c --- writing filesystem contents into a portable 'dumpfile' format.
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -16,6 +16,7 @@
  */
 
 
+#include "svn_private_config.h"
 #include "svn_pools.h"
 #include "svn_error.h"
 #include "svn_fs.h"
@@ -33,15 +34,18 @@
 
 /** A variant of our hash-writing routine in libsvn_subr;  this one
     writes to a stringbuf instead of a file, and outputs PROPS-END
-    instead of END. **/
+    instead of END.  If OLDHASH is not NULL, then only properties
+    which vary from OLDHASH will be written, and properties which
+    exist only in OLDHASH will be written out with "D" entries
+    (like "K" entries but with no corresponding value). **/
 
 static void
-write_hash_to_stringbuf (apr_hash_t *hash, 
+write_hash_to_stringbuf (apr_hash_t *hash,
+                         apr_hash_t *oldhash,
                          svn_stringbuf_t **strbuf,
                          apr_pool_t *pool)
 {
   apr_hash_index_t *this;      /* current hash entry */
-  char buf[SVN_KEYLINE_MAXLEN];
 
   *strbuf = svn_stringbuf_create ("", pool);
 
@@ -50,40 +54,112 @@ write_hash_to_stringbuf (apr_hash_t *hash,
       const void *key;
       void *val;
       apr_ssize_t keylen;
-      int bytes_used;
       svn_string_t *value;
 
       /* Get this key and val. */
       apr_hash_this (this, &key, &keylen, &val);
+      value = val;
+
+      /* Don't output properties equal to the ones in oldhash, if present. */
+      if (oldhash)
+        {
+          svn_string_t *oldvalue = apr_hash_get (oldhash, key, keylen);
+
+          if (oldvalue && svn_string_compare (value, oldvalue))
+            continue;
+        }
 
       /* Output name length, then name. */
 
-      svn_stringbuf_appendbytes (*strbuf, "K ", 2);
-
-      sprintf (buf, "%" APR_SSIZE_T_FMT "%n", keylen, &bytes_used);
-      svn_stringbuf_appendbytes (*strbuf, buf, bytes_used);
-      svn_stringbuf_appendbytes (*strbuf, "\n", 1);
+      svn_stringbuf_appendcstr (*strbuf,
+                                apr_psprintf (pool, "K %" APR_SSIZE_T_FMT "\n",
+                                              keylen));
 
       svn_stringbuf_appendbytes (*strbuf, (const char *) key, keylen);
       svn_stringbuf_appendbytes (*strbuf, "\n", 1);
 
       /* Output value length, then value. */
-      value = val;
 
-      svn_stringbuf_appendbytes (*strbuf, "V ", 2);
-
-      sprintf (buf, "%" APR_SIZE_T_FMT "%n", value->len, &bytes_used);
-      svn_stringbuf_appendbytes (*strbuf, buf, bytes_used);
-      svn_stringbuf_appendbytes (*strbuf, "\n", 1);
+      svn_stringbuf_appendcstr (*strbuf,
+                                apr_psprintf (pool, "V %" APR_SIZE_T_FMT "\n",
+                                              value->len));
 
       svn_stringbuf_appendbytes (*strbuf, value->data, value->len);
       svn_stringbuf_appendbytes (*strbuf, "\n", 1);
     }
 
+  if (oldhash)
+    {
+      /* Output a "D " entry for each property in oldhash but not hash. */
+      for (this = apr_hash_first (pool, oldhash); this;
+           this = apr_hash_next (this))
+        {
+          const void *key;
+          void *val;
+          apr_ssize_t keylen;
+
+          /* Get this key and val. */
+          apr_hash_this (this, &key, &keylen, &val);
+
+          /* Only output values deleted in hash. */
+          if (apr_hash_get (hash, key, keylen))
+            continue;
+
+          /* Output name length, then name. */
+
+          svn_stringbuf_appendcstr (*strbuf, 
+                                    apr_psprintf (pool,
+                                                  "D %" APR_SSIZE_T_FMT "\n",
+                                                  keylen));
+
+          svn_stringbuf_appendbytes (*strbuf, (const char *) key, keylen);
+          svn_stringbuf_appendbytes (*strbuf, "\n", 1);
+        }
+    }
   svn_stringbuf_appendbytes (*strbuf, "PROPS-END\n", 10);
 }
 
 
+/* Compute the delta between OLDROOT/OLDPATH and NEWROOT/NEWPATH and
+   store it into a new temporary file *TEMPFILE.  OLDROOT may be NULL,
+   in which case the delta will be computed against an empty file, as
+   per the svn_fs_get_file_delta_stream docstring.  Record the length
+   of the temporary file in *LEN, and rewind the file before
+   returning. */
+static svn_error_t *
+store_delta (apr_file_t **tempfile, svn_filesize_t *len,
+             svn_fs_root_t *oldroot, const char *oldpath,
+             svn_fs_root_t *newroot, const char *newpath, apr_pool_t *pool)
+{
+  const char *tempdir, *name;
+  svn_stream_t *temp_stream;
+  apr_off_t offset = 0;
+  svn_txdelta_stream_t *delta_stream;
+  svn_txdelta_window_handler_t wh;
+  void *whb;
+
+  /* Create a temporary file and open a stream to it. */
+  SVN_ERR (svn_io_temp_dir (&tempdir, pool));
+  SVN_ERR (svn_io_open_unique_file (tempfile, &name,
+                                    apr_psprintf (pool, "%s/dump", tempdir),
+                                    ".tmp", TRUE, pool));
+  temp_stream = svn_stream_from_aprfile (*tempfile, pool);
+
+  /* Compute the delta and send it to the temporary file. */
+  SVN_ERR (svn_fs_get_file_delta_stream (&delta_stream, oldroot, oldpath,
+                                         newroot, newpath, pool));
+  svn_txdelta_to_svndiff (temp_stream, pool, &wh, &whb);
+  SVN_ERR (svn_txdelta_send_txstream (delta_stream, wh, whb, pool));
+
+  /* Get the length of the temporary file and rewind it. */
+  SVN_ERR (svn_io_file_seek (*tempfile, APR_CUR, &offset, pool));
+  *len = offset;
+  offset = 0;
+  SVN_ERR (svn_io_file_seek (*tempfile, APR_SET, &offset, pool));
+  return SVN_NO_ERROR;
+}
+
+
 /*----------------------------------------------------------------------*/
 
 /** An editor which dumps node-data in 'dumpfile format' to a file. **/
@@ -106,6 +182,9 @@ struct edit_baton
   svn_fs_root_t *fs_root;
   svn_revnum_t current_rev;
 
+  /* True if dumped nodes should output deltas instead of full text. */
+  svn_boolean_t use_deltas;
+
   /* The first revision dumped in this dumpstream. */
   svn_revnum_t oldest_dumped_rev;
 
@@ -182,6 +261,10 @@ make_dir_baton (const char *path,
   else
     full_path = apr_pstrdup (pool, eb->path);
 
+  /* Remove leading slashes from copyfrom paths. */
+  if (cmp_path)
+    cmp_path = ((*cmp_path == '/') ? cmp_path + 1 : cmp_path);
+
   new_db->edit_baton = eb;
   new_db->parent_dir_baton = pb;
   new_db->path = full_path;
@@ -219,23 +302,31 @@ dump_node (struct edit_baton *eb,
            apr_pool_t *pool)
 {
   svn_stringbuf_t *propstring;
-  apr_hash_t *prophash;
-  svn_filesize_t textlen = 0, content_length = 0;
-  apr_size_t proplen = 0, len;
+  svn_filesize_t content_length = 0;
+  apr_size_t len;
   svn_boolean_t must_dump_text = FALSE, must_dump_props = FALSE;
   const char *compare_path = path;
   svn_revnum_t compare_rev = eb->current_rev - 1;
+  svn_fs_root_t *compare_root = NULL;
+  apr_file_t *delta_file = NULL;
 
   /* Write out metadata headers for this file node. */
-  SVN_ERR (svn_stream_printf (eb->stream, pool,
-                              SVN_REPOS_DUMPFILE_NODE_PATH ": %s\n", path));
-  
-  if (kind == svn_node_file)
-    SVN_ERR (svn_stream_printf (eb->stream, pool,
-                                SVN_REPOS_DUMPFILE_NODE_KIND ": file\n"));
-  else if (kind == svn_node_dir)
-    SVN_ERR (svn_stream_printf (eb->stream, pool,
-                                SVN_REPOS_DUMPFILE_NODE_KIND ": dir\n"));
+  if (eb->stream)
+    {
+      SVN_ERR (svn_stream_printf (eb->stream, pool,
+                                  SVN_REPOS_DUMPFILE_NODE_PATH ": %s\n", 
+                                  (*path == '/') ? path + 1 : path));
+      if (kind == svn_node_file)
+        SVN_ERR (svn_stream_printf (eb->stream, pool,
+                                    SVN_REPOS_DUMPFILE_NODE_KIND ": file\n"));
+      else if (kind == svn_node_dir)
+        SVN_ERR (svn_stream_printf (eb->stream, pool,
+                                    SVN_REPOS_DUMPFILE_NODE_KIND ": dir\n"));
+    }
+
+  /* Remove leading slashes from copyfrom paths. */
+  if (cmp_path)
+    cmp_path = ((*cmp_path == '/') ? cmp_path + 1 : cmp_path);
 
   /* Validate the comparison path/rev. */
   if (ARE_VALID_COPY_ARGS (cmp_path, cmp_rev))
@@ -246,38 +337,33 @@ dump_node (struct edit_baton *eb,
 
   if (action == svn_node_action_change)
     {
-      svn_fs_root_t *compare_root;
-      int text_changed = 0, props_changed = 0;
-
-      SVN_ERR (svn_stream_printf (eb->stream, pool,
-                                  SVN_REPOS_DUMPFILE_NODE_ACTION
-                                  ": change\n"));
+      if (eb->stream)
+        SVN_ERR (svn_stream_printf (eb->stream, pool,
+                                    SVN_REPOS_DUMPFILE_NODE_ACTION
+                                    ": change\n"));
 
       /* either the text or props changed, or possibly both. */
       SVN_ERR (svn_fs_revision_root (&compare_root, 
                                      svn_fs_root_fs (eb->fs_root),
                                      compare_rev, pool));
       
-      SVN_ERR (svn_fs_props_changed (&props_changed,
+      SVN_ERR (svn_fs_props_changed (&must_dump_props,
                                      compare_root, compare_path,
                                      eb->fs_root, path, pool));
       if (kind == svn_node_file)
-        SVN_ERR (svn_fs_contents_changed (&text_changed,
+        SVN_ERR (svn_fs_contents_changed (&must_dump_text,
                                           compare_root, compare_path,
                                           eb->fs_root, path, pool));
-      if (props_changed)
-        must_dump_props = TRUE;
-      if (text_changed)
-        must_dump_text = TRUE;        
     }
   else if (action == svn_node_action_replace)
     {
       if (! is_copy)
         {
           /* a simple delete+add, implied by a single 'replace' action. */
-          SVN_ERR (svn_stream_printf (eb->stream, pool,
-                                      SVN_REPOS_DUMPFILE_NODE_ACTION
-                                      ": replace\n")); 
+          if (eb->stream)
+            SVN_ERR (svn_stream_printf (eb->stream, pool,
+                                        SVN_REPOS_DUMPFILE_NODE_ACTION
+                                        ": replace\n")); 
 
           /* definitely need to dump all content for a replace. */
           if (kind == svn_node_file)
@@ -290,9 +376,10 @@ dump_node (struct edit_baton *eb,
 
           /* the path & kind headers have already been printed;  just
              add a delete action, and end the current record.*/
-          SVN_ERR (svn_stream_printf (eb->stream, pool,
-                                      SVN_REPOS_DUMPFILE_NODE_ACTION
-                                      ": delete\n\n"));  
+          if (eb->stream)
+            SVN_ERR (svn_stream_printf (eb->stream, pool,
+                                        SVN_REPOS_DUMPFILE_NODE_ACTION
+                                        ": delete\n\n"));  
 
           /* recurse:  print an additional add-with-history record. */
           SVN_ERR (dump_node (eb, path, kind, svn_node_action_add,
@@ -306,9 +393,10 @@ dump_node (struct edit_baton *eb,
     }
   else if (action == svn_node_action_delete)
     {
-      SVN_ERR (svn_stream_printf (eb->stream, pool,
-                                  SVN_REPOS_DUMPFILE_NODE_ACTION
-                                  ": delete\n"));  
+      if (eb->stream)
+        SVN_ERR (svn_stream_printf (eb->stream, pool,
+                                    SVN_REPOS_DUMPFILE_NODE_ACTION
+                                    ": delete\n"));  
 
       /* we can leave this routine quietly now, don't need to dump
          any content. */
@@ -317,8 +405,9 @@ dump_node (struct edit_baton *eb,
     }
   else if (action == svn_node_action_add)
     {
-      SVN_ERR (svn_stream_printf (eb->stream, pool,
-                                  SVN_REPOS_DUMPFILE_NODE_ACTION ": add\n"));
+      if (eb->stream)
+        SVN_ERR (svn_stream_printf (eb->stream, pool,
+                                    SVN_REPOS_DUMPFILE_NODE_ACTION ": add\n"));
 
       if (! is_copy)
         {
@@ -329,42 +418,37 @@ dump_node (struct edit_baton *eb,
         }
       else
         {
-          svn_fs_root_t *src_root;
-          int text_changed = 0, props_changed = 0;
-
           if ((cmp_rev < eb->oldest_dumped_rev)
               && eb->feedback_stream)
             svn_stream_printf 
               (eb->feedback_stream, pool,
-               "WARNING: cmp_rev %" SVN_REVNUM_T_FMT
-               " is older than oldest dumped rev %" SVN_REVNUM_T_FMT 
-               "\n... loading this dump into an empty repository will fail.\n",
+               _("WARNING: Referencing data in revision %ld" 
+                 ", which is older than the oldest\nWARNING: dumped revision "
+                 "(%ld).  Loading this dump into an empty "
+                 "repository\nWARNING: will fail.\n"),
                cmp_rev, eb->oldest_dumped_rev);
 
-          SVN_ERR (svn_stream_printf (eb->stream, pool,
-                                      SVN_REPOS_DUMPFILE_NODE_COPYFROM_REV 
-                                      ": %" SVN_REVNUM_T_FMT "\n"
-                                      SVN_REPOS_DUMPFILE_NODE_COPYFROM_PATH
-                                      ": %s\n",                  
-                                      cmp_rev, cmp_path));
+          if (eb->stream)
+            SVN_ERR (svn_stream_printf (eb->stream, pool,
+                                        SVN_REPOS_DUMPFILE_NODE_COPYFROM_REV 
+                                        ": %ld\n"
+                                        SVN_REPOS_DUMPFILE_NODE_COPYFROM_PATH
+                                        ": %s\n",
+                                        cmp_rev, cmp_path));
 
-          SVN_ERR (svn_fs_revision_root (&src_root, 
+          SVN_ERR (svn_fs_revision_root (&compare_root, 
                                          svn_fs_root_fs (eb->fs_root),
                                          compare_rev, pool));
 
           /* Need to decide if the copied node had any extra textual or
              property mods as well.  */
-          SVN_ERR (svn_fs_props_changed (&props_changed,
-                                         src_root, compare_path,
+          SVN_ERR (svn_fs_props_changed (&must_dump_props,
+                                         compare_root, compare_path,
                                          eb->fs_root, path, pool));
           if (kind == svn_node_file)
-            SVN_ERR (svn_fs_contents_changed (&text_changed,
-                                              src_root, compare_path,
+            SVN_ERR (svn_fs_contents_changed (&must_dump_text,
+                                              compare_root, compare_path,
                                               eb->fs_root, path, pool));
-          if (props_changed)
-            must_dump_props = TRUE;
-          if (text_changed)
-            must_dump_text = TRUE;
           
           /* ### someday write a node-copyfrom-source-checksum. */
         }
@@ -377,8 +461,12 @@ dump_node (struct edit_baton *eb,
          then our dumpstream format demands that at a *minimum*, we
          see a lone "PROPS-END" as a divider between text and props
          content within the content-block. */
-      len = 2;
-      return svn_stream_write (eb->stream, "\n\n", &len); /* ### needed? */
+      if (eb->stream)
+        {
+          len = 2;
+          return svn_stream_write (eb->stream, "\n\n", &len); /* ### needed? */
+        }
+      return SVN_NO_ERROR;
     }
 
   /*** Start prepping content to dump... ***/
@@ -388,31 +476,65 @@ dump_node (struct edit_baton *eb,
      property values here. */
   if (must_dump_props)
     {
+      apr_hash_t *prophash, *oldhash = NULL;
+      apr_size_t proplen;
+
       SVN_ERR (svn_fs_node_proplist (&prophash, eb->fs_root, path, pool));
-      write_hash_to_stringbuf (prophash, &propstring, pool);
+      if (eb->use_deltas && compare_root)
+        {
+          /* Fetch the old property hash to diff against and output a header
+             saying that our property contents are a delta. */
+          SVN_ERR (svn_fs_node_proplist (&oldhash, compare_root, compare_path,
+                                         pool));
+          if (eb->stream)
+            SVN_ERR (svn_stream_printf (eb->stream, pool,
+                                        SVN_REPOS_DUMPFILE_PROP_DELTA 
+                                        ": true\n"));
+        }
+      write_hash_to_stringbuf (prophash, oldhash, &propstring, pool);
       proplen = propstring->len;
       content_length += proplen;
-      SVN_ERR (svn_stream_printf (eb->stream, pool,
-                                  SVN_REPOS_DUMPFILE_PROP_CONTENT_LENGTH 
-                                  ": %" APR_SIZE_T_FMT "\n", proplen));
+      if (eb->stream)
+        SVN_ERR (svn_stream_printf (eb->stream, pool,
+                                    SVN_REPOS_DUMPFILE_PROP_CONTENT_LENGTH 
+                                    ": %" APR_SIZE_T_FMT "\n", proplen));
     }
 
   /* If we are supposed to dump text, write out a text length header
      here, and a md5 checksum (if available.) */
   if (must_dump_text && (kind == svn_node_file))
     {
-      unsigned char md5_digest[MD5_DIGESTSIZE];
+      unsigned char md5_digest[APR_MD5_DIGESTSIZE];
       const char *hex_digest;
+      svn_filesize_t textlen;
+
+      if (eb->use_deltas)
+        {
+          /* Compute the text delta now and write it into a temporary
+             file, so that we can find its length.  Output a header
+             saying our text contents are a delta. */
+          SVN_ERR (store_delta (&delta_file, &textlen, compare_root,
+                                compare_path, eb->fs_root, path, pool));
+          if (eb->stream)
+            SVN_ERR (svn_stream_printf (eb->stream, pool,
+                                        SVN_REPOS_DUMPFILE_TEXT_DELTA
+                                        ": true\n"));
+        }
+      else
+        {
+          /* Just fetch the length of the file. */
+          SVN_ERR (svn_fs_file_length (&textlen, eb->fs_root, path, pool));
+        }
 
-      SVN_ERR (svn_fs_file_length (&textlen, eb->fs_root, path, pool));
       content_length += textlen;
-      SVN_ERR (svn_stream_printf (eb->stream, pool,
-                                  SVN_REPOS_DUMPFILE_TEXT_CONTENT_LENGTH 
-                                  ": %" SVN_FILESIZE_T_FMT "\n", textlen));
+      if (eb->stream)
+        SVN_ERR (svn_stream_printf (eb->stream, pool,
+                                    SVN_REPOS_DUMPFILE_TEXT_CONTENT_LENGTH 
+                                    ": %" SVN_FILESIZE_T_FMT "\n", textlen));
 
       SVN_ERR (svn_fs_file_md5_checksum (md5_digest, eb->fs_root, path, pool));
       hex_digest = svn_md5_digest_to_cstring (md5_digest, pool);
-      if (hex_digest)
+      if (hex_digest && eb->stream)
         SVN_ERR (svn_stream_printf (eb->stream, pool,
                                     SVN_REPOS_DUMPFILE_TEXT_CONTENT_CHECKSUM 
                                     ": %s\n", hex_digest));
@@ -422,60 +544,37 @@ dump_node (struct edit_baton *eb,
      and is the summation of the text and prop contents lengths.  We
      write this only for the benefit of non-Subversion RFC-822
      parsers. */
-  SVN_ERR (svn_stream_printf (eb->stream, pool,
-                              SVN_REPOS_DUMPFILE_CONTENT_LENGTH 
-                              ": %" SVN_FILESIZE_T_FMT "\n\n",
-                              content_length));
+  if (eb->stream)
+    SVN_ERR (svn_stream_printf (eb->stream, pool,
+                                SVN_REPOS_DUMPFILE_CONTENT_LENGTH 
+                                ": %" SVN_FILESIZE_T_FMT "\n\n",
+                                content_length));
 
   /* Dump property content if we're supposed to do so. */
-  if (must_dump_props)
+  if (must_dump_props && eb->stream)
     {
       len = propstring->len;
       SVN_ERR (svn_stream_write (eb->stream, propstring->data, &len));
     }
 
   /* Dump text content */
-  /*    (this stream "pull and push" code was stolen from
-        libsvn_ra_local/ra_plugin.c:get_file().  */
-  if (must_dump_text && (kind == svn_node_file))
+  if (must_dump_text && (kind == svn_node_file) && eb->stream)
     {
-      apr_size_t rlen, wlen;
       svn_stream_t *contents;
-          
-      SVN_ERR (svn_fs_file_contents (&contents, eb->fs_root, path, pool));
-      
-      while (1)
-        {
-          /* read a maximum number of bytes from the file, please. */
-          rlen = eb->bufsize; 
-          SVN_ERR (svn_stream_read (contents, eb->buffer, &rlen));
-          
-          /* write however many bytes you read, please. */
-          wlen = rlen;
-          SVN_ERR (svn_stream_write (eb->stream, eb->buffer, &wlen));
-          if (wlen != rlen)
-            {
-              /* Uh oh, didn't write as many bytes as we read, and no
-                 error was returned.  According to the docstring, this
-                 should never happen. */
-              return 
-                svn_error_createf (SVN_ERR_STREAM_UNEXPECTED_EOF, NULL,
-                                   "Error dumping textual contents of '%s'.",
-                                   path);
-            }
-        
-        if (rlen != eb->bufsize)
-          {
-            /* svn_stream_read didn't throw an error, yet it didn't read
-               all the bytes requested.  According to the docstring,
-               this means a plain old EOF happened, so we're done. */
-            break;
-          }
-        }
+
+      if (delta_file)
+        contents = svn_stream_from_aprfile (delta_file, pool);
+      else
+        SVN_ERR (svn_fs_file_contents (&contents, eb->fs_root, path, pool));
+
+      SVN_ERR (svn_stream_copy (contents, eb->stream, pool));
     }
   
-  len = 2;
-  SVN_ERR (svn_stream_write (eb->stream, "\n\n", &len)); /* ### needed? */
+  if (eb->stream)
+    {
+      len = 2;
+      SVN_ERR (svn_stream_write (eb->stream, "\n\n", &len)); /* ### needed? */
+    }
   
   return SVN_NO_ERROR;
 }
@@ -711,6 +810,7 @@ get_dump_editor (const svn_delta_editor_t **editor,
                  svn_stream_t *stream,
                  svn_stream_t *feedback_stream,
                  svn_revnum_t oldest_dumped_rev,
+                 svn_boolean_t use_deltas,
                  apr_pool_t *pool)
 {
   /* Allocate an edit baton to be stored in every directory baton.
@@ -727,6 +827,7 @@ get_dump_editor (const svn_delta_editor_t **editor,
   eb->path = apr_pstrdup (pool, root_path);
   SVN_ERR (svn_fs_revision_root (&(eb->fs_root), fs, to_rev, pool));
   eb->current_rev = to_rev;
+  eb->use_deltas = use_deltas;
 
   /* Set up the editor. */
   dump_editor->open_root = open_root;
@@ -762,41 +863,44 @@ write_revision_record (svn_stream_t *stream,
   apr_size_t len;
   apr_hash_t *props;
   svn_stringbuf_t *encoded_prophash;
+  apr_time_t timetemp;
+  svn_string_t *datevalue;
 
+  /* Read the revision props even if we're aren't going to dump
+     them for verification purposes */
   SVN_ERR (svn_fs_revision_proplist (&props, fs, rev, pool));
 
-  {
-    /* Run revision date properties through the time conversion to
-       canonize them. */
-    /* ### Remove this when it is no longer needed for sure. */
-    apr_time_t timetemp;
-    svn_string_t *datevalue = apr_hash_get (props,
-                                            SVN_PROP_REVISION_DATE,
-                                            APR_HASH_KEY_STRING);
-    if (datevalue)
-      {
-        SVN_ERR (svn_time_from_cstring (&timetemp, datevalue->data, pool));
-        datevalue = svn_string_create (svn_time_to_cstring (timetemp, pool),
-                                       pool);
-        apr_hash_set (props,
-                      SVN_PROP_REVISION_DATE,
-                      APR_HASH_KEY_STRING,
-                      datevalue);
-      }
-  }
+  /* If we have no stream, and we aren't validating anything, we might
+     as well just go home now. */
+  if (! stream)
+    return SVN_NO_ERROR;
 
-  write_hash_to_stringbuf (props, &encoded_prophash, pool);
+  /* Run revision date properties through the time conversion to
+     canonize them. */
+  /* ### Remove this when it is no longer needed for sure. */
+  datevalue = apr_hash_get (props, SVN_PROP_REVISION_DATE,
+                            APR_HASH_KEY_STRING);
+  if (datevalue)
+    {
+      SVN_ERR (svn_time_from_cstring (&timetemp, datevalue->data, pool));
+      datevalue = svn_string_create (svn_time_to_cstring (timetemp, pool),
+                                     pool);
+      apr_hash_set (props, SVN_PROP_REVISION_DATE, APR_HASH_KEY_STRING,
+                    datevalue);
+    }
+
+  write_hash_to_stringbuf (props, NULL, &encoded_prophash, pool);
 
   /* ### someday write a revision-content-checksum */
 
   SVN_ERR (svn_stream_printf (stream, pool,
                               SVN_REPOS_DUMPFILE_REVISION_NUMBER 
-                              ": %" SVN_REVNUM_T_FMT "\n", rev));
+                              ": %ld\n", rev));
   SVN_ERR (svn_stream_printf (stream, pool,
                               SVN_REPOS_DUMPFILE_PROP_CONTENT_LENGTH
                               ": %" APR_SIZE_T_FMT "\n",
                               encoded_prophash->len));
-
+  
   /* Write out a regular Content-length header for the benefit of
      non-Subversion RFC-822 parsers. */
   SVN_ERR (svn_stream_printf (stream, pool,
@@ -806,7 +910,7 @@ write_revision_record (svn_stream_t *stream,
   
   len = encoded_prophash->len;
   SVN_ERR (svn_stream_write (stream, encoded_prophash->data, &len));
-
+  
   len = 1;
   SVN_ERR (svn_stream_write (stream, "\n", &len));
   
@@ -817,13 +921,16 @@ write_revision_record (svn_stream_t *stream,
 
 /* The main dumper. */
 svn_error_t *
-svn_repos_dump_fs (svn_repos_t *repos,
-                   svn_stream_t *stream,
-                   svn_stream_t *feedback_stream,
-                   svn_revnum_t start_rev,
-                   svn_revnum_t end_rev,
-                   svn_boolean_t incremental,
-                   apr_pool_t *pool)
+svn_repos_dump_fs2 (svn_repos_t *repos,
+                    svn_stream_t *stream,
+                    svn_stream_t *feedback_stream,
+                    svn_revnum_t start_rev,
+                    svn_revnum_t end_rev,
+                    svn_boolean_t incremental,
+                    svn_boolean_t use_deltas,
+                    svn_cancel_func_t cancel_func,
+                    void *cancel_baton,
+                    apr_pool_t *pool)
 {
   const svn_delta_editor_t *dump_editor;
   void *dump_edit_baton;
@@ -832,6 +939,7 @@ svn_repos_dump_fs (svn_repos_t *repos,
   apr_pool_t *subpool = svn_pool_create (pool);
   svn_revnum_t youngest;
   const char *uuid;
+  svn_boolean_t dumping = (stream != NULL);
 
   /* Determine the current youngest revision of the filesystem. */
   SVN_ERR (svn_fs_youngest_rev (&youngest, fs, pool));
@@ -841,38 +949,55 @@ svn_repos_dump_fs (svn_repos_t *repos,
     start_rev = 0;
   if (! SVN_IS_VALID_REVNUM(end_rev))
     end_rev = youngest;
+  if (! stream)
+    stream = svn_stream_empty (pool);
 
   /* Validate the revisions. */
   if (start_rev > end_rev)
     return svn_error_createf (SVN_ERR_REPOS_BAD_ARGS, NULL,
-                              "start_rev %" SVN_REVNUM_T_FMT
-                              " is greater than end_rev %" SVN_REVNUM_T_FMT,
+                              _("Start revision %ld"
+                                " is greater than end revision %ld"),
                               start_rev, end_rev);
   if (end_rev > youngest)
     return svn_error_createf (SVN_ERR_REPOS_BAD_ARGS, NULL,
-                              "end_rev %" SVN_REVNUM_T_FMT " is invalid "
-                              "(youngest rev is %" SVN_REVNUM_T_FMT ")",
+                              _("End revision %ld is invalid "
+                                "(youngest revision is %ld)"),
                               end_rev, youngest);
   if ((start_rev == 0) && incremental)
     incremental = FALSE; /* revision 0 looks the same regardless of
                             whether or not this is an incremental
                             dump, so just simplify things. */
 
-  /* Write out "general" metadata for the dumpfile.  In this case, a
-     magic header followed by a dumpfile format version. */
-  SVN_ERR (svn_stream_printf (stream, pool, SVN_REPOS_DUMPFILE_MAGIC_HEADER
-                              ": %d\n\n", SVN_REPOS_DUMPFILE_FORMAT_VERSION));
   /* Write out the UUID. */
   SVN_ERR (svn_fs_get_uuid(fs, &uuid, pool));
 
-  SVN_ERR (svn_stream_printf (stream, pool, SVN_REPOS_DUMPFILE_UUID
-                              ": %s\n\n", uuid));
-                   
+  /* Write out "general" metadata for the dumpfile.  In this case, a
+     magic header followed by a dumpfile format version. */
+  if (stream)
+    {
+      int version = SVN_REPOS_DUMPFILE_FORMAT_VERSION;
+
+      /* If we're not using deltas, use the previous version, for
+         compatibility with svn 1.0.x. */
+      if (!use_deltas)
+        version--;
+      SVN_ERR (svn_stream_printf (stream, pool, 
+                                  SVN_REPOS_DUMPFILE_MAGIC_HEADER ": %d\n\n", 
+                                  version));
+      SVN_ERR (svn_stream_printf (stream, pool, SVN_REPOS_DUMPFILE_UUID
+                                  ": %s\n\n", uuid));
+    }
+
   /* Main loop:  we're going to dump revision i.  */
   for (i = start_rev; i <= end_rev; i++)
     {
       svn_revnum_t from_rev, to_rev;
       svn_fs_root_t *to_root;
+      svn_boolean_t use_deltas_for_rev;
+
+      /* Check for cancellation. */
+      if (cancel_func)
+        SVN_ERR (cancel_func (cancel_baton));
 
       /* Special-case the initial revision dump: it needs to contain
          *all* nodes, because it's the foundation of all future
@@ -904,10 +1029,13 @@ svn_repos_dump_fs (svn_repos_t *repos,
       /* Write the revision record. */
       SVN_ERR (write_revision_record (stream, fs, to_rev, subpool));
 
-      /* The editor which dumps nodes to a file. */
-      SVN_ERR (get_dump_editor (&dump_editor, &dump_edit_baton, 
-                                fs, to_rev, "/", stream, feedback_stream,
-                                start_rev, subpool));
+      /* Fetch the editor which dumps nodes to a file.  Regardless of
+         what we've been told, don't use deltas for the first rev of a
+         non-incremental dump. */
+      use_deltas_for_rev = use_deltas && (incremental || i != start_rev);
+      SVN_ERR (get_dump_editor (&dump_editor, &dump_edit_baton, fs, to_rev,
+                                "/", stream, feedback_stream, start_rev,
+                                use_deltas_for_rev, subpool));
 
       /* Drive the editor in one way or another. */
       SVN_ERR (svn_fs_revision_root (&to_root, fs, to_rev, subpool));
@@ -919,9 +1047,11 @@ svn_repos_dump_fs (svn_repos_t *repos,
         {
           svn_fs_root_t *from_root;
           SVN_ERR (svn_fs_revision_root (&from_root, fs, from_rev, subpool));
-          SVN_ERR (svn_repos_dir_delta (from_root, "/", NULL, 
+          SVN_ERR (svn_repos_dir_delta (from_root, "/", "",
                                         to_root, "/",
                                         dump_editor, dump_edit_baton,
+                                        NULL,
+                                        NULL,
                                         FALSE, /* don't send text-deltas */
                                         TRUE, /* recurse */
                                         FALSE, /* don't send entry props */
@@ -938,12 +1068,29 @@ svn_repos_dump_fs (svn_repos_t *repos,
       /* Reuse all memory consumed by the dump of this one revision. */
       svn_pool_clear (subpool);
       if (feedback_stream)
-        svn_stream_printf (feedback_stream, pool,
-                           "* Dumped revision %" SVN_REVNUM_T_FMT ".\n",
-                           to_rev);
+        SVN_ERR (svn_stream_printf (feedback_stream, pool,
+                                    _("* %s revision %ld.\n"),
+                                    dumping ? "Dumped" : "Verified",
+                                    to_rev));
     }
 
   svn_pool_destroy (subpool);
 
   return SVN_NO_ERROR;
 }
+
+svn_error_t *
+svn_repos_dump_fs (svn_repos_t *repos,
+                   svn_stream_t *stream,
+                   svn_stream_t *feedback_stream,
+                   svn_revnum_t start_rev,
+                   svn_revnum_t end_rev,
+                   svn_boolean_t incremental,
+                   svn_cancel_func_t cancel_func,
+                   void *cancel_baton,
+                   apr_pool_t *pool)
+{
+  return svn_repos_dump_fs2 (repos, stream, feedback_stream, start_rev,
+                             end_rev, incremental, FALSE, cancel_func,
+                             cancel_baton, pool);
+}
