@@ -1,7 +1,7 @@
 /* load.c --- parsing a 'dumpfile'-formatted stream.
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -16,6 +16,7 @@
  */
 
 
+#include "svn_private_config.h"
 #include "svn_pools.h"
 #include "svn_error.h"
 #include "svn_fs.h"
@@ -41,6 +42,8 @@ struct parse_baton
   svn_stream_t *outstream;
   enum svn_repos_load_uuid uuid_action;
   const char *parent_dir;
+  apr_pool_t *pool;
+  apr_hash_t *rev_map;
 };
 
 struct revision_baton
@@ -74,6 +77,31 @@ struct node_baton
 
 
 
+/*----------------------------------------------------------------------*/
+
+/** A conversion function between the two vtable types. **/
+static
+svn_repos_parser_fns2_t *fns2_from_fns (const svn_repos_parser_fns_t *fns,
+                                        apr_pool_t *pool)
+{
+  svn_repos_parser_fns2_t *fns2;
+
+  fns2 = apr_palloc (pool, sizeof (*fns2));
+  fns2->new_revision_record = fns->new_revision_record;
+  fns2->uuid_record = fns->uuid_record;
+  fns2->new_node_record = fns->new_node_record;
+  fns2->set_revision_property = fns->set_revision_property;
+  fns2->set_node_property = fns->set_node_property;
+  fns2->remove_node_props = fns->remove_node_props;
+  fns2->set_fulltext = fns->set_fulltext;
+  fns2->close_node = fns->close_node;
+  fns2->close_revision = fns->close_revision;
+  fns2->delete_node_property = NULL;
+  fns2->apply_textdelta = NULL;
+  return fns2;
+}
+
+
 /*----------------------------------------------------------------------*/
 
 /** The parser and related helper funcs **/
@@ -101,19 +129,21 @@ read_header_block (svn_stream_t *stream,
     {
       svn_stringbuf_t *header_str;
       const char *name, *value; 
+      svn_boolean_t eof;
       apr_size_t i = 0;
 
       if (first_header != NULL)
         {
           header_str = first_header;
           first_header = NULL;  /* so we never visit this block again. */
+          eof = FALSE;
         }
 
       else
         /* Read the next line into a stringbuf. */
-        SVN_ERR (svn_stream_readline (stream, &header_str, pool));
+        SVN_ERR (svn_stream_readline (stream, &header_str, "\n", &eof, pool));
       
-      if ((header_str == NULL) || (svn_stringbuf_isempty (header_str)))
+      if (eof || svn_stringbuf_isempty(header_str))
         break;    /* end of header block */
 
       /* Find the next colon in the stringbuf. */
@@ -122,7 +152,7 @@ read_header_block (svn_stream_t *stream,
           if (header_str->data[i] == '\0')
             return svn_error_create (SVN_ERR_STREAM_MALFORMED_DATA, NULL,
                                      "Found malformed header block "
-                                     "in dumpfile stream.");
+                                     "in dumpfile stream");
           i++;
         }
       /* Create a 'name' string and point to it. */
@@ -134,7 +164,7 @@ read_header_block (svn_stream_t *stream,
       if (i > header_str->len)
         return svn_error_create (SVN_ERR_STREAM_MALFORMED_DATA, NULL,
                                  "Found malformed header block "
-                                 "in dumpfile stream.");
+                                 "in dumpfile stream");
 
       /* Point to the 'value' string. */
       value = header_str->data + i;
@@ -151,14 +181,14 @@ static svn_error_t *
 stream_ran_dry (void)
 {
   return svn_error_create (SVN_ERR_INCOMPLETE_DATA, NULL,
-                           "Premature end of content data in dumpstream.");
+                           "Premature end of content data in dumpstream");
 }
 
 static svn_error_t *
 stream_malformed (void)
 {
   return svn_error_create (SVN_ERR_STREAM_MALFORMED_DATA, NULL,
-                           "Dumpstream data appears to be malformed.");
+                           "Dumpstream data appears to be malformed");
 }
 
 /* Read CONTENT_LENGTH bytes from STREAM, parsing the bytes as an
@@ -170,7 +200,7 @@ stream_malformed (void)
 static svn_error_t *
 parse_property_block (svn_stream_t *stream,
                       svn_filesize_t content_length,
-                      const svn_repos_parser_fns_t *parse_fns,
+                      const svn_repos_parser_fns2_t *parse_fns,
                       void *record_baton,
                       svn_boolean_t is_node,
                       apr_pool_t *pool)
@@ -180,17 +210,18 @@ parse_property_block (svn_stream_t *stream,
   while (content_length)
     {
       char *buf;  /* a pointer into the stringbuf's data */
+      svn_boolean_t eof;
 
       /* Read a key length line.  (Actually, it might be PROPS_END). */
-      SVN_ERR (svn_stream_readline (stream, &strbuf, pool));
+      SVN_ERR (svn_stream_readline (stream, &strbuf, "\n", &eof, pool));
 
-      if (strbuf == NULL)
+      if (eof)
         {
           /* We could just use stream_ran_dry() or stream_malformed(),
              but better to give a non-generic property block error. */ 
           return svn_error_create
             (SVN_ERR_STREAM_MALFORMED_DATA, NULL,
-             "incomplete or unterminated property block");
+             "Incomplete or unterminated property block");
         }
 
       content_length -= (strbuf->len + 1); /* +1 because we read a \n too. */
@@ -227,7 +258,7 @@ parse_property_block (svn_stream_t *stream,
             return stream_malformed ();
 
           /* Read a val length line */
-          SVN_ERR (svn_stream_readline (stream, &strbuf, pool));
+          SVN_ERR (svn_stream_readline (stream, &strbuf, "\n", &eof, pool));
           content_length -= (strbuf->len + 1); /* +1 because we read \n too */
           buf = strbuf->data;
 
@@ -273,6 +304,41 @@ parse_property_block (svn_stream_t *stream,
           else
             return stream_malformed (); /* didn't find expected 'V' line */
         }
+      else if ((buf[0] == 'D') && (buf[1] == ' '))
+        {
+          apr_size_t numread;
+          char *keybuf;
+          char c;
+          
+          /* Get the length of the key */
+          apr_size_t keylen = (apr_size_t) atoi (buf + 2);
+
+          /* Now read that much into a buffer, + 1 byte for null terminator */
+          keybuf = apr_pcalloc (pool, keylen + 1);
+          numread = keylen;
+          SVN_ERR (svn_stream_read (stream, keybuf, &numread));
+          content_length -= numread;
+          if (numread != keylen)
+            return stream_ran_dry ();
+          keybuf[keylen] = '\0';
+
+          /* Suck up extra newline after key data */
+          numread = 1;
+          SVN_ERR (svn_stream_read (stream, &c, &numread));
+          content_length -= numread;
+          if (numread != 1)
+            return stream_ran_dry ();
+          if (c != '\n') 
+            return stream_malformed ();
+
+          /* We don't expect these in revision properties, and if we see
+             one when we don't have a delete_node_property callback,
+             then we're seeing a v3 feature in a v2 dump. */
+          if (!is_node || !parse_fns->delete_node_property)
+            return stream_malformed ();
+
+          SVN_ERR (parse_fns->delete_node_property (record_baton, keybuf));
+        }
       else
         return stream_malformed (); /* didn't find expected 'K' line */
       
@@ -290,7 +356,8 @@ parse_property_block (svn_stream_t *stream,
 static svn_error_t *
 parse_text_block (svn_stream_t *stream,
                   svn_filesize_t content_length,
-                  const svn_repos_parser_fns_t *parse_fns,
+                  svn_boolean_t is_delta,
+                  const svn_repos_parser_fns2_t *parse_fns,
                   void *record_baton,
                   char *buffer,
                   apr_size_t buflen,
@@ -298,9 +365,21 @@ parse_text_block (svn_stream_t *stream,
 {
   svn_stream_t *text_stream = NULL;
   apr_size_t num_to_read, rlen, wlen;
-  
-  /* Get a stream to which we can push the data. */
-  SVN_ERR (parse_fns->set_fulltext (&text_stream, record_baton));
+
+  if (is_delta)
+    {
+      svn_txdelta_window_handler_t wh;
+      void *whb;
+
+      SVN_ERR (parse_fns->apply_textdelta (&wh, &whb, record_baton));
+      if (wh)
+        text_stream = svn_txdelta_parse_svndiff (wh, whb, TRUE, pool);
+    }
+  else
+    {
+      /* Get a stream to which we can push the data. */
+      SVN_ERR (parse_fns->set_fulltext (&text_stream, record_baton));
+    }
 
   /* If there are no contents to read, just write an empty buffer
      through our callback. */
@@ -335,7 +414,7 @@ parse_text_block (svn_stream_t *stream,
             {
               /* Uh oh, didn't write as many bytes as we read. */
               return svn_error_create (SVN_ERR_STREAM_UNEXPECTED_EOF, NULL,
-                                       "Error pushing textual contents.");
+                                       "Unexpected EOF writing contents");
             }
         }
     }
@@ -364,13 +443,13 @@ parse_format_version (const char *versionstring, int *version)
                   SVN_REPOS_DUMPFILE_MAGIC_HEADER,
                   magic_len))
     return svn_error_create (SVN_ERR_STREAM_MALFORMED_DATA, NULL,
-                             "malformed dumpfile header.");
+                             "Malformed dumpfile header");
 
   value = atoi (p+1);
 
   if (value > SVN_REPOS_DUMPFILE_FORMAT_VERSION)
     return svn_error_createf (SVN_ERR_STREAM_MALFORMED_DATA, NULL,
-                              "unsupported dumpfile version: %d",
+                              "Unsupported dumpfile version: %d",
                               value);
 
   *version = value;
@@ -381,11 +460,14 @@ parse_format_version (const char *versionstring, int *version)
 
 /* The Main Parser Logic */
 svn_error_t *
-svn_repos_parse_dumpstream (svn_stream_t *stream,
-                            const svn_repos_parser_fns_t *parse_fns,
-                            void *parse_baton,
-                            apr_pool_t *pool)
+svn_repos_parse_dumpstream2 (svn_stream_t *stream,
+                             const svn_repos_parser_fns2_t *parse_fns,
+                             void *parse_baton,
+                             svn_cancel_func_t cancel_func,
+                             void *cancel_baton,
+                             apr_pool_t *pool)
 {
+  svn_boolean_t eof;
   svn_stringbuf_t *linebuf;
   void *rev_baton = NULL;
   char *buffer = apr_palloc (pool, SVN_STREAM_CHUNK_SIZE);
@@ -393,17 +475,24 @@ svn_repos_parse_dumpstream (svn_stream_t *stream,
   apr_pool_t *linepool = svn_pool_create (pool);
   apr_pool_t *revpool = svn_pool_create (pool);
   apr_pool_t *nodepool = svn_pool_create (pool);
-  const char *uuid;
   int version;
 
-  SVN_ERR (svn_stream_readline (stream, &linebuf, linepool));
-  if (linebuf == NULL)
+  SVN_ERR (svn_stream_readline (stream, &linebuf, "\n", &eof, linepool));
+  if (eof)
     return stream_ran_dry ();
     
   /* The first two lines of the stream are the dumpfile-format version
      number, and a blank line. */
   SVN_ERR (parse_format_version (linebuf->data, &version));
 
+  /* If we were called from svn_repos_parse_dumpstream(), the
+     callbacks to handle delta contents will be NULL, so we have to
+     reject dumpfiles with the current version. */
+  if (version == SVN_REPOS_DUMPFILE_FORMAT_VERSION
+      && (!parse_fns->delete_node_property || !parse_fns->apply_textdelta))
+    return svn_error_createf (SVN_ERR_STREAM_MALFORMED_DATA, NULL,
+                              "Unsupported dumpfile version: %d", version);
+
   /* A dumpfile "record" is defined to be a header-block of
      rfc822-style headers, possibly followed by a content-block.
 
@@ -424,15 +513,20 @@ svn_repos_parse_dumpstream (svn_stream_t *stream,
       void *node_baton;
       const char *valstr;
       svn_boolean_t found_node = FALSE;
+      const char *value;
 
       /* Clear our per-line pool. */
       svn_pool_clear (linepool);
 
+      /* Check for cancellation. */
+      if (cancel_func)
+        SVN_ERR (cancel_func (cancel_baton));
+
       /* Keep reading blank lines until we discover a new record, or until
          the stream runs out. */
-      SVN_ERR (svn_stream_readline (stream, &linebuf, linepool));
+      SVN_ERR (svn_stream_readline (stream, &linebuf, "\n", &eof, linepool));
       
-      if (linebuf == NULL)
+      if (eof)
         break;   /* end of stream, go home. */
 
       if ((linebuf->len == 0) || (apr_isspace (linebuf->data[0])))
@@ -444,7 +538,7 @@ svn_repos_parse_dumpstream (svn_stream_t *stream,
          Read the whole header-block into a hash. */
       SVN_ERR (read_header_block (stream, linebuf, &headers, linepool));
 
-      /* Create some kind of new record object. */
+      /*** Handle the various header blocks. ***/
 
       /* Is this a revision record? */
       if (apr_hash_get (headers, SVN_REPOS_DUMPFILE_REVISION_NUMBER,
@@ -462,7 +556,6 @@ svn_repos_parse_dumpstream (svn_stream_t *stream,
                                                    headers, parse_baton,
                                                    revpool));
         }
-
       /* Or is this, perhaps, a node record? */
       else if (apr_hash_get (headers, SVN_REPOS_DUMPFILE_NODE_PATH,
                              APR_HASH_KEY_STRING))
@@ -473,20 +566,26 @@ svn_repos_parse_dumpstream (svn_stream_t *stream,
                                                nodepool));
           found_node = TRUE;
         }
-
       /* Or is this the repos UUID? */
-      else if (NULL != (uuid = apr_hash_get (headers, SVN_REPOS_DUMPFILE_UUID,
-                                             APR_HASH_KEY_STRING)))
+      else if ((value = apr_hash_get (headers, SVN_REPOS_DUMPFILE_UUID,
+                                      APR_HASH_KEY_STRING)))
         {
-          SVN_ERR (parse_fns->uuid_record (uuid, parse_baton, pool));
+          SVN_ERR (parse_fns->uuid_record (value, parse_baton, pool));
+        }
+      /* Or perhaps a dumpfile format? */
+      else if ((value = apr_hash_get (headers, 
+                                      SVN_REPOS_DUMPFILE_MAGIC_HEADER,
+                                      APR_HASH_KEY_STRING)))
+        {
+          /* ### someday, switch modes of operation here. */
+          version = atoi (value);
         }
-
       /* Or is this bogosity?! */
       else
         {
           /* What the heck is this record?!? */
           return svn_error_create (SVN_ERR_STREAM_MALFORMED_DATA, NULL,
-                                   "Unrecognized record type in stream.");
+                                   "Unrecognized record type in stream");
         }
       
       /* Is there a props content-block to parse? */
@@ -494,8 +593,18 @@ svn_repos_parse_dumpstream (svn_stream_t *stream,
                                   SVN_REPOS_DUMPFILE_PROP_CONTENT_LENGTH,
                                   APR_HASH_KEY_STRING)))
         {
+          const char *delta = apr_hash_get (headers,
+                                            SVN_REPOS_DUMPFILE_PROP_DELTA,
+                                            APR_HASH_KEY_STRING);
+          svn_boolean_t is_delta = (delta && strcmp (delta, "true") == 0);
+
+          /* First, remove all node properties, unless this is a delta
+             property block. */
+          if (found_node && !is_delta)
+            SVN_ERR (parse_fns->remove_node_props (node_baton));
+
           SVN_ERR (parse_property_block (stream,
-                                         apr_atoui64 (valstr),
+                                         svn__atoui64 (valstr),
                                          parse_fns,
                                          found_node ? node_baton : rev_baton,
                                          found_node,
@@ -507,11 +616,17 @@ svn_repos_parse_dumpstream (svn_stream_t *stream,
                                   SVN_REPOS_DUMPFILE_TEXT_CONTENT_LENGTH,
                                   APR_HASH_KEY_STRING)))
         {
-          SVN_ERR (parse_text_block (stream, 
-                                     apr_atoui64 (valstr),
+          const char *delta = apr_hash_get (headers,
+                                            SVN_REPOS_DUMPFILE_TEXT_DELTA,
+                                            APR_HASH_KEY_STRING);
+          svn_boolean_t is_delta = (delta && strcmp (delta, "true") == 0);
+
+          SVN_ERR (parse_text_block (stream,
+                                     svn__atoui64 (valstr),
+                                     is_delta,
                                      parse_fns,
                                      found_node ? node_baton : rev_baton,
-                                     buffer, 
+                                     buffer,
                                      buflen,
                                      found_node ? nodepool : revpool));
         }
@@ -539,6 +654,19 @@ svn_repos_parse_dumpstream (svn_stream_t *stream,
 }
 
 
+svn_error_t *
+svn_repos_parse_dumpstream (svn_stream_t *stream,
+                            const svn_repos_parser_fns_t *parse_fns,
+                            void *parse_baton,
+                            svn_cancel_func_t cancel_func,
+                            void *cancel_baton,
+                            apr_pool_t *pool)
+{
+  svn_repos_parser_fns2_t *fns2 = fns2_from_fns (parse_fns, pool);
+
+  return svn_repos_parse_dumpstream2 (stream, fns2, parse_baton,
+                                      cancel_func, cancel_baton, pool);
+}
 
 /*----------------------------------------------------------------------*/
 
@@ -598,7 +726,11 @@ make_node_baton (apr_hash_t *headers,
   if ((val = apr_hash_get (headers, SVN_REPOS_DUMPFILE_NODE_COPYFROM_PATH,
                            APR_HASH_KEY_STRING)))
     {
-      nb->copyfrom_path = apr_pstrdup (pool, val);
+      if (rb->pb->parent_dir)
+        nb->copyfrom_path = svn_path_join (rb->pb->parent_dir,
+                                           (*val == '/' ? val + 1 : val), pool);
+      else
+        nb->copyfrom_path = apr_pstrdup (pool, val);
     }
 
   if ((val = apr_hash_get (headers, SVN_REPOS_DUMPFILE_TEXT_CONTENT_CHECKSUM,
@@ -646,10 +778,12 @@ new_revision_record (void **revision_baton,
   rb = make_revision_baton (headers, pb, pool);
   SVN_ERR (svn_fs_youngest_rev (&head_rev, pb->fs, pool));
 
-  /* Calculate the revision 'offset' for finding copyfrom sources.
+  /* FIXME: This is a lame fallback loading multiple segments of dump in
+     several seperate operations. It is highly susceptible to race conditions.
+     Calculate the revision 'offset' for finding copyfrom sources.
      It might be positive or negative. */
   rb->rev_offset = (rb->rev) - (head_rev + 1);
-
+ 
   if (rb->rev > 0)
     {
       /* Create a new fs txn. */
@@ -657,9 +791,9 @@ new_revision_record (void **revision_baton,
       SVN_ERR (svn_fs_txn_root (&(rb->txn_root), rb->txn, pool));
       
       if (pb->outstream)
-        svn_stream_printf (pb->outstream, pool,
-                           "<<< Started new txn, based on original revision %"
-                           SVN_REVNUM_T_FMT "\n", rb->rev);
+        SVN_ERR (svn_stream_printf (pb->outstream, pool,
+                                    _("<<< Started new transaction, based on "
+                                      "original revision %ld\n"), rb->rev));
     }
 
   /* If we're parsing revision 0, only the revision are (possibly)
@@ -695,11 +829,15 @@ maybe_add_with_history (struct node_baton *nb,
       /* Hunt down the source revision in this fs. */
       svn_fs_root_t *copy_root;
       svn_revnum_t src_rev = nb->copyfrom_rev - rb->rev_offset;
+      svn_revnum_t *src_rev_from_map;
+      if ((src_rev_from_map = apr_hash_get (pb->rev_map, &nb->copyfrom_rev,
+                                            sizeof(nb->copyfrom_rev))))
+        src_rev = *src_rev_from_map;
 
       if (! SVN_IS_VALID_REVNUM(src_rev))
         return svn_error_createf (SVN_ERR_FS_NO_SUCH_REVISION, NULL,
-                                  "Relative copyfrom_rev %" SVN_REVNUM_T_FMT
-                                  " is not available in current repository.",
+                                  _("Relative source revision %ld is not"
+                                    " available in current repository"),
                                   src_rev);
 
       SVN_ERR (svn_fs_revision_root (&copy_root, pb->fs, src_rev, pool));
@@ -709,7 +847,7 @@ maybe_add_with_history (struct node_baton *nb,
       if (pb->outstream)
         {
           apr_size_t len = 9;
-          svn_stream_write (pb->outstream, "COPIED...", &len);
+          SVN_ERR (svn_stream_write (pb->outstream, "COPIED...", &len));
         }
     }
 
@@ -746,30 +884,40 @@ new_node_record (void **node_baton,
 {
   struct revision_baton *rb = revision_baton;
   struct parse_baton *pb = rb->pb;
-  struct node_baton *nb = make_node_baton (headers, rb, pool);
+  struct node_baton *nb;
+  
+  if (rb->rev == 0)
+    return svn_error_create (SVN_ERR_STREAM_MALFORMED_DATA, NULL,
+                             "Malformed dumpstream: "
+                             "Revision 0 must not contain node records");
+
+  nb = make_node_baton (headers, rb, pool);
 
   switch (nb->action)
     {
     case svn_node_action_change:
       {
         if (pb->outstream)
-          svn_stream_printf (pb->outstream, pool,
-                             "     * editing path : %s ...", nb->path);
+          SVN_ERR (svn_stream_printf (pb->outstream, pool,
+                                      "     * editing path : %s ...",
+                                      nb->path));
         break;
       }
     case svn_node_action_delete:
       {
         if (pb->outstream)
-          svn_stream_printf (pb->outstream, pool,
-                             "     * deleting path : %s ...", nb->path);
-        SVN_ERR (svn_fs_delete_tree (rb->txn_root, nb->path, pool));
+          SVN_ERR (svn_stream_printf (pb->outstream, pool,
+                                      "     * deleting path : %s ...",
+                                      nb->path));
+        SVN_ERR (svn_fs_delete (rb->txn_root, nb->path, pool));
         break;
       }
     case svn_node_action_add:
       {
         if (pb->outstream)
-          svn_stream_printf (pb->outstream, pool,
-                             "     * adding path : %s ...", nb->path);
+          SVN_ERR (svn_stream_printf (pb->outstream, pool,
+                                      "     * adding path : %s ...",
+                                      nb->path));
 
         SVN_ERR (maybe_add_with_history (nb, rb, pool));
         break;
@@ -777,17 +925,18 @@ new_node_record (void **node_baton,
     case svn_node_action_replace:
       {
         if (pb->outstream)
-          svn_stream_printf (pb->outstream, pool,
-                             "     * replacing path : %s ...", nb->path);
+          SVN_ERR (svn_stream_printf (pb->outstream, pool,
+                                      "     * replacing path : %s ...",
+                                      nb->path));
 
-        SVN_ERR (svn_fs_delete_tree (rb->txn_root, nb->path, pool));
+        SVN_ERR (svn_fs_delete (rb->txn_root, nb->path, pool));
 
         SVN_ERR (maybe_add_with_history (nb, rb, pool));
         break;
       }
     default:
       return svn_error_createf (SVN_ERR_STREAM_UNRECOGNIZED_DATA, NULL,
-                                "Unrecognized node-action on node '%s'.",
+                                "Unrecognized node-action on node '%s'",
                                 nb->path);
     }
 
@@ -844,6 +993,20 @@ set_node_property (void *baton,
 }
 
 
+static svn_error_t *
+delete_node_property (void *baton,
+                      const char *name)
+{
+  struct node_baton *nb = baton;
+  struct revision_baton *rb = nb->rb;
+
+  SVN_ERR (svn_fs_change_node_prop (rb->txn_root, nb->path,
+                                    name, NULL, nb->pool));
+  
+  return SVN_NO_ERROR;
+}
+
+
 static svn_error_t *
 remove_node_props (void *baton)
 {
@@ -872,6 +1035,21 @@ remove_node_props (void *baton)
 }
 
 
+static svn_error_t *
+apply_textdelta (svn_txdelta_window_handler_t *handler,
+                 void **handler_baton,
+                 void *node_baton)
+{
+  struct node_baton *nb = node_baton;
+  struct revision_baton *rb = nb->rb;
+
+  return svn_fs_apply_textdelta (handler, handler_baton,
+                                 rb->txn_root, nb->path,
+                                 NULL, nb->md5_checksum,
+                                 nb->pool);
+}
+
+
 static svn_error_t *
 set_fulltext (svn_stream_t **stream,
               void *node_baton)
@@ -896,7 +1074,7 @@ close_node (void *baton)
   if (pb->outstream)
     {
       apr_size_t len = 7;
-      svn_stream_write (pb->outstream, " done.\n", &len);
+      SVN_ERR (svn_stream_write (pb->outstream, " done.\n", &len));
     }
   
   return SVN_NO_ERROR;
@@ -909,45 +1087,58 @@ close_revision (void *baton)
   struct revision_baton *rb = baton;
   struct parse_baton *pb = rb->pb;
   const char *conflict_msg = NULL;
-  svn_revnum_t new_rev;
+  svn_revnum_t *old_rev, *new_rev;
   svn_error_t *err;
 
   if (rb->rev <= 0)
     return SVN_NO_ERROR;
 
-  err = svn_fs_commit_txn (&conflict_msg, &new_rev, rb->txn);
+  /* Prepare memory for saving dump-rev -> in-repos-rev mapping. */
+  old_rev = apr_palloc (pb->pool, sizeof(svn_revnum_t) * 2);
+  new_rev = old_rev + 1;
+  *old_rev = rb->rev;
+
+  err = svn_fs_commit_txn (&conflict_msg, &(*new_rev), rb->txn, rb->pool);
 
   if (err)
     {
-      svn_fs_abort_txn (rb->txn);
+      svn_error_clear (svn_fs_abort_txn (rb->txn, rb->pool));
       if (conflict_msg)
         return svn_error_quick_wrap (err, conflict_msg);
       else
         return err;
     }
 
+  /* After a successful commit, must record the dump-rev -> in-repos-rev
+     mapping, so that copyfrom instructions in the dump file can look up the
+     correct repository revision to copy from. */
+  apr_hash_set (pb->rev_map, old_rev, sizeof(svn_revnum_t), new_rev);
+
+  /* Deltify the predecessors of paths changed in this revision. */
+  SVN_ERR (svn_fs_deltify_revision (pb->fs, *new_rev, rb->pool));
+
   /* Grrr, svn_fs_commit_txn rewrites the datestamp property to the
      current clock-time.  We don't want that, we want to preserve
      history exactly.  Good thing revision props aren't versioned! */
   if (rb->datestamp)
-    SVN_ERR (svn_fs_change_rev_prop (pb->fs, new_rev,
+    SVN_ERR (svn_fs_change_rev_prop (pb->fs, *new_rev,
                                      SVN_PROP_REVISION_DATE, rb->datestamp,
                                      rb->pool));
 
   if (pb->outstream)
     {
-      if (new_rev == rb->rev)
+      if (*new_rev == rb->rev)
         {
-          svn_stream_printf (pb->outstream, rb->pool,
-                             "\n------- Committed revision %" SVN_REVNUM_T_FMT
-                             " >>>\n\n", new_rev);
+          SVN_ERR (svn_stream_printf (pb->outstream, rb->pool,
+                                      _("\n------- Committed revision %ld"
+                                        " >>>\n\n"), *new_rev));
         }
       else
         {
-          svn_stream_printf (pb->outstream, rb->pool,
-                             "\n------- Committed new rev %" SVN_REVNUM_T_FMT
-                             " (loaded from original rev %" SVN_REVNUM_T_FMT
-                             ") >>>\n\n", new_rev, rb->rev);
+          SVN_ERR (svn_stream_printf (pb->outstream, rb->pool,
+                                      _("\n------- Committed new rev %ld"
+                                        " (loaded from original rev %ld"
+                                        ") >>>\n\n"), *new_rev, rb->rev));
         }
     }
 
@@ -960,6 +1151,33 @@ close_revision (void *baton)
 /** The public routines **/
 
 
+svn_error_t *
+svn_repos_get_fs_build_parser2 (const svn_repos_parser_fns2_t **callbacks,
+                                void **parse_baton,
+                                svn_repos_t *repos,
+                                svn_boolean_t use_history,
+                                enum svn_repos_load_uuid uuid_action,
+                                svn_stream_t *outstream,
+                                const char *parent_dir,
+                                apr_pool_t *pool)
+{
+  const svn_repos_parser_fns_t *fns;
+  svn_repos_parser_fns2_t *parser;
+
+  /* Fetch the old-style vtable and baton, convert the vtable to a
+   * new-style vtable, and set the new callbacks. */
+  SVN_ERR (svn_repos_get_fs_build_parser (&fns, parse_baton, repos,
+                                          use_history, uuid_action, outstream,
+                                          parent_dir, pool));
+  parser = fns2_from_fns (fns, pool);
+  parser->delete_node_property = delete_node_property;
+  parser->apply_textdelta = apply_textdelta;
+  *callbacks = parser;
+  return SVN_NO_ERROR;
+}
+
+
+
 svn_error_t *
 svn_repos_get_fs_build_parser (const svn_repos_parser_fns_t **parser_callbacks,
                                void **parse_baton,
@@ -989,6 +1207,8 @@ svn_repos_get_fs_build_parser (const svn_repos_parser_fns_t **parser_callbacks,
   pb->outstream = outstream;
   pb->uuid_action = uuid_action;
   pb->parent_dir = parent_dir;
+  pb->pool = pool;
+  pb->rev_map = apr_hash_make (pool);
 
   *parser_callbacks = parser;
   *parse_baton = pb;
@@ -1003,22 +1223,25 @@ svn_repos_load_fs (svn_repos_t *repos,
                    svn_stream_t *feedback_stream,
                    enum svn_repos_load_uuid uuid_action,
                    const char *parent_dir,
+                   svn_cancel_func_t cancel_func,
+                   void *cancel_baton,
                    apr_pool_t *pool)
 {
-  const svn_repos_parser_fns_t *parser;
+  const svn_repos_parser_fns2_t *parser;
   void *parse_baton;
   
   /* This is really simple. */  
 
-  SVN_ERR (svn_repos_get_fs_build_parser (&parser, &parse_baton,
-                                          repos,
-                                          TRUE, /* look for copyfrom revs */
-                                          uuid_action,
-                                          feedback_stream,
-                                          parent_dir,
-                                          pool));
+  SVN_ERR (svn_repos_get_fs_build_parser2 (&parser, &parse_baton,
+                                           repos,
+                                           TRUE, /* look for copyfrom revs */
+                                           uuid_action,
+                                           feedback_stream,
+                                           parent_dir,
+                                           pool));
 
-  SVN_ERR (svn_repos_parse_dumpstream (dumpstream, parser, parse_baton, pool));
+  SVN_ERR (svn_repos_parse_dumpstream2 (dumpstream, parser, parse_baton,
+                                        cancel_func, cancel_baton, pool));
 
   return SVN_NO_ERROR;
 }
