@@ -157,6 +157,15 @@ svn_config__open_file (FILE **pfile,
                        const char *mode,
                        apr_pool_t *pool);
 
+
+/* Stubs for allowing 1.0.x Apache modules to be mixed with 1.1.x libraries. */
+typedef svn_boolean_t (*svn_config__section_enumerator_t)
+        (const char *name, void *baton);
+
+int svn_config__enumerate_sections (svn_config_t *cfg,
+                                    svn_config__section_enumerator_t callback,
+                                    void *baton);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
