@@ -75,22 +75,24 @@
 #ifdef WIN32
 #define WIN32_RETRY_LOOP(err, expr)                                        \
   do                                                                       \
-  {                                                                        \
-    int retries = 0;                                                       \
-    int sleep_count = 1000;                                                \
-                                                                           \
-    for ( retries = 0;                                                     \
-          APR_TO_OS_ERROR (err) == ERROR_ACCESS_DENIED && retries < 100;   \
-          ++retries )                                                      \
     {                                                                      \
-      apr_sleep (sleep_count);                                             \
-      if (sleep_count < 128000)                                            \
-        sleep_count *= 2;                                                  \
-      err = expr;                                                          \
+      apr_status_t os_err = APR_TO_OS_ERROR (err);                         \
+      int sleep_count = 1000;                                              \
+      int retries;                                                         \
+      for (retries = 0;                                                    \
+           retries < 100 && (os_err == ERROR_ACCESS_DENIED                 \
+                             || os_err == ERROR_SHARING_VIOLATION);        \
+           ++retries, os_err = APR_TO_OS_ERROR (err))                      \
+        {                                                                  \
+          apr_sleep (sleep_count);                                         \
+          if (sleep_count < 128000)                                        \
+            sleep_count *= 2;                                              \
+          (err) = (expr);                                                  \
+        }                                                                  \
     }                                                                      \
-  } while (0)
+  while (0)
 #else
-#define WIN32_RETRY_LOOP(err, expr) do {} while(0)
+#define WIN32_RETRY_LOOP(err, expr) ((void)0)
 #endif
 
 /* Helper for svn_io_check_path() and svn_io_check_resolved_path();
@@ -663,43 +665,33 @@ svn_error_t *svn_io_copy_dir_recursively (const char *src,
         }
       else
         {
-          const char *src_target, *src_target_utf8, *dst_target_utf8;
+          const char *src_target, *entryname_utf8;
 
           if (cancel_func)
             SVN_ERR (cancel_func (cancel_baton));
 
-          /* Telescope the entryname onto the source dir. */
-          src_target = svn_path_join (src, this_entry.name, subpool);
-          SVN_ERR (svn_path_cstring_to_utf8 (&src_target_utf8, src_target,
-                                             subpool));
+          SVN_ERR (svn_path_cstring_to_utf8 (&entryname_utf8,
+                                             this_entry.name, subpool));
+          src_target = svn_path_join (src, entryname_utf8, subpool);
           
           if (this_entry.filetype == APR_REG) /* regular file */
             {
-              const char *dst_target = svn_path_join (dst_path,
-                                                      this_entry.name,
+              const char *dst_target = svn_path_join (dst_path, entryname_utf8,
                                                       subpool);
-              SVN_ERR (svn_path_cstring_to_utf8 (&dst_target_utf8, dst_target,
-                                                 subpool));
-              SVN_ERR (svn_io_copy_file (src_target_utf8, dst_target_utf8,
+              SVN_ERR (svn_io_copy_file (src_target, dst_target,
                                          copy_perms, subpool));
             }
           else if (this_entry.filetype == APR_LNK) /* symlink */
             {
-              const char *dst_target = svn_path_join (dst_path,
-                                                      this_entry.name,
+              const char *dst_target = svn_path_join (dst_path, entryname_utf8,
                                                       subpool);
-              SVN_ERR (svn_path_cstring_to_utf8 (&dst_target_utf8, dst_target,
-                                                 subpool));
-              SVN_ERR (svn_io_copy_link (src_target_utf8, dst_target_utf8,
+              SVN_ERR (svn_io_copy_link (src_target, dst_target,
                                          subpool));
             }
           else if (this_entry.filetype == APR_DIR) /* recurse */
             {
-              const char *entryname_utf8;
-              SVN_ERR (svn_path_cstring_to_utf8 (&entryname_utf8,
-                                                 this_entry.name, subpool));
               SVN_ERR (svn_io_copy_dir_recursively 
-                       (src_target_utf8,
+                       (src_target,
                         dst_path,
                         entryname_utf8,
                         copy_perms,
