@@ -740,7 +740,9 @@ svn_opt_print_help (apr_getopt_t *os,
   if (os)
     SVN_ERR (svn_opt_parse_all_args (&targets, os, pool));
 
-  if (targets && targets->nelts)  /* help on subcommand(s) requested */
+  if (cmd_table
+      && targets
+      && targets->nelts)  /* help on subcommand(s) requested */
     for (i = 0; i < targets->nelts; i++)
       {
         svn_opt_subcommand_help (((const char **) (targets->elts))[i],
