@@ -50,19 +50,22 @@
 #include "props.h"
 #include "translate.h"
 
+#include "svn_private_config.h"
+
 
 /*** Finishing updates and commits. ***/
 
 
-/* The main recursive body of svn_wc__do_update_cleanup. */
+/* The main body of svn_wc__do_update_cleanup. */
 static svn_error_t *
-recursively_tweak_entries (svn_wc_adm_access_t *dirpath,
-                           const char *base_url,
-                           svn_revnum_t new_rev,
-                           svn_wc_notify_func_t notify_func,
-                           void *notify_baton,
-                           svn_boolean_t remove_missing_dirs,
-                           apr_pool_t *pool)
+tweak_entries (svn_wc_adm_access_t *dirpath,
+               const char *base_url,
+               svn_revnum_t new_rev,
+               svn_wc_notify_func_t notify_func,
+               void *notify_baton,
+               svn_boolean_t remove_missing_dirs,
+               svn_boolean_t recurse,
+               apr_pool_t *pool)
 {
   apr_hash_t *entries;
   apr_hash_index_t *hi;
@@ -70,7 +73,7 @@ recursively_tweak_entries (svn_wc_adm_access_t *dirpath,
   svn_boolean_t write_required = FALSE;
   
   /* Read DIRPATH's entries. */
-  SVN_ERR (svn_wc_entries_read (&entries, dirpath, TRUE, subpool));
+  SVN_ERR (svn_wc_entries_read (&entries, dirpath, TRUE, pool));
 
   /* Tweak "this_dir" */
   SVN_ERR (svn_wc__tweak_entry (entries, SVN_WC_ENTRY_THIS_DIR,
@@ -78,7 +81,7 @@ recursively_tweak_entries (svn_wc_adm_access_t *dirpath,
                                 svn_wc_adm_access_pool (dirpath)));
 
   /* Recursively loop over all children. */
-  for (hi = apr_hash_first (subpool, entries); hi; hi = apr_hash_next (hi))
+  for (hi = apr_hash_first (pool, entries); hi; hi = apr_hash_next (hi))
     {
       const void *key;
       void *val;
@@ -86,6 +89,8 @@ recursively_tweak_entries (svn_wc_adm_access_t *dirpath,
       svn_wc_entry_t *current_entry;
       const char *child_url = NULL;
 
+      svn_pool_clear (subpool);
+
       apr_hash_this (hi, &key, NULL, &val);
       name = key;
       current_entry = val;
@@ -98,19 +103,18 @@ recursively_tweak_entries (svn_wc_adm_access_t *dirpath,
       if (base_url)
         child_url = svn_path_url_add_component (base_url, name, subpool);
       
-      /* If a file, or deleted or absent dir, then tweak the entry but
-         don't recurse. */
+      /* If a file, or deleted or absent dir in recursive mode, then tweak the
+         entry but don't recurse. */
       if ((current_entry->kind == svn_node_file)
-          || current_entry->deleted
-          || current_entry->absent)
+          || (recurse && (current_entry->deleted || current_entry->absent)))
         {
           SVN_ERR (svn_wc__tweak_entry (entries, name,
                                         child_url, new_rev, &write_required,
                                         svn_wc_adm_access_pool (dirpath)));
         }
       
-      /* If a directory... */
-      else if (current_entry->kind == svn_node_dir)        
+      /* If a directory and recursive... */
+      else if (recurse && (current_entry->kind == svn_node_dir))
         {
           const char *child_path
             = svn_path_join (svn_wc_adm_access_path (dirpath), name, subpool);
@@ -142,9 +146,9 @@ recursively_tweak_entries (svn_wc_adm_access_t *dirpath,
               svn_wc_adm_access_t *child_access;
               SVN_ERR (svn_wc_adm_retrieve (&child_access, dirpath, child_path,
                                             subpool));
-              SVN_ERR (recursively_tweak_entries 
+              SVN_ERR (tweak_entries 
                        (child_access, child_url, new_rev, notify_func, 
-                        notify_baton, remove_missing_dirs, subpool));
+                        notify_baton, remove_missing_dirs, recurse, subpool));
             }
         }
     }
@@ -199,26 +203,14 @@ svn_wc__do_update_cleanup (const char *path,
       svn_wc_adm_access_t *dir_access;
       SVN_ERR (svn_wc_adm_retrieve (&dir_access, adm_access, path, pool));
 
-      if (! recursive) 
-        {
-          svn_boolean_t write_required = FALSE;
-          SVN_ERR (svn_wc_entries_read (&entries, dir_access, TRUE, pool));
-          SVN_ERR (svn_wc__tweak_entry (entries, SVN_WC_ENTRY_THIS_DIR,
-                                        base_url, new_revision, &write_required,
-                                        svn_wc_adm_access_pool (dir_access)));
-          if (write_required)
-            SVN_ERR (svn_wc__entries_write (entries, dir_access, pool));
-        }
-      else
-        SVN_ERR (recursively_tweak_entries (dir_access, base_url,
-                                            new_revision, notify_func, 
-                                            notify_baton, remove_missing_dirs,
-                                            pool));
+      SVN_ERR (tweak_entries (dir_access, base_url, new_revision,
+                              notify_func, notify_baton, remove_missing_dirs,
+                              recursive, pool));
     }
 
   else
     return svn_error_createf (SVN_ERR_NODE_UNKNOWN_KIND, NULL,
-                              "Unrecognized node kind: '%s'", path);
+                              _("Unrecognized node kind: '%s'"), path);
 
   return SVN_NO_ERROR;
 }
@@ -237,7 +229,7 @@ svn_wc_process_committed (const char *path,
   const char *base_name;
   svn_stringbuf_t *logtags;
   apr_file_t *log_fp = NULL;
-  char *revstr = apr_psprintf (pool, "%" SVN_REVNUM_T_FMT, new_revnum);
+  char *revstr = apr_psprintf (pool, "%ld", new_revnum);
   const char *hex_digest = NULL;
 
   SVN_ERR (svn_wc__adm_write_check (adm_access));
@@ -383,7 +375,7 @@ svn_wc_process_committed (const char *path,
 
   SVN_ERR_W (svn_io_file_write_full (log_fp, logtags->data, 
                                      logtags->len, NULL, pool),
-             apr_psprintf (pool, "Error writing log file for '%s'", path));
+             apr_psprintf (pool, _("Error writing log file for '%s'"), path));
       
   SVN_ERR (svn_wc__close_adm_file (log_fp, svn_wc_adm_access_path (adm_access),
                                    SVN_WC__ADM_LOG,
@@ -620,7 +612,7 @@ erase_unversioned_from_wc (const char *path,
  * deletion from the physical filesystem.  For directories it involves the
  * deletion from the filesystem of all unversioned children, and all
  * versioned children that are files. By the time we get here, added but
- * not committed items will have been scheduled for detetion which means
+ * not committed items will have been scheduled for deletion which means
  * they have become unversioned.
  *
  * The result is that all that remains are versioned directories, each with
@@ -740,8 +732,8 @@ svn_wc_delete (const char *path,
   svn_node_kind_t was_kind;
   svn_boolean_t was_deleted = FALSE; /* Silence a gcc uninitialized warning */
 
-  SVN_ERR (svn_wc_adm_probe_try (&dir_access, adm_access, path,
-                                 TRUE, TRUE, pool));
+  SVN_ERR (svn_wc_adm_probe_try2 (&dir_access, adm_access, path,
+                                  TRUE, -1, pool));
   if (dir_access)
     SVN_ERR (svn_wc_entry (&entry, path, dir_access, FALSE, pool));
   else
@@ -763,7 +755,7 @@ svn_wc_delete (const char *path,
       apr_hash_t *entries;
       const svn_wc_entry_t *entry_in_parent;
 
-      /* The deleted state is only avaliable in the entry in parent's
+      /* The deleted state is only available in the entry in parent's
          entries file */
       SVN_ERR (svn_wc_adm_retrieve (&parent_access, adm_access, parent, pool));
       SVN_ERR (svn_wc_entries_read (&entries, parent_access, TRUE, pool));
@@ -891,10 +883,10 @@ svn_wc_add (const char *path,
   SVN_ERR (svn_io_check_path (path, &kind, pool));
   if (kind == svn_node_none)
     return svn_error_createf (SVN_ERR_WC_PATH_NOT_FOUND, NULL,
-                              "'%s' not found", path);
+                              _("'%s' not found"), path);
   if (kind == svn_node_unknown)
     return svn_error_createf (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-                              "Unsupported node kind for path '%s'", path);
+                              _("Unsupported node kind for path '%s'"), path);
 
   /* Get the original entry for this path if one exists (perhaps
      this is actually a replacement of a previously deleted thing).
@@ -902,8 +894,9 @@ svn_wc_add (const char *path,
      Note that this is one of the few functions that is allowed to see
     'deleted' entries;  it's totally fine to have an entry that is
      scheduled for addition and still previously 'deleted'.  */
-  SVN_ERR (svn_wc_adm_probe_try (&adm_access, parent_access, path,
-                                 TRUE, copyfrom_url != NULL, pool));
+  SVN_ERR (svn_wc_adm_probe_try2 (&adm_access, parent_access, path,
+                                  TRUE, copyfrom_url != NULL ? -1 : 0,
+                                  pool));
   if (adm_access)
     SVN_ERR (svn_wc_entry (&orig_entry, path, adm_access, TRUE, pool));
   else
@@ -922,7 +915,7 @@ svn_wc_add (const char *path,
         {
           return svn_error_createf 
             (SVN_ERR_ENTRY_EXISTS, NULL,
-             "'%s' is already under version control", path);
+             _("'%s' is already under version control"), path);
         }
       else if (orig_entry->kind != kind)
         {
@@ -933,8 +926,8 @@ svn_wc_add (const char *path,
              error so that clients can detect it. */
           return svn_error_createf 
             (SVN_ERR_WC_NODE_KIND_CHANGE, NULL,
-             "Can't replace '%s' with a node of a differing type; "
-             "commit the deletion, update the parent, and then add '%s'",
+             _("Can't replace '%s' with a node of a differing type; "
+               "commit the deletion, update the parent, and then add '%s'"),
              path, path);
         }
       if (orig_entry->schedule == svn_wc_schedule_delete)
@@ -948,12 +941,12 @@ svn_wc_add (const char *path,
   if (! parent_entry)
     return svn_error_createf 
       (SVN_ERR_ENTRY_NOT_FOUND, NULL,
-       "Can't not find parent directory's entry while trying to add '%s'",
+       _("Can't find parent directory's entry while trying to add '%s'"),
        path);
   if (parent_entry->schedule == svn_wc_schedule_delete)
     return svn_error_createf 
       (SVN_ERR_WC_SCHEDULE_CONFLICT, NULL,
-       "Can't add '%s' to a parent directory scheduled for deletion",
+       _("Can't add '%s' to a parent directory scheduled for deletion"),
        path);
 
   /* Init the modify flags. */
@@ -1022,9 +1015,12 @@ svn_wc_add (const char *path,
       
       /* We want the locks to persist, so use the access baton's pool */
       if (! orig_entry || orig_entry->deleted)
-        SVN_ERR (svn_wc_adm_open (&adm_access, parent_access, path,
-                                  TRUE, copyfrom_url != NULL,
-                                  svn_wc_adm_access_pool (parent_access)));
+        {
+          apr_pool_t* access_pool = svn_wc_adm_access_pool (parent_access);
+          SVN_ERR (svn_wc_adm_open2 (&adm_access, parent_access, path,
+                                     TRUE, copyfrom_url != NULL ? -1 : 0,
+                                     access_pool));
+        }
 
       /* We're making the same mods we made above, but this time we'll
          force the scheduling.  Also make sure to undo the
@@ -1135,20 +1131,6 @@ svn_wc_add (const char *path,
 */
 
 
-/* Return a new wrapping of error ERR regarding the revert subcommand,
-   while doing VERB on PATH.  Use POOL for allocations.
-*/
-static svn_error_t *
-revert_error (svn_error_t *err,
-              const char *path,
-              const char *verb,
-              apr_pool_t *pool)
-{
-  return svn_error_quick_wrap 
-    (err, apr_psprintf (pool, "Error '%s' for '%s'", verb, path));
-}
-
-
 /* Revert ENTRY for NAME in directory represented by ADM_ACCESS, altering
    *MODIFY_FLAGS to indicate what parts of the entry were reverted
    (for example, if property changes were reverted, then set the
@@ -1168,6 +1150,8 @@ revert_admin_things (svn_wc_adm_access_t *adm_access,
   svn_boolean_t modified_p;
   svn_error_t *err;
   apr_time_t tstamp;
+  apr_array_header_t *propchanges;
+  svn_boolean_t magic_props_changed = FALSE;
 
   /* Build the full path of the thing we're reverting. */
   fullpath = svn_wc_adm_access_path (adm_access);
@@ -1180,6 +1164,28 @@ revert_admin_things (svn_wc_adm_access_t *adm_access,
     {
       svn_node_kind_t working_props_kind;
 
+      /* Get the full list of property changes and see if any magic
+         properties were changed. */
+      SVN_ERR (svn_wc_get_prop_diffs (&propchanges, NULL, name ? name : "",
+                                      adm_access, pool));
+      
+      /* Determine if any of the propchanges are the "magic" ones that
+         might require changing the working file. */
+      {
+        int i;
+        for (i = 0; i < propchanges->nelts; i++)
+          {
+            svn_prop_t *propchange
+              = &APR_ARRAY_IDX (propchanges, i, svn_prop_t);
+            
+            if ((! strcmp (propchange->name, SVN_PROP_EXECUTABLE))
+                || (! strcmp (propchange->name, SVN_PROP_KEYWORDS))
+                || (! strcmp (propchange->name, SVN_PROP_EOL_STYLE))
+                || (! strcmp (propchange->name, SVN_PROP_SPECIAL)))
+              magic_props_changed = TRUE;
+          }
+      }
+  
       SVN_ERR (svn_wc__prop_path (&thing, fullpath, adm_access, FALSE, pool)); 
       SVN_ERR (svn_wc__prop_base_path (&base_thing, fullpath, adm_access, FALSE,
                                        pool));
@@ -1197,10 +1203,14 @@ revert_admin_things (svn_wc_adm_access_t *adm_access,
         {
           if ((working_props_kind == svn_node_file)
               && (err = svn_wc__prep_file_for_replacement (thing, FALSE, pool)))
-            return revert_error (err, fullpath, "restoring props", pool);
+            return svn_error_quick_wrap 
+              (err, apr_psprintf (pool, _("Error restoring props for '%s'"),
+                                  fullpath));
 
           if ((err = svn_io_copy_file (base_thing, thing, FALSE, pool)))
-            return revert_error (err, fullpath, "restoring props", pool);
+            return svn_error_quick_wrap 
+              (err, apr_psprintf (pool, _("Error restoring props for '%s'"),
+                                  fullpath));
 
           SVN_ERR (svn_io_file_affected_time (&tstamp, thing, pool));
           entry->prop_time = tstamp;
@@ -1208,7 +1218,9 @@ revert_admin_things (svn_wc_adm_access_t *adm_access,
       else if (working_props_kind == svn_node_file)
         {
           if ((err = svn_io_remove_file (thing, pool)))
-            return revert_error (err, fullpath, "removing props", pool);
+            return svn_error_quick_wrap 
+              (err, apr_psprintf (pool, _("Error removing props for '%s'"),
+                                  fullpath));
         }
 
       /* Modify our entry structure. */
@@ -1232,7 +1244,9 @@ revert_admin_things (svn_wc_adm_access_t *adm_access,
       if (kind == svn_node_file)
         {
           if ((err = svn_io_copy_file (base_thing, thing, FALSE, pool)))
-            return revert_error (err, fullpath, "restoring props", pool);
+            return svn_error_quick_wrap 
+              (err, apr_psprintf (pool, _("Error restoring props for '%s'"),
+                                  fullpath));
       
           SVN_ERR (svn_io_file_affected_time (&tstamp, thing, pool));
           entry->prop_time = tstamp;
@@ -1243,9 +1257,12 @@ revert_admin_things (svn_wc_adm_access_t *adm_access,
   if (entry->kind == svn_node_file)
     {
       SVN_ERR (svn_io_check_path (fullpath, &kind, pool));
-      SVN_ERR (svn_wc_text_modified_p (&modified_p, fullpath, FALSE,
-                                       adm_access, pool));
-      if ((modified_p) || (kind == svn_node_none))
+      base_thing = svn_wc__text_base_path (fullpath, 0, pool);
+
+      if (! magic_props_changed)
+        SVN_ERR (svn_wc_text_modified_p (&modified_p, fullpath, TRUE,
+                                         adm_access, pool));
+      if ((modified_p) || (kind == svn_node_none) || (magic_props_changed))
         {
           /* If there are textual mods (or if the working file is
              missing altogether), copy the text-base out into
@@ -1253,26 +1270,29 @@ revert_admin_things (svn_wc_adm_access_t *adm_access,
              file. */
           svn_subst_keywords_t *keywords;
           const char *eol;
+          svn_boolean_t special;
           
-          base_thing = svn_wc__text_base_path (fullpath, 0, pool);
-
           SVN_ERR (svn_wc__get_eol_style (NULL, &eol, fullpath, adm_access,
                                           pool));
           SVN_ERR (svn_wc__get_keywords (&keywords, fullpath, adm_access, NULL,
                                          pool));
+          SVN_ERR (svn_wc__get_special (&special, fullpath, adm_access, pool));
 
           /* When copying the text-base out to the working copy, make
              sure to do any eol translations or keyword substitutions,
              as dictated by the property values.  If these properties
              are turned off, then this is just a normal copy. */
-          if ((err = svn_subst_copy_and_translate (base_thing,
-                                                   fullpath,
-                                                   eol, 
-                                                   FALSE, /* don't repair */
-                                                   keywords,
-                                                   TRUE, /* expand keywords */
-                                                   pool)))
-            return revert_error (err, fullpath, "restoring text", pool);
+          if ((err = svn_subst_copy_and_translate2 (base_thing,
+                                                    fullpath,
+                                                    eol, 
+                                                    FALSE, /* don't repair */
+                                                    keywords,
+                                                    TRUE, /* expand keywords */
+                                                    special,
+                                                    pool)))
+            return svn_error_quick_wrap 
+              (err, apr_psprintf (pool, _("Error restoring text for '%s'"),
+                                  fullpath));
 
           /* If necessary, tweak the new working file's executable bit. */
           SVN_ERR (svn_wc__maybe_set_executable (NULL, fullpath, adm_access,
@@ -1280,7 +1300,7 @@ revert_admin_things (svn_wc_adm_access_t *adm_access,
 
           /* Possibly set the timestamp to last-commit-time, rather
              than the 'now' time that already exists. */
-          if (use_commit_times)
+          if (use_commit_times && (! special))
             {
               SVN_ERR (svn_io_set_file_affected_time (entry->cmt_date,
                                                       fullpath, pool));
@@ -1372,7 +1392,7 @@ svn_wc_revert (const char *path,
   if (! entry)
     return svn_error_createf 
       (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
-       "Cannot revert: '%s' is not under version control", path);
+       _("Cannot revert: '%s' is not under version control"), path);
 
   /* Safeguard 1.5: is this a missing versioned directory? */
   if (entry->kind == svn_node_dir)
@@ -1403,7 +1423,7 @@ svn_wc_revert (const char *path,
   if ((entry->kind != svn_node_file) && (entry->kind != svn_node_dir))
     return svn_error_createf 
       (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-       "Cannot revert '%s': unsupported entry node kind", path);
+       _("Cannot revert '%s': unsupported entry node kind"), path);
 
   /* Safeguard 3:  can we deal with the node kind of PATH current in
      the working copy? */
@@ -1413,7 +1433,7 @@ svn_wc_revert (const char *path,
       && (kind != svn_node_dir))
     return svn_error_createf 
       (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-       "Cannot revert '%s': unsupported node kind in working copy", path);
+       _("Cannot revert '%s': unsupported node kind in working copy"), path);
 
   /* For directories, determine if PATH is a WC root so that we can
      tell if it is safe to split PATH into a parent directory and
@@ -1460,8 +1480,8 @@ svn_wc_revert (const char *path,
           if (path[0] == '\0')
             {
               return svn_error_create (SVN_ERR_WC_INVALID_OP_ON_CWD, NULL,
-                "Cannot revert addition of current directory; "
-                "please try again from the parent directory");
+                _("Cannot revert addition of current directory; "
+                  "please try again from the parent directory"));
             }
               
           SVN_ERR (svn_wc_entries_read (&entries, parent_access, TRUE, pool));
@@ -1482,7 +1502,7 @@ svn_wc_revert (const char *path,
       else  /* Else it's `none', or something exotic like a symlink... */
         return svn_error_createf
           (SVN_ERR_NODE_UNKNOWN_KIND, NULL,
-           "Unknown or unexpected kind for path '%s'", path);
+           _("Unknown or unexpected kind for path '%s'"), path);
 
       /* Recursivity is taken care of by svn_wc_remove_from_revision_control, 
          and we've definitely reverted PATH at this point. */
@@ -1667,7 +1687,7 @@ svn_wc_remove_from_revision_control (svn_wc_adm_access_t *adm_access,
                                        FALSE, adm_access, pool));
       if (text_modified_p && instant_error)
         return svn_error_createf (SVN_ERR_WC_LEFT_LOCAL_MOD, NULL,
-                                  "File '%s' has local modifications",
+                                  _("File '%s' has local modifications"),
                                   name);
 
       /* Remove NAME from PATH's entries file: */
@@ -2072,7 +2092,7 @@ svn_wc_resolved_conflict (const char *path,
       SVN_ERR (svn_wc_entry (&entry, path, adm_access, FALSE, pool));
       if (! entry)
         return svn_error_createf (SVN_ERR_ENTRY_NOT_FOUND, NULL,
-                                  "'%s' is not under version control", path);
+                                  _("'%s' is not under version control"), path);
 
       SVN_ERR (resolve_found_entry_callback (path, entry, baton, pool));
     }
