@@ -3,7 +3,7 @@
  *           repository.
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -141,6 +141,9 @@ struct edit_baton {
      recursive. */
   svn_boolean_t recurse;
 
+  /* Should this diff ignore node ancestry. */
+  svn_boolean_t ignore_ancestry;
+
   /* Possibly diff repos against text-bases instead of working files. */
   svn_boolean_t use_text_base;
 
@@ -209,6 +212,9 @@ struct file_baton {
   svn_txdelta_window_handler_t apply_handler;
   void *apply_baton;
 
+  /* Is this file scheduled to be deleted? */
+  svn_boolean_t schedule_delete;
+
   /* The overall crawler editor baton. */
   struct edit_baton *edit_baton;
 
@@ -219,9 +225,10 @@ struct file_baton {
 /* Create a new edit baton. TARGET/ANCHOR are working copy paths that
  * describe the root of the comparison. CALLBACKS/CALLBACK_BATON
  * define the callbacks to compare files. RECURSE defines whether to
- * descend into subdirectories.  USE_TEXT_BASE defines whether to
- * compare against working files or text-bases.  REVERSE_ORDER defines
- * which direction to perform the diff.
+ * descend into subdirectories.  IGNORE_ANCESTRY defines whether to
+ * utilize node ancestry when calculating diffs.  USE_TEXT_BASE
+ * defines whether to compare against working files or text-bases.
+ * REVERSE_ORDER defines which direction to perform the diff.
  */
 static struct edit_baton *
 make_editor_baton (svn_wc_adm_access_t *anchor,
@@ -229,6 +236,7 @@ make_editor_baton (svn_wc_adm_access_t *anchor,
                    const svn_wc_diff_callbacks_t *callbacks,
                    void *callback_baton,
                    svn_boolean_t recurse,
+                   svn_boolean_t ignore_ancestry,
                    svn_boolean_t use_text_base,
                    svn_boolean_t reverse_order,
                    apr_pool_t *pool)
@@ -241,6 +249,7 @@ make_editor_baton (svn_wc_adm_access_t *anchor,
   eb->callbacks = callbacks;
   eb->callback_baton = callback_baton;
   eb->recurse = recurse;
+  eb->ignore_ancestry = ignore_ancestry;
   eb->use_text_base = use_text_base;
   eb->reverse_order = reverse_order;
   eb->pool = pool;
@@ -298,6 +307,7 @@ make_file_baton (const char *path,
   file_baton->baseprops = apr_hash_make (file_baton->pool);
   file_baton->propchanges  = apr_array_make (pool, 1, sizeof (svn_prop_t));
   file_baton->path = path;
+  file_baton->schedule_delete = FALSE;
 
   /* If the parent directory is added rather than replaced it does not
      exist in the working copy.  Determine a working copy path whose
@@ -473,6 +483,11 @@ file_diff (struct dir_baton *dir_baton,
   if (copied)
     schedule = svn_wc_schedule_normal;
 
+  /* If this was scheduled replace and we are ignoring ancestry,
+     report it as a normal file modification. */
+  if (eb->ignore_ancestry && (schedule == svn_wc_schedule_replace))
+    schedule = svn_wc_schedule_normal;
+
   /* Prep these two paths early. */
   pristine_copy = svn_wc__text_base_path (path, FALSE, pool);
   empty_file = svn_wc__empty_file_path (path, pool);
@@ -487,13 +502,13 @@ file_diff (struct dir_baton *dir_baton,
       /* Delete compares text-base against empty file, modifications to the
          working-copy version of the deleted file are not wanted. */
 
-      /* Get svn:mimetype from pristine props of PATH. */
+      /* Get svn:mime-type from pristine props of PATH. */
       SVN_ERR (get_local_mimetypes (&pristine_mimetype, NULL, NULL,
                                     adm_access, path, pool));
 
       if (! eb->use_text_base)
         SVN_ERR (dir_baton->edit_baton->callbacks->file_deleted
-                 (NULL, path, 
+                 (NULL, NULL, path, 
                   pristine_copy, 
                   empty_file,
                   pristine_mimetype,
@@ -507,15 +522,16 @@ file_diff (struct dir_baton *dir_baton,
     case svn_wc_schedule_add:
       empty_file = svn_wc__empty_file_path (path, pool);
 
-      /* Get svn:mimetype from working props of PATH. */
+      /* Get svn:mime-type from working props of PATH. */
       SVN_ERR (get_local_mimetypes (NULL, &working_mimetype, NULL,
                                     adm_access, path, pool));
 
       if (! eb->use_text_base)
         SVN_ERR (dir_baton->edit_baton->callbacks->file_added
-                 (NULL, path,
+                 (NULL, NULL, path,
                   empty_file,
                   path,
+                  0, entry->revision,
                   NULL,
                   working_mimetype,
                   dir_baton->edit_baton->callback_baton));
@@ -545,7 +561,7 @@ file_diff (struct dir_baton *dir_baton,
       if (modified && (! eb->use_text_base))
         {
           const char *translated;
-          svn_error_t *err;
+          svn_error_t *err, *err2 = SVN_NO_ERROR;
 
           pristine_copy = svn_wc__text_base_path (path, FALSE, pool);   
 
@@ -557,7 +573,7 @@ file_diff (struct dir_baton *dir_baton,
           SVN_ERR (svn_wc_translated_file (&translated, path, adm_access,
                                            TRUE, pool));
           
-          /* Get svn:mimetype for both pristine and working file. */
+          /* Get svn:mime-type for both pristine and working file. */
           SVN_ERR (get_local_mimetypes (&pristine_mimetype, &working_mimetype,
                                         NULL, adm_access, path, pool));
 
@@ -573,10 +589,17 @@ file_diff (struct dir_baton *dir_baton,
              dir_baton->edit_baton->callback_baton);
           
           if (translated != path)
-            SVN_ERR (svn_io_remove_file (translated, pool));
+            err2 = svn_io_remove_file (translated, pool);
 
+          if (err && err2)
+            {
+              svn_error_compose (err, err2);
+              return err;
+            }
           if (err)
             return err;
+          if (err2)
+            return err2;
         }
 
       SVN_ERR (svn_wc_props_modified_p (&modified, path, adm_access, pool));
@@ -629,7 +652,7 @@ directory_elements_diff (struct dir_baton *dir_baton,
      directory and if this is that directory the non-target entries must be
      skipped. */
   in_anchor_not_target =
-    (dir_baton->edit_baton->target
+    (*dir_baton->edit_baton->target
      && (! svn_path_compare_paths
          (dir_baton->path,
           svn_wc_adm_access_path (dir_baton->edit_baton->anchor))));
@@ -804,7 +827,7 @@ delete_entry (const char *path,
                                                          FALSE, pool);
 
           SVN_ERR (pb->edit_baton->callbacks->file_deleted
-                   (NULL, full_path,
+                   (NULL, NULL, full_path,
                     textbase,
                     svn_wc__empty_file_path (full_path, pool),
                     pristine_mimetype,
@@ -815,9 +838,10 @@ delete_entry (const char *path,
         {
           /* Or normally, show the working file being added. */
           SVN_ERR (pb->edit_baton->callbacks->file_added
-                   (NULL, full_path,
+                   (NULL, NULL, full_path,
                     svn_wc__empty_file_path (full_path, pool),
                     full_path,
+                    0, entry->revision,
                     NULL,
                     working_mimetype,
                     pb->edit_baton->callback_baton));
@@ -1052,6 +1076,22 @@ apply_textdelta (void *file_baton,
                  void **handler_baton)
 {
   struct file_baton *b = file_baton;
+  struct edit_baton *eb = b->edit_baton;
+  const svn_wc_entry_t *entry;
+
+  SVN_ERR (svn_wc_entry (&entry, b->wc_path, eb->anchor, FALSE, b->pool));
+  
+  /* Check to see if there is a schedule-add with history entry in
+     the current working copy.  If so, then this is not actually
+     an add, but instead a modification.*/
+  if (entry && entry->copyfrom_url)
+    b->added = FALSE;
+
+  /* Check to see if this entry is scheduled to be deleted, if so,
+     then we need to remember so that we don't attempt to open the
+     non-existent file when doing the final diff. */
+  if (entry && entry->schedule == svn_wc_schedule_delete)
+    b->schedule_delete = TRUE;
 
   if (b->added)
     {
@@ -1135,9 +1175,11 @@ close_file (void *file_baton,
          show an add. */
       if (eb->reverse_order)
         SVN_ERR (b->edit_baton->callbacks->file_added
-                 (NULL, b->path,
+                 (NULL, NULL, b->path,
                   svn_wc__empty_file_path (b->wc_path, b->pool),
                   temp_file_path,
+                  0, 
+                  entry ? entry->revision : SVN_INVALID_REVNUM,
                   NULL,
                   pristine_mimetype,
                   b->edit_baton->callback_baton));
@@ -1146,7 +1188,7 @@ close_file (void *file_baton,
          diff should show this as a delete. Thus compare the repository
          file against the empty file. */
         SVN_ERR (b->edit_baton->callbacks->file_deleted
-                 (NULL, b->path,
+                 (NULL, NULL, b->path,
                   temp_file_path,
                   svn_wc__empty_file_path (b->wc_path, b->pool),
                   pristine_mimetype,
@@ -1164,6 +1206,8 @@ close_file (void *file_baton,
 
           if (eb->use_text_base)
             localfile = svn_wc__text_base_path (b->path, FALSE, b->pool);
+          else if (b->schedule_delete)
+            localfile = svn_wc__empty_file_path (b->wc_path, b->pool);
           else
             /* a detranslated version of the working file */
             SVN_ERR (svn_wc_translated_file (&localfile, b->path, adm_access,
@@ -1180,7 +1224,7 @@ close_file (void *file_baton,
              eb->reverse_order ? pristine_mimetype: working_mimetype,
              b->edit_baton->callback_baton);
       
-          if (! eb->use_text_base)
+          if ((! eb->use_text_base) && (! b->schedule_delete))
             if (localfile != b->path)
               err2 = svn_io_remove_file (localfile, b->pool);
 
@@ -1289,24 +1333,26 @@ close_edit (void *edit_baton,
 
 /* Create a diff editor and baton. */
 svn_error_t *
-svn_wc_get_diff_editor (svn_wc_adm_access_t *anchor,
-                        const char *target,
-                        const svn_wc_diff_callbacks_t *callbacks,
-                        void *callback_baton,
-                        svn_boolean_t recurse,
-                        svn_boolean_t use_text_base,
-                        svn_boolean_t reverse_order,
-                        svn_cancel_func_t cancel_func,
-                        void *cancel_baton,
-                        const svn_delta_editor_t **editor,
-                        void **edit_baton,
-                        apr_pool_t *pool)
+svn_wc_get_diff_editor2 (svn_wc_adm_access_t *anchor,
+                         const char *target,
+                         const svn_wc_diff_callbacks_t *callbacks,
+                         void *callback_baton,
+                         svn_boolean_t recurse,
+                         svn_boolean_t ignore_ancestry,
+                         svn_boolean_t use_text_base,
+                         svn_boolean_t reverse_order,
+                         svn_cancel_func_t cancel_func,
+                         void *cancel_baton,
+                         const svn_delta_editor_t **editor,
+                         void **edit_baton,
+                         apr_pool_t *pool)
 {
   struct edit_baton *eb;
   svn_delta_editor_t *tree_editor;
 
   eb = make_editor_baton (anchor, target, callbacks, callback_baton,
-                          recurse, use_text_base, reverse_order, pool);
+                          recurse, ignore_ancestry, use_text_base,
+                          reverse_order, pool);
   tree_editor = svn_delta_default_editor (eb->pool);
 
   tree_editor->set_target_revision = set_target_revision;
@@ -1334,14 +1380,36 @@ svn_wc_get_diff_editor (svn_wc_adm_access_t *anchor,
   return SVN_NO_ERROR;
 }
 
+svn_error_t *
+svn_wc_get_diff_editor (svn_wc_adm_access_t *anchor,
+                        const char *target,
+                        const svn_wc_diff_callbacks_t *callbacks,
+                        void *callback_baton,
+                        svn_boolean_t recurse,
+                        svn_boolean_t use_text_base,
+                        svn_boolean_t reverse_order,
+                        svn_cancel_func_t cancel_func,
+                        void *cancel_baton,
+                        const svn_delta_editor_t **editor,
+                        void **edit_baton,
+                        apr_pool_t *pool)
+{
+  return svn_wc_get_diff_editor2 (anchor, target, callbacks, callback_baton,
+                                  recurse, FALSE, use_text_base, reverse_order,
+                                  cancel_func, cancel_baton,
+                                  editor, edit_baton, pool);
+}
+
+
 /* Compare working copy against the text-base. */
 svn_error_t *
-svn_wc_diff (svn_wc_adm_access_t *anchor,
-             const char *target,
-             const svn_wc_diff_callbacks_t *callbacks,
-             void *callback_baton,
-             svn_boolean_t recurse,
-             apr_pool_t *pool)
+svn_wc_diff2 (svn_wc_adm_access_t *anchor,
+              const char *target,
+              const svn_wc_diff_callbacks_t *callbacks,
+              void *callback_baton,
+              svn_boolean_t recurse,
+              svn_boolean_t ignore_ancestry,
+              apr_pool_t *pool)
 {
   struct edit_baton *eb;
   struct dir_baton *b;
@@ -1350,13 +1418,10 @@ svn_wc_diff (svn_wc_adm_access_t *anchor,
   svn_wc_adm_access_t *adm_access;
 
   eb = make_editor_baton (anchor, target, callbacks, callback_baton,
-                          recurse, FALSE, FALSE, pool);
+                          recurse, ignore_ancestry, FALSE, FALSE, pool);
 
-  if (target)
-    target_path = svn_path_join (svn_wc_adm_access_path (anchor), target,
-                                 eb->pool);
-  else
-    target_path = apr_pstrdup (eb->pool, svn_wc_adm_access_path (anchor));
+  target_path = svn_path_join (svn_wc_adm_access_path (anchor), target,
+                               eb->pool);
 
   SVN_ERR (svn_wc_adm_probe_retrieve (&adm_access, anchor, target_path,
                                       eb->pool));
@@ -1371,3 +1436,16 @@ svn_wc_diff (svn_wc_adm_access_t *anchor,
 
   return SVN_NO_ERROR;
 }
+
+
+svn_error_t *
+svn_wc_diff (svn_wc_adm_access_t *anchor,
+             const char *target,
+             const svn_wc_diff_callbacks_t *callbacks,
+             void *callback_baton,
+             svn_boolean_t recurse,
+             apr_pool_t *pool)
+{
+  return svn_wc_diff2 (anchor, target, callbacks, callback_baton,
+                       recurse, FALSE, pool);
+}
