@@ -2,7 +2,7 @@
  * entries.c :  manipulating the administrative `entries' file.
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -17,7 +17,6 @@
  */
 
 
-
 #include <string.h>
 #include <assert.h>
 
@@ -35,6 +34,8 @@
 #include "adm_ops.h"
 #include "entries.h"
 
+#include "svn_private_config.h"
+
 
 /** Overview **/
 
@@ -48,27 +49,20 @@
 
 /*--------------------------------------------------------------- */
 
-/*** XML Attribute names and values ***/
-
-/* Attribute values for 'schedule' */
-#define SVN_WC__ENTRY_VALUE_ADD        "add"
-#define SVN_WC__ENTRY_VALUE_DELETE     "delete"
-#define SVN_WC__ENTRY_VALUE_REPLACE    "replace"
-
-
 
 /*** Initialization of the entries file. ***/
 
 svn_error_t *
 svn_wc__entries_init (const char *path,
+                      const char *uuid,
                       const char *url,
                       svn_revnum_t initial_rev,
                       apr_pool_t *pool)
 {
-  apr_status_t apr_err;
   apr_file_t *f = NULL;
   svn_stringbuf_t *accum = NULL;
-  char *initial_revstr =  apr_psprintf (pool, "%" SVN_REVNUM_T_FMT,
+  apr_hash_t *atts = apr_hash_make (pool);
+  char *initial_revstr =  apr_psprintf (pool, "%ld",
                                         initial_rev);
 
   /* Create the entries file, which must not exist prior to this. */
@@ -87,45 +81,42 @@ svn_wc__entries_init (const char *path,
                          SVN_XML_NAMESPACE,
                          NULL);
 
-  /* Add an entry for the dir itself -- name is absent, only the
-     revision and default ancestry are present as xml attributes, and
-     possibly an 'incomplete' flag if  the revnum is > 0. */
-  if ((initial_rev == 0) || (! SVN_IS_VALID_REVNUM(initial_rev)))
-    svn_xml_make_open_tag 
-      (&accum,
-       pool,
-       svn_xml_self_closing,
-       SVN_WC__ENTRIES_ENTRY,
-       SVN_WC__ENTRY_ATTR_KIND, SVN_WC__ENTRIES_ATTR_DIR_STR,
-       SVN_WC__ENTRY_ATTR_REVISION, initial_revstr,
-       SVN_WC__ENTRY_ATTR_URL, url,
-       NULL);    
-  else
-    svn_xml_make_open_tag 
-      (&accum,
-       pool,
-       svn_xml_self_closing,
-       SVN_WC__ENTRIES_ENTRY,
-       SVN_WC__ENTRY_ATTR_KIND, SVN_WC__ENTRIES_ATTR_DIR_STR,
-       SVN_WC__ENTRY_ATTR_REVISION, initial_revstr,
-       SVN_WC__ENTRY_ATTR_URL, url,
-       SVN_WC__ENTRY_ATTR_INCOMPLETE, "true",
-       NULL);
+  /* Add an entry for the dir itself.  The directory has no name.  It
+     might have a UUID, but otherwise only the revision and default
+     ancestry are present as XML attributes, and possibly an
+     'incomplete' flag if the revnum is > 0. */
+
+  apr_hash_set (atts, SVN_WC__ENTRY_ATTR_KIND, 
+                sizeof (SVN_WC__ENTRY_ATTR_KIND) - 1,
+                SVN_WC__ENTRIES_ATTR_DIR_STR);
+  apr_hash_set (atts, SVN_WC__ENTRY_ATTR_URL,
+                sizeof (SVN_WC__ENTRY_ATTR_URL) - 1,
+                url);
+  apr_hash_set (atts, SVN_WC__ENTRY_ATTR_REVISION,
+                sizeof (SVN_WC__ENTRY_ATTR_REVISION) - 1,
+                initial_revstr);
+
+  if (uuid)
+    apr_hash_set (atts, SVN_WC__ENTRY_ATTR_UUID,
+                  sizeof (SVN_WC__ENTRY_ATTR_UUID) - 1,
+                  uuid);
+    
+  if (initial_rev > 0)
+    apr_hash_set (atts, SVN_WC__ENTRY_ATTR_INCOMPLETE,
+                  sizeof (SVN_WC__ENTRY_ATTR_INCOMPLETE) - 1,
+                  "true");
+  
+  svn_xml_make_open_tag_hash (&accum, pool, svn_xml_self_closing,
+                              SVN_WC__ENTRIES_ENTRY, atts);
 
   /* Close the top-level form. */
   svn_xml_make_close_tag (&accum,
                           pool,
                           SVN_WC__ENTRIES_TOPLEVEL);
 
-  apr_err = apr_file_write_full (f, accum->data, accum->len, NULL);
-  if (apr_err)
-    {
-      apr_file_close (f);
-      return svn_error_createf (apr_err, NULL,
-                                "svn_wc__entries_init: "
-                                "error writing entries file for '%s'.",
-                                path);
-    }
+  SVN_ERR_W (svn_io_file_write_full (f, accum->data, accum->len, NULL, pool),
+             apr_psprintf (pool, 
+                           _("Error writing entries file for '%s'"), path));
 
   /* Now we have a `entries' file with exactly one entry, an entry
      for this dir.  Close the file and sync it up. */
@@ -148,7 +139,7 @@ struct entries_accumulator
   svn_xml_parser_t *parser;
 
   /* Should we include 'deleted' entries in the hash? */
-  svn_boolean_t show_deleted;
+  svn_boolean_t show_hidden;
 
   /* Don't leave home without one. */
   apr_pool_t *pool;
@@ -180,6 +171,12 @@ svn_wc__atts_to_entry (svn_wc_entry_t **new_entry,
 
   /* Find the name and set up the entry under that name. */
   name = apr_hash_get (atts, SVN_WC__ENTRY_ATTR_NAME, APR_HASH_KEY_STRING);
+  /* XXX Replace the obsolete "svn:this_dir".
+     XXX This code should go away by 1.0 */
+  {
+    if (name && strcmp (name, "svn:this_dir") == 0)
+      name = SVN_WC_ENTRY_THIS_DIR;
+  }
   entry->name = name ? name : SVN_WC_ENTRY_THIS_DIR;
 
   /* Attempt to set revision (resolve_to_defaults may do it later, too) */
@@ -220,7 +217,7 @@ svn_wc__atts_to_entry (svn_wc_entry_t **new_entry,
         else
           return svn_error_createf 
             (SVN_ERR_NODE_UNKNOWN_KIND, NULL,
-             "Entry '%s' has invalid node kind",
+             _("Entry '%s' has invalid node kind"),
              (name ? name : SVN_WC_ENTRY_THIS_DIR));
         *modify_flags |= SVN_WC__ENTRY_MODIFY_KIND;
       }
@@ -245,7 +242,7 @@ svn_wc__atts_to_entry (svn_wc_entry_t **new_entry,
         else
           return svn_error_createf 
             (SVN_ERR_ENTRY_ATTRIBUTE_INVALID, NULL,
-             "Entry '%s' has invalid '%s' value",
+             _("Entry '%s' has invalid '%s' value"),
              (name ? name : SVN_WC_ENTRY_THIS_DIR),
              SVN_WC__ENTRY_ATTR_SCHEDULE);
 
@@ -295,7 +292,7 @@ svn_wc__atts_to_entry (svn_wc_entry_t **new_entry,
         else
           return svn_error_createf 
             (SVN_ERR_ENTRY_ATTRIBUTE_INVALID, NULL,
-             "Entry '%s' has invalid '%s' value",
+             _("Entry '%s' has invalid '%s' value"),
              (name ? name : SVN_WC_ENTRY_THIS_DIR),
              SVN_WC__ENTRY_ATTR_COPIED);
 
@@ -304,12 +301,16 @@ svn_wc__atts_to_entry (svn_wc_entry_t **new_entry,
 
     entry->copyfrom_url = apr_hash_get (atts, SVN_WC__ENTRY_ATTR_COPYFROM_URL,
                                         APR_HASH_KEY_STRING);
+    if (entry->copyfrom_url)
+      *modify_flags |= SVN_WC__ENTRY_MODIFY_COPYFROM_URL;
 
     revstr = apr_hash_get (atts, SVN_WC__ENTRY_ATTR_COPYFROM_REV, 
                            APR_HASH_KEY_STRING);
     if (revstr)
-      entry->copyfrom_rev = SVN_STR_TO_REV (revstr);
-    
+      {
+        entry->copyfrom_rev = SVN_STR_TO_REV (revstr);
+        *modify_flags |= SVN_WC__ENTRY_MODIFY_COPYFROM_REV;
+      }
   }
 
   /* Is this entry deleted? */
@@ -331,7 +332,7 @@ svn_wc__atts_to_entry (svn_wc_entry_t **new_entry,
         else
           return svn_error_createf 
             (SVN_ERR_ENTRY_ATTRIBUTE_INVALID, NULL,
-             "Entry '%s' has invalid '%s' value",
+             _("Entry '%s' has invalid '%s' value"),
              (name ? name : SVN_WC_ENTRY_THIS_DIR),
              SVN_WC__ENTRY_ATTR_DELETED);
 
@@ -339,6 +340,33 @@ svn_wc__atts_to_entry (svn_wc_entry_t **new_entry,
       }
   }
 
+  /* Is this entry absent? */
+  {
+    const char *absentstr;
+      
+    absentstr = apr_hash_get (atts, SVN_WC__ENTRY_ATTR_ABSENT, 
+                               APR_HASH_KEY_STRING);
+        
+    entry->absent = FALSE;
+    if (absentstr)
+      {
+        if (! strcmp (absentstr, "true"))
+          entry->absent = TRUE;
+        else if (! strcmp (absentstr, "false"))
+          entry->absent = FALSE;
+        else if (! strcmp (absentstr, ""))
+          entry->absent = FALSE;
+        else
+          return svn_error_createf 
+            (SVN_ERR_ENTRY_ATTRIBUTE_INVALID, NULL,
+             _("Entry '%s' has invalid '%s' value"),
+             (name ? name : SVN_WC_ENTRY_THIS_DIR),
+             SVN_WC__ENTRY_ATTR_ABSENT);
+
+        *modify_flags |= SVN_WC__ENTRY_MODIFY_ABSENT;
+      }
+  }
+
   /* Is this entry incomplete? */
   {
     const char *incompletestr;
@@ -358,7 +386,7 @@ svn_wc__atts_to_entry (svn_wc_entry_t **new_entry,
         else
           return svn_error_createf 
             (SVN_ERR_ENTRY_ATTRIBUTE_INVALID, NULL,
-             "Entry '%s' has invalid '%s' value",
+             _("Entry '%s' has invalid '%s' value"),
              (name ? name : SVN_WC_ENTRY_THIS_DIR),
              SVN_WC__ENTRY_ATTR_INCOMPLETE);
 
@@ -487,10 +515,13 @@ handle_start_tag (void *userData, const char *tagname, const char **atts)
         
   /* Find the name and set up the entry under that name.  This
      should *NOT* be NULL, since svn_wc__atts_to_entry() should
-     have made it into SVN_WC_ENTRY_THIS_DIR.  */
-  if (entry->deleted
+     have made it into SVN_WC_ENTRY_THIS_DIR.  (Note that technically,
+     an entry can't be both absent and scheduled for addition, but we
+     don't need a sanity check for that here.) */
+  if ((entry->deleted || entry->absent)
       && (entry->schedule != svn_wc_schedule_add)
-      && (! accum->show_deleted))
+      && (entry->schedule != svn_wc_schedule_replace)
+      && (! accum->show_hidden))
     ;
   else
     apr_hash_set (accum->entries, entry->name, APR_HASH_KEY_STRING, entry);
@@ -536,17 +567,17 @@ resolve_to_defaults (apr_hash_t *entries,
   if (! default_entry)
     return svn_error_create (SVN_ERR_ENTRY_NOT_FOUND,
                              NULL,
-                             "missing default entry");
+                             _("Missing default entry"));
 
   if (default_entry->revision == SVN_INVALID_REVNUM)
     return svn_error_create (SVN_ERR_ENTRY_MISSING_REVISION,
                              NULL,
-                             "default entry has no revision number");
+                             _("Default entry has no revision number"));
 
   if (! default_entry->url)
     return svn_error_create (SVN_ERR_ENTRY_MISSING_URL,
                              NULL,
-                             "default entry missing url");
+                             _("Default entry is missing URL"));
   
     
   /* Then use it to fill in missing information in other entries. */
@@ -584,19 +615,19 @@ resolve_to_defaults (apr_hash_t *entries,
 
 
 
-/* Fill the entries cache in ADM_ACCESS. Either the full hash cache will
-   populated, if SHOW_DELETED is TRUE, or the truncated hash cache will be
-   populated if SHOW_DELETED is FALSE.  POOL is used for local memory
+/* Fill the entries cache in ADM_ACCESS. Either the full hash cache will be
+   populated, if SHOW_HIDDEN is TRUE, or the truncated hash cache will be
+   populated if SHOW_HIDDEN is FALSE.  POOL is used for local memory
    allocation, the access baton pool is used for the cache. */
 static svn_error_t *
 read_entries (svn_wc_adm_access_t *adm_access,
-              svn_boolean_t show_deleted,
+              svn_boolean_t show_hidden,
               apr_pool_t *pool)
 {
   svn_error_t *err;
+  svn_boolean_t is_final;
   apr_file_t *infile = NULL;
   svn_xml_parser_t *svn_parser;
-  apr_status_t apr_err;
   char buf[BUFSIZ];
   apr_size_t bytes_read;
   struct entries_accumulator accum;
@@ -609,7 +640,7 @@ read_entries (svn_wc_adm_access_t *adm_access,
 
   /* Set up userData for the XML parser. */
   accum.entries = entries;
-  accum.show_deleted = show_deleted;
+  accum.show_hidden = show_hidden;
   accum.pool = svn_wc_adm_access_pool (adm_access);
 
   /* Create the XML parser */
@@ -625,30 +656,30 @@ read_entries (svn_wc_adm_access_t *adm_access,
 
   /* Parse. */
   do {
-    apr_err = apr_file_read_full (infile, buf, sizeof(buf), &bytes_read);
-    if (apr_err && !APR_STATUS_IS_EOF(apr_err))
-      return svn_error_create 
-        (apr_err, NULL, "read_entries: apr_file_read_full choked");
+    err = svn_io_file_read_full (infile, buf, sizeof(buf), &bytes_read, pool);
+    if (err && !APR_STATUS_IS_EOF(err->apr_err))
+      return err;
+
+    is_final = (err != NULL); /* EOF is only possible error */
+    svn_error_clear (err);
     
-    err = svn_xml_parse (svn_parser, buf, bytes_read,
-                         APR_STATUS_IS_EOF(apr_err));
-    if (err)
-      return svn_error_createf (err->apr_err, err, 
-                                "read_entries: xml parser failed (%s).", 
-                                svn_wc_adm_access_path (adm_access));
-  } while (!APR_STATUS_IS_EOF(apr_err));
+    SVN_ERR_W (svn_xml_parse (svn_parser, buf, bytes_read, is_final),
+               apr_psprintf (pool,
+                             _("XML parser failed in '%s'"),
+                             svn_wc_adm_access_path (adm_access)));
+  } while (! is_final);
 
   /* Close the entries file. */
   SVN_ERR (svn_wc__close_adm_file (infile, svn_wc_adm_access_path (adm_access),
                                    SVN_WC__ADM_ENTRIES, 0, pool));
 
-  /* Clean up the xml parser */
+  /* Clean up the XML parser */
   svn_xml_free_parser (svn_parser);
 
   /* Fill in any implied fields. */
   SVN_ERR (resolve_to_defaults (entries, svn_wc_adm_access_pool (adm_access)));
 
-  svn_wc__adm_access_set_entries (adm_access, show_deleted, entries);
+  svn_wc__adm_access_set_entries (adm_access, show_hidden, entries);
 
   return SVN_NO_ERROR;
 }
@@ -671,29 +702,19 @@ svn_error_t *
 svn_wc_entry (const svn_wc_entry_t **entry,
               const char *path,
               svn_wc_adm_access_t *adm_access,
-              svn_boolean_t show_deleted,
+              svn_boolean_t show_hidden,
               apr_pool_t *pool)
 {
   const char *entry_name;
   svn_wc_adm_access_t *dir_access;
 
-  svn_error_t *err = svn_wc_adm_retrieve (&dir_access, adm_access, path, pool);
-  if (err && err->apr_err != SVN_ERR_WC_NOT_LOCKED)
-    return err;
-
-  if (err)
+  SVN_ERR (svn_wc__adm_retrieve_internal (&dir_access, adm_access, path, pool));
+  if (! dir_access)
     {
       const char *dir_path, *base_name;
       svn_path_split (path, &dir_path, &base_name, pool);
-      svn_error_clear (err);
-      err = svn_wc_adm_retrieve (&dir_access, adm_access, dir_path, pool);
-      if (err)
-        {
-          if (err->apr_err != SVN_ERR_WC_NOT_LOCKED)
-            return err;
-          dir_access = NULL;
-          svn_error_clear (err);
-        }
+      SVN_ERR (svn_wc__adm_retrieve_internal (&dir_access, adm_access, dir_path,
+                                              pool));
       entry_name = base_name;
     }
   else
@@ -702,7 +723,7 @@ svn_wc_entry (const svn_wc_entry_t **entry,
   if (dir_access)
     {
       apr_hash_t *entries;
-      SVN_ERR (svn_wc_entries_read (&entries, dir_access, show_deleted, pool));
+      SVN_ERR (svn_wc_entries_read (&entries, dir_access, show_hidden, pool));
       *entry = apr_hash_get (entries, entry_name, APR_HASH_KEY_STRING);
     }
   else
@@ -731,8 +752,7 @@ check_entries (apr_hash_t *entries,
   if (! default_entry)
     return svn_error_createf
       (SVN_ERR_WC_CORRUPT, NULL,
-       "'%s' has no default entry",
-       path);
+       _("Corrupt working copy: '%s' has no default entry"), path);
 
   /* Validate DEFAULT_ENTRY's current schedule. */
   switch (default_entry->schedule)
@@ -748,7 +768,7 @@ check_entries (apr_hash_t *entries,
       /* This is an invalid state */
       return svn_error_createf
         (SVN_ERR_WC_CORRUPT, NULL,
-         "Directory '%s' has an invalid schedule",
+         _("Corrupt working copy: directory '%s' has an invalid schedule"),
          path);
     }
   
@@ -782,7 +802,8 @@ check_entries (apr_hash_t *entries,
           /* This is an invalid state */
           return svn_error_createf
             (SVN_ERR_WC_CORRUPT, NULL,
-             "'%s' in directory '%s' has an invalid schedule",
+             _("Corrupt working copy: "
+               "'%s' in directory '%s' has an invalid schedule"),
              name, path);
         }
 
@@ -790,24 +811,24 @@ check_entries (apr_hash_t *entries,
           && (this_entry->schedule != svn_wc_schedule_add))
         return svn_error_createf
           (SVN_ERR_WC_CORRUPT, NULL,
-           "'%s' in directory '%s' (which is scheduled for addition) "
-           "is not itself scheduled for addition",
+           _("Corrupt working copy: '%s' in directory '%s' (which is "
+             "scheduled for addition) is not itself scheduled for addition"),
            name, path);
   
       if ((default_entry->schedule == svn_wc_schedule_delete)
           && (this_entry->schedule != svn_wc_schedule_delete))
         return svn_error_createf
           (SVN_ERR_WC_CORRUPT, NULL,
-           "'%s' in directory '%s' (which is scheduled for deletion) "
-           "is not itself scheduled for deletion",
+           _("Corrupt working copy: '%s' in directory '%s' (which is "
+             "scheduled for deletion) is not itself scheduled for deletion"),
            name, path);
 
       if ((default_entry->schedule == svn_wc_schedule_replace)
           && (this_entry->schedule == svn_wc_schedule_normal))
         return svn_error_createf
           (SVN_ERR_WC_CORRUPT, NULL,
-           "'%s' in directory '%s' (which is scheduled for replacement) "
-           "has in invalid schedule",
+           _("Corrupt working copy: '%s' in directory '%s' (which is "
+             "scheduled for replacement) has an invalid schedule"),
            name, path);
     }
   
@@ -819,16 +840,16 @@ check_entries (apr_hash_t *entries,
 svn_error_t *
 svn_wc_entries_read (apr_hash_t **entries,
                      svn_wc_adm_access_t *adm_access,
-                     svn_boolean_t show_deleted,
+                     svn_boolean_t show_hidden,
                      apr_pool_t *pool)
 {
   apr_hash_t *new_entries;
 
-  new_entries = svn_wc__adm_access_entries (adm_access, show_deleted, pool);
+  new_entries = svn_wc__adm_access_entries (adm_access, show_hidden, pool);
   if (! new_entries)
     {
-      SVN_ERR (read_entries (adm_access, show_deleted, pool));
-      new_entries = svn_wc__adm_access_entries (adm_access, show_deleted, pool);
+      SVN_ERR (read_entries (adm_access, show_hidden, pool));
+      new_entries = svn_wc__adm_access_entries (adm_access, show_hidden, pool);
     }
 
   *entries = new_entries;
@@ -860,7 +881,7 @@ write_entry (svn_stringbuf_t **output,
   /* Revision */
   if (SVN_IS_VALID_REVNUM (entry->revision))
     apr_hash_set (atts, SVN_WC__ENTRY_ATTR_REVISION, APR_HASH_KEY_STRING,
-                  apr_psprintf (pool, "%" SVN_REVNUM_T_FMT, entry->revision));
+                  apr_psprintf (pool, "%ld", entry->revision));
   
   /* URL */
   if (entry->url)
@@ -932,7 +953,7 @@ write_entry (svn_stringbuf_t **output,
 
   if (SVN_IS_VALID_REVNUM (entry->copyfrom_rev))
     apr_hash_set (atts, SVN_WC__ENTRY_ATTR_COPYFROM_REV, APR_HASH_KEY_STRING,
-                  apr_psprintf (pool, "%" SVN_REVNUM_T_FMT,
+                  apr_psprintf (pool, "%ld",
                                 entry->copyfrom_rev));
 
   if (entry->copyfrom_url)
@@ -943,6 +964,10 @@ write_entry (svn_stringbuf_t **output,
   apr_hash_set (atts, SVN_WC__ENTRY_ATTR_DELETED, APR_HASH_KEY_STRING,
                 (entry->deleted ? "true" : NULL));
 
+  /* Absent state */
+  apr_hash_set (atts, SVN_WC__ENTRY_ATTR_ABSENT, APR_HASH_KEY_STRING,
+                (entry->absent ? "true" : NULL));
+
   /* Incomplete state */
   apr_hash_set (atts, SVN_WC__ENTRY_ATTR_INCOMPLETE, APR_HASH_KEY_STRING,
                 (entry->incomplete ? "true" : NULL));
@@ -967,7 +992,7 @@ write_entry (svn_stringbuf_t **output,
   /* Last-commit Stuff */
   if (SVN_IS_VALID_REVNUM (entry->cmt_rev))
     apr_hash_set (atts, SVN_WC__ENTRY_ATTR_CMT_REV, APR_HASH_KEY_STRING,
-                  apr_psprintf (pool, "%" SVN_REVNUM_T_FMT, entry->cmt_rev));
+                  apr_psprintf (pool, "%ld", entry->cmt_rev));
 
   if (entry->cmt_author)
     apr_hash_set (atts, SVN_WC__ENTRY_ATTR_CMT_AUTHOR, APR_HASH_KEY_STRING,
@@ -1068,11 +1093,11 @@ svn_wc__entries_write (apr_hash_t *entries,
   svn_error_t *err = SVN_NO_ERROR;
   svn_stringbuf_t *bigstr = NULL;
   apr_file_t *outfile = NULL;
-  apr_status_t apr_err;
   apr_hash_index_t *hi;
   svn_wc_entry_t *this_dir;
+  apr_pool_t *subpool = svn_pool_create (pool);
 
-  SVN_ERR (svn_wc_adm_write_check (adm_access));
+  SVN_ERR (svn_wc__adm_write_check (adm_access));
 
   /* Get a copy of the "this dir" entry for comparison purposes. */
   this_dir = apr_hash_get (entries, SVN_WC_ENTRY_THIS_DIR, 
@@ -1081,7 +1106,7 @@ svn_wc__entries_write (apr_hash_t *entries,
   /* If there is no "this dir" entry, something is wrong. */
   if (! this_dir)
     return svn_error_createf (SVN_ERR_ENTRY_NOT_FOUND, NULL,
-                              "No default entry in directory `%s'", 
+                              _("No default entry in directory '%s'"),
                               svn_wc_adm_access_path (adm_access));
 
   /* Open entries file for writing.  It's important we don't use APR_EXCL
@@ -1092,7 +1117,8 @@ svn_wc__entries_write (apr_hash_t *entries,
    * tags such as SVN_WC__LOG_MV to move entries files so any existing file
    * is not "valuable".
    */
-  SVN_ERR (svn_wc__open_adm_file (&outfile, svn_wc_adm_access_path (adm_access),
+  SVN_ERR (svn_wc__open_adm_file (&outfile, 
+                                  svn_wc_adm_access_path (adm_access),
                                   SVN_WC__ADM_ENTRIES,
                                   (APR_WRITE | APR_CREATE),
                                   pool));
@@ -1123,20 +1149,22 @@ svn_wc__entries_write (apr_hash_t *entries,
         continue;
 
       /* Append the entry to BIGSTR */
-      write_entry (&bigstr, this_entry, key, this_dir, pool);
+      write_entry (&bigstr, this_entry, key, this_dir, subpool);
+      svn_pool_clear (subpool);
     }
 
+  svn_pool_destroy (subpool);
   svn_xml_make_close_tag (&bigstr, pool, SVN_WC__ENTRIES_TOPLEVEL);
 
-  apr_err = apr_file_write_full (outfile, bigstr->data, bigstr->len, NULL);
-  if (apr_err)
-    err = svn_error_createf (apr_err, NULL,
-                             "svn_wc__entries_write: %s",
-                             svn_wc_adm_access_path (adm_access));
-  else
-    err = svn_wc__close_adm_file (outfile,
-                                   svn_wc_adm_access_path (adm_access),
-                                   SVN_WC__ADM_ENTRIES, 1, pool);
+  SVN_ERR_W (svn_io_file_write_full (outfile, bigstr->data, 
+                                     bigstr->len, NULL, pool),
+             apr_psprintf (pool,
+                           _("Error writing to '%s'"),
+                           svn_wc_adm_access_path (adm_access)));
+
+  err = svn_wc__close_adm_file (outfile,
+                                svn_wc_adm_access_path (adm_access),
+                                SVN_WC__ADM_ENTRIES, 1, pool);
 
   svn_wc__adm_access_set_entries (adm_access, TRUE, entries);
   svn_wc__adm_access_set_entries (adm_access, FALSE, NULL);
@@ -1211,6 +1239,10 @@ fold_entry (apr_hash_t *entries,
   if (modify_flags & SVN_WC__ENTRY_MODIFY_DELETED)
     cur_entry->deleted = entry->deleted;
 
+  /* Absent state */
+  if (modify_flags & SVN_WC__ENTRY_MODIFY_ABSENT)
+    cur_entry->absent = entry->absent;
+
   /* Incomplete state */
   if (modify_flags & SVN_WC__ENTRY_MODIFY_INCOMPLETE)
     cur_entry->incomplete = entry->incomplete;
@@ -1326,10 +1358,7 @@ fold_scheduling (apr_hash_t *entries,
           return SVN_NO_ERROR;
 
         default:
-          return 
-            svn_error_createf 
-            (SVN_ERR_WC_SCHEDULE_CONFLICT, NULL,
-             "fold_state_changes: Illegal schedule in state set operation");
+          return svn_error_create (SVN_ERR_WC_SCHEDULE_CONFLICT, NULL, NULL);
         }
     }
 
@@ -1341,10 +1370,9 @@ fold_scheduling (apr_hash_t *entries,
         return SVN_NO_ERROR;
       else
         return 
-          svn_error_createf 
-          (SVN_ERR_WC_SCHEDULE_CONFLICT, NULL,
-           "fold_state_changes: '%s' is not a versioned resource",
-           name);
+          svn_error_createf (SVN_ERR_WC_SCHEDULE_CONFLICT, NULL,
+                             _("'%s' is not under version control"),
+                             name);
     }
 
   /* Get the default entry */
@@ -1369,18 +1397,24 @@ fold_scheduling (apr_hash_t *entries,
     {
       if (*schedule == svn_wc_schedule_add)
         return 
-          svn_error_createf 
-          (SVN_ERR_WC_SCHEDULE_CONFLICT, NULL,
-           "fold_state_changes: Can't add '%s' to deleted directory"
-           "--try undeleting its parent directory first",
-           name);
+          svn_error_createf (SVN_ERR_WC_SCHEDULE_CONFLICT, NULL,
+                             _("Can't add '%s' to deleted directory; "
+                               "try undeleting its parent directory first"),
+                             name);
       if (*schedule == svn_wc_schedule_replace)
         return 
-          svn_error_createf 
-          (SVN_ERR_WC_SCHEDULE_CONFLICT, NULL,
-           "fold_state_changes: Can't replace '%s' in deleted directory"
-           "--try undeleting its parent directory first",
-           name);
+          svn_error_createf (SVN_ERR_WC_SCHEDULE_CONFLICT, NULL,
+                             _("Can't replace '%s' in deleted directory; "
+                               "try undeleting its parent directory first"),
+                             name);
+    }
+
+  if (entry->absent && (*schedule == svn_wc_schedule_add))
+    {
+      return svn_error_createf 
+        (SVN_ERR_WC_SCHEDULE_CONFLICT, NULL,
+         _("'%s' is marked as absent, so it cannot be scheduled for addition"),
+         name);
     }
 
   switch (entry->schedule)
@@ -1408,8 +1442,7 @@ fold_scheduling (apr_hash_t *entries,
             return 
               svn_error_createf 
               (SVN_ERR_WC_SCHEDULE_CONFLICT, NULL,
-               "fold_state_changes: Entry '%s' already under revision control",
-               name);
+               _("Entry '%s' is already under version control"), name);
         }
       break;
 
@@ -1501,8 +1534,7 @@ fold_scheduling (apr_hash_t *entries,
       return 
         svn_error_createf 
         (SVN_ERR_WC_SCHEDULE_CONFLICT, NULL,
-         "fold_state_changes: Entry '%s' has illegal schedule",
-         name);
+         _("Entry '%s' has illegal schedule"), name);
     }
   return SVN_NO_ERROR;
 }
@@ -1517,7 +1549,7 @@ svn_wc__entry_modify (svn_wc_adm_access_t *adm_access,
                       svn_boolean_t do_sync,
                       apr_pool_t *pool)
 {
-  apr_hash_t *entries;
+  apr_hash_t *entries, *entries_nohidden;
   svn_boolean_t entry_was_deleted_p = FALSE;
 
   /* ENTRY is rather necessary, and ENTRY->kind is required to be valid! */
@@ -1525,6 +1557,7 @@ svn_wc__entry_modify (svn_wc_adm_access_t *adm_access,
 
   /* Load PATH's whole entries file. */
   SVN_ERR (svn_wc_entries_read (&entries, adm_access, TRUE, pool));
+  SVN_ERR (svn_wc_entries_read (&entries_nohidden, adm_access, FALSE, pool));
 
   /* Ensure that NAME is valid. */
   if (name == NULL)
@@ -1533,6 +1566,8 @@ svn_wc__entry_modify (svn_wc_adm_access_t *adm_access,
   if (modify_flags & SVN_WC__ENTRY_MODIFY_SCHEDULE)
     {
       svn_wc_entry_t *entry_before, *entry_after;
+      apr_uint32_t orig_modify_flags = modify_flags;
+      svn_wc_schedule_t orig_schedule = entry->schedule;
 
       /* Keep a copy of the unmodified entry on hand. */
       entry_before = apr_hash_get (entries, name, APR_HASH_KEY_STRING);
@@ -1541,6 +1576,17 @@ svn_wc__entry_modify (svn_wc_adm_access_t *adm_access,
          manage those modifications. */
       SVN_ERR (fold_scheduling (entries, name, &modify_flags, 
                                 &entry->schedule, pool));
+      
+      if (entries != entries_nohidden)
+        {
+          SVN_ERR (fold_scheduling (entries_nohidden, name, &orig_modify_flags,
+                                    &orig_schedule, pool));
+          
+          /* Make certain that both folding operations had the same
+             result. */
+          assert(orig_modify_flags == modify_flags);
+          assert(orig_schedule == entry->schedule);
+        }
 
       /* Special case:  fold_state_changes() may have actually REMOVED
          the entry in question!  If so, don't try to fold_entry, as
@@ -1556,14 +1602,17 @@ svn_wc__entry_modify (svn_wc_adm_access_t *adm_access,
   /* If the entry wasn't just removed from the entries hash, fold the
      changes into the entry. */
   if (! entry_was_deleted_p)
-    fold_entry (entries, name, modify_flags, entry,
-                svn_wc_adm_access_pool (adm_access));
+    {
+      fold_entry (entries, name, modify_flags, entry,
+                  svn_wc_adm_access_pool (adm_access));
+      if (entries != entries_nohidden)
+        fold_entry (entries_nohidden, name, modify_flags, entry,
+                    svn_wc_adm_access_pool (adm_access));
+    }
 
   /* Sync changes to disk. */
   if (do_sync)
     SVN_ERR (svn_wc__entries_write (entries, adm_access, pool));
-  else
-    svn_wc__adm_access_set_entries (adm_access, FALSE, NULL);
 
   return SVN_NO_ERROR;
 }
@@ -1609,7 +1658,8 @@ svn_error_t *
 svn_wc__tweak_entry (apr_hash_t *entries,
                      const char *name,
                      const char *new_url,
-                     const svn_revnum_t new_rev,
+                     svn_revnum_t new_rev,
+                     svn_boolean_t *write_required,
                      apr_pool_t *pool)
 {
   svn_wc_entry_t *entry;
@@ -1617,24 +1667,41 @@ svn_wc__tweak_entry (apr_hash_t *entries,
   entry = apr_hash_get (entries, name, APR_HASH_KEY_STRING);
   if (! entry)
     return svn_error_createf (SVN_ERR_ENTRY_NOT_FOUND, NULL,
-                              "No such entry: '%s'", name);
+                              _("No such entry: '%s'"), name);
 
-  if (new_url != NULL)
-    entry->url = apr_pstrdup (pool, new_url);
+  if (new_url != NULL
+      && (! entry->url || strcmp (new_url, entry->url)))
+    {
+      *write_required = TRUE;
+      entry->url = apr_pstrdup (pool, new_url);
+    }
 
   if ((SVN_IS_VALID_REVNUM (new_rev))
       && (entry->schedule != svn_wc_schedule_add)
-      && (entry->schedule != svn_wc_schedule_replace))
-    entry->revision = new_rev;
+      && (entry->schedule != svn_wc_schedule_replace)
+      && (entry->revision != new_rev))
+    {
+      *write_required = TRUE;
+      entry->revision = new_rev;
+    }
 
   /* As long as this function is only called as a helper to
-     svn_wc__do_update_cleanup, then it's okay to totally remove any
-     'deleted' entry.  The rationale is: if the server didn't
-     overwrite the 'deleted' entry with something new during the
-     update, then it *must* have meant for the entry to be permanently
-     gone in the parent dir's revision. */
-  if (entry->deleted)
-    apr_hash_set (entries, name, APR_HASH_KEY_STRING, NULL);
+     svn_wc__do_update_cleanup, then it's okay to remove any entry
+     under certain circumstances:
+
+     If the entry is still marked 'deleted', then the server did not
+     re-add it.  So it's really gone in this revision, thus we remove
+     the entry.
+
+     If the entry is still marked 'absent' and yet is not the same
+     revision as new_rev, then the server did not re-add it, nor
+     re-absent it, so we can remove the entry. */
+  if ((entry->deleted)
+      || (entry->absent && (entry->revision != new_rev)))
+    {
+      *write_required = TRUE;
+      apr_hash_set (entries, name, APR_HASH_KEY_STRING, NULL);
+    }
 
   return SVN_NO_ERROR;
 }
@@ -1651,7 +1718,7 @@ walker_helper (const char *dirpath,
                svn_wc_adm_access_t *adm_access,
                const svn_wc_entry_callbacks_t *walk_callbacks,
                void *walk_baton,
-               svn_boolean_t show_deleted,
+               svn_boolean_t show_hidden,
                apr_pool_t *pool)
 {
   apr_pool_t *subpool = svn_pool_create (pool);
@@ -1659,20 +1726,20 @@ walker_helper (const char *dirpath,
   apr_hash_index_t *hi;
   svn_wc_entry_t *dot_entry;
 
-  SVN_ERR (svn_wc_entries_read (&entries, adm_access, show_deleted, subpool));
+  SVN_ERR (svn_wc_entries_read (&entries, adm_access, show_hidden, pool));
   
   /* As promised, always return the '.' entry first. */
   dot_entry = apr_hash_get (entries, SVN_WC_ENTRY_THIS_DIR, 
                             APR_HASH_KEY_STRING);
   if (! dot_entry)
     return svn_error_createf (SVN_ERR_ENTRY_NOT_FOUND, NULL,
-                              "Directory '%s' has no THIS_DIR entry!",
+                              _("Directory '%s' has no THIS_DIR entry"),
                               dirpath);
 
-  SVN_ERR (walk_callbacks->found_entry (dirpath, dot_entry, walk_baton));
+  SVN_ERR (walk_callbacks->found_entry (dirpath, dot_entry, walk_baton, pool));
 
   /* Loop over each of the other entries. */
-  for (hi = apr_hash_first (subpool, entries); hi; hi = apr_hash_next (hi))
+  for (hi = apr_hash_first (pool, entries); hi; hi = apr_hash_next (hi))
     {
       const void *key;
       apr_ssize_t klen;
@@ -1688,7 +1755,7 @@ walker_helper (const char *dirpath,
 
       entrypath = svn_path_join (dirpath, key, subpool);
       SVN_ERR (walk_callbacks->found_entry (entrypath, current_entry,
-                                            walk_baton));
+                                            walk_baton, subpool));
 
       if (current_entry->kind == svn_node_dir)
         {
@@ -1697,8 +1764,10 @@ walker_helper (const char *dirpath,
                                         subpool));
           SVN_ERR (walker_helper (entrypath, entry_access,
                                   walk_callbacks, walk_baton,
-                                  show_deleted, subpool));
+                                  show_hidden, subpool));
         }
+
+      svn_pool_clear (subpool);
     }
 
   svn_pool_destroy (subpool);
@@ -1712,27 +1781,27 @@ svn_wc_walk_entries (const char *path,
                      svn_wc_adm_access_t *adm_access,
                      const svn_wc_entry_callbacks_t *walk_callbacks,
                      void *walk_baton,
-                     svn_boolean_t show_deleted,
+                     svn_boolean_t show_hidden,
                      apr_pool_t *pool)
 {
   const svn_wc_entry_t *entry;
   
-  SVN_ERR (svn_wc_entry (&entry, path, adm_access, show_deleted, pool));
+  SVN_ERR (svn_wc_entry (&entry, path, adm_access, show_hidden, pool));
 
   if (! entry)
     return svn_error_createf (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
-                              "'%s' is not under revision control.", path);
+                              _("'%s' is not under version control"), path);
 
   if (entry->kind == svn_node_file)
-    return walk_callbacks->found_entry (path, entry, walk_baton);
+    return walk_callbacks->found_entry (path, entry, walk_baton, pool);
 
   else if (entry->kind == svn_node_dir)
     return walker_helper (path, adm_access, walk_callbacks, walk_baton,
-                          show_deleted, pool);
+                          show_hidden, pool);
 
   else
     return svn_error_createf (SVN_ERR_NODE_UNKNOWN_KIND, NULL,
-                              "%s: unrecognized node kind.", path);
+                              _("'%s' has an unrecognized node kind"), path);
 }
 
 
@@ -1749,22 +1818,24 @@ svn_wc_mark_missing_deleted (const char *path,
     {
       const char *parent_path, *bname;
       svn_wc_adm_access_t *adm_access;
-      svn_wc_entry_t *newent
-        = apr_pcalloc (pool, sizeof(*newent));
+      svn_wc_entry_t newent;
       
-      newent->deleted = TRUE;
+      newent.deleted = TRUE;
+      newent.schedule = svn_wc_schedule_normal;
 
       svn_path_split (path, &parent_path, &bname, pool);
 
       SVN_ERR (svn_wc_adm_retrieve (&adm_access, parent, parent_path, pool));
-      SVN_ERR (svn_wc__entry_modify (adm_access, bname, newent,
-                                     SVN_WC__ENTRY_MODIFY_DELETED,
+      SVN_ERR (svn_wc__entry_modify (adm_access, bname, &newent,
+                                     (SVN_WC__ENTRY_MODIFY_DELETED
+                                      | SVN_WC__ENTRY_MODIFY_SCHEDULE
+                                      | SVN_WC__ENTRY_MODIFY_FORCE),
                                      TRUE, /* sync right away */ pool));
 
       return SVN_NO_ERROR;
     }
   else
     return svn_error_createf (SVN_ERR_WC_PATH_FOUND, NULL,
-                              "svn_wc_mark_missing_deleted: path '%s' isn't "
-                              "missing.", path);
+                              _("Unexpectedly found '%s': "
+                                "path is marked 'missing'"), path);
 }
