@@ -28,6 +28,7 @@
 #include "translate.h"
 #include "questions.h"
 
+#include "svn_private_config.h"
 
 
 
@@ -51,7 +52,7 @@ svn_wc_merge (const char *left,
   svn_subst_keywords_t *keywords;
   const char *eol;
   const svn_wc_entry_t *entry;
-  svn_boolean_t contains_conflicts;
+  svn_boolean_t contains_conflicts, special;
 
   svn_path_split (merge_target, &mt_pt, &mt_bn, pool);
 
@@ -233,14 +234,18 @@ svn_wc_merge (const char *left,
                                          NULL, pool));
           SVN_ERR (svn_wc__get_eol_style (NULL, &eol, merge_target, adm_access,
                                           pool));
-          SVN_ERR (svn_subst_copy_and_translate (left, 
-                                                 left_copy,
-                                                 eol, eol ? TRUE : FALSE, 
-                                                 keywords, TRUE, pool));
-          SVN_ERR (svn_subst_copy_and_translate (right,
-                                                 right_copy,
-                                                 eol, eol ? TRUE : FALSE, 
-                                                 keywords, TRUE, pool));
+          SVN_ERR (svn_wc__get_special (&special, merge_target, adm_access,
+                                        pool));
+          SVN_ERR (svn_subst_copy_and_translate2 (left, 
+                                                  left_copy,
+                                                  eol, eol ? TRUE : FALSE, 
+                                                  keywords, TRUE, special,
+                                                  pool));
+          SVN_ERR (svn_subst_copy_and_translate2 (right,
+                                                  right_copy,
+                                                  eol, eol ? TRUE : FALSE, 
+                                                  keywords, TRUE, special,
+                                                  pool));
 
           /* Back up MERGE_TARGET verbatim (it's already in expanded form.) */
           SVN_ERR (svn_io_copy_file (merge_target,
@@ -288,21 +293,24 @@ svn_wc_merge (const char *left,
                                          NULL, pool));
           SVN_ERR (svn_wc__get_eol_style (NULL, &eol, merge_target, adm_access,
                                           pool));
-          SVN_ERR (svn_subst_copy_and_translate (result_target, merge_target,
-                                                 eol, eol ? TRUE : FALSE, 
-                                                 keywords, TRUE, pool));
+          SVN_ERR (svn_wc__get_special (&special, merge_target, adm_access,
+                                        pool));
+          SVN_ERR (svn_subst_copy_and_translate2 (result_target, merge_target,
+                                                  eol, eol ? TRUE : FALSE, 
+                                                  keywords, TRUE, special,
+                                                  pool));
         }
 
       /* Don't forget to clean up tmp_target, result_target, tmp_left,
          tmp_right.  There are a lot of scratch files lying around. */
       SVN_ERR_W (svn_io_remove_file (tmp_target, pool),
-                 "Unable to delete temporary file");
+                 _("Unable to delete temporary file"));
       SVN_ERR_W (svn_io_remove_file (result_target, pool),
-                 "Unable to delete temporary file");
+                 _("Unable to delete temporary file"));
       SVN_ERR_W (svn_io_remove_file (tmp_left, pool),
-                 "Unable to delete temporary file");
+                 _("Unable to delete temporary file"));
       SVN_ERR_W (svn_io_remove_file (tmp_right, pool),
-                 "Unable to delete temporary file");
+                 _("Unable to delete temporary file"));
 
     } /* end of merging for text files */
 
