@@ -836,7 +836,6 @@ svn_wc_prop_list (apr_hash_t **props,
                   svn_wc_adm_access_t *adm_access,
                   apr_pool_t *pool)
 {
-  svn_node_kind_t pkind;
   const char *prop_path;
 
   *props = apr_hash_make (pool);
@@ -844,18 +843,8 @@ svn_wc_prop_list (apr_hash_t **props,
   /* Construct a path to the relevant property file */
   SVN_ERR (svn_wc__prop_path (&prop_path, path, adm_access, FALSE, pool));
 
-  /* Does the property file exist? */
-  SVN_ERR (svn_io_check_path (prop_path, &pkind, pool));
-  
-  if (pkind == svn_node_none)
-    /* No property file exists.  Just go home, with an empty hash. */
-    return SVN_NO_ERROR;
-  
-  /* else... */
-
-  SVN_ERR (svn_wc__load_prop_file (prop_path, *props, pool));
-
-  return SVN_NO_ERROR;
+  /* svn_wc__load_prop_file checks if the prop file exists */
+  return svn_wc__load_prop_file (prop_path, *props, pool);
 }
 
 
