@@ -2,7 +2,7 @@
  * liveprops.c: mod_dav_svn live property provider functions for Subversion
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -24,12 +24,14 @@
 #include <apr_md5.h>
 #include <mod_dav.h>
 
-#include "mod_dav_svn.h"
+#include "dav_svn.h"
+
 #include "svn_pools.h"
 #include "svn_time.h"
 #include "svn_dav.h"
 #include "svn_md5.h"
 
+
 /*
 ** The namespace URIs that we use. This list and the enumeration must
 ** stay in sync.
@@ -43,7 +45,7 @@ static const char * const dav_svn_namespace_uris[] =
 };
 enum {
     DAV_SVN_NAMESPACE_URI_DAV,  /* the DAV: namespace URI */
-    DAV_SVN_NAMESPACE_URI,      /* the dav<->ra_dav namespace URI */
+    DAV_SVN_NAMESPACE_URI       /* the dav<->ra_dav namespace URI */
 };
 
 #define SVN_RO_DAV_PROP(name) \
@@ -64,7 +66,7 @@ enum {
 enum {
   SVN_PROPID_baseline_relative_path = 1,
   SVN_PROPID_md5_checksum,
-  SVN_PROPID_repository_uuid,
+  SVN_PROPID_repository_uuid
 };
 
 static const dav_liveprop_spec dav_svn_props[] =
@@ -104,6 +106,56 @@ static const dav_liveprop_group dav_svn_liveprop_group =
     &dav_svn_hooks_liveprop
 };
 
+/* Return the revision property PROPNAME's value in PROPVAL for
+   RESOURCE in the revision COMMITTED_REV after verifying that the
+   path is readable.  If the property if inaccessible, SVN_NO_ERROR is
+   returned and *PROPVAL is NULL.
+
+   This function must only be used to retrieve properties for which it
+   is sufficient to have read access to a single changed path in the
+   revision to have access to the revprop, e.g.
+   SVN_PROP_REVISION_AUTHOR or SVN_PROP_REVISION_DATE.
+
+   The reason for this is that we only check readability of the
+   current path (which is one of the revisions' changed paths per
+   definition).  If the current path is readable, the revprop is also
+   readable.  While it's possible that the property is readable even
+   though the current path is not readable (because another path in
+   the same revision is readable), it's a silly situation worth
+   ignoring to gain the extra performance. */
+static svn_error_t *svn_svn_get_path_revprop(svn_string_t **propval,
+                                             const dav_resource *resource,
+                                             svn_revnum_t committed_rev,
+                                             const char *propname,
+                                             apr_pool_t *pool)
+{
+  dav_svn_authz_read_baton arb;
+  svn_boolean_t allowed;
+  svn_fs_root_t *root;
+
+  *propval = NULL;
+
+  arb.r = resource->info->r;
+  arb.repos = resource->info->repos;
+  SVN_ERR(svn_fs_revision_root(&root,
+                               resource->info->repos->fs,
+                               committed_rev, pool));
+  SVN_ERR(dav_svn_authz_read(&allowed,
+                             root,
+                             resource->info->repos_path,
+                             &arb, pool));
+
+  if (! allowed)
+    return SVN_NO_ERROR;
+
+  /* Get the property of the created revision. The authz is already
+     performed, so we don't need to do it here too. */
+  return svn_repos_fs_revision_prop(propval,
+                                    resource->info->repos->repos,
+                                    committed_rev,
+                                    propname,
+                                    NULL, NULL, pool);
+}
 
 static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
                                            int propid, dav_prop_insert what,
@@ -145,94 +197,35 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
            svn_fs_revisions_changed.  gstein, is it a bad thing that
            we're currently using 'creationdate' to mean the same thing
            as 'last modified date'?  */
-        svn_revnum_t committed_rev = SVN_INVALID_REVNUM;
-        svn_string_t *committed_date = NULL;
+        const char *datestring;
+        apr_time_t timeval;
+        enum dav_svn_time_format format;
 
-        if (resource->baselined && resource->type == DAV_RESOURCE_TYPE_VERSION)
-          {
-            /* A baseline URI. */
-            committed_rev = resource->info->root.rev;
-          }
-        else if (resource->type == DAV_RESOURCE_TYPE_REGULAR
-                 || resource->type == DAV_RESOURCE_TYPE_WORKING
-                 || resource->type == DAV_RESOURCE_TYPE_VERSION)
-          {
-            /* Get the CR field out of the node's skel.  Notice that the
-               root object might be an ID root -or- a revision root. */
-            serr = svn_fs_node_created_rev(&committed_rev,
-                                           resource->info->root.root,
-                                           resource->info->repos_path, p);
-            if (serr != NULL)
-              {
-                /* ### what to do? */
-                value = "###error###";
-                break;
-              }
-          }        
-        else
+        /* ### for now, our global VCC has no such property. */
+        if (resource->type == DAV_RESOURCE_TYPE_PRIVATE
+            && resource->info->restype == DAV_SVN_RESTYPE_VCC)
           {
             return DAV_PROP_INSERT_NOTSUPP;
           }
-        
-        /* Get the date property of the created revision. */
-        serr = svn_fs_revision_prop(&committed_date,
-                                    resource->info->repos->fs,
-                                    committed_rev,
-                                    SVN_PROP_REVISION_DATE, p);
-        if (serr != NULL)
-          {
-            /* ### what to do? */
-            value = "###error###";
-            break;
-          }
-        
-        if (committed_date == NULL)
-          return DAV_PROP_INSERT_NOTDEF;
-
+       
         if (propid == DAV_PROPID_creationdate)
           {
             /* Return an ISO8601 date; this is what the svn client
                expects, and rfc2518 demands it. */
-
-            /* Revision datestamps are already ISO8601 format. */
-            value = apr_xml_quote_string(p, committed_date->data, 1);
+            format = dav_svn_time_format_iso8601;
           }
-
-        else if (propid == DAV_PROPID_getlastmodified)
+        else /* propid == DAV_PROPID_getlastmodified */
           {
-            /* Return an rfc1123 date string, as rfc2518 demands an
-               'http date' for this property.  */
-            apr_time_t timeval;
-            apr_time_exp_t tms;
-            apr_status_t status;
-            const char *nicedate;
-
-            /* convert the ISO8601 date into an apr_time_t */
-            serr = svn_time_from_cstring(&timeval, committed_date->data, p);
-            if (serr != NULL)
-              {
-                value = "###error###";
-                break;
-              }
-            
-            /* convert the apr_time_t into a apr_time_exp_t */
-            status = apr_time_exp_gmt(&tms, timeval);
-            if (status != APR_SUCCESS)
-              {
-                value = "###error###";
-                break;
-              }
-              
-            /* stolen from dav/fs/repos.c   :-)  */
-            nicedate = apr_psprintf(p, "%s, %.2d %s %d %.2d:%.2d:%.2d GMT",
-                                    apr_day_snames[tms.tm_wday],
-                                    tms.tm_mday, apr_month_snames[tms.tm_mon],
-                                    tms.tm_year + 1900,
-                                    tms.tm_hour, tms.tm_min, tms.tm_sec);
-
-            value = apr_xml_quote_string(p, nicedate, 1);
+            format = dav_svn_time_format_rfc1123;
           }
 
+        if (0 != dav_svn_get_last_modified_time (&datestring, &timeval,
+                                                 resource, format, p))
+          {
+            return DAV_PROP_INSERT_NOTDEF;
+          }
+
+        value = apr_xml_quote_string(p, datestring, 1);
         break;
       }
 
@@ -241,6 +234,13 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
         svn_revnum_t committed_rev = SVN_INVALID_REVNUM;
         svn_string_t *last_author = NULL;
 
+        /* ### for now, our global VCC has no such property. */
+        if (resource->type == DAV_RESOURCE_TYPE_PRIVATE
+            && resource->info->restype == DAV_SVN_RESTYPE_VCC)
+          {
+            return DAV_PROP_INSERT_NOTSUPP;
+          }
+
         if (resource->baselined && resource->type == DAV_RESOURCE_TYPE_VERSION)
           {
             /* A baseline URI. */
@@ -258,6 +258,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
             if (serr != NULL)
               {
                 /* ### what to do? */
+                svn_error_clear(serr);
                 value = "###error###";
                 break;
               }
@@ -266,15 +267,16 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
           {
             return DAV_PROP_INSERT_NOTSUPP;
           }
-        
-        /* Get the date property of the created revision. */
-        serr = svn_fs_revision_prop(&last_author,
-                                    resource->info->repos->fs,
-                                    committed_rev,
-                                    SVN_PROP_REVISION_AUTHOR, p);
-        if (serr != NULL)
+
+        serr = svn_svn_get_path_revprop(&last_author,
+                                        resource,
+                                        committed_rev,
+                                        SVN_PROP_REVISION_AUTHOR,
+                                        p);
+        if (serr)
           {
             /* ### what to do? */
+            svn_error_clear(serr);
             value = "###error###";
             break;
           }
@@ -303,6 +305,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
                                   resource->info->repos_path, p);
         if (serr != NULL)
           {
+            svn_error_clear(serr);
             value = "0";  /* ### what to do? */
             break;
           }
@@ -321,12 +324,19 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
         if (resource->baselined && resource->type == DAV_RESOURCE_TYPE_VERSION)
           return DAV_PROP_INSERT_NOTSUPP;
 
+        if (resource->type == DAV_RESOURCE_TYPE_PRIVATE
+            && resource->info->restype == DAV_SVN_RESTYPE_VCC)
+          {
+            return DAV_PROP_INSERT_NOTSUPP;
+          }
+
         serr = svn_fs_node_prop (&pval, resource->info->root.root,
                                  resource->info->repos_path,
                                  SVN_PROP_MIME_TYPE, p);
 
         if ((serr != NULL) || (pval == NULL))
           {
+            svn_error_clear(serr);
             if (resource->collection) /* defaults for directories */
               {
                 if (resource->info->repos->xslt_uri)
@@ -348,7 +358,8 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
                    there's no point even checking.  No matter what the
                    error is, we can't claim to have a mime type for
                    this resource. */
-                  return DAV_PROP_INSERT_NOTDEF;
+                svn_error_clear(serr);
+                return DAV_PROP_INSERT_NOTDEF;
               }
             else
               value = pval->data;
@@ -358,6 +369,12 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
       }
 
     case DAV_PROPID_getetag:
+      if (resource->type == DAV_RESOURCE_TYPE_PRIVATE
+          && resource->info->restype == DAV_SVN_RESTYPE_VCC)
+        {
+          return DAV_PROP_INSERT_NOTSUPP;
+        }
+
       value = dav_svn_getetag(resource, p);
       break;
 
@@ -393,6 +410,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
           if (serr != NULL)
             {
               /* ### what to do? */
+              svn_error_clear(serr);
               value = "###error###";
               break;
             }
@@ -440,10 +458,16 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
           && (! resource->versioned))
         return DAV_PROP_INSERT_NOTSUPP;
 
+      if (resource->type == DAV_RESOURCE_TYPE_PRIVATE
+          && resource->info->restype == DAV_SVN_RESTYPE_VCC)
+        {
+          return DAV_PROP_INSERT_NOTSUPP;
+        }
+
       if (resource->baselined)
         {
           /* just the revision number for baselines */
-          value = apr_psprintf(p, "%" SVN_REVNUM_T_FMT,
+          value = apr_psprintf(p, "%ld",
                                resource->info->root.rev);
         }
       else
@@ -458,12 +482,13 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
           if (serr != NULL)
             {
               /* ### what to do? */
+              svn_error_clear(serr);
               value = "###error###";
               break;
             }
           
           /* Convert the revision into a quoted string */
-          s = apr_psprintf(p, "%" SVN_REVNUM_T_FMT, committed_rev);
+          s = apr_psprintf(p, "%ld", committed_rev);
           value = apr_xml_quote_string(p, s, 1);
         }
       break;
@@ -487,7 +512,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
               || resource->type == DAV_RESOURCE_TYPE_WORKING
               || resource->type == DAV_RESOURCE_TYPE_VERSION))
         {
-          unsigned char digest[MD5_DIGESTSIZE];
+          unsigned char digest[APR_MD5_DIGESTSIZE];
 
           serr = svn_fs_file_md5_checksum(digest,
                                           resource->info->root.root,
@@ -495,6 +520,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
           if (serr != NULL)
             {
               /* ### what to do? */
+              svn_error_clear(serr);
               value = "###error###";
               break;
             }
@@ -514,6 +540,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
       if (serr != NULL)
         {
           /* ### what to do? */
+          svn_error_clear(serr);
           value = "###error###";
           break;
         }
@@ -671,3 +698,99 @@ void dav_svn_register_uris(apr_pool_t *p)
     /* register the namespace URIs */
     dav_register_liveprop_group(p, &dav_svn_liveprop_group);
 }
+
+
+int dav_svn_get_last_modified_time (const char **datestring,
+                                    apr_time_t *timeval,
+                                    const dav_resource *resource,
+                                    enum dav_svn_time_format format,
+                                    apr_pool_t *pool)
+{
+  svn_revnum_t committed_rev = SVN_INVALID_REVNUM;
+  svn_string_t *committed_date = NULL;
+  svn_error_t *serr;
+  apr_time_t timeval_tmp;
+
+  if ((datestring == NULL) && (timeval == NULL))
+    return 0;
+
+  if (resource->baselined && resource->type == DAV_RESOURCE_TYPE_VERSION)
+    {
+      /* A baseline URI. */
+      committed_rev = resource->info->root.rev;
+    }
+  else if (resource->type == DAV_RESOURCE_TYPE_REGULAR
+           || resource->type == DAV_RESOURCE_TYPE_WORKING
+           || resource->type == DAV_RESOURCE_TYPE_VERSION)
+    {
+      serr = svn_fs_node_created_rev(&committed_rev,
+                                     resource->info->root.root,
+                                     resource->info->repos_path, pool);
+      if (serr != NULL)
+        {
+          svn_error_clear(serr);
+          return 1;
+        }
+    }
+  else
+    {
+      /* unsupported resource kind -- has no mod-time */
+      return 1;
+    }
+
+  serr = svn_svn_get_path_revprop(&committed_date,
+                                  resource,
+                                  committed_rev,
+                                  SVN_PROP_REVISION_DATE,
+                                  pool);
+  if (serr)
+    {
+      svn_error_clear(serr);
+      return 1;
+    }
+
+  if (committed_date == NULL)
+    return 1;
+
+  /* return the ISO8601 date as an apr_time_t */
+  serr = svn_time_from_cstring(&timeval_tmp, committed_date->data, pool);
+  if (serr != NULL)
+    {
+      svn_error_clear(serr);
+      return 1;
+    }
+
+  if (timeval)
+    memcpy(timeval, &timeval_tmp, sizeof(*timeval));
+
+  if (! datestring)
+    return 0;
+
+  if (format == dav_svn_time_format_iso8601)
+    {
+      *datestring = committed_date->data;
+    }
+  else if (format == dav_svn_time_format_rfc1123)
+    {
+      apr_time_exp_t tms;
+      apr_status_t status;
+      
+      /* convert the apr_time_t into an apr_time_exp_t */
+      status = apr_time_exp_gmt(&tms, timeval_tmp);
+      if (status != APR_SUCCESS)
+        return 1;
+              
+      /* stolen from dav/fs/repos.c   :-)  */
+      *datestring = apr_psprintf(pool, "%s, %.2d %s %d %.2d:%.2d:%.2d GMT",
+                                 apr_day_snames[tms.tm_wday],
+                                 tms.tm_mday, apr_month_snames[tms.tm_mon],
+                                 tms.tm_year + 1900,
+                                 tms.tm_hour, tms.tm_min, tms.tm_sec);      
+    }
+  else /* unknown time format */
+    {
+      return 1;
+    }
+
+  return 0;
+}                                              
