@@ -44,6 +44,8 @@
 #include "svn_props.h"
 #include "svn_diff.h"
 
+#include "svn_private_config.h"
+
 
 /*** Some convenience macros and types. ***/
 
@@ -84,28 +86,28 @@ enum
 static const apr_getopt_option_t options_table[] =
   {
     {"help",          'h', 0,
-     "show help on a subcommand"},
+     N_("show help on a subcommand")},
 
     {NULL,            '?', 0,
-     "show help on a subcommand"},
+     N_("show help on a subcommand")},
 
     {"version",       svnlook__version, 0,
-     "show version information"},
+     N_("show version information")},
 
     {"revision",      'r', 1,
-     "specify revision number ARG"},
+     N_("specify revision number ARG")},
 
     {"transaction",  't', 1,
-     "specify transaction name ARG"},
+     N_("specify transaction name ARG")},
 
     {"verbose",  'v', 0,
-     "be verbose"},
+     N_("be verbose")},
 
     {"show-ids",      svnlook__show_ids, 0,
-     "show node revision ids for each path"},
+     N_("show node revision ids for each path")},
 
     {"no-diff-deleted", svnlook__no_diff_deleted, 0,
-     "do not print differences for deleted files"},
+     N_("do not print differences for deleted files")},
 
     {0,               0, 0, 0}
   };
@@ -117,82 +119,83 @@ static const apr_getopt_option_t options_table[] =
 static const svn_opt_subcommand_desc_t cmd_table[] =
   {
     {"author", subcommand_author, {0},
-     "usage: svnlook author REPOS_PATH\n\n"
-     "Print the author.\n",
+     N_("usage: svnlook author REPOS_PATH\n\n"
+        "Print the author.\n"),
      {'r', 't'} },
     
     {"cat", subcommand_cat, {0},
-     "usage: svnlook cat REPOS_PATH FILE_PATH\n\n"
-     "Print the contents of a file.  Leading '/' on FILE_PATH is optional.\n",
+     N_("usage: svnlook cat REPOS_PATH FILE_PATH\n\n"
+        "Print the contents of a file.  Leading '/' on FILE_PATH is "
+        "optional.\n"),
      {'r', 't'} },
     
     {"changed", subcommand_changed, {0},
-     "usage: svnlook changed REPOS_PATH\n\n"
-     "Print the paths that were changed.\n",
+     N_("usage: svnlook changed REPOS_PATH\n\n"
+        "Print the paths that were changed.\n"),
      {'r', 't'} },
     
     {"date", subcommand_date, {0},
-     "usage: svnlook date REPOS_PATH\n\n"
-     "Print the datestamp.\n",
+     N_("usage: svnlook date REPOS_PATH\n\n"
+        "Print the datestamp.\n"),
      {'r', 't'} },
 
     {"diff", subcommand_diff, {0},
-     "usage: svnlook diff REPOS_PATH\n\n"
-     "Print GNU-style diffs of changed files and properties.\n",
+     N_("usage: svnlook diff REPOS_PATH\n\n"
+        "Print GNU-style diffs of changed files and properties.\n"),
      {'r', 't', svnlook__no_diff_deleted} },
 
     {"dirs-changed", subcommand_dirschanged, {0},
-     "usage: svnlook dirs-changed REPOS_PATH\n\n"
-     "Print the directories that were themselves changed (property edits)\n"
-     "or whose file children were changed.\n",
+     N_("usage: svnlook dirs-changed REPOS_PATH\n\n"
+        "Print the directories that were themselves changed (property edits)\n"
+        "or whose file children were changed.\n"),
      {'r', 't'} },
     
     {"help", subcommand_help, {"?", "h"},
-     "usage: svnlook help [SUBCOMMAND...]\n\n"
-     "Describe the usage of this program or its subcommands.\n",
+     N_("usage: svnlook help [SUBCOMMAND...]\n\n"
+        "Describe the usage of this program or its subcommands.\n"),
      {svnlook__version} },
 
     {"history", subcommand_history, {0},
-     "usage: svnlook history REPOS_PATH [PATH_IN_REPOS]\n\n"
-     "Print information about the history of a path in the repository (or\n"
-     "the root directory if no path is supplied).\n",
+     N_("usage: svnlook history REPOS_PATH [PATH_IN_REPOS]\n\n"
+        "Print information about the history of a path in the repository (or\n"
+        "the root directory if no path is supplied).\n"),
      {'r', svnlook__show_ids} },
 
     {"info", subcommand_info, {0},
-     "usage: svnlook info REPOS_PATH\n\n"
-     "Print the author, datestamp, log message size, and log message.\n",
+     N_("usage: svnlook info REPOS_PATH\n\n"
+        "Print the author, datestamp, log message size, and log message.\n"),
      {'r', 't'} },
 
     {"log", subcommand_log, {0},
-     "usage: svnlook log REPOS_PATH\n\n"
-     "Print the log message.\n",
+     N_("usage: svnlook log REPOS_PATH\n\n"
+        "Print the log message.\n"),
      {'r', 't'} },
 
     {"propget", subcommand_pget, {"pget", "pg"},
-     "usage: svnlook propget REPOS_PATH PROPNAME PATH_IN_REPOS\n\n"
-     "Print the raw value of a property on a path in the repository.\n",
+     N_("usage: svnlook propget REPOS_PATH PROPNAME PATH_IN_REPOS\n\n"
+        "Print the raw value of a property on a path in the repository.\n"),
      {'r', 't'} },
 
     {"proplist", subcommand_plist, {"plist", "pl"},
-     "usage: svnlook proplist REPOS_PATH PATH_IN_REPOS\n\n"
-     "List the properties of a path in the repository.\n"
-     "With -v, show the property values too.\n",
+     N_("usage: svnlook proplist REPOS_PATH PATH_IN_REPOS\n\n"
+        "List the properties of a path in the repository.\n"
+        "With -v, show the property values too.\n"),
      {'r', 't', 'v'} },
 
     {"tree", subcommand_tree, {0},
-     "usage: svnlook tree REPOS_PATH [PATH_IN_REPOS]\n\n"
-     "Print the tree, starting at PATH_IN_REPOS (if supplied, at the root\n"
-     "of the tree otherwise), optionally showing node revision ids.\n",
+     N_("usage: svnlook tree REPOS_PATH [PATH_IN_REPOS]\n\n"
+        "Print the tree, starting at PATH_IN_REPOS (if supplied, at the root\n"
+        "of the tree otherwise), optionally showing node revision ids.\n"),
      {'r', 't', svnlook__show_ids} },
 
     {"uuid", subcommand_uuid, {0},
-     "usage: svnlook uuid REPOS_PATH\n\n"
-     "Print the repository's UUID.\n",
+     N_("usage: svnlook uuid REPOS_PATH\n\n"
+        "Print the repository's UUID.\n"),
      {0} },
 
     {"youngest", subcommand_youngest, {0},
-     "usage: svnlook youngest REPOS_PATH\n\n"
-     "Print the youngest revision number.\n",
+     N_("usage: svnlook youngest REPOS_PATH\n\n"
+        "Print the youngest revision number.\n"),
      {0} },
 
     { NULL, NULL, {0}, NULL, {0} }
@@ -252,11 +255,33 @@ check_cancel (void *baton)
     return SVN_NO_ERROR;
 }
 
+
+/* Version compatibility check */
 static svn_error_t *
-get_property (svn_string_t **prop_value /* native */,
-              svn_boolean_t need_translation,
+check_lib_versions (void)
+{
+  static const svn_version_checklist_t checklist[] =
+    {
+      { "svn_subr",  svn_subr_version },
+      { "svn_repos", svn_repos_version },
+      { "svn_fs",    svn_fs_version },
+      { "svn_delta", svn_delta_version },
+      { "svn_diff",  svn_diff_version },
+      { NULL, NULL }
+    };
+
+   SVN_VERSION_DEFINE (my_version);
+   return svn_ver_check_list (&my_version, checklist);
+}
+
+
+/* Get revision or transaction property PROP_NAME for the revision or
+   transaction specified in C, allocating in in POOL and placing it in
+   *PROP_VALUE. */
+static svn_error_t *
+get_property (svn_string_t **prop_value,
               svnlook_ctxt_t *c, 
-              const char *prop_name /* UTF-8! */,
+              const char *prop_name,
               apr_pool_t *pool)
 {
   svn_string_t *raw_value;
@@ -270,9 +295,6 @@ get_property (svn_string_t **prop_value /* native */,
     SVN_ERR (svn_fs_revision_prop (&raw_value, c->fs, c->rev_id,
                                    prop_name, pool));
 
-  if (need_translation && svn_prop_needs_translation (prop_name))
-    SVN_ERR (svn_subst_detranslate_string (&raw_value, raw_value, TRUE, pool));
-
   *prop_value = raw_value;
 
   return SVN_NO_ERROR;
@@ -396,9 +418,7 @@ print_dirs_changed_tree (svn_repos_node_t *node,
   /* Print the node if it qualifies. */
   if (print_me)
     {
-      const char *path_stdout;
-      SVN_ERR (svn_cmdline_cstring_from_utf8 (&path_stdout, path, pool));
-      printf ("%s/\n", path_stdout);
+      SVN_ERR (svn_cmdline_printf (pool, "%s/\n", path));
     }
 
   /* Return here if the node has no children. */
@@ -460,12 +480,10 @@ print_changed_tree (svn_repos_node_t *node,
   /* Print this node unless told to skip it. */
   if (print_me)
     {
-      const char *path_stdout;
-      SVN_ERR (svn_cmdline_cstring_from_utf8 (&path_stdout, path, pool));
-      printf ("%s  %s%s\n",
-              status,
-              path_stdout,
-              node->kind == svn_node_dir ? "/" : "");
+      SVN_ERR (svn_cmdline_printf (pool, "%s  %s%s\n",
+                                   status,
+                                   path,
+                                   node->kind == svn_node_dir ? "/" : ""));
     }
   
   /* Return here if the node has no children. */
@@ -554,32 +572,12 @@ dump_contents (apr_file_t *fh,
                const char *path /* UTF-8! */,
                apr_pool_t *pool)
 {
-  apr_status_t apr_err;
-  apr_size_t len, len2;
-  svn_stream_t *stream;
-  char buffer[1024];
+  svn_stream_t *contents, *file_stream;
 
-  /* Get a stream to the current file's contents. */
-  SVN_ERR (svn_fs_file_contents (&stream, root, path, pool));
-  
-  /* Now, route that data into our temporary file. */
-  while (1)
-    {
-      SVN_ERR (check_cancel (NULL));
-      len = sizeof (buffer);
-      SVN_ERR (svn_stream_read (stream, buffer, &len));
-      len2 = len;
-      apr_err = apr_file_write (fh, buffer, &len2);
-      if ((apr_err) || (len2 != len))
-        return svn_error_wrap_apr 
-          (apr_err ? apr_err : SVN_ERR_INCOMPLETE_DATA,
-           "Can't write contents of '%s'", path);
-      if (len != sizeof (buffer))
-        break;
-    }
-
-  /* And close the file. */
-  apr_file_close (fh);
+  /* Grab the contents and copy them into fh. */
+  SVN_ERR (svn_fs_file_contents (&contents, root, path, pool));
+  file_stream = svn_stream_from_aprfile (fh, pool);
+  SVN_ERR (svn_stream_copy (contents, file_stream, pool));
   return SVN_NO_ERROR;
 }
 
@@ -702,7 +700,7 @@ generate_label (const char **label,
     *label = apr_psprintf (pool, "%s\t%s UTC (txn %s)", 
                            path, datestr, name); 
   else
-    *label = apr_psprintf (pool, "%s\t%s UTC (rev %" SVN_REVNUM_T_FMT ")",
+    *label = apr_psprintf (pool, "%s\t%s UTC (rev %ld)",
                            path, datestr, rev);
   return SVN_NO_ERROR;
 }
@@ -726,7 +724,8 @@ display_prop_diffs (const apr_array_header_t *prop_diffs,
 {
   int i;
 
-  printf ("\nProperty changes on: %s\n%s\n", path, under_string);
+  SVN_ERR (svn_cmdline_printf (pool, "\nProperty changes on: %s\n%s\n",
+                               path, under_string));
 
   for (i = 0; i < prop_diffs->nelts; i++)
     {
@@ -740,12 +739,14 @@ display_prop_diffs (const apr_array_header_t *prop_diffs,
       else
         orig_value = NULL;
 
-      printf ("Name: %s\n", pc->name);
+      SVN_ERR (svn_cmdline_printf (pool, _("Name: %s\n"), pc->name));
 
       /* For now, we have a rather simple heuristic: if this is an
          "svn:" property, then assume the value is UTF-8 and must
          therefore be converted before printing.  Otherwise, just
-         print whatever's there and hope for the best. */
+         print whatever's there and hope for the best.
+         ### We don't use svn_cmdline_printf here, since we don't know if the
+         values are UTF-8. */
       {
         svn_boolean_t val_to_utf8 = svn_prop_is_svn_prop (pc->name);
         const char *printable_val;
@@ -772,9 +773,8 @@ display_prop_diffs (const apr_array_header_t *prop_diffs,
       }
     }
 
-  printf ("\n");
-  fflush (stdout);
-  return SVN_NO_ERROR;
+  SVN_ERR (svn_cmdline_printf (pool, "\n"));
+  return svn_cmdline_fflush (stdout);
 }
 
 
@@ -791,7 +791,7 @@ print_diff_tree (svn_fs_root_t *root,
                  const char *tmpdir,
                  apr_pool_t *pool)
 {
-  const char *orig_path = NULL, *new_path = NULL, *path_native;
+  const char *orig_path = NULL, *new_path = NULL;
   svn_boolean_t do_diff = FALSE;
   svn_boolean_t is_copy = FALSE;
   svn_boolean_t binary = FALSE;
@@ -802,14 +802,10 @@ print_diff_tree (svn_fs_root_t *root,
   if (! node)
     return SVN_NO_ERROR;
 
-  SVN_ERR (svn_utf_cstring_from_utf8 (&path_native, path, pool));
-
   /* Print copyfrom history for the top node of a copied tree. */
   if ((SVN_IS_VALID_REVNUM (node->copyfrom_rev))
       && (node->copyfrom_path != NULL))
     {
-      const char *base_path_native;
-
       /* This is ... a copy. */
       is_copy = TRUE;
 
@@ -822,10 +818,8 @@ print_diff_tree (svn_fs_root_t *root,
       else
         base_path = apr_pstrdup (pool, node->copyfrom_path);
 
-      SVN_ERR (svn_utf_cstring_from_utf8 (&base_path_native, base_path, pool));
-
-      printf ("Copied: %s (from rev %" SVN_REVNUM_T_FMT ", %s)\n",
-              path_native, node->copyfrom_rev, base_path_native);
+      SVN_ERR (svn_cmdline_printf (pool, _("Copied: %s (from rev %ld, %s)\n"),
+                                   path, node->copyfrom_rev, base_path));
 
       SVN_ERR (svn_fs_revision_root (&base_root,
                                      svn_fs_root_fs (base_root),
@@ -887,22 +881,24 @@ print_diff_tree (svn_fs_root_t *root,
   if (do_diff)
     {
       if (! is_copy)
-        printf ("%s: %s\n", 
-                ((node->action == 'A') ? "Added" : 
-                 ((node->action == 'D') ? "Deleted" :
-                  ((node->action == 'R') ? "Modified" : "Index"))),
-                path_native);
+        SVN_ERR (svn_cmdline_printf (pool, "%s: %s\n", 
+                                     ((node->action == 'A') ? _("Added") :
+                                      ((node->action == 'D') ? _("Deleted") :
+                                       ((node->action == 'R') ? _("Modified")
+                                        : _("Index")))),
+                                     path));
 
       if ((! no_diff_deleted) || (node->action != 'D'))
         {
           svn_diff_t *diff;
 
-          printf ("%s\n", equal_string);
-          fflush (stdout);
+          SVN_ERR (svn_cmdline_printf (pool, "%s\n", equal_string));
+          SVN_ERR (svn_cmdline_fflush (stdout));
 
           if (binary)
             {
-              printf ("(Binary files differ)\n");
+              SVN_ERR (svn_cmdline_printf (pool,
+                                           _("(Binary files differ)\n")));
             }
           else
             {
@@ -926,19 +922,19 @@ print_diff_tree (svn_fs_root_t *root,
             }
         }
 
-      printf ("\n");
-      fflush (stdout);
+      SVN_ERR (svn_cmdline_printf (pool, "\n"));
+      SVN_ERR (svn_cmdline_fflush (stdout));
     }
   else if (is_copy)
     {
-      printf ("\n");
+      SVN_ERR (svn_cmdline_printf (pool, "\n"));
     }
 
   /* Make sure we delete any temporary files. */
   if (orig_path)
-    svn_io_remove_file (orig_path, pool);
+    SVN_ERR (svn_io_remove_file (orig_path, pool));
   if (new_path)
-    svn_io_remove_file (new_path, pool);
+    SVN_ERR (svn_io_remove_file (new_path, pool));
 
   /*** Now handle property diffs ***/
   if ((node->prop_mod) && (node->action != 'D'))
@@ -1019,23 +1015,28 @@ print_tree (svn_fs_root_t *root,
   /* Print the indentation. */
   for (i = 0; i < indentation; i++)
     {
-      printf (" ");
+      SVN_ERR (svn_cmdline_fputs (" ", stdout, pool));
     }
 
   /* Print the node. */
   SVN_ERR (svn_utf_cstring_from_utf8 (&name_native, 
                                       svn_path_basename (path, pool), 
                                       pool));
-  printf ("%s%s", name_native, is_dir ? "/" : "");
+  SVN_ERR (svn_cmdline_printf (pool, "%s%s",
+                               svn_path_basename (path, pool),
+                               is_dir ? "/" : ""));
 
   if (show_ids)
     {
       svn_string_t *unparsed_id = NULL;
       if (id)
         unparsed_id = svn_fs_unparse_id (id, pool);
-      printf (" <%s>", unparsed_id ? unparsed_id->data : "unknown");
+      SVN_ERR (svn_cmdline_printf (pool, " <%s>",
+                                   unparsed_id
+                                   ? unparsed_id->data
+                                   : _("unknown")));
     }
-  printf ("\n");
+  SVN_ERR (svn_cmdline_fputs ("\n", stdout, pool));
 
   /* Return here if PATH is not a directory. */
   if (! is_dir)
@@ -1072,18 +1073,26 @@ static svn_error_t *
 do_log (svnlook_ctxt_t *c, svn_boolean_t print_size, apr_pool_t *pool)
 {
   svn_string_t *prop_value;
+  svn_string_t *prop_value_native;
 
-  SVN_ERR (get_property (&prop_value, TRUE, c, SVN_PROP_REVISION_LOG, pool));
+  SVN_ERR (get_property (&prop_value, c, SVN_PROP_REVISION_LOG, pool));
   if (! (prop_value && prop_value->data))
     {
-      printf ("%s\n", print_size ? "0" : "");
+      SVN_ERR (svn_cmdline_printf (pool, "%s\n", print_size ? "0" : ""));
       return SVN_NO_ERROR;
     }
   
   if (print_size)
-    printf ("%" APR_SIZE_T_FMT "\n", prop_value->len);
+    {
+      /* svn_cmdline_printf will convert to the native locale and eol-style
+         for us, but we need the size of the converted message. */
+      SVN_ERR (svn_subst_detranslate_string (&prop_value_native, prop_value,
+                                             TRUE, pool));
+      SVN_ERR (svn_cmdline_printf (pool, "%" APR_SIZE_T_FMT "\n",
+                                   prop_value_native->len));
+    }
 
-  printf ("%s\n", prop_value->data);
+  SVN_ERR (svn_cmdline_printf (pool, "%s\n", prop_value->data));
   return SVN_NO_ERROR;
 }
 
@@ -1096,24 +1105,21 @@ do_date (svnlook_ctxt_t *c, apr_pool_t *pool)
 {
   svn_string_t *prop_value;
 
-  SVN_ERR (get_property (&prop_value, FALSE, c, SVN_PROP_REVISION_DATE, pool));
+  SVN_ERR (get_property (&prop_value, c, SVN_PROP_REVISION_DATE, pool));
   if (prop_value && prop_value->data)
     {
       /* Convert the date for humans. */
       apr_time_t aprtime;
-      const char *time_native;
       const char *time_utf8;
       
       SVN_ERR (svn_time_from_cstring (&aprtime, prop_value->data, pool));
 
       time_utf8 = svn_time_to_human_cstring (aprtime, pool);
       
-      SVN_ERR (svn_utf_cstring_from_utf8 (&time_native, time_utf8, pool));
-      
-      printf ("%s", time_native);
+      SVN_ERR (svn_cmdline_printf (pool, "%s", time_utf8));
     }
 
-  printf ("\n");
+  SVN_ERR (svn_cmdline_printf (pool, "\n"));
   return SVN_NO_ERROR;
 }
 
@@ -1124,12 +1130,12 @@ do_author (svnlook_ctxt_t *c, apr_pool_t *pool)
 {
   svn_string_t *prop_value;
 
-  SVN_ERR (get_property (&prop_value, TRUE, c,
+  SVN_ERR (get_property (&prop_value, c,
                          SVN_PROP_REVISION_AUTHOR, pool));
   if (prop_value && prop_value->data) 
-    printf ("%s", prop_value->data);
+    SVN_ERR (svn_cmdline_printf (pool, "%s", prop_value->data));
   
-  printf ("\n");
+  SVN_ERR (svn_cmdline_printf (pool, "\n"));
   return SVN_NO_ERROR;
 }
 
@@ -1152,7 +1158,7 @@ do_dirs_changed (svnlook_ctxt_t *c, apr_pool_t *pool)
   if (! SVN_IS_VALID_REVNUM (base_rev_id))
     return svn_error_createf 
       (SVN_ERR_FS_NO_SUCH_REVISION, NULL,
-       "Transaction '%s' is not based on a revision; how odd",
+       _("Transaction '%s' is not based on a revision; how odd"),
        c->txn_name);
   
   SVN_ERR (generate_delta_tree (&tree, c->repos, root, base_rev_id, 
@@ -1182,10 +1188,10 @@ verify_path (svn_node_kind_t *kind,
       if (svn_path_is_url (path))  /* check for a common mistake. */
         return svn_error_createf
           (SVN_ERR_FS_NOT_FOUND, NULL,
-           "'%s' is a URL, probably should be a path", path);
+           _("'%s' is a URL, probably should be a path"), path);
       else
         return svn_error_createf 
-          (SVN_ERR_FS_NOT_FOUND, NULL, "Path '%s' does not exist", path);
+          (SVN_ERR_FS_NOT_FOUND, NULL, _("Path '%s' does not exist"), path);
     }
 
   return SVN_NO_ERROR;
@@ -1209,17 +1215,19 @@ do_cat (svnlook_ctxt_t *c, const char *path, apr_pool_t *pool)
 
   if (kind != svn_node_file)
     return svn_error_createf 
-      (SVN_ERR_FS_NOT_FILE, NULL, "Path '%s' is not a file", path);
+      (SVN_ERR_FS_NOT_FILE, NULL, _("Path '%s' is not a file"), path);
 
   /* Else. */
 
   SVN_ERR (svn_fs_file_contents (&fstream, root, path, pool));
   SVN_ERR (svn_stream_for_stdout (&stdout_stream, pool));
-  do {
-    SVN_ERR (check_cancel (NULL));
-    SVN_ERR (svn_stream_read (fstream, buf, &len));
-    SVN_ERR (svn_stream_write (stdout_stream, buf, &len));
-  } while (len == BUFSIZ);
+  do 
+    {
+      SVN_ERR (check_cancel (NULL));
+      SVN_ERR (svn_stream_read (fstream, buf, &len));
+      SVN_ERR (svn_stream_write (stdout_stream, buf, &len));
+    } 
+  while (len == BUFSIZ);
 
   return SVN_NO_ERROR;
 }
@@ -1243,7 +1251,7 @@ do_changed (svnlook_ctxt_t *c, apr_pool_t *pool)
   if (! SVN_IS_VALID_REVNUM (base_rev_id))
     return svn_error_createf 
       (SVN_ERR_FS_NO_SUCH_REVISION, NULL,
-       "Transaction '%s' is not based on a revision; how odd",
+       _("Transaction '%s' is not based on a revision; how odd"),
        c->txn_name);
   
   SVN_ERR (generate_delta_tree (&tree, c->repos, root, base_rev_id, 
@@ -1258,45 +1266,36 @@ do_changed (svnlook_ctxt_t *c, apr_pool_t *pool)
 static svn_error_t *
 create_unique_tmpdir (const char **name, apr_pool_t *pool)
 {
-  const char *unique_name_apr;
   const char *unique_name;
   const char *sys_tmp_dir;
-  const char *base_apr;
   const char *base;
   unsigned int i;
 
   SVN_ERR (svn_io_temp_dir (&sys_tmp_dir, pool));
   base = svn_path_join (sys_tmp_dir, "svnlook", pool);
-  SVN_ERR (svn_path_cstring_from_utf8 (&base_apr, base, pool));
 
   for (i = 1; i <= 99999; i++)
     {
-      apr_status_t apr_err;
+      svn_error_t *err;
 
-      unique_name_apr = apr_psprintf (pool, "%s.%u", base_apr, i);
-      apr_err = apr_dir_make (unique_name_apr, APR_OS_DEFAULT, pool);
+      unique_name = apr_psprintf (pool, "%s.%u", base, i);
+      err = svn_io_dir_make (unique_name, APR_OS_DEFAULT, pool);
 
-      if (APR_STATUS_IS_EEXIST (apr_err))
-        continue;
-
-      SVN_ERR (svn_path_cstring_to_utf8 (&unique_name, unique_name_apr, pool));
-
-      if (apr_err)
-        {
-          *name = NULL;
-          return svn_error_wrap_apr (apr_err, "Can't create directory '%s'",
-                                     unique_name);
-        }
-      else
+      if (!err)
         {
           *name = unique_name;
           return SVN_NO_ERROR;
         }
+
+      if (! APR_STATUS_IS_EEXIST (err->apr_err))
+        return err;
+
+      svn_error_clear (err);
     }
 
   *name = NULL;
   return svn_error_createf (SVN_ERR_IO_UNIQUE_NAMES_EXHAUSTED,
-                            NULL, "Can't create temporary directory");
+                            NULL, _("Can't create temporary directory"));
 }
 
 /* Print some diff-y stuff in a TBD way. :-) */
@@ -1316,7 +1315,7 @@ do_diff (svnlook_ctxt_t *c, apr_pool_t *pool)
   if (! SVN_IS_VALID_REVNUM (base_rev_id))
     return svn_error_createf 
       (SVN_ERR_FS_NO_SUCH_REVISION, NULL,
-       "Transaction '%s' is not based on a revision; how odd",
+       _("Transaction '%s' is not based on a revision; how odd"),
        c->txn_name);
   
   SVN_ERR (generate_delta_tree (&tree, c->repos, root, base_rev_id, 
@@ -1371,12 +1370,12 @@ print_history (void *baton,
       SVN_ERR (svn_fs_revision_root (&rev_root, phb->fs, revision, pool));
       SVN_ERR (svn_fs_node_id (&node_id, rev_root, path, pool));
       id_string = svn_fs_unparse_id (node_id, pool);
-      printf ("%8" SVN_REVNUM_T_FMT "   %s <%s>\n", 
-              revision, path, id_string->data);
+      SVN_ERR (svn_cmdline_printf (pool, "%8ld   %s <%s>\n", 
+                                   revision, path, id_string->data));
     }
   else
     {
-      printf ("%8" SVN_REVNUM_T_FMT "   %s\n", revision, path);
+      SVN_ERR (svn_cmdline_printf (pool, "%8ld   %s\n", revision, path));
     }
 
   return SVN_NO_ERROR;
@@ -1396,13 +1395,13 @@ do_history (svnlook_ctxt_t *c,
 
   if (show_ids)
     {
-      printf ("REVISION   PATH <ID>\n");
-      printf ("--------   ---------\n");
+      SVN_ERR (svn_cmdline_printf (pool, _("REVISION   PATH <ID>\n"
+                                           "--------   ---------\n")));
     }
   else
     {
-      printf ("REVISION   PATH\n");
-      printf ("--------   ----\n");
+      SVN_ERR (svn_cmdline_printf (pool, _("REVISION   PATH\n"
+                                           "--------   ----\n")));
     }
 
   /* Call our history crawler.  We want the whole lifetime of the path
@@ -1438,7 +1437,7 @@ do_pget (svnlook_ctxt_t *c,
   if (prop == NULL)
     return svn_error_createf
       (SVN_ERR_PROPERTY_NOT_FOUND, NULL,
-       "Property '%s' not found on path '%s'", propname, path);
+       _("Property '%s' not found on path '%s'"), propname, path);
 
   /* Else. */
 
@@ -1493,6 +1492,8 @@ do_plist (svnlook_ctxt_t *c,
          colon and some spaces) anyway, just mimic the output of the
          command line client proplist.   Compare to 'svnlook propget',
          which sends the raw bytes to stdout, untranslated. */
+      /* We leave printf calls here, since we don't always know the encoding
+         of the prop value. */
       if (svn_prop_needs_translation (pname))
         SVN_ERR (svn_subst_detranslate_string (&propval, propval, TRUE, pool));
 
@@ -1592,7 +1593,7 @@ subcommand_cat (apr_getopt_t *os, void *baton, apr_pool_t *pool)
   if (opt_state->arg1 == NULL)
     return svn_error_createf
       (SVN_ERR_CL_INSUFFICIENT_ARGS, NULL,
-       "Missing repository path argument");
+       _("Missing repository path argument"));
 
   SVN_ERR (get_ctxt_baton (&c, opt_state, pool));
   SVN_ERR (do_cat (c, opt_state->arg1, pool));
@@ -1653,13 +1654,13 @@ subcommand_help (apr_getopt_t *os, void *baton, apr_pool_t *pool)
 {
   struct svnlook_opt_state *opt_state = baton;
   const char *header =
-    "general usage: svnlook SUBCOMMAND REPOS_PATH [ARGS & OPTIONS ...]\n"
-    "Note: any subcommand which takes the '--revision' and '--transaction'\n"
-    "      options will, if invoked without one of those options, act on\n"
-    "      the repository's youngest revision.\n"
-    "Type \"svnlook help <subcommand>\" for help on a specific subcommand.\n"
-    "\n"
-    "Available subcommands:\n";
+    _("general usage: svnlook SUBCOMMAND REPOS_PATH [ARGS & OPTIONS ...]\n"
+      "Note: any subcommand which takes the '--revision' and '--transaction'\n"
+      "      options will, if invoked without one of those options, act on\n"
+      "      the repository's youngest revision.\n"
+      "Type \"svnlook help <subcommand>\" for help on a specific subcommand.\n"
+      "\n"
+      "Available subcommands:\n");
 
   SVN_ERR (svn_opt_print_help (os, "svnlook", 
                                opt_state ? opt_state->version : FALSE, 
@@ -1723,13 +1724,13 @@ subcommand_pget (apr_getopt_t *os, void *baton, apr_pool_t *pool)
     {
       return svn_error_createf
         (SVN_ERR_CL_INSUFFICIENT_ARGS, NULL,
-         "Missing propname and repository path arguments");
+         _("Missing propname and repository path arguments"));
     }
   else if (opt_state->arg2 == NULL)
     {
       return svn_error_createf
         (SVN_ERR_CL_INSUFFICIENT_ARGS, NULL,
-         "Missing propname or repository path argument");
+         _("Missing propname or repository path argument"));
     }
 
   SVN_ERR (get_ctxt_baton (&c, opt_state, pool));
@@ -1747,7 +1748,7 @@ subcommand_plist (apr_getopt_t *os, void *baton, apr_pool_t *pool)
   if (opt_state->arg1 == NULL)
     return svn_error_createf
       (SVN_ERR_CL_INSUFFICIENT_ARGS, NULL,
-       "Missing repository path argument");
+       _("Missing repository path argument"));
 
   SVN_ERR (get_ctxt_baton (&c, opt_state, pool));
   SVN_ERR (do_plist (c, opt_state->arg1, opt_state->verbose, pool));
@@ -1775,7 +1776,7 @@ subcommand_youngest (apr_getopt_t *os, void *baton, apr_pool_t *pool)
   svnlook_ctxt_t *c;
 
   SVN_ERR (get_ctxt_baton (&c, opt_state, pool));
-  printf ("%" SVN_REVNUM_T_FMT "\n", c->rev_id);
+  SVN_ERR (svn_cmdline_printf (pool, "%ld\n", c->rev_id));
   return SVN_NO_ERROR;
 }
 
@@ -1789,7 +1790,7 @@ subcommand_uuid (apr_getopt_t *os, void *baton, apr_pool_t *pool)
 
   SVN_ERR (get_ctxt_baton (&c, opt_state, pool));
   SVN_ERR (svn_fs_get_uuid (c->fs, &uuid, pool));
-  printf ("%s\n", uuid);
+  SVN_ERR (svn_cmdline_printf (pool, "%s\n", uuid));
   return SVN_NO_ERROR;
 }
 
@@ -1827,6 +1828,16 @@ main (int argc, const char * const *argv)
   pool = svn_pool_create_ex (NULL, allocator);
   apr_allocator_owner_set (allocator, pool);
 
+  /* Check library versions */
+  err = check_lib_versions ();
+  if (err)
+    {
+      svn_handle_error (err, stderr, FALSE);
+      svn_error_clear (err);
+      svn_pool_destroy (pool);
+      return EXIT_FAILURE;
+    }
+
   if (argc <= 1)
     {
       subcommand_help (NULL, NULL, pool);
@@ -1863,11 +1874,16 @@ main (int argc, const char * const *argv)
       switch (opt_id) 
         {
         case 'r':
-          opt_state.rev = atoi (opt_arg);
-          if (! SVN_IS_VALID_REVNUM (opt_state.rev))
-            SVN_INT_ERR (svn_error_create
-                         (SVN_ERR_CL_ARG_PARSING_ERROR, NULL,
-                          "Invalid revision number supplied"));
+          {
+            char *digits_end = NULL;
+            opt_state.rev = strtol (opt_arg, &digits_end, 10);
+            if ((! SVN_IS_VALID_REVNUM (opt_state.rev))
+                || (! digits_end)
+                || *digits_end)
+              SVN_INT_ERR (svn_error_create
+                           (SVN_ERR_CL_ARG_PARSING_ERROR, NULL,
+                            _("Invalid revision number supplied")));
+          }
           break;
 
         case 't':
@@ -1908,8 +1924,8 @@ main (int argc, const char * const *argv)
   if ((opt_state.rev != SVN_INVALID_REVNUM) && opt_state.txn)
     SVN_INT_ERR (svn_error_create 
                  (SVN_ERR_CL_MUTUALLY_EXCLUSIVE_ARGS, NULL,
-                  "The '--transaction' (-t) and '--revision' (-r) arguments "
-                  "can not co-exist"));
+                  _("The '--transaction' (-t) and '--revision' (-r) arguments "
+                    "can not co-exist")));
 
   /* If the user asked for help, then the rest of the arguments are
      the names of subcommands to get help on (if any), or else they're
@@ -1924,7 +1940,9 @@ main (int argc, const char * const *argv)
     {
       if (os->ind >= os->argc)
         {
-          fprintf (stderr, "subcommand argument required\n");
+          svn_error_clear
+            (svn_cmdline_fprintf (stderr, pool,
+                                  _("subcommand argument required\n")));
           subcommand_help (NULL, NULL, pool);
           svn_pool_destroy (pool);
           return EXIT_FAILURE;
@@ -1935,7 +1953,20 @@ main (int argc, const char * const *argv)
           subcommand = svn_opt_get_canonical_subcommand (cmd_table, first_arg);
           if (subcommand == NULL)
             {
-              fprintf (stderr, "unknown command: '%s'\n", first_arg);
+              const char *first_arg_utf8;
+              err = svn_utf_cstring_to_utf8 (&first_arg_utf8, first_arg,
+                                             pool);
+              if (err)
+                {
+                  svn_handle_error (err, stderr, FALSE);
+                  svn_pool_destroy (pool);
+                  svn_error_clear (err);
+                  return EXIT_FAILURE;
+                }
+              svn_error_clear
+                (svn_cmdline_fprintf (stderr, pool,
+                                      _("Unknown command: '%s'\n"),
+                                      first_arg_utf8));
               subcommand_help (NULL, NULL, pool);
               svn_pool_destroy (pool);
               return EXIT_FAILURE;
@@ -1965,16 +1996,19 @@ main (int argc, const char * const *argv)
 
       if (repos_path == NULL)
         {
-          fprintf (stderr, "repository argument required\n");
+          svn_error_clear
+            (svn_cmdline_fprintf (stderr, pool,
+                                  _("repository argument required\n")));
           subcommand_help (NULL, NULL, pool);
           svn_pool_destroy (pool);
           return EXIT_FAILURE;
         }
       else if (svn_path_is_url (repos_path))
         {
-          fprintf (stderr,
-                   "'%s' is a URL when it should be a path\n",
-                   repos_path);
+          svn_error_clear
+            (svn_cmdline_fprintf (stderr, pool,
+                      _("'%s' is a URL when it should be a path\n"),
+                      repos_path));
           svn_pool_destroy (pool);
           return EXIT_FAILURE;
         }
@@ -2018,10 +2052,12 @@ main (int argc, const char * const *argv)
           const apr_getopt_option_t *badopt = 
             svn_opt_get_option_from_code (opt_id, options_table);
           svn_opt_format_option (&optstr, badopt, FALSE, pool);
-          fprintf (stderr,
-                   "subcommand '%s' doesn't accept option '%s'\n"
-                   "Type 'svnlook help %s' for usage.\n",
-                   subcommand->name, optstr, subcommand->name);
+          svn_error_clear
+            (svn_cmdline_fprintf
+             (stderr, pool,
+              _("subcommand '%s' doesn't accept option '%s'\n"
+                "Type 'svnlook help %s' for usage.\n"),
+              subcommand->name, optstr, subcommand->name));
           svn_pool_destroy (pool);
           return EXIT_FAILURE;
         }
@@ -2063,6 +2099,9 @@ main (int argc, const char * const *argv)
   else
     {
       svn_pool_destroy (pool);
+      /* Ensure everything is printed on stdout, so the user sees any
+         print errors. */
+      SVN_INT_ERR (svn_cmdline_fflush (stdout));
       return EXIT_SUCCESS;
     }
 }
