@@ -6,7 +6,7 @@
 #  See http://subversion.tigris.org for more information.
 #    
 # ====================================================================
-# Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2004 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -21,6 +21,7 @@ import string, sys, re, os, shutil
 
 # Our testing module
 import svntest
+from svntest import SVNAnyOutput
 
 
 ######################################################################
@@ -67,7 +68,6 @@ def guarantee_repos_and_wc(sbox):
   global max_revision
 
   sbox.build()
-
   wc_path = sbox.wc_dir
 
   # Now we have a repos and wc at revision 1.
@@ -194,30 +194,30 @@ def parse_log_output(log_lines):
   # Here's some log output to look at while writing this function:
   
   # ------------------------------------------------------------------------
-  # rev 5:  kfogel | Tue 6 Nov 2001 17:18:19 | 1 line
+  # r5 | kfogel | Tue 6 Nov 2001 17:18:19 | 1 line
   # 
   # Log message for revision 5.
   # ------------------------------------------------------------------------
-  # rev 4:  kfogel | Tue 6 Nov 2001 17:18:18 | 1 line
+  # r4 | kfogel | Tue 6 Nov 2001 17:18:18 | 1 line
   # 
   # Log message for revision 4.
   # ------------------------------------------------------------------------
-  # rev 3:  kfogel | Tue 6 Nov 2001 17:18:17 | 1 line
+  # r3 | kfogel | Tue 6 Nov 2001 17:18:17 | 1 line
   # 
   # Log message for revision 3.
   # ------------------------------------------------------------------------
-  # rev 2:  kfogel | Tue 6 Nov 2001 17:18:16 | 1 line
+  # r2 | kfogel | Tue 6 Nov 2001 17:18:16 | 1 line
   # 
   # Log message for revision 2.
   # ------------------------------------------------------------------------
-  # rev 1:  foo | Tue 6 Nov 2001 15:27:57 | 1 line
+  # r1 | foo | Tue 6 Nov 2001 15:27:57 | 1 line
   # 
   # Log message for revision 1.
   # ------------------------------------------------------------------------
 
   # Regular expression to match the header line of a log message, with
   # these groups: (revision number), (author), (date), (num lines).
-  header_re = re.compile ('^rev ([0-9]+):  ' \
+  header_re = re.compile ('^r([0-9]+) \| ' \
                           + '([^|]*) \| ([^|]*) \| ([0-9]+) lines?')
 
   # The log chain to return.
@@ -311,27 +311,22 @@ def check_log_chain (chain, revlist):
 
 #----------------------------------------------------------------------
 def plain_log(sbox):
-  "'svn log', no args, top of wc."
+  "'svn log', no args, top of wc"
 
   guarantee_repos_and_wc(sbox)
 
-  result = 0
-
   was_cwd = os.getcwd()
   os.chdir(sbox.wc_dir)
 
-  output, errput = svntest.main.run_svn (None, 'log')
+  try:
+    output, err = svntest.actions.run_and_verify_svn ("", None, [], 'log')
 
-  if errput:
+    log_chain = parse_log_output (output)
+    if check_log_chain (log_chain, range(max_revision, 1 - 1, -1)):
+      raise svntest.Failure
+    
+  finally:
     os.chdir (was_cwd)
-    raise svntest.Failure
-
-  log_chain = parse_log_output (output)
-  if check_log_chain (log_chain, range(max_revision, 1 - 1, -1)):
-    os.chdir (was_cwd)
-    raise svntest.Failure
-
-  os.chdir (was_cwd)
 
 
 #----------------------------------------------------------------------
@@ -343,73 +338,55 @@ def versioned_log_message(sbox):
   was_cwd = os.getcwd ()
   os.chdir (sbox.wc_dir)
 
-  iota_path = os.path.join ('iota')
-  mu_path = os.path.join ('A', 'mu')
-  log_path = os.path.join ('A', 'D', 'H', 'omega')
+  try:
+    iota_path = os.path.join ('iota')
+    mu_path = os.path.join ('A', 'mu')
+    log_path = os.path.join ('A', 'D', 'H', 'omega')
+    
+    svntest.main.file_append (iota_path, "2")
+    
+    # try to check in a change using a versioned file as your log entry.
+    svntest.actions.run_and_verify_svn("", None, SVNAnyOutput,
+                                       'ci', '-F', log_path)
 
-  svntest.main.file_append (iota_path, "2")
+    # force it.  should not produce any errors.
+    svntest.actions.run_and_verify_svn ("", None, [],
+                                        'ci', '-F', log_path, '--force-log')
 
-  # try to check in a change using a versioned file as your log entry.
-  stdout_lines, stderr_lines = svntest.main.run_svn (1, 'ci', '-F', log_path)
+    svntest.main.file_append (mu_path, "2")
 
-  # make sure we failed.
-  if (len(stderr_lines) <= 0):
+    # try the same thing, but specifying the file to commit explicitly.
+    svntest.actions.run_and_verify_svn("", None, SVNAnyOutput,
+                                       'ci', '-F', log_path, mu_path)
+
+    # force it...  should succeed.
+    svntest.actions.run_and_verify_svn ("", None, [],
+                                        'ci',
+                                        '-F', log_path,
+                                        '--force-log', mu_path)
+
+  finally:
     os.chdir (was_cwd)
-    raise svntest.Failure
-
-  # force it.  should not produce any errors.
-  stdout_lines, stderr_lines = \
-    svntest.main.run_svn (None, 'ci', '-F', log_path, '--force')
-
-  if (len(stderr_lines) != 0):
-    os.chdir (was_cwd)
-    raise svntest.Failure
-
-  svntest.main.file_append (mu_path, "2")
-
-  # try the same thing, but specifying the file to commit explicitly.
-  stdout_lines, stderr_lines = \
-    svntest.main.run_svn (1, 'ci', '-F', log_path, mu_path)
-
-  # make sure it failed.
-  if (len(stderr_lines) <= 0):
-    os.chdir (was_cwd)
-    raise svntest.Failure
-
-  # force it...  should succeed.
-  stdout_lines, stderr_lines = \
-    svntest.main.run_svn (None, 'ci', '-F', log_path, '--force', mu_path)
-
-  if (len(stderr_lines) != 0):
-    os.chdir (was_cwd)
-    raise svntest.Failure
-
-  os.chdir (was_cwd)
 
 
 #----------------------------------------------------------------------
 def log_with_empty_repos(sbox):
-  "test 'svn log' on an empty repository"
+  "'svn log' on an empty repository"
 
   # Create virgin repos
-  if os.path.exists(sbox.repo_dir):
-    shutil.rmtree(sbox.repo_dir)
+  svntest.main.safe_rmtree(sbox.repo_dir, 1)
   svntest.main.create_repos(sbox.repo_dir)
   svntest.main.set_repos_paths(sbox.repo_dir)
 
-  stdout_lines, stderr_lines = svntest.main.run_svn\
-                               (None, 'log',
-                                '--username', svntest.main.wc_author,
-                                '--password', svntest.main.wc_passwd,
-                                svntest.main.current_repo_url)
-
-  if (len(stderr_lines) != 0):
-    raise svntest.Failure
-
+  svntest.actions.run_and_verify_svn ("", None, [],
+                                      'log',
+                                      '--username', svntest.main.wc_author,
+                                      '--password', svntest.main.wc_passwd,
+                                      svntest.main.current_repo_url)
 
 #----------------------------------------------------------------------
 def log_where_nothing_changed(sbox):
-  "test 'svn log -rN some_dir_unchanged_in_N'"
+  "'svn log -rN some_dir_unchanged_in_N'"
   sbox.build()
 
   # Fix bug whereby running 'svn log -rN SOMEPATH' would result in an
@@ -434,41 +411,111 @@ def log_where_nothing_changed(sbox):
 
 #----------------------------------------------------------------------
 def log_to_revision_zero(sbox):
-  "Make sure 'svn log -v -r 1:0 wc_root' doesn't seg fault the server."
+  "'svn log -v -r 1:0 wc_root'"
   sbox.build()
-  stdout_lines, stderr_lines = svntest.main.run_svn(0, 'log', '-v',
-                                                    '-r', '1:0', sbox.wc_dir)
-  if stderr_lines:
-    raise svntest.Failure
 
+  # This used to segfault the server.
+  
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'log', '-v',
+                                     '-r', '1:0', sbox.wc_dir)
 
 #----------------------------------------------------------------------
 def log_with_path_args(sbox):
-  "'svn log', no args, top of wc."
+  "'svn log', with args, top of wc"
 
   guarantee_repos_and_wc(sbox)
 
-  result = 0
-
   was_cwd = os.getcwd()
   os.chdir(sbox.wc_dir)
 
-  output, errput = svntest.main.run_svn (None, 'log',
-                                         svntest.main.current_repo_url,
-                                         'A/D/G', 'A/D/H')
+  try:
+    output, err = svntest.actions.run_and_verify_svn(
+      None, None, [],
+      'log', svntest.main.current_repo_url, 'A/D/G', 'A/D/H')
 
-  if errput:
+    log_chain = parse_log_output (output)
+    if check_log_chain (log_chain, [8, 6, 5, 3, 1]):
+      raise svntest.Failure
+
+  finally:
     os.chdir (was_cwd)
-    raise svntest.Failure
 
+#----------------------------------------------------------------------
+def url_missing_in_head(sbox):
+  "'svn log -r N URL' when URL is not in HEAD "
+
+  guarantee_repos_and_wc(sbox)
+
+  my_url = svntest.main.current_repo_url + "/A/B/E/alpha"
+  
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'log', '-r', '8', my_url)
+
+#----------------------------------------------------------------------
+def log_through_copyfrom_history(sbox):
+  "'svn log TGT' with copyfrom history"
+  sbox.build()
+  wc_dir = sbox.wc_dir
+
+  mu_path = os.path.join (wc_dir, 'A', 'mu')
+  mu2_path = os.path.join (wc_dir, 'A', 'mu2')
+  mu_URL = svntest.main.current_repo_url + '/A/mu'
+  mu2_URL = svntest.main.current_repo_url + '/A/mu2'
+
+  svntest.main.file_append (mu_path, "2")
+  svntest.actions.run_and_verify_svn (None, None, [], 'ci', wc_dir,
+                                      '-m', "Log message for revision 2")
+  svntest.main.file_append (mu2_path, "this is mu2")
+  svntest.actions.run_and_verify_svn (None, None, [], 'add', mu2_path)
+  svntest.actions.run_and_verify_svn (None, None, [], 'ci', wc_dir,
+                                      '-m', "Log message for revision 3")
+  svntest.actions.run_and_verify_svn (None, None, [], 'rm', mu2_path)
+  svntest.actions.run_and_verify_svn (None, None, [], 'ci', wc_dir,
+                                      '-m', "Log message for revision 4")
+  svntest.main.file_append (mu_path, "5")
+  svntest.actions.run_and_verify_svn (None, None, [], 'ci', wc_dir,
+                                      '-m', "Log message for revision 5")
+
+  svntest.actions.run_and_verify_svn (None, None, [],
+                                      'cp', '-r', '5', mu_URL, mu2_URL,
+                                      '-m', "Log message for revision 6")
+  svntest.actions.run_and_verify_svn (None, None, [], 'up', wc_dir)
+
+  # The full log for mu2 is relatively unsurprising
+  output, err = svntest.actions.run_and_verify_svn (None, None, [],
+                                                    'log', mu2_path)
   log_chain = parse_log_output (output)
-  if check_log_chain (log_chain, [8, 6, 5, 3, 1]):
-    os.chdir (was_cwd)
+  if check_log_chain (log_chain, [6, 5, 2, 1]):
     raise svntest.Failure
 
-  os.chdir (was_cwd)
+  output, err = svntest.actions.run_and_verify_svn (None, None, [],
+                                                    'log', mu2_URL)
+  log_chain = parse_log_output (output)
+  if check_log_chain (log_chain, [6, 5, 2, 1]):
+    raise svntest.Failure
 
+  # First "oddity", the full log for mu2 doesn't include r3, but the -r3
+  # log works!
+  output, err = svntest.actions.run_and_verify_svn (None, None, [],
+                                                    'log', '-r', '3', mu2_path)
+  log_chain = parse_log_output (output)
+  if check_log_chain (log_chain, [3]):
+    raise svntest.Failure
 
+  output, err = svntest.actions.run_and_verify_svn (None, None, [],
+                                                    'log', '-r', '3', mu2_URL)
+  log_chain = parse_log_output (output)
+  if check_log_chain (log_chain, [3]):
+    raise svntest.Failure
+
+  # Second "oddity", the full log for mu2 includes r2, but the -r2 log
+  # fails!
+  svntest.actions.run_and_verify_svn (None, [], SVNAnyOutput,
+                                      'log', '-r', '2', mu2_path)
+  svntest.actions.run_and_verify_svn (None, [], SVNAnyOutput,
+                                      'log', '-r', '2', mu2_URL)
+  
 ########################################################################
 # Run the tests
 
@@ -481,6 +528,8 @@ test_list = [ None,
               log_where_nothing_changed,
               log_to_revision_zero,
               log_with_path_args,
+              url_missing_in_head,
+              log_through_copyfrom_history,
              ]
 
 if __name__ == '__main__':
