@@ -6,7 +6,7 @@
 #  See http://subversion.tigris.org for more information.
 #    
 # ====================================================================
-# Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2004 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -32,8 +32,10 @@ Item = svntest.wc.StateItem
 # Helper functions
 def check_prop(name, path, exp_out):
   """Verify that property NAME on PATH has a value of EXP_OUT"""
+  # Not using run_svn because binary_mode must be set
   out, err = svntest.main.run_command(svntest.main.svn_binary, None, 1,
-                                      'pg', '--strict', name, path)
+                                      'pg', '--strict', name, path,
+                                      '--config-dir', svntest.main.config_dir)
   if out != exp_out:
     print "svn pg --strict", name, "output does not match expected."
     print "Expected standard output: ", exp_out, "\n"
@@ -51,7 +53,6 @@ def make_local_props(sbox):
 
   # Bootstrap
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   # Add properties to one file and one directory
@@ -86,7 +87,7 @@ def make_local_props(sbox):
   actual_disk_tree = svntest.tree.build_tree_from_wc(wc_dir, 1)
 
   # Compare actual vs. expected disk trees.
-  svntest.tree.compare_trees(expected_disk.old_tree(), actual_disk_tree)
+  svntest.tree.compare_trees(actual_disk_tree, expected_disk.old_tree())
 
 #----------------------------------------------------------------------
 
@@ -95,7 +96,6 @@ def commit_props(sbox):
 
   # Bootstrap
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   # Add a property to a file and a directory
@@ -133,7 +133,6 @@ def update_props(sbox):
 
   # Bootstrap
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   # Make a backup copy of the working copy
@@ -197,7 +196,6 @@ def downdate_props(sbox):
 
   # Bootstrap
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   iota_path = os.path.join(wc_dir, 'iota') 
@@ -270,7 +268,6 @@ def remove_props(sbox):
 
   # Bootstrap
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   # Add a property to a file
@@ -312,17 +309,16 @@ def detect_conflict_files(node, extra_files):
     mo = re.match(pattern, node.name)
     if mo:
       extra_files.pop(extra_files.index(pattern)) # delete pattern from list
-      return
-
-  print "Found unexpected disk object:", node.name
-  raise svntest.tree.SVNTreeUnequal
+      break
+  else:
+    print "Found unexpected disk object:", node.name
+    raise svntest.tree.SVNTreeUnequal
 
 def update_conflict_props(sbox):
   "update with conflicting props"
 
   # Bootstrap
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   # Add a property to a file and a directory
@@ -368,11 +364,11 @@ def update_conflict_props(sbox):
 
   if len(extra_files) != 0:
     print "didn't get expected conflict files"
-    raise svntest.SVNUnexpectedOutput
+    raise svntest.actions.SVNUnexpectedOutput
 
   # Resolve the conflicts
-  svntest.main.run_svn(None, 'resolve', mu_path)
-  svntest.main.run_svn(None, 'resolve', A_path)
+  svntest.main.run_svn(None, 'resolved', mu_path)
+  svntest.main.run_svn(None, 'resolved', A_path)
 
   expected_status = svntest.actions.get_virginal_state(wc_dir, 2)
   expected_status.tweak('A/mu', 'A', status=' M')
@@ -392,7 +388,6 @@ def commit_replacement_props(sbox):
 
   # Bootstrap
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   # Add a property to two files
@@ -451,7 +446,6 @@ def revert_replacement_props(sbox):
 
   # Bootstrap
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   # Add a property to two files
@@ -519,8 +513,8 @@ def inappropriate_props(sbox):
 
   # Bootstrap
   sbox.build()
-
   wc_dir = sbox.wc_dir
+  
   A_path = os.path.join(wc_dir, 'A')
   E_path = os.path.join(wc_dir, 'A', 'B', 'E')
   iota_path = os.path.join(wc_dir, 'iota')
@@ -555,6 +549,26 @@ def inappropriate_props(sbox):
                                      'svn:externals',
                                      'foo http://host.com/repos', iota_path)
 
+  svntest.actions.run_and_verify_svn('Illegal target', None,
+                                     svntest.SVNAnyOutput, 'propset',
+                                     'svn:author', 'socrates', iota_path)
+  
+  svntest.actions.run_and_verify_svn('Illegal target', None,
+                                     svntest.SVNAnyOutput, 'propset',
+                                     'svn:log', 'log message', iota_path)
+ 
+  svntest.actions.run_and_verify_svn('Illegal target', None,
+                                     svntest.SVNAnyOutput, 'propset',
+                                     'svn:date', 
+				     'Tue Jan 19 04:14:07 2038',
+				     iota_path)
+  
+  svntest.actions.run_and_verify_svn('Illegal target', None,
+                                     svntest.SVNAnyOutput, 'propset',
+                                     'svn:original-date',
+				     'Thu Jan  1 01:00:00 1970',
+				     iota_path)
+
   # Status unchanged
   svntest.actions.run_and_verify_status(wc_dir, expected_status)
 
@@ -615,12 +629,11 @@ def inappropriate_props(sbox):
 # and svn:mime-type values as though the file is brand new, instead
 # use the copied file's property values.
 
-def copy_should_use_copied_executable_and_mime_type_values(sbox):
-  "copying a file should use the original svn:executable and svn:mime-type"
+def copy_inherits_special_props(sbox):
+  "file copies inherit (not re-derive) special props"
 
   # Bootstrap
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   orig_mime_type = 'image/fake_image'
@@ -664,7 +677,7 @@ def copy_should_use_copied_executable_and_mime_type_values(sbox):
     print "svn pg svn:mime-type output does not match expected."
     print "Expected standard output: ", expected_stdout, "\n"
     print "Actual standard output: ", actual_stdout, "\n"
-    raise svntest.SVNUnexpectedOutput
+    raise svntest.actions.SVNUnexpectedOutput
 
   # Check the svn:executable value.
   # The value of the svn:executable property is now always forced to '*'
@@ -678,12 +691,12 @@ def copy_should_use_copied_executable_and_mime_type_values(sbox):
       print "svn pg svn:executable output does not match expected."
       print "Expected standard output: ", expected_stdout, "\n"
       print "Actual standard output: ", actual_stdout, "\n"
-      raise svntest.SVNUnexpectedOutput
+      raise svntest.actions.SVNUnexpectedOutput
 
 #----------------------------------------------------------------------
 
 def revprop_change(sbox):
-  "set and get a revprop change"
+  "set, get, and delete a revprop change"
   
   sbox.build()
 
@@ -700,6 +713,23 @@ def revprop_change(sbox):
                                      'propget', '--revprop', '-r', '0',
                                      'cash-sound', sbox.wc_dir)
 
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'propdel', '--revprop', '-r', '0',
+                                     'cash-sound', sbox.wc_dir)
+
+  actual_stdout, actual_stderr = svntest.main.run_svn(None,
+                                                      'pg', '--revprop',
+                                                      '-r', '0',
+                                                      'cash-sound',
+                                                      sbox.wc_dir)
+
+  # The property should have been deleted.
+  regex = 'cha-ching'
+  for line in actual_stdout:
+    if re.match(regex, line):
+      raise svntest.Failure
+
+
 #----------------------------------------------------------------------
 
 def prop_value_conversions(sbox):
@@ -707,8 +737,8 @@ def prop_value_conversions(sbox):
 
   # Bootstrap
   sbox.build()
-
   wc_dir = sbox.wc_dir
+  
   A_path = os.path.join(wc_dir, 'A')
   B_path = os.path.join(wc_dir, 'A', 'B')
   iota_path = os.path.join(wc_dir, 'iota')
@@ -841,32 +871,6 @@ def binary_props(sbox):
   prop_xml  = "This property has an <xml> tag."
   prop_binx = "This property has an <xml> tag and a zer\000 byte."
   
-  if re.search(r'^http', sbox.repo_url):
-
-    ### Remove this block of code after the final step of issue 1015
-    ### is implemented.  In its current intermediate state, ra-dav
-    ### servers can send binary props, but clients can't.  So, for
-    ### now, use ra_local to set some binary properties when ra-dav is
-    ### the main test schema.  Once the clients learn to transmit
-    ### binary props, we can lose this custom replacement of our main
-    ### working copy.
-    
-    wc_url = svntest.main.file_schema_prefix + \
-             os.path.abspath(os.getcwd()) + \
-             '/' + sbox.repo_dir
-
-    # Blow away our existing main working copy in favor of a new
-    # ra-local-based one.
-    svntest.main.safe_rmtree(wc_dir)
-
-    # Checkout over ra-local.
-    output, errput = \
-            svntest.main.run_svn (None, 'checkout',
-                                  '--username', svntest.main.wc_author,
-                                  '--password', svntest.main.wc_passwd,
-                                  wc_url, wc_dir)
-    if errput: raise svntest.Failure
-
   # Set some binary properties.
   propval_path = os.path.join(wc_dir, 'propval.tmp')
   propval_file = open(propval_path, 'wb')
@@ -932,6 +936,159 @@ def binary_props(sbox):
   check_prop('prop_binx', mu_path_bak, [prop_binx])
   check_prop('prop_binx', A_path_bak, [prop_binx])
 
+#----------------------------------------------------------------------
+
+def recursive_base_wc_ops(sbox):
+  "recursive property operations in BASE and WC"
+
+  # Bootstrap
+  sbox.build()
+  wc_dir = sbox.wc_dir
+
+  # Files with which to test, in alphabetical order
+  f_add = os.path.join('A', 'added')
+  f_del = os.path.join('A', 'mu')
+  f_keep= os.path.join('iota')
+  fp_add = os.path.join(wc_dir, f_add)
+  fp_del = os.path.join(wc_dir, f_del)
+  fp_keep= os.path.join(wc_dir, f_keep)
+
+  # Set up properties
+  svntest.main.run_svn(None, 'propset', 'p', 'old-del', fp_del)
+  svntest.main.run_svn(None, 'propset', 'p', 'old-keep',fp_keep)
+  svntest.main.run_svn(None, 'commit', '-m', '', wc_dir)
+  svntest.main.file_append(fp_add, 'blah')
+  svntest.main.run_svn(None, 'add', fp_add)
+  svntest.main.run_svn(None, 'propset', 'p', 'new-add', fp_add)
+  svntest.main.run_svn(None, 'propset', 'p', 'new-del', fp_del)
+  svntest.main.run_svn(None, 'propset', 'p', 'new-keep',fp_keep)
+  svntest.main.run_svn(None, 'del', '--force', fp_del)
+
+  # Ensure that each line of output contains the corresponding string of
+  # expected_out, and that errput is empty.
+  def verify_output(expected_out, output, errput):
+    if errput != []:
+      print 'Error: stderr:'
+      print errput
+      raise svntest.Failure
+    output.sort()
+    ln = 0
+    for line in output:
+      if ((line.find(expected_out[ln]) == -1) or
+          (line != '' and expected_out[ln] == '')):
+        print 'Error: expected keywords: ', expected_out
+        print '       actual full output:', output
+        raise svntest.Failure
+      ln = ln + 1
+
+  # Test recursive proplist
+  output, errput = svntest.main.run_svn(None, 'proplist', '-R', '-v', wc_dir,
+                                        '-rBASE') 
+  verify_output([ 'old-del', 'old-keep', 'Properties on ', 'Properties on ' ],
+                output, errput)
+  
+  output, errput = svntest.main.run_svn(None, 'proplist', '-R', '-v', wc_dir)
+  verify_output([ 'new-add', 'new-keep', 'Properties on ', 'Properties on ' ],
+                output, errput)
+  
+  # Test recursive propget
+  output, errput = svntest.main.run_svn(None, 'propget', '-R', 'p', wc_dir,
+                                        '-rBASE') 
+  verify_output([ 'old-del', 'old-keep' ], output, errput)
+  
+  output, errput = svntest.main.run_svn(None, 'propget', '-R', 'p', wc_dir)
+  verify_output([ 'new-add', 'new-keep' ], output, errput)
+
+  # Test recursive propset (issue 1794)
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
+  expected_status.tweak(repos_rev=2)
+  expected_status.tweak('A/mu', status='D ', wc_rev=2)
+  expected_status.tweak('iota', status=' M', wc_rev=2)
+  expected_status.add({
+    'A/added'     : Item(status='A ', repos_rev=2, wc_rev=0),
+    })
+  svntest.actions.run_and_verify_status(wc_dir, expected_status)
+
+  svntest.actions.run_and_verify_svn(None, None, [], 
+                                     'propset', '-R', 'svn:keywords', 'Date',
+                                     os.path.join(wc_dir, 'A', 'B'))
+  expected_status.tweak('A/B/lambda', 'A/B/E/alpha', 'A/B/E/beta', status=' M')
+  svntest.actions.run_and_verify_status(wc_dir, expected_status)
+
+#----------------------------------------------------------------------
+
+def url_props_ops(sbox):
+  "property operations on an URL"
+
+  # Bootstrap
+  sbox.build()
+  wc_dir = sbox.wc_dir
+
+  prop1 = 'prop1'
+  propval1 = 'propval1'
+  prop2 = 'prop2'
+  propval2 = 'propval2'
+
+  iota_path = os.path.join(sbox.wc_dir, 'iota')
+  iota_url = svntest.main.current_repo_url + '/iota'
+  A_path = os.path.join(sbox.wc_dir, 'A')
+  A_url = svntest.main.current_repo_url + '/A'
+
+  # Add a couple of properties
+  svntest.main.run_svn(None, 'propset', prop1, propval1, iota_path)
+  svntest.main.run_svn(None, 'propset', prop1, propval1, A_path)
+
+  # Commit
+  svntest.main.run_svn(None, 'ci', '-m', 'logmsg', sbox.wc_dir)
+
+  # Add a few more properties
+  svntest.main.run_svn(None, 'propset', prop2, propval2, iota_path)
+  svntest.main.run_svn(None, 'propset', prop2, propval2, A_path)
+
+  # Commit again
+  svntest.main.run_svn(None, 'ci', '-m', 'logmsg', sbox.wc_dir)
+
+  # Ensure that each line of output contains the corresponding string of
+  # expected_out, and that errput is empty.
+  def verify_output(expected_out, output, errput):
+    if errput != []:
+      print 'Error: stderr:'
+      print errput
+      raise svntest.Failure
+    output.sort()
+    ln = 0
+    for line in output:
+      if ((line.find(expected_out[ln]) == -1) or
+          (line != '' and expected_out[ln] == '')):
+        print 'Error: expected keywords: ', expected_out
+        print '       actual full output:', output
+        raise svntest.Failure
+      ln = ln + 1
+
+  # Test propget
+  svntest.actions.run_and_verify_svn(None, [ propval1 + '\n' ], [],
+                                     'propget', prop1, iota_url)
+  svntest.actions.run_and_verify_svn(None, [ propval1 + '\n' ], [],
+                                     'propget', prop1, A_url)
+
+  # Test normal proplist
+  output, errput = svntest.main.run_svn(None, 'proplist', iota_url)
+  verify_output([ prop1, prop2, 'Properties on ' ],
+                output, errput)
+
+  output, errput = svntest.main.run_svn(None, 'proplist', A_url)
+  verify_output([ prop1, prop2, 'Properties on ' ],
+                output, errput)
+
+  # Test verbose proplist
+  output, errput = svntest.main.run_svn(None, 'proplist', '-v', iota_url)
+  verify_output([ prop1 + ' : ' + propval1, prop2 + ' : ' + propval2,
+                  'Properties on ' ], output, errput)
+
+  output, errput = svntest.main.run_svn(None, 'proplist', '-v', A_url)
+  verify_output([ prop1 + ' : ' + propval1, prop2 + ' : ' + propval2,
+                  'Properties on ' ], output, errput)
+
   
 ########################################################################
 # Run the tests
@@ -948,12 +1105,14 @@ test_list = [ None,
               commit_replacement_props,
               revert_replacement_props,
               inappropriate_props,
-              copy_should_use_copied_executable_and_mime_type_values,
+              copy_inherits_special_props,
               # If we learn how to write a pre-revprop-change hook for
               # non-Posix platforms, we won't have to skip here:
               Skip(revprop_change, (os.name != 'posix')),
               prop_value_conversions,
               binary_props,
+              recursive_base_wc_ops,
+              url_props_ops,
              ]
 
 if __name__ == '__main__':
