@@ -7,7 +7,7 @@
 #  See http://subversion.tigris.org for more information.
 #    
 # ====================================================================
-# Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2004 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -25,8 +25,10 @@ import stat    # for ST_MODE
 import string  # for atof()
 import copy    # for deepcopy()
 import time    # for time()
+import getopt
 
 from svntest import Failure
+from svntest import Skip
 from svntest import testcase
 from svntest import wc
 
@@ -81,32 +83,6 @@ if sys.platform == 'win32':
   windows = 1
   file_schema_prefix = 'file:///'
   _exe = '.exe'
-
-  # svn on windows doesn't support backslashes in path names
-  _os_path_abspath_orig = os.path.abspath
-  def _os_path_abspath(arg):
-    path = _os_path_abspath_orig(arg)
-    return path.replace('\\', '/')
-  os.path.abspath = _os_path_abspath
-
-  _os_path_join_orig = os.path.join
-  def _os_path_join(*args):
-    path = apply(_os_path_join_orig, args)
-    return path.replace('\\', '/')
-  os.path.join = _os_path_join
-
-  _os_path_normpath_orig = os.path.normpath
-  def _os_path_normpath(arg):
-    path = _os_path_normpath_orig(arg)
-    return path.replace('\\', '/')
-  os.path.normpath = _os_path_normpath
-
-  _os_path_dirname_orig = os.path.dirname
-  def _os_path_dirname(arg):
-    path = _os_path_dirname_orig(arg)
-    return path.replace('\\', '/')
-  os.path.dirname = _os_path_dirname
-
 else:
   windows = 0
   file_schema_prefix = 'file://'
@@ -131,6 +107,11 @@ cleanup_mode = 0
 
 # Global URL to testing area.  Default to ra_local, current working dir.
 test_area_url = file_schema_prefix + os.path.abspath(os.getcwd())
+if windows == 1:
+  test_area_url = string.replace(test_area_url, '\\', '/')
+
+# Global variable indicating the FS type for repository creations.
+fs_type = "bdb"
 
 # Where we want all the repositories and working copies to live.
 # Each test will have its own!
@@ -149,6 +130,8 @@ temp_dir = 'local_tmp'
 # (derivatives of the tmp dir.)
 pristine_dir = os.path.join(temp_dir, "repos")
 greek_dump_dir = os.path.join(temp_dir, "greekfiles")
+config_dir = os.path.abspath(os.path.join(temp_dir, "config"))
+default_config_dir = config_dir
 
 
 #
@@ -245,9 +228,24 @@ def run_command(command, error_expected, binary_mode=0, *varargs):
 
   if (not error_expected) and (stderr_lines):
     map(sys.stdout.write, stderr_lines)
+    raise Failure
 
   return stdout_lines, stderr_lines
 
+def set_config_dir(cfgdir):
+  "Set the config directory."
+
+  global config_dir
+  config_dir = cfgdir
+
+def reset_config_dir():
+  "Reset the config directory to the default value."
+
+  global config_dir
+  global default_config_dir
+
+  config_dir = default_config_dir
+
 # For running subversion and returning the output
 def run_svn(error_expected, *varargs):
   """Run svn with VARARGS; return stdout, stderr as lists of lines.
@@ -255,7 +253,9 @@ def run_svn(error_expected, *varargs):
   you're just checking that something does/doesn't come out of
   stdout/stderr, you might want to use actions.run_and_verify_svn() or
   actions.run_and_verify_svn_error()."""
-  return run_command(svn_binary, error_expected, 0, *varargs)
+  global config_dir
+  return run_command(svn_binary, error_expected, 0,
+                     *varargs + ('--config-dir', config_dir))
 
 # For running svnadmin.  Ignores the output.
 def run_svnadmin(*varargs):
@@ -283,13 +283,27 @@ def chmod_tree(path, mode, mask):
   os.path.walk(path, visit, (mode, mask))
 
 # For clearing away working copies
-def safe_rmtree(dirname):
+def safe_rmtree(dirname, retry=0):
   "Remove the tree at DIRNAME, making it writable first"
-
-  if os.path.exists(dirname):
+  def rmtree(dirname):
     chmod_tree(dirname, 0666, 0666)
     shutil.rmtree(dirname)
 
+  if not os.path.exists(dirname):
+    return
+
+  if retry:
+    for delay in (0.5, 1, 2, 4):
+      try:
+        rmtree(dirname)
+        break
+      except:
+        time.sleep(delay)
+    else:
+      rmtree(dirname)
+  else:
+    rmtree(dirname)
+
 # For making local mods to files
 def file_append(path, new_text):
   "Append NEW_TEXT to file at PATH"
@@ -305,7 +319,19 @@ def create_repos(path):
 
   if not(os.path.exists(path)):
     os.makedirs(path) # this creates all the intermediate dirs, if neccessary
-  run_svnadmin("create", path, "--bdb-txn-nosync")
+
+  stdout, stderr = run_command(svnadmin_binary, 1, 0, "create", path,
+                               "--bdb-txn-nosync", "--fs-type="+fs_type)
+
+  # Skip tests if we can't create the repository (most likely, because
+  # Subversion was built without BDB and this is a default test run).
+  for line in stderr:
+    if line.find('Unknown FS type') != -1:
+      raise Skip
+
+  # Allow unauthenticated users to write to the repos, for ra_svn testing.
+  file_append(os.path.join(path, "conf", "svnserve.conf"),
+              "[general]\nanon-access = write\n");
 
   # make the repos world-writeable, for mod_dav_svn's sake.
   chmod_tree(path, 0666, 0666)
@@ -379,6 +405,8 @@ def set_repos_paths(repo_dir):
   global current_repo_dir, current_repo_url
   current_repo_dir = repo_dir
   current_repo_url = test_area_url + '/' + repo_dir
+  if windows == 1:
+    current_repo_url = string.replace(current_repo_url, '\\', '/')
 
 
 ######################################################################
@@ -392,6 +420,8 @@ class Sandbox:
     self.wc_dir = os.path.join(general_wc_dir, self.name)
     self.repo_dir = os.path.join(general_repo_dir, self.name)
     self.repo_url = test_area_url + '/' + self.repo_dir
+    if windows == 1:
+      self.repo_url = string.replace(self.repo_url, '\\', '/')
     self.test_paths = [self.wc_dir, self.repo_dir]
 
   def build(self):
@@ -416,9 +446,29 @@ class Sandbox:
 
   def cleanup_test_paths(self):
     for path in self.test_paths:
-      if verbose_mode:
-        print "CLEANUP: " + path
-      safe_rmtree(path)
+      _cleanup_test_path(path)
+
+
+_deferred_test_paths = []
+def _cleanup_deferred_test_paths():
+  global _deferred_test_paths
+  test_paths = _deferred_test_paths[:]
+  _deferred_test_paths = []
+  for path in test_paths:
+    _cleanup_test_path(path, 1)
+
+def _cleanup_test_path(path, retrying=None):
+  if verbose_mode:
+    if retrying:
+      print "CLEANUP: RETRY:", path
+    else:
+      print "CLEANUP:", path
+  try:
+    safe_rmtree(path)
+  except:
+    if verbose_mode:
+      print "WARNING: cleanup failed, will try again later"
+    _deferred_test_paths.append(path)
 
 ######################################################################
 # Main testing functions
@@ -458,6 +508,7 @@ def run_one_test(n, test_list):
   exit_code = tc.run(args)
   if sandbox is not None and not exit_code and cleanup_mode:
     sandbox.cleanup_test_paths()
+  reset_config_dir()
   return exit_code
 
 def _internal_run_tests(test_list, testnum=None):
@@ -470,8 +521,10 @@ def _internal_run_tests(test_list, testnum=None):
   else:
     exit_code = run_one_test(testnum, test_list)
 
+  _cleanup_deferred_test_paths()
   return exit_code
 
+
 # Main func.  This is the "entry point" that all the test scripts call
 # to run their list of tests.
 #
@@ -490,14 +543,22 @@ def run_tests(test_list):
   """
 
   global test_area_url
+  global fs_type
   global verbose_mode
   global cleanup_mode
   testnum = None
 
-  url_re = re.compile('^(?:--url|BASE_URL)=(.+)')
+  # Explicitly set this so that commands that commit but don't supply a
+  # log message will fail rather than invoke an editor.
+  os.environ['SVN_EDITOR'] = ''
 
-  for arg in sys.argv:
+  try:
+    opts, args = getopt.getopt(sys.argv[1:], 'v',
+                               ['url=', 'fs-type=', 'verbose', 'cleanup'])
+  except getopt.GetoptError:
+    args = []
 
+  for arg in args:
     if arg == "list":
       print "Test #  Mode   Test Description"
       print "------  -----  ----------------"
@@ -509,33 +570,34 @@ def run_tests(test_list):
       # done. just exit with success.
       sys.exit(0)
 
-    elif arg == "--url":
-      index = sys.argv.index(arg)
-      test_area_url = sys.argv[index + 1]
+    if arg.startswith('BASE_URL='):
+      test_area_url = arg[9:]
+    else:
+      try:
+        testnum = int(arg)
+      except ValueError:
+        pass
 
-    elif arg == "-v" or arg == "--verbose":
+  for opt, val in opts:
+    if opt == "--url":
+      test_area_url = val
+
+    elif opt == "--fs-type":
+      fs_type = val
+
+    elif opt == "-v" or opt == "--verbose":
       verbose_mode = 1
 
-    elif arg == "--cleanup":
+    elif opt == "--cleanup":
       cleanup_mode = 1
 
-    else:
-      match = url_re.search(arg)
-      if match:
-        test_area_url = match.group(1)
-
-      else:
-        try:
-          testnum = int(arg)
-        except ValueError:
-          pass
-
   exit_code = _internal_run_tests(test_list, testnum)
 
   # remove all scratchwork: the 'pristine' repository, greek tree, etc.
   # This ensures that an 'import' will happen the next time we run.
-  if os.path.exists(temp_dir):
-    shutil.rmtree(temp_dir)
+  safe_rmtree(temp_dir)
+
+  _cleanup_deferred_test_paths()
 
   # return the appropriate exit code from the tests.
   sys.exit(exit_code)
@@ -547,8 +609,7 @@ def run_tests(test_list):
 # Cleanup: if a previous run crashed or interrupted the python
 # interpreter, then `temp_dir' was never removed.  This can cause wonkiness.
 
-if os.path.exists(temp_dir):
-  shutil.rmtree(temp_dir)
+safe_rmtree(temp_dir)
 
 # the modules import each other, so we do this import very late, to ensure
 # that the definitions in "main" have been completed.
