@@ -1,7 +1,7 @@
 /* fs-helpers.c --- tests for the filesystem
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -26,12 +26,12 @@
 #include "svn_test.h"
 #include "fs-helpers.h"
 
-#include "../libsvn_fs/fs.h"
-#include "../libsvn_fs/dag.h"
-#include "../libsvn_fs/trail.h"
+#include "../libsvn_fs_base/fs.h"
+#include "../libsvn_fs_base/dag.h"
+#include "../libsvn_fs_base/trail.h"
 
-#include "../libsvn_fs/bdb/rev-table.h"
-#include "../libsvn_fs/bdb/nodes-table.h"
+#include "../libsvn_fs_base/bdb/rev-table.h"
+#include "../libsvn_fs_base/bdb/nodes-table.h"
 
 
 /*-------------------------------------------------------------------*/
@@ -90,7 +90,7 @@ svn_test__create_fs (svn_fs_t **fs_p,
         SVN_ERR (svn_fs_delete_berkeley (name, pool));
       else
         return svn_error_createf (SVN_ERR_TEST_FAILED, NULL,
-                                  "there is already a file named `%s'", name);
+                                  "there is already a file named '%s'", name);
     }
 
   SVN_ERR (svn_test__fs_new (fs_p, pool));
@@ -125,18 +125,22 @@ svn_test__create_repos (svn_repos_t **repos_p,
         SVN_ERR (svn_repos_delete (name, pool));
       else
         return svn_error_createf (SVN_ERR_TEST_FAILED, NULL,
-                                  "there is already a file named `%s'", name);
+                                  "there is already a file named '%s'", name);
     }
 
   fs_config = apr_hash_make (pool);
   apr_hash_set (fs_config, SVN_FS_CONFIG_BDB_TXN_NOSYNC,
                 APR_HASH_KEY_STRING, "1");
+  if (getenv ("FS_TYPE"))
+    apr_hash_set (fs_config, SVN_FS_CONFIG_FS_TYPE, APR_HASH_KEY_STRING,
+                  getenv ("FS_TYPE"));
   SVN_ERR (svn_repos_create (repos_p, name, NULL, NULL, NULL,
                              fs_config, pool));
-  
-  /* Provide a handler for Berkeley DB error messages.  */
-  SVN_ERR (svn_fs_set_berkeley_errcall (svn_repos_fs (*repos_p), 
-                                        berkeley_error_handler));
+
+  /* Provide a handler for Berkeley DB error messages if we're using bdb.  */
+  if (getenv ("FS_TYPE") == NULL || strcmp (getenv ("FS_TYPE"), "bdb") == 0)
+    SVN_ERR (svn_fs_set_berkeley_errcall (svn_repos_fs (*repos_p), 
+                                          berkeley_error_handler));
 
   /* Register this repo for cleanup. */
   svn_test_add_dir_cleanup (name);
@@ -240,7 +244,6 @@ get_dir_entries (apr_hash_t *tree_entries,
       void *val;
       svn_fs_dirent_t *dirent;
       const char *full_path;
-      int is_dir;
  
       apr_hash_this (hi, &key, &keylen, &val);
       dirent = val;
@@ -254,8 +257,7 @@ get_dir_entries (apr_hash_t *tree_entries,
       apr_hash_set (tree_entries, full_path, APR_HASH_KEY_STRING, dirent);
 
       /* If this entry is a directory, recurse into the tree. */
-      SVN_ERR (svn_fs_is_dir (&is_dir, root, full_path, pool));
-      if (is_dir)
+      if (dirent->kind == svn_node_dir)
         SVN_ERR (get_dir_entries (tree_entries, root, full_path, pool));
     }
 
@@ -271,14 +273,14 @@ validate_tree_entry (svn_fs_root_t *root,
 {
   svn_stream_t *rstream;
   svn_stringbuf_t *rstring;
-  int is_dir;
+  svn_boolean_t is_dir;
 
   /* Verify that this is the expected type of node */
   SVN_ERR (svn_fs_is_dir (&is_dir, root, path, pool));
   if ((!is_dir && !contents) || (is_dir && contents))
     return svn_error_createf
       (SVN_ERR_FS_GENERAL, NULL,
-       "node `%s' in tree was of unexpected node type", 
+       "node '%s' in tree was of unexpected node type", 
        path);
 
   /* Verify that the contents are as expected (files only) */
@@ -290,7 +292,7 @@ validate_tree_entry (svn_fs_root_t *root,
                                    svn_stringbuf_create (contents, pool)))
         return svn_error_createf 
           (SVN_ERR_FS_GENERAL, NULL,
-           "node `%s' in tree had unexpected contents",
+           "node '%s' in tree had unexpected contents",
            path);
     }
 
@@ -464,7 +466,7 @@ svn_test__txn_script_exec (svn_fs_root_t *txn_root,
           break;
 
         case 'd':
-          SVN_ERR (svn_fs_delete_tree (txn_root, path, pool));
+          SVN_ERR (svn_fs_delete (txn_root, path, pool));
           break;
 
         case 'e':
@@ -518,7 +520,7 @@ svn_test__check_greek_tree (svn_fs_root_t *root,
       content = svn_stringbuf_create (file_contents[i][1], pool);
       if (! svn_stringbuf_compare (rstring, content))
         return svn_error_createf (SVN_ERR_FS_GENERAL, NULL,
-                                 "data read != data written in file `%s'.",
+                                 "data read != data written in file '%s'.",
                                  file_contents[i][0]);
     }
   return SVN_NO_ERROR;
