@@ -2,7 +2,7 @@
  * Incomplete regression tests for the diff/diff3 library.
  *
  * ====================================================================
- * Copyright (c) 2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2003-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -144,6 +144,7 @@ three_way_merge (const char *filename1,
 {
   svn_diff_t *diff;
   apr_file_t *output;
+  svn_stream_t *ostream;
   apr_status_t status;
   svn_stringbuf_t *actual;
   char *merge_name = apr_psprintf (pool, "merge-%s-%s-%s",
@@ -159,12 +160,15 @@ three_way_merge (const char *filename1,
                           pool);
   if (status)
     return svn_error_createf (status, NULL, "failed to open '%s'", merge_name);
-  SVN_ERR (svn_diff_file_output_merge (output, diff,
+
+  ostream = svn_stream_from_aprfile (output, pool);
+  SVN_ERR (svn_diff_file_output_merge (ostream, diff,
                                        filename1, filename2, filename3,
                                        NULL, NULL, NULL, NULL,
                                        FALSE,
                                        FALSE,
                                        pool));
+  SVN_ERR (svn_stream_close (ostream));
   status = apr_file_close (output);
   if (status)
     return svn_error_createf (status, NULL, "failed to close '%s'", merge_name);
@@ -202,6 +206,7 @@ two_way_diff (const char *filename1,
 {
   svn_diff_t *diff;
   apr_file_t *output;
+  svn_stream_t *ostream;
   apr_status_t status;
   svn_stringbuf_t *actual;
   char *diff_name = apr_psprintf (pool, "diff-%s-%s", filename1, filename2);
@@ -217,10 +222,13 @@ two_way_diff (const char *filename1,
                           pool);
   if (status)
     return svn_error_createf (status, NULL, "failed to open '%s'", diff_name);
-  SVN_ERR (svn_diff_file_output_unified (output, diff,
+
+  ostream = svn_stream_from_aprfile (output, pool);
+  SVN_ERR (svn_diff_file_output_unified (ostream, diff,
                                          filename1, filename2,
                                          filename1, filename2,
                                          pool));
+  SVN_ERR (svn_stream_close (ostream));
   status = apr_file_close (output);
   if (status)
     return svn_error_createf (status, NULL, "failed to close '%s'", diff_name);
@@ -365,9 +373,9 @@ dump_core (const char **msg,
 
                          "",
 
-                         "--- foo2\n"
-                         "+++ bar2\n"
-                         "@@ -1,3 +0,0 @@\n"
+                         "--- foo2" APR_EOL_STR
+                         "+++ bar2" APR_EOL_STR
+                         "@@ -1,3 +0,0 @@" APR_EOL_STR
                          "-Aa\n"
                          "-Bb\n"
                          "-Cc\n",
@@ -380,9 +388,9 @@ dump_core (const char **msg,
                          "Bb\n"
                          "Cc\n",
 
-                         "--- foo3\n"
-                         "+++ bar3\n"
-                         "@@ -0,0 +1,3 @@\n"
+                         "--- foo3" APR_EOL_STR
+                         "+++ bar3" APR_EOL_STR
+                         "@@ -0,0 +1,3 @@" APR_EOL_STR
                          "+Aa\n"
                          "+Bb\n"
                          "+Cc\n",
@@ -397,7 +405,7 @@ test_two_way_unified (const char **msg,
                       svn_boolean_t msg_only,
                       apr_pool_t *pool)
 {
-  *msg = "two way unified diff and trivial merge";
+  *msg = "2-way unified diff and trivial merge";
   if (msg_only)
     return SVN_NO_ERROR;
 
@@ -408,9 +416,9 @@ test_two_way_unified (const char **msg,
                          "Bb\n"
                          "Cc\n",
 
-                         "--- foo4\n"
-                         "+++ bar4\n"
-                         "@@ -1 +1,3 @@\n"
+                         "--- foo4" APR_EOL_STR
+                         "+++ bar4" APR_EOL_STR
+                         "@@ -1 +1,3 @@" APR_EOL_STR
                          " Aa\n"
                          "+Bb\n"
                          "+Cc\n",
@@ -423,9 +431,9 @@ test_two_way_unified (const char **msg,
                          "Bb\n"
                          "Cc\n",
 
-                         "--- foo4b\n"
-                         "+++ bar4b\n"
-                         "@@ -1 +1,3 @@\n"
+                         "--- foo4b" APR_EOL_STR
+                         "+++ bar4b" APR_EOL_STR
+                         "@@ -1 +1,3 @@" APR_EOL_STR
                          "+Aa\n"
                          "+Bb\n"
                          " Cc\n",
@@ -438,9 +446,9 @@ test_two_way_unified (const char **msg,
 
                          "Aa\n",
 
-                         "--- foo5\n"
-                         "+++ bar5\n"
-                         "@@ -1,3 +1 @@\n"
+                         "--- foo5" APR_EOL_STR
+                         "+++ bar5" APR_EOL_STR
+                         "@@ -1,3 +1 @@" APR_EOL_STR
                          " Aa\n"
                          "-Bb\n"
                          "-Cc\n",
@@ -453,9 +461,9 @@ test_two_way_unified (const char **msg,
 
                          "Cc\n",
 
-                         "--- foo5b\n"
-                         "+++ bar5b\n"
-                         "@@ -1,3 +1 @@\n"
+                         "--- foo5b" APR_EOL_STR
+                         "+++ bar5b" APR_EOL_STR
+                         "@@ -1,3 +1 @@" APR_EOL_STR
                          "-Aa\n"
                          "-Bb\n"
                          " Cc\n",
@@ -482,9 +490,9 @@ test_two_way_unified (const char **msg,
                          "Xx\n"
                          "Cc\n",
 
-                         "--- foo6b\n"
-                         "+++ bar6b\n"
-                         "@@ -1,3 +1,3 @@\n"
+                         "--- foo6b" APR_EOL_STR
+                         "+++ bar6b" APR_EOL_STR
+                         "@@ -1,3 +1,3 @@" APR_EOL_STR
                          " Aa\n"
                          "-Bb\n"
                          "+Xx\n"
@@ -500,9 +508,9 @@ test_two_way_unified (const char **msg,
                          "Xx\r\n"
                          "Cc\r\n",
 
-                         "--- foo6c\n"
-                         "+++ bar6c\n"
-                         "@@ -1,3 +1,3 @@\n"
+                         "--- foo6c" APR_EOL_STR
+                         "+++ bar6c" APR_EOL_STR
+                         "@@ -1,3 +1,3 @@" APR_EOL_STR
                          " Aa\r\n"
                          "-Bb\r\n"
                          "+Xx\r\n"
@@ -514,9 +522,9 @@ test_two_way_unified (const char **msg,
 
                          "Bb\n",
 
-                         "--- foo7\n"
-                         "+++ bar7\n"
-                         "@@ -1 +1 @@\n"
+                         "--- foo7" APR_EOL_STR
+                         "+++ bar7" APR_EOL_STR
+                         "@@ -1 +1 @@" APR_EOL_STR
                          "-Aa\n"
                          "+Bb\n",
                          pool));
@@ -528,9 +536,9 @@ test_two_way_unified (const char **msg,
                          "Bb\n"
                          "Cc\n",
 
-                         "--- foo7a\n"
-                         "+++ bar7a\n"
-                         "@@ -1,2 +1,2 @@\n"
+                         "--- foo7a" APR_EOL_STR
+                         "+++ bar7a" APR_EOL_STR
+                         "@@ -1,2 +1,2 @@" APR_EOL_STR
                          "-Aa\n"
                          "+Bb\n"
                          " Cc\n",
@@ -544,9 +552,9 @@ test_two_way_unified (const char **msg,
                          "Xx\n"
                          "Yy\n",
 
-                         "--- foo8\n"
-                         "+++ bar8\n"
-                         "@@ -1,3 +1,2 @@\n"
+                         "--- foo8" APR_EOL_STR
+                         "+++ bar8" APR_EOL_STR
+                         "@@ -1,3 +1,2 @@" APR_EOL_STR
                          "-Aa\n"
                          "-Bb\n"
                          "-Cc\n"
@@ -561,9 +569,9 @@ test_two_way_unified (const char **msg,
 
                          "Bb\n",
 
-                         "--- foo9\n"
-                         "+++ bar9\n"
-                         "@@ -1,3 +1 @@\n"
+                         "--- foo9" APR_EOL_STR
+                         "+++ bar9" APR_EOL_STR
+                         "@@ -1,3 +1 @@" APR_EOL_STR
                          "-Aa\n"
                          " Bb\n"
                          "-Cc\n",
@@ -578,13 +586,13 @@ test_two_way_unified (const char **msg,
                          "Xx\n"
                          "Yy\n",
 
-                         "--- foo10\n"
-                         "+++ bar10\n"
-                         "@@ -1,3 +1,3 @@\n"
+                         "--- foo10" APR_EOL_STR
+                         "+++ bar10" APR_EOL_STR
+                         "@@ -1,3 +1,3 @@" APR_EOL_STR
                          " Aa\n"
                          "-Bb\n"
-                         "-Cc\n"
-                         "\\ No newline at end of file\n"
+                         "-Cc" APR_EOL_STR
+                         "\\ No newline at end of file" APR_EOL_STR
                          "+Xx\n"
                          "+Yy\n",
                          pool));
@@ -598,15 +606,15 @@ test_two_way_unified (const char **msg,
                          "Bb\n"
                          "Cc",
 
-                         "--- foo11\n"
-                         "+++ bar11\n"
-                         "@@ -1,3 +1,3 @@\n"
+                         "--- foo11" APR_EOL_STR
+                         "+++ bar11" APR_EOL_STR
+                         "@@ -1,3 +1,3 @@" APR_EOL_STR
                          " Aa\n"
                          "-Xx\n"
                          "-Yy\n"
                          "+Bb\n"
-                         "+Cc\n"
-                         "\\ No newline at end of file\n",
+                         "+Cc" APR_EOL_STR
+                         "\\ No newline at end of file" APR_EOL_STR,
                          pool));
 
   SVN_ERR (two_way_diff ("foo12", "bar12",
@@ -618,16 +626,16 @@ test_two_way_unified (const char **msg,
                          "Bb\n"
                          "Cc",
 
-                         "--- foo12\n"
-                         "+++ bar12\n"
-                         "@@ -1,3 +1,3 @@\n"
+                         "--- foo12" APR_EOL_STR
+                         "+++ bar12" APR_EOL_STR
+                         "@@ -1,3 +1,3 @@" APR_EOL_STR
                          " Aa\n"
                          "-Xx\n"
-                         "-Yy\n"
-                         "\\ No newline at end of file\n"
+                         "-Yy" APR_EOL_STR
+                         "\\ No newline at end of file" APR_EOL_STR
                          "+Bb\n"
-                         "+Cc\n"
-                         "\\ No newline at end of file\n",
+                         "+Cc" APR_EOL_STR
+                         "\\ No newline at end of file" APR_EOL_STR,
                          pool));
 
   SVN_ERR (two_way_diff ("foo13", "bar13",
@@ -649,14 +657,14 @@ test_two_way_unified (const char **msg,
                          "Gg\n"
                          "Yy\n",
 
-                         "--- foo13\n"
-                         "+++ bar13\n"
-                         "@@ -1,3 +1,4 @@\n"
+                         "--- foo13" APR_EOL_STR
+                         "+++ bar13" APR_EOL_STR
+                         "@@ -1,3 +1,4 @@" APR_EOL_STR
                          "+Xx\n"
                          " Aa\n"
                          " Bb\n"
                          " Cc\n"
-                         "@@ -5,3 +6,4 @@\n"
+                         "@@ -5,3 +6,4 @@" APR_EOL_STR
                          " Ee\n"
                          " Ff\n"
                          " Gg\n"
@@ -680,9 +688,9 @@ test_two_way_unified (const char **msg,
                          "Gg\n"
                          "Ff\n",
 
-                         "--- foo14\n"
-                         "+++ bar14\n"
-                         "@@ -1,7 +1,7 @@\n"
+                         "--- foo14" APR_EOL_STR
+                         "+++ bar14" APR_EOL_STR
+                         "@@ -1,7 +1,7 @@" APR_EOL_STR
                          "+Bb\n"
                          " Aa\n"
                          "-Bb\n"
@@ -703,9 +711,9 @@ test_two_way_unified (const char **msg,
                          "Bb\n"
                          "Cc\n",
 
-                         "--- foo16\n"
-                         "+++ bar16\n"
-                         "@@ -1,3 +1,3 @@\n"
+                         "--- foo16" APR_EOL_STR
+                         "+++ bar16" APR_EOL_STR
+                         "@@ -1,3 +1,3 @@" APR_EOL_STR
                          " Aa\n"
                          "-\n"
                          "+Bb\n"
@@ -721,7 +729,7 @@ test_two_way_unified_suspect (const char **msg,
                               svn_boolean_t msg_only,
                               apr_pool_t *pool)
 {
-  *msg = "two way unified diff where output is suspect";
+  *msg = "2-way unified diff where output is suspect";
   if (msg_only)
     return SVN_NO_ERROR;
 
@@ -745,9 +753,9 @@ test_two_way_unified_suspect (const char **msg,
                          "Hh\n"
                          "Ii\n",
 
-                         "--- foo15a\n"
-                         "+++ bar15a\n"
-                         "@@ -2,7 +2,6 @@\n"
+                         "--- foo15a" APR_EOL_STR
+                         "+++ bar15a" APR_EOL_STR
+                         "@@ -2,7 +2,6 @@" APR_EOL_STR
                          " Bb\n"
                          " Cc\n"
                          " Dd\n"
@@ -780,9 +788,9 @@ test_two_way_unified_suspect (const char **msg,
                          "Hh\n"
                          "Ii\n",
 
-                         "--- foo15b\n"
-                         "+++ bar15b\n"
-                         "@@ -3,6 +3,8 @@\n"
+                         "--- foo15b" APR_EOL_STR
+                         "+++ bar15b" APR_EOL_STR
+                         "@@ -3,6 +3,8 @@" APR_EOL_STR
                          " Cc\n"
                          " Dd\n"
                          " Ee\n"
@@ -815,9 +823,9 @@ test_two_way_unified_suspect (const char **msg,
                          "Hh\n"
                          "Ii\n",
 
-                         "--- foo15c\n"
-                         "+++ bar15c\n"
-                         "@@ -2,7 +2,8 @@\n"
+                         "--- foo15c" APR_EOL_STR
+                         "+++ bar15c" APR_EOL_STR
+                         "@@ -2,7 +2,8 @@" APR_EOL_STR
                          " Bb\n"
                          " Cc\n"
                          " Dd\n"
@@ -838,7 +846,7 @@ test_three_way_merge_no_overlap (const char **msg,
                                  svn_boolean_t msg_only,
                                  apr_pool_t *pool)
 {
-  *msg = "three way merge where the changes don't overlap";
+  *msg = "3-way merge, non-overlapping changes";
   if (msg_only)
     return SVN_NO_ERROR;
 
@@ -1037,7 +1045,7 @@ test_three_way_merge_with_overlap (const char **msg,
                                    svn_boolean_t msg_only,
                                    apr_pool_t *pool)
 {
-  *msg = "three way merge with non-conflicting overlapping changes";
+  *msg = "3-way merge, non-conflicting overlapping changes";
   if (msg_only)
     return SVN_NO_ERROR;
 
@@ -1194,7 +1202,7 @@ test_three_way_merge_with_conflict (const char **msg,
                                     svn_boolean_t msg_only,
                                     apr_pool_t *pool)
 {
-  *msg = "three way merge with conflicting overlapping changes";
+  *msg = "3-way merge, conflicting overlapping changes";
   if (msg_only)
     return SVN_NO_ERROR;
 
@@ -1224,15 +1232,15 @@ test_three_way_merge_with_conflict (const char **msg,
 
                             "",
 
-                            "<<<<<<< dug2\n"
+                            "<<<<<<< dug2" APR_EOL_STR
                             "Aa\n"
                             "Bb\n"
                             "Cc\n"
                             "Dd\n"
                             "Ee\n"
                             "Ff\n"
-                            "=======\n"
-                            ">>>>>>> dag2\n",
+                            "=======" APR_EOL_STR
+                            ">>>>>>> dag2" APR_EOL_STR,
                             pool));
 
   SVN_ERR (three_way_merge ("dig3", "dug3", "dag3",
@@ -1252,13 +1260,13 @@ test_three_way_merge_with_conflict (const char **msg,
 
                             "Aa\n"
                             "Bb\n"
-                            "<<<<<<< dug3\n"
+                            "<<<<<<< dug3" APR_EOL_STR
                             "Cc\n"
                             "Dd\n"
                             "Ee\n"
                             "Ff\n"
-                            "=======\n"
-                            ">>>>>>> dag3\n",
+                            "=======" APR_EOL_STR
+                            ">>>>>>> dag3" APR_EOL_STR,
                             pool));
 
   SVN_ERR (three_way_merge ("dig4", "dug4", "dag4",
@@ -1279,9 +1287,9 @@ test_three_way_merge_with_conflict (const char **msg,
                             "Aa\n"
                             "Bb\n"
                             "Cc\n"
-                            "<<<<<<< dug4\n"
-                            "Dd=======\n"
-                            "Ee>>>>>>> dag4\n",
+                            "<<<<<<< dug4" APR_EOL_STR
+                            "Dd=======" APR_EOL_STR
+                            "Ee>>>>>>> dag4" APR_EOL_STR,
                             pool));
 
   return SVN_NO_ERROR;
@@ -1334,7 +1342,7 @@ random_trivial_merge (const char **msg,
 }
 
 
-/* The "original" file has has a number of distinct lines.  We generate two
+/* The "original" file has a number of distinct lines.  We generate two
    random modifications by selecting two subsets of the original lines and
    for each selected line either adding an additional line, replacing the
    line, or deleting the line.  The two subsets are chosen so that each
@@ -1348,7 +1356,7 @@ random_three_way_merge (const char **msg,
   int i;
   apr_pool_t *subpool = svn_pool_create (pool);
 
-  *msg = apr_psprintf (pool, "random three way merge (seed:%u)", seed_val());
+  *msg = apr_psprintf (pool, "random 3-way merge (seed:%u)", seed_val());
   if (msg_only)
     return SVN_NO_ERROR;
 
@@ -1486,7 +1494,7 @@ merge_adjacent_changes (const char **msg,
                         svn_boolean_t msg_only,
                         apr_pool_t *pool)
 {
-  *msg = "three way merge with adjacent changes";
+  *msg = "3-way merge, adjacent changes";
   if (msg_only)
     return SVN_NO_ERROR;
 
