@@ -85,12 +85,13 @@ use Cwd;
 # Repository check/create
 sub init_repo
   {
-    my ( $repo, $create, $no_sync ) = @_;
+    my ( $repo, $create, $no_sync, $fsfs ) = @_;
     if ( $create )
       {
         rmtree([$repo]) if -e $repo;
         my $svnadmin_cmd = "svnadmin create $repo";
-        $svnadmin_cmd = "$svnadmin_cmd --bdb-txn-nosync" if $no_sync;
+        $svnadmin_cmd .= " --fs-type fsfs" if $fsfs;
+        $svnadmin_cmd .= " --bdb-txn-nosync" if $no_sync;
         system( $svnadmin_cmd) and die "$svnadmin_cmd: failed: $?\n";
       }
     else
@@ -159,13 +160,13 @@ sub status_update_commit
         s/\r*$//;               # [Windows compat] Remove trailing \r's
         $acceptable_error = 1 if ( /^svn:[ ]
                                    (
-                                    Transaction[ ]is[ ]out[ ]of[ ]date
+                                    Out[ ]of[ ]date
                                     |
-                                    Merge[ ]conflict[ ]during[ ]commit
+                                    Conflict[ ]at
                                     |
                                     Baseline[ ]incorrect
                                    )
-                                   $/x );
+                                   /x );
       }
     close COMMIT_ERR_READ or die "close COMMIT_ERR_READ: $!\n";
     close COMMIT_WRITE or die "close COMMIT_WRITE: $!\n";
@@ -264,11 +265,14 @@ sub ParseCommandLine
   {
     my %cmd_opts;
     my $usage = "
-usage: stress.pl [-c] [-i num] [-n num] [-s secs] [-x num] [-D num] [-F num]
-                 [-N num] [-P num] [-R path] [-S path] [-U url] [-W]
+usage: stress.pl [-c] [-h] [-i num] [-n num] [-s secs] [-x num] [-D num] 
+                 [-F num] [-N num] [-P num] [-R path] [-S path] [-U url] [-W]
+
 where
   -c cause repository creation
   -d don't make the status calls
+  -f use --fs-type fsfs during repository creation
+  -h show this help information (other options will be ignored)
   -i the ID (valid IDs are 0 to 9, default is 0 if -c given, 1 otherwise)
   -n the number of sets of changes to commit
   -p add svn:eol-style and svn:keywords properties to the files
@@ -295,17 +299,25 @@ where
     $cmd_opts{'W'} = 0;            # create with --bdb-txn-nosync
     $cmd_opts{'c'} = 0;            # create repository
     $cmd_opts{'d'} = 0;            # disable status
+    $cmd_opts{'f'} = 0;            # create with --fs-type fsfs
+    $cmd_opts{'h'} = 0;            # help
     $cmd_opts{'i'} = 0;            # ID
     $cmd_opts{'n'} = 200;          # sets of changes
     $cmd_opts{'p'} = 0;            # add file properties
     $cmd_opts{'s'} = -1;           # sleep interval
     $cmd_opts{'x'} = 4;            # files to modify
 
-    getopts( 'cdi:n:ps:x:D:F:N:P:R:U:W', \%cmd_opts ) or die $usage;
+    getopts( 'cdfhi:n:ps:x:D:F:N:P:R:U:W', \%cmd_opts ) or die $usage;
+
+    # print help info (and exit nicely) if requested
+    if ( $cmd_opts{'h'} )
+      {
+        print( $usage );
+        exit 0;
+      }
 
     # default ID if not set
     $cmd_opts{'i'} = 1 - $cmd_opts{'c'} if not $cmd_opts{'i'};
-
     die $usage if $cmd_opts{'i'} !~ /^[0-9]$/;
 
     return %cmd_opts;
@@ -328,7 +340,8 @@ srand 123456789;
 
 my %cmd_opts = ParseCommandLine();
 
-my $repo = init_repo $cmd_opts{'R'}, $cmd_opts{'c'}, $cmd_opts{'W'};
+my $repo = init_repo( $cmd_opts{'R'}, $cmd_opts{'c'}, $cmd_opts{'W'},
+                      $cmd_opts{'f'} );
 
 # [Windows compat]
 # Replace backslashes in the path, and tweak the number of slashes
